pro MSP_PPOL_Restore_Group_File, Event, ierr
	;
	;	restores the group summary file to the group structure
	;
	;	and, if present, restores history save file
	;
	;	DPC		20110828	added test for S11 queues. If absent, creates them
	;	DPC		20121230	added LLQ flags, if absent
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	;
	file = (*s_ptr).analysis_dir + 'S1_Image_Groups_and_Meta_Groups' + PATH_SEP() + 'Group_Summary.sav'
	print_text, Event, "Restoring Group File = "+file
	;
	CATCH, ierr
	if(ierr ne 0) then begin
		print_text, Event, "ERROR - Cannot restore group file "+file+". Does not exist."
		ierr = 1
		goto, MBRNF_end
	endif
	;
	RESTORE, DESCRIPTION=description, FILENAME=file
	;
	;	release the group structure memory locations, so can reassign
	;
	;	and reassign from newly read structure
	;
	PTR_struct.PTR_group = g_ptr
	;
	;help, PTR_struct.PTR_night
	;help, (*PTR_struct.PTR_night), /struct
	;
	;print, "After restore, number of images = ",(*PTR_struct.PTR_night).num_images
	;
	;	must update pointer structure
	;
	WIDGET_CONTROL, Event.top, SET_UVALUE=PTR_struct
	;
	MBRNF_end:
	CATCH, /CANCEL
	;
	;	fix any common problems with group data
	;
	;	1. if number of meta-files = 0, clear all meta-buffers
	;
	if((*g_ptr).n_meta_groups le 0) then begin
		(*g_ptr).group_meta[*] = 0
		(*g_ptr).meta_group_groups[*,*] = 0
		(*g_ptr).meta_group_group_flags[*,*] = -1
		(*g_ptr).meta_group_names[*] = ''
		(*g_ptr).meta_group_numbers[*] = 0
		(*g_ptr).meta_group_step[*] = 0
		;
		print_text, Event, "Cleared Meta-Group Inconsistencies in Group Save file"
	endif
	;
	gtags = TAG_NAMES( (*g_ptr) )
	;
	;	2. look for presence of S11 queues - if missing, add them
	;
	ind = WHERE(gtags eq STRUPCASE('S11_group_queue'),nind)
	if(nind eq 0) then begin
		print_text, Event, "Restored Group File Lacking S11 queues. Instating them"
		(*g_ptr) = CREATE_STRUCT( (*g_ptr),'S11_group_queue',intarr(1000),'S11_meta_group_queue',intarr(100) )
	endif
	;
	;	3. look for presence of group_llq_flags - if missing, add them
	;
	jnd = WHERE(gtags eq 'GROUP_LLQ_FLAGS',njnd)
	if(njnd eq 0) then begin
		print_text, Event, 'adding GROUP_LLQ_FLAGS array'
		(*g_ptr) = CREATE_STRUCT( (*g_ptr),'group_llq_flags',bytarr(1000,512))
	endif
	;
	;	history restore
	;
	file = (*s_ptr).analysis_dir + 'S1_Image_Groups_and_Meta_Groups' + PATH_SEP() + 'Problem_History.sav'
	print_text, Event, "Restoring Problem History File = "+file
	;
	CATCH, ierr
	if(ierr ne 0) then begin
		print_text, Event, "Warning - Cannot restore problem history file "+file+". Does not exist."
		ierr = 1
		goto, MBRNF_end_2
	endif
	;
	RESTORE, DESCRIPTION=description, FILENAME=file
	;
	;	if the history structure did not include enough room for
	;	all the tabs, increase its size
	;
	nh = N_ELEMENTS((*h_ptr).n_history)
	if(nh lt 12 ) then begin
		;
		;	must increase its size
		;
		hh_ptr = PTR_NEW( {history_buffer:			strarr(12,500),  $	; one buffer per main tab
							n_history:				lonarr(12),		$	; number of entries on each buffer
							max_lines:					500L} )
		for ih = 0, nh-1 do begin
			(*hh_ptr).history_buffer[ih,*] = (*h_ptr).history_buffer[ih,*]
			(*hh_ptr).n_history[ih] = (*h_ptr).n_history[ih]
		endfor
		PTR_FREE, h_ptr
		h_ptr = hh_ptr
		;
		print_text, Event, 'Extended History Structure Buffer'
	endif
	;
	;	release the group structure memory locations, so can reassign
	;
	;	and reassign from newly read structure
	;
	PTR_struct.PTR_history = h_ptr
	;
	;help, PTR_struct.PTR_night
	;help, (*PTR_struct.PTR_night), /struct
	;
	;print, "After restore, number of images = ",(*PTR_struct.PTR_night).num_images
	;
	;	must update pointer structure
	;
	WIDGET_CONTROL, Event.top, SET_UVALUE=PTR_struct
	;
	MBRNF_end_2:
	CATCH, /CANCEL
	;
end
