;	Step 0 Event Handlers
;
;---------------------------------------------------------------
pro S0_Create_New, Event
	;
	;	creates a new directory structure
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	launch file chooser
	;
	(*s_ptr).analysis_dir = ''
	;
	print_text, Event, "Create Root Analysis Directory"
	new_dir = dialog_pickfile(Title="Create Analysis Directory",/directory)
	;
	;	check for cancel, as empty directory name
	;
	if(new_dir eq '') then begin
		print_text, Event, "ERROR - No directory specified - try again."
		goto, S0CN_end
	endif
	;
	;	does directory already exist? If so, don't create. If not, error
	;
	test = FILE_INFO(new_dir)
	if(test.exists and test.directory) then begin
		print_text, Event, "Created Analysis Directory "+new_dir
		(*s_ptr).analysis_dir = new_dir
		;
		;	and update the window title to reflect this analysis directory
		;
		(*s_ptr).version_string = (*s_ptr).version_string + " dir - " + STRTRIM(new_dir,2)
		;
		TLB_Title = (*s_ptr).version_string
		tlb_wid = WIDGET_INFO( Event.top, FIND_BY_UNAME='WID_BASE')
		WIDGET_CONTROL, tlb_wid, TLB_SET_TITLE=TLB_Title
		;
		;	clear key variables
		;
		(*g_ptr).n_groups = 0L
		(*g_ptr).n_meta_groups = 0L
		(*g_ptr).S2_group_queue[*] = -2
		(*g_ptr).S3_group_queue[*] = -2
		(*g_ptr).S4_group_queue[*] = -2
		(*g_ptr).S5_group_queue[*] = -2
		(*g_ptr).S6_group_queue[*] = -2
		(*g_ptr).S7_group_queue[*] = -2
		(*g_ptr).S8_group_queue[*] = -2
		(*g_ptr).S9_group_queue[*] = -2
		(*g_ptr).S6_meta_group_queue[*] = -2
		(*g_ptr).S7_meta_group_queue[*] = -2
		(*g_ptr).S8_meta_group_queue[*] = -2
		(*g_ptr).S9_meta_group_queue[*] = -2
		;
		;	write to the text boxes
		;
		S0_Write_Text_Boxes, Event
		;
		; if not already there, create the subdirectories needed by the rest of the program
		;
		S0_Mkdir, Event, 'S0_Analysis_logs'
		S0_Mkdir, Event, 'S1_Image_Groups_and_Meta_Groups'
		S0_Mkdir, Event, 'S2_Ski_Jump_Fixes'
		S0_Mkdir, Event, 'S3_Astrometry'
		S0_Mkdir, Event, 'S4_First_PSF_Fits'
		S0_Mkdir, Event, 'S5_Star_Matching'
		S0_Mkdir, Event, 'S6_Scaling'
		S0_Mkdir, Event, 'S7_Coadding'
		S0_Mkdir, Event, 'S8_Deep_PSF_Fitting'
		S0_Mkdir, Event, 'S9_Polarimetry'
		S0_Mkdir, Event, 'S8_PHOTCAT_entries'
		S0_Mkdir, Event, 'S9_POLCAT_entries'
	endif else begin
		print_text, Event, "ERROR - Failed to create Analysis directory "+new_dir
		(*s_ptr).analysis_dir = ''
	endelse
	;
	;	create a new log file
	;
	S0_Create_New_Log_File, Event
	;
	;	save the group structure to disk, if one isn't there already
	;
	group_file = new_dir + PATH_SEP() + 'S1_Image_Groups_and_Meta_Groups' + PATH_SEP() + 'Group_Summary.sav'
	test = FILE_INFO(group_file)
	if(test.exists) then begin
		;
		;	whoops! file already there -
		;
		print_text, Event, "WARNING - Group Summary File Exists - Opening instead of Clearing"
		MSP_PPOL_Restore_Group_File, Event
	endif else begin
		print_text, Event, "Creating new Group Summary File - Analysis.sav"
		MSP_PPOL_Save_Group_File, Event
	endelse
	;
	S0CN_end:
	;WIDGET_CONTROL, Event.top, SET_UVALUE=PTR_struc
end
;
;------------------------------------------------------------------
pro S0_Create_New_Log_File, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	;
	date_time = SYSTIME()
	mon = ''
	reads, date_time, format='(4x,A3,1x,i2,1x,i2,1x,i2,1x,i2,1x,i4)',mon,day,hh,mm,ss,year
	amon = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
	imon = WHERE(mon eq amon[*])
	ymdhm = ulong64(year)*100000000L + ulong64(imon[0]+1)*1000000L + ulong64(day)*10000L + $
			ulong64(hh)*100L + ulong64(mm)
	;
	log_name = 'log_'+STRING(format='(I12.12)',ymdhm)+'.dat'
	log_dir = (*s_ptr).analysis_dir + PATH_SEP() + 'S0_Analysis_logs'
	log_path = log_dir + PATH_SEP() + log_name
	openw, lu_log, log_path, /GET_LUN
	printf, lu_log, 'Log file for MSP_PPOL opened at ', SYSTIME()
	;
	(*s_ptr).lu_log = lu_log
	(*s_ptr).log_path = log_path
end
;
;-------------------------------------------------------------------
pro S0_Open_Existing, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	;
	;	launch file chooser
	;
	print_text, Event, "Select Root Analysis Directory"
	old_dir = dialog_pickfile(Title="Open Analysis Directory",/directory)
	;
	;	does directory already exist? If so, don't create. If not, create
	;
	test = FILE_INFO(old_dir)
	if (test.exists and test.directory) then begin
		print_text, Event, "Opening Analysis Directory "+old_dir
		;
		(*s_ptr).analysis_dir = old_dir
		;
		;	fetch the summary save file
		;
		sav_file = old_dir + PATH_SEP() + 'S1_Image_Groups_and_Meta_Groups' + PATH_SEP() + 'Group_Summary.sav'
		;
		;	check that the file is present
		;
		test = FILE_INFO(sav_file)
		if(test.exists) then begin
			;
			MSP_PPOL_Restore_Group_File, Event
			;
			S0_Write_Text_Boxes, Event
			;
			;	and update the window title to reflect this analysis directory
			;
			(*s_ptr).version_string = (*s_ptr).version_string + " dir - " + STRTRIM(old_dir,2)
			;
			TLB_Title = (*s_ptr).version_string
    		tlb_wid = WIDGET_INFO( Event.top, FIND_BY_UNAME='WID_BASE')
    		WIDGET_CONTROL, tlb_wid, TLB_SET_TITLE=TLB_Title
		endif else begin
			print_text, Event, "ERROR - Group_Summary.sav file was not found"
			;
		endelse
	endif else begin
		print_text, Event, "ERROR- Failed to open Analysis directory "+FILE_DIRNAME(new_dir)
		(*s_ptr).analysis_dir = ''
	endelse
	;
	;	create a new log file
	;
	S0_Create_New_Log_File, Event
	;
	;	update group file
	;
	MSP_PPOL_Save_Group_File, Event
	;
	WIDGET_CONTROL, Event.top, SET_UVALUE=PTR_struc
end
;
;-------------------------------------------------------------------------
pro S0_Write_Text_Boxes, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	n_groups = STRING(format='(I5.3)',(*g_ptr).n_groups)
	n_meta_groups = STRING(format='(I5.3)',(*g_ptr).n_meta_groups)
	;
	WIDGET_CONTROL, (*w_ptr).s0_group_text_box_wid, SET_VALUE=n_groups
	WIDGET_CONTROL, (*w_ptr).s0_meta_group_text_box_wid, SET_VALUE=n_meta_groups
end
;
;--------------------------------------------------------------------------
pro S0_Mkdir, Event, name
	;
	;	creates a subdirectory from the current analysis directory,
	;	after first checking that one isn't already there.
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	;
	analysis_dir = (*s_ptr).analysis_dir
	;
	new_dir = analysis_dir + PATH_SEP() + name
	test = FILE_INFO(new_dir)
	if(test.exists) then begin
		print_text, Event, "ERROR - subdirectory "+name+" already exists. Will not create anew."
	endif else begin
		print_text, Event, "Creating subdirectory "+name
		FILE_MKDIR, new_dir
	endelse
end

