;
;	Step 11 Event Handler Routines
;
;		full frame polarization analysis and background region identification
;
pro S11_Init, Event
	;
	;	initialize S11 stuff when tabbing onto Step 11
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	h_wid = (*w_ptr).S11_history_textbox_wid
	;
	;	populate the group, metagroup table based on current
	;		meta-group mode
	;
	mg_mode = (*w_ptr).S11_meta_group_mode
	;
	case mg_mode of
		0:	begin
				S11_Select_group_mode, Event, 0
				n_groups = (*g_ptr).n_groups
				if(n_groups gt 0) then begin
					ind = WHERE((*g_ptr).S11_group_queue[0:n_groups-1] ge 1,count)
					if(count gt 0) then (*g_ptr).S11_group_queue[ind] = -2
				endif
				S11_populate_group_list, Event
			end
		1:	begin
				S11_Select_group_mode, Event, 1
				n_meta_groups = (*g_ptr).n_meta_groups
				if(n_meta_groups gt 0) then begin
					ind = WHERE((*g_ptr).S11_meta_group_queue[1:n_meta_groups] ge 1,count)
					if(count gt 0) then (*g_ptr).S11_meta_group_queue[ind+1] = -2
				endif
				S11_populate_meta_group_list, Event
			end
		else: print_text, Event, "ERROR - illegal meta-group mode number for Step 9"
	endcase
	;
	;	clear LEDs to red
	;
	S9_LED_sensitize, Event, 1, 0
	;
	;	initialize options buttons and buffers
	;
	(*w_ptr).S11_Pinst_pathname = 'Default H-Pol P_inst Files'
	S11_Show_Textbox, Event					; PInst table pathname
	;
	(*w_ptr).S11_Pinst_mode = 1 - (*w_ptr).S11_Pinst_mode
	S11_Toggle_pinst, Event
	;
	;S11_Show_P_Scale_Textbox,Event
	;
	;	show pinst values already cataloged
	;
	S9_Read_Pinst_Files, Event, 11
	;
	SAll_populate_history, Event, h_wid, 11
end
;

;------------------------------------------------
;
pro S11_Pinst_override, Event
	;
	;	throws up file chooser to select new P_Inst files,
	;	updates text box
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	new_dir = dialog_pickfile(Title="Select Directory with New P_Inst Files",/directory)
	if(new_dir ne '') then begin
		(*w_ptr).S11_pinst_pathname = FILE_DIRNAME(new_dir) + PATH_SEP() + FILE_BASENAME(new_dir)
		S11_Show_Textbox, Event
	endif
	;
	;	echo to screen
	;
	S9_Read_Pinst_Files, Event, 11		; the 11 tells the code to use the S11 pathname
end
;-----------------------------------------------
pro S11_Show_Textbox, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	wid = (*w_ptr).S11_Pinst_textbox_wid
	str = STRTRIM((*w_ptr).S11_Pinst_pathname,2)
	;
	WIDGET_CONTROL, wid, SET_VALUE=str
end
;-----------------------------------------------
pro S11_Show_P_Scale_Textbox, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	wid = (*w_ptr).S11_P_scale_textbox_wid
	str = STRTRIM(STRING( (*w_ptr).S11_textbox_value,format='(i)') ,2)
	;
	WIDGET_CONTROL, wid, SET_VALUE=str
end
;----------------------------------------------
pro S11_Select_group_mode, Event, meta_group_mode
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S11_meta_group_mode = meta_group_mode
	;
	WIDGET_CONTROL, (*w_ptr).S11_show_groups_wid, SET_BUTTON=(1-meta_group_mode)
	WIDGET_CONTROL, (*w_ptr).S11_show_meta_groups_wid, SET_BUTTON=meta_group_mode
	;
	case meta_group_mode of
		0: begin
			print_text, Event, "Selecting Single Group Mode"
			S11_populate_group_list, Event
		   end
		1: begin
			print_text, Event, "Selecting Meta-Group Mode"
			S11_populate_meta_group_list, Event
		   end
	endcase
end
;
;-----------------------------------------------
pro S11_Toggle_pinst, Event
	;
	;	toggle this mode
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S11_pinst_mode = 1 - (*w_ptr).S11_pinst_mode
	;
	WIDGET_CONTROL, (*w_ptr).S11_use_pinst_correction_wid, SET_BUTTON=(*w_ptr).S11_pinst_mode
	;
	case (*w_ptr).S11_pinst_mode of
		0: print_text, Event, "Changing mode to NOT use P_Inst corrections"
		1: print_text, Event, "Changing mode to USE P_Inst corrections"
	endcase
	;
	;	update echo to screen
	;
	S9_Read_Pinst_Files, Event, 11
	;
end
;
;-----------------------------------------------
pro S11_Comments_text, Event
	;
	;	handle insert character events for FITS comments
	;		- text is always valid, CRs ignored --
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	first, fetch current pointer into text box buffer
	;
	nch = (*w_ptr).S11_comments_nch
	wid = (*w_ptr).S11_comments_textbox_wid
	;
	;	if it is zero, check whether something is already there
	;
	if(nch eq 0) then begin
		mch = WIDGET_INFO(wid,/TEXT_NUMBER)
		if(mch ne 0) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			mch = STRLEN(existing_text[0])
			;
			;	pack into character buffer
			;
			(*w_ptr).S11_comments_buffer[*] = ''
			;
			for im = 0, mch-1 do begin
				(*w_ptr).S11_comments_buffer[im] = STRMID(existing_text, im, 1)
			endfor
			(*w_ptr).S11_comments_nch = mch
			nch = mch
		endif
	endif
	;
	if(Event.type eq 0) then begin
		;
		;	if character not CR, put it into buffer
		;
		ch = Event.ch
		if(ch ne 10B) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			(*w_ptr).S11_comments_nch = STRLEN(existing_text)
			for il = 0, STRLEN(existing_text[0])-1 do (*w_ptr).S11_comments_buffer[il] = STRMID(existing_text, il, 1)
			WIDGET_CONTROL, wid, SET_TEXT_SELECT=[Event.offset,0]
		endif else begin
			;
			;	was a character return, ignore
		endelse
		;
	endif else begin
		if(Event.type eq 1) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			;
			name = STRTRIM(Event.str,2)
			length = STRLEN(name)
			if(length gt 1) then begin
				this_name = strarr(length)
				for il=0, length-1 do begin
					this_name[il] = STRMID(name,il,1)
				endfor
			endif else begin
				if(length eq 0) then goto, S11CT_end
				;
				this_name = strarr(1)
				this_name[0] = name
			endelse
		endif else begin
			if(Event.type eq 2) then begin
				WIDGET_CONTROL, wid, GET_VALUE=existing_text
				if(Event.length eq 0) then goto, S11CT_end
			endif else begin
				print_text, Event, "Event not recognized"
			endelse
			new_name = 0
		endelse
		(*w_ptr).S11_comments_nch = STRLEN(existing_text)
		for il = 0, STRLEN(existing_text[0])-1 do (*w_ptr).S11_comments_buffer[il] = STRMID(existing_text, il, 1)
		WIDGET_CONTROL, wid, SET_TEXT_SELECT=[Event.offset,0]
	endelse
	;
	S11CT_end:
end
;
;----------------------------------------------------------------
pro S11_Modify_textbox, Event
	;
	;	handle insert character events for P scale override value
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	first, fetch current pointer into text box buffer
	;
	wid = (*w_ptr).S11_P_scale_textbox_wid
	;
	nch = (*w_ptr).S11_textbox_nch
	;
	;	if it is zero, check whether something is already there
	;
	;** debug - dump some stuff
	;
	;print, "wid, nch = ",wid, nch
	;help, Event, /struct
	;
	existing_text = ''
	if(nch eq 0) then begin
		mch = WIDGET_INFO(wid,/TEXT_NUMBER)
		if(mch gt 1) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			mch = STRLEN(existing_text[0])
			;
			;	pack into character buffer
			;
			(*w_ptr).S11_textbox_buffer[*] = ''
			;
			for im = 0, mch-1 do begin
				(*w_ptr).S11_textbox_buffer[im] = STRMID(existing_text, im, 1)
			endfor
			(*w_ptr).S11_textbox_nch = mch
			nch = mch
		endif
	endif
	;
	;print, "after looking at existing text on arrival, "
	;print, "existing text = ",existing_text
	;print, "nch = ",nch
	;
	if(Event.type eq 0) then begin
		;
		;	if character not CR, put it into buffer
		;
		ch = Event.ch
		if(ch ne 10B) then begin
;			WIDGET_CONTROL, wid, GET_VALUE=existing_text
;			print, "existing text = ",existing_text
;			(*w_ptr).S9_textbox_nch = STRLEN(existing_text)
;			print, "string length = ",(*w_ptr).S9_textbox_nch
;			for il = 0, STRLEN(existing_text[0])-1 do (*w_ptr).S9_textbox_buffer[il] = STRMID(existing_text, il, 1)
;			WIDGET_CONTROL, wid, SET_TEXT_SELECT=[Event.offset,0]
			(*w_ptr).S11_textbox_nch = 1
			(*w_ptr).S11_textbox_buffer[0] = STRING(Event.ch)
			reads, STRING(Event.ch),format='(i)',value
			;print, "decoded value = ",value
			(*w_ptr).S11_textbox_value = value
		endif else begin
			;
			;	was a character return, interpret and load value buffer
			;
			;		- first, check for non-zero buffer length
			;
			if((*w_ptr).S11_textbox_nch gt 0) then begin
				;
				;		- load the buffer into a single string
				;
				name = ''
				for ich = 0, (*w_ptr).S11_textbox_nch-1 do begin
					name = name + (*w_ptr).S11_textbox_buffer[ich]
				endfor
				;
				name = STRTRIM(name,2)
				;
				;	new name available - read it into value buffer
				;
				reads, name, format='(i)',value
				(*w_ptr).S11_textbox_value = value
				;
				print_text, Event, "Setting P Scale Value to "+name+" percent"
				S11_Show_P_Scale_Textbox, Event
				;
			endif else begin
				print_text, Event, "Warning - no characters in textbox"
			endelse
			;
		endelse
		;
		;print_text, Event, "Insert Character Event - Character = "+STRING(format='(a)',Event.ch)
	endif else begin
		if(Event.type eq 1) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			;
			name = STRTRIM(Event.str,2)
			length = STRLEN(name)
			if(length gt 1) then begin
				this_name = strarr(length)
				for il=0, length-1 do begin
					this_name[il] = STRMID(name,il,1)
				endfor
			endif else begin
				if(length eq 0) then goto, S11PT_end
				;
				this_name = strarr(1)
				this_name[0] = name
			endelse
			existing_text = this_name
		endif else begin
			if(Event.type eq 2) then begin
				;
				;	delete type event, go to offset, delete length
				;
				WIDGET_CONTROL, wid, GET_VALUE=existing_text
				if(Event.length eq 0) then goto, S11PT_end
				existing_text = ''
			endif else begin
				print_text, Event, "Event not recognized"
			endelse
		endelse
		(*w_ptr).S11_textbox_nch = STRLEN(existing_text)
		if( (*w_ptr).S11_textbox_nch gt 0) then begin
			for il = 0, STRLEN(existing_text[0])-1 do (*w_ptr).S11_textbox_buffer[il] = STRMID(existing_text, il, 1)
			WIDGET_CONTROL, wid, SET_TEXT_SELECT=[Event.offset,0]
		endif else begin
			(*w_ptr).S11_textbox_buffer[*] = ''
			(*w_ptr).S11_textbox_buffer[0] = ''
			(*w_ptr).S11_textbox_nch = 0
			(*w_ptr).S11_textbox_value = -1
		endelse
	endelse
	;
	S11PT_end:
	S11_Show_P_Scale_Textbox, Event
end
;-----------------------------------------------
pro S11_Select_Groups, Event
	;
	;	select groups or metagroups
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	get group/metagroup mode
	;
	tab_no = (*w_ptr).S11_meta_group_mode
	;
	wid = (*w_ptr).S11_group_listbox_wid
	;
	index = WIDGET_INFO( wid, /LIST_SELECT)
    n = N_ELEMENTS(index)
    skip = 0
	if(n eq 1 and index[0] eq -1) then skip = 1
	if(skip eq 0) then begin
		;
		case tab_no of
			0:	begin
					ind = WHERE((*g_ptr).group_step[index] ge 7 and $
			    				(*g_ptr).group_step[index] ne 31 and $
			    				(*g_ptr).group_step[index] ne 32, count)
					if(count gt 0) then begin
						index = index[ind]
						if(n ne count) then print_text, Event, "Warning - some groups not processed through Step 7 - skipping those"
						n = count
					endif else begin
						print_text, Event, "Warning - no groups processed through Step 7. Go back"
						goto, S11_SG_end
					endelse
					;
					(*g_ptr).n_groups_selected = n
					(*g_ptr).selected_groups[0:n-1] = index[*]
					if(n eq 1) then begin
						group = (*g_ptr).group_names[index[0]]
						print_text, Event, "Selected Group "+group+" from Group List"
					endif else begin
						print_text, Event, "Selected "+STRING(format='(I)',n)+" groups from list"
					endelse
				end
			1:	begin
					ind = WHERE((*g_ptr).meta_group_step[index+1] ge 7, count)
					if(count gt 0) then begin
						index = index[ind]
						if(n ne count) then print_text, Event, "Warning - some meta-groups not processed through Step 7 - skipping those"
						n = count
					endif else begin
						print_text, Event, "Warning - no meta-groups processed through Step 7. Go back"
						goto, S11_SG_end
					endelse
					(*g_ptr).n_meta_groups_selected = n
					(*g_ptr).selected_meta_groups[1:n] = index[*]+1
					if(n eq 1) then begin
						group = (*g_ptr).meta_group_names[index[0]+1]
						print_text, Event, "Selected Group "+group+" from Group List"
					endif else begin
						print_text, Event, "Selected "+STRING(format='(I)',n)+" groups from list"
					endelse
				end
		endcase
	   ;
	endif else print_text, Event, "Warning - no groups selected"
	S11_SG_end:
end
;
;-----------------------------------------------
pro S11_Add_groups, Event
	;
	;	selected groups are added to the group or metagroup queues
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	get mode
	;
	tab_no = (*w_ptr).S11_meta_group_mode
	;
	case tab_no of
		0:	begin
				;
				;	how many selected?
				;
				n = (*g_ptr).n_groups_selected
				;
				if(n gt 0) then begin
					for ig = 0, n-1 do begin
						j = (*g_ptr).selected_groups[ig]
						;
						;	set queue pointer to 2 (enqueued)
						;
						(*g_ptr).S11_group_queue[j] = 2
					endfor
				endif
				;
				;	reload listbox
				;
				S11_populate_group_list, Event
				;
				;	clear selections
				;
				(*g_ptr).n_groups_selected = 0
				if(n gt 0) then (*g_ptr).selected_groups[0:n-1] = 0
			end
		1: 	begin
				;
				n = (*g_ptr).n_meta_groups_selected
				;
				if(n gt 0) then begin
					(*g_ptr).S11_meta_group_queue[0]= -1
					for ig = 1, n do begin
						j = (*g_ptr).selected_meta_groups[ig]
						;
						;	set queue pointer to 2 (enqueued)
						;
						(*g_ptr).S11_meta_group_queue[j] = 2
					endfor
					print_text, Event, "Added "+STRING(n,format='(I)')+" meta-groups to queue"
				endif
				;
				;	reload listbox
				;
				S11_populate_meta_group_list, Event
				;
				(*g_ptr).n_meta_groups_selected = 0
				if(n gt 0) then (*g_ptr).selected_meta_groups[1:n] = 0
			end
	endcase

end
;-----------------------------------------------
pro S11_Remove_groups, Event
	;
	;	selected groups are removed (skipped) indicated queue
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	get mode
	;
	tab_no = (*w_ptr).S11_meta_group_mode
	;
	case tab_no of
		0:	begin
				;
				;	how many selected?
				;
				n = (*g_ptr).n_groups_selected
				;
				if(n gt 0) then begin
					for ig = 0, n-1 do begin
						j = (*g_ptr).selected_groups[ig]
						;
						(*g_ptr).S11_group_queue[j] = -1
					endfor
				endif
				;
				;	reload listbox
				;
				S11_populate_group_list, Event
				;
				;	clear selections
				;
				(*g_ptr).n_groups_selected = 0
				if(n gt 0) then (*g_ptr).selected_groups[0:n-1] = 0
			end
		1:	begin
				;
				;	how many selected?
				;
				n = (*g_ptr).n_meta_groups_selected
				;
				if(n gt 0) then begin
					for ig = 1, n do begin
						j = (*g_ptr).selected_meta_groups[ig]
						;
						(*g_ptr).S11_meta_group_queue[j] = -1
					endfor
				endif
				;
				S11_populate_meta_group_list, Event
				;
				(*g_ptr).n_meta_groups_selected = 0
				if(n gt 0) then (*g_ptr).selected_meta_groups[1:n] = 0
			end
	endcase
end
;
;-----------------------------------------------
pro S11_LED_sensitize, Event, S11_LED_number, sense
	;
	;	initializes LED stacks
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	if(S11_LED_number eq 1) then begin
		wid = (*w_ptr).S11_LED1_groups_wid
	endif
	;
	WIDGET_CONTROL, wid, SENSITIVE=sense
	;
	;	set all colors to red upon sensitization
	;
	WIDGET_CONTROL, wid, GET_UVALUE=nLED
	;
	n_ele = N_ELEMENTS(nLED)
	if(n_ele eq 0) then nLED = 25
	if(nLED eq 0) then nLED = 25
	WIDGET_CONTROL, wid, SET_UVALUE=nLED
	colors = strarr(nLED)
	if(sense eq 1) then colors[*] = "red" else colors[*] = "gray"
	WCT_LED_stack, wid, colors
end
;
;-----------------------------------------------
pro S11_populate_group_list, Event
	;
	;	lists current groups and step status by name on the listbox
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	n_groups = (*g_ptr).n_groups
	if(n_groups gt 0) then begin
		;
		;	load group name,step and meta status, and queue status
		;
		text_vec = strarr(n_groups)
		for ig = 0, n_groups-1 do begin
			if((*g_ptr).group_step[ig] lt 7) then (*g_ptr).S11_group_queue[ig] = -2
			i_queue = (*g_ptr).S11_group_queue[ig]
			;
			case i_queue of
				-3: queue = "to Meta"
				-2:	queue = '?'
				-1: queue = 'Skip'
				 0: queue = 'Done'
				 1: queue = 'Active'
				 2: queue = 'Queued'
				else: queue = '...'
			endcase
			;
			grp_step = (*g_ptr).group_step[ig]
			if((grp_step ge 30 and grp_step le 40) or $
			   (grp_step ge 60 and grp_step le 70) or $
			   (grp_step ge 110 and grp_step le 120)) then begin
			   step_0 = FLOOR(grp_step / 10)
			   step_1 = grp_step - 10 * step_0
			   text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,".",I1.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],step_0,step_1,(*g_ptr).group_meta[ig],queue),1)

			endif else begin
				text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],(*g_ptr).group_step[ig],(*g_ptr).group_meta[ig],queue),1)
			endelse
			;
			;text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,", Meta=",I2.1,", Action= ",A)',$
			;		(*g_ptr).group_names[ig],(*g_ptr).group_step[ig],(*g_ptr).group_meta[ig],queue),1)
		endfor
	endif else begin
		text_vec = ['','']
	endelse
	;
	WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_VALUE=text_vec
	;
	(*g_ptr).n_groups_selected = 0
	(*g_ptr).selected_groups[*] = 0
end
;
;------------------------------------------------------------------
pro S11_populate_meta_group_list, Event
	;
	;	loads the meta-name listbox on the indicated tab_number
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	wid = (*w_ptr).S11_group_listbox_wid
	;
	n_meta = (*g_ptr).n_meta_groups
	if(n_meta gt 0) then begin
		;
		text_vec = strarr(n_meta)
		for img = 1, n_meta do begin
			;
			(*g_ptr).S11_meta_group_queue[0] = -1
			if((*g_ptr).meta_group_step[img] lt 7) then (*g_ptr).S11_meta_group_queue[img] = -2
			case (*g_ptr).S11_meta_group_queue[img] of
				-2:	queue = '?'
				-1: queue = 'Skip'
				 0: queue = 'Done'
				 1: queue = 'Active'
				 2: queue = 'Queued'
				else: queue = '...'
			endcase
			;
			grp_step = (*g_ptr).meta_group_step[img]
			if((grp_step ge 30 and grp_step le 40) or $
			   (grp_step ge 60 and grp_step le 70) or $
			   (grp_step ge 110 and grp_step le 120)) then begin
			   step_0 = FLOOR(grp_step / 10)
			   step_1 = grp_step - 10 * step_0
			   text_vec[img-1] = STRTRIM(STRING(format='(A," Step=",I2.1,".",I1.1,", Action= ",A)',$
					(*g_ptr).meta_group_names[img],step_0,step_1,queue),1)

			endif else begin
				text_vec[img-1] = STRTRIM(STRING(format='(A," Step=",I2.1," Action= ",A)',$
					(*g_ptr).meta_group_names[img],(*g_ptr).meta_group_step[img],queue),1)
			endelse
		endfor
	endif else begin
		text_vec = ['','']
	endelse
	;
	WIDGET_CONTROL, wid, SET_VALUE=text_vec
	;
	(*g_ptr).n_meta_groups_selected = 0
	(*g_ptr).selected_meta_groups[*] = 0
end
;
;------------------------------------------------
pro S11_Find_backgrounds, Event
	;
	;	uses the queue to send groups to the background
	;	finder and to store the results to the S11 directory
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	h_wid = (*w_ptr).S11_history_textbox_wid
	;
	inst_corr_flag = (*w_ptr).S11_Pinst_mode
	;
	meta_flag = (*w_ptr).S11_meta_group_mode
	;
	LED1_wid = (*w_ptr).S11_LED1_groups_wid
	;
	case meta_flag of
		0:	begin
				this_queue = (*g_ptr).S11_group_queue[0:(*g_ptr).n_groups-1]
				these_names = (*g_ptr).group_names[0:(*g_ptr).n_groups-1]
			end
		1:  begin
				this_queue = (*g_ptr).S11_meta_group_queue[1:(*g_ptr).n_meta_groups]
				these_names = (*g_ptr).meta_group_names[1:(*g_ptr).n_meta_groups]
			end
	endcase
	;
	;	how many active items on the queue?
	;
	ind_queue = WHERE(this_queue[*] eq 2,count)
	if(count le 0) then begin
		; if none, quit
		print_text, Event, "WARNING - no items in queue to process."
		goto, S11_FB_end
	endif
	;
	print_text, Event, "Found "+STRING(format='(I)',count)+" groups in the queue. Processing starting"
	;
	;	Clear LED1 to red with the number of groups to process
	;
	WIDGET_CONTROL, LED1_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "red"
	S11_LED_sensitize, Event, 1, 1
	;
	;	for each of the enqueued groups:
	;
	for ig = 0, count-1 do begin
		;
		g_ind = ind_queue[ig]
		group_name = these_names[g_ind]
		;
		case meta_flag of
			0: begin
				print_text, Event, "Starting Background Analysis for Group = "+group_name
				(*g_ptr).S11_group_queue[g_ind] = 1
				S11_populate_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
			1: begin
				print_text, Event, "Starting Background Analysis for Meta-Group = "+group_name
				(*g_ptr).S11_meta_group_queue[g_ind+1] = 1
				S11_populate_meta_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
		endcase
		;
		;	change the LED1 bar to blue
		;
		group_colors[ig] = "blue"
		WCT_LED_stack, LED1_wid, group_colors
		;
		;	for the pathname to the deep coadd image
		;
		deep_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
		deep_path = deep_dir + group_name + '_Deep_Coadd.fits'
		;
		;	push to heap, set up pointer in structure
		;
		my_ptr = PTR_NEW(deep_path)
		;
		;	call the background fitter
		;
		ID_gui, GROUP_LEADER=Event.top, _Extra={ptr_path: my_ptr}
		;
		;	get status from the heap variable
		;
		if( (*my_ptr) eq '1') then status = 0 else status = -1
		;
		;	done, clear heap
		;
		PTR_FREE, my_ptr
		;
		wait, 1
		;
		;	evaluate completion status: 0=OK, -1=bad
		;
		case status of
			0: begin
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S11_group_queue[g_ind] = 0
							;
							;	and update step status value for this group

					;
							(*g_ptr).group_step[g_ind] = 111
							S11_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S11_meta_group_queue[g_ind+1] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 111
							S11_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
				;
				;	change color of group LED to green
				;
				group_colors[ig] = "green"
				WCT_LED_stack, LED1_wid, group_colors
				;
			  end
			  ;
		  -1: begin
		  		;
				;	change color of group LED to yellow
				;
				group_colors[ig] = "yellow"
				WCT_LED_stack, LED1_wid, group_colors
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S11_group_queue[g_ind] = -2
							;
							;	and don't step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 7
							S11_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S11_meta_group_queue[g_ind+1] = -2
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 7
							S11_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
			 end
		endcase
		MSP_PPOL_Save_Group_File, Event
	endfor
	;
	;	all groups done.
	;
	print_text, Event, "All Queued Groups have had backgrounds found and fit"
	;
	S11_FB_end:
end
;
;------------------------------------------------
pro S11_Start_background_removal, Event
	;
	;	uses the queue to send groups to the background
	;	fitter and to store the results to the S11 directory
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	h_wid = (*w_ptr).S11_history_textbox_wid
	;
	inst_corr_flag = (*w_ptr).S11_Pinst_mode
	;
	meta_flag = (*w_ptr).S11_meta_group_mode
	;
	LED1_wid = (*w_ptr).S11_LED1_groups_wid
	;
	case meta_flag of
		0:	begin
				this_queue = (*g_ptr).S11_group_queue[0:(*g_ptr).n_groups-1]
				these_names = (*g_ptr).group_names[0:(*g_ptr).n_groups-1]
			end
		1:  begin
				this_queue = (*g_ptr).S11_meta_group_queue[1:(*g_ptr).n_meta_groups]
				these_names = (*g_ptr).meta_group_names[1:(*g_ptr).n_meta_groups]
			end
	endcase
	;
	;	how many active items on the queue?
	;
	ind_queue = WHERE(this_queue[*] eq 2,count)
	if(count le 0) then begin
		; if none, quit
		print_text, Event, "WARNING - no items in queue to process."
		goto, S11_FBR_end
	endif
	;
	print_text, Event, "Found "+STRING(format='(I)',count)+" groups in the queue. Processing starting"
	;
	;	Clear LED1 to red with the number of groups to process
	;
	WIDGET_CONTROL, LED1_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "red"
	S11_LED_sensitize, Event, 1, 1
	;
	;	for each of the enqueued groups:
	;
	for ig = 0, count-1 do begin
		;
		g_ind = ind_queue[ig]
		group_name = these_names[g_ind]
		;
		case meta_flag of
			0: begin
				print_text, Event, "Starting Background Removal for Group = "+group_name
				(*g_ptr).S11_group_queue[g_ind] = 1
				S11_populate_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
			1: begin
				print_text, Event, "Starting Background Removal for Meta-Group = "+group_name
				(*g_ptr).S11_meta_group_queue[g_ind+1] = 1
				S11_populate_meta_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
		endcase
		;
		;	change the LED1 bar to blue
		;
		group_colors[ig] = "blue"
		WCT_LED_stack, LED1_wid, group_colors
		;
		;	for the pathname to the deep coadd image
		;
		deep_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
		deep_path = deep_dir + group_name + '_Deep_Coadd.fits'
		;
		;	remove backgrounds from all of the HWP images, storing them
		;	in the S11 directory
		;
		S11_remove_backgrounds, Event, deep_path, group_colors, ig, status
		;
		case meta_flag of
			0:	begin
					print_text, Event, "Finished Background Removal for Group = "+group_name
				end
			1:	begin
					print_text, Event, "Finished Background Removal for Meta-Group = "+group_name
				end
		endcase
		;
		wait, 1
		;
		;	evaluate completion status: 0=OK, -1=bad
		;
		case status of
			0: begin
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S11_group_queue[g_ind] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 113
							S11_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S11_meta_group_queue[g_ind+1] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 113
							S11_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
				;
				;	change color of group LED to green
				;
				group_colors[ig] = "green"
				WCT_LED_stack, LED1_wid, group_colors
				;
			  end
			  ;
		  -1: begin
		  		;
				;	change color of group LED to yellow
				;
				group_colors[ig] = "yellow"
				WCT_LED_stack, LED1_wid, group_colors
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S11_group_queue[g_ind] = -2
							;
							;	and don't step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 7
							S11_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S11_meta_group_queue[g_ind+1] = -2
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 7
							S11_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
			 end
		endcase
		MSP_PPOL_Save_Group_File, Event
	endfor
	;
	;	all groups done.
	;
	print_text, Event, "All Queued Groups have had backgrounds found and fit"
	;
	S11_FBR_end:
end
;
;
;------------------------------------------------
pro S11_Skip_background_removal, Event
	;
	;	uses the queue to send groups to the background
	;	fitter and to store the results to the S11 directory
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	h_wid = (*w_ptr).S11_history_textbox_wid
	;
	inst_corr_flag = (*w_ptr).S11_Pinst_mode
	;
	meta_flag = (*w_ptr).S11_meta_group_mode
	;
	LED1_wid = (*w_ptr).S11_LED1_groups_wid
	;
	case meta_flag of
		0:	begin
				this_queue = (*g_ptr).S11_group_queue[0:(*g_ptr).n_groups-1]
				these_names = (*g_ptr).group_names[0:(*g_ptr).n_groups-1]
			end
		1:  begin
				this_queue = (*g_ptr).S11_meta_group_queue[1:(*g_ptr).n_meta_groups]
				these_names = (*g_ptr).meta_group_names[1:(*g_ptr).n_meta_groups]
			end
	endcase
	;
	;	how many active items on the queue?
	;
	ind_queue = WHERE(this_queue[*] eq 2,count)
	if(count le 0) then begin
		; if none, quit
		print_text, Event, "WARNING - no items in queue to process."
		goto, S11_FBS_end
	endif
	;
	print_text, Event, "Found "+STRING(format='(I)',count)+" groups in the queue. Processing starting"
	;
	;	Clear LED1 to red with the number of groups to process
	;
	WIDGET_CONTROL, LED1_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "red"
	S11_LED_sensitize, Event, 1, 1
	;
	;	for each of the enqueued groups:
	;
	for ig = 0, count-1 do begin
		;
		g_ind = ind_queue[ig]
		group_name = these_names[g_ind]
		;
		case meta_flag of
			0: begin
				print_text, Event, "Starting Skipping of Background Removal for Group = "+group_name
				(*g_ptr).S11_group_queue[g_ind] = 1
				S11_populate_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
			1: begin
				print_text, Event, "Starting Skipping of Background Removal for Meta-Group = "+group_name
				(*g_ptr).S11_meta_group_queue[g_ind+1] = 1
				S11_populate_meta_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
		endcase
		;
		;	change the LED1 bar to yellow
		;
		group_colors[ig] = "yellow"
		WCT_LED_stack, LED1_wid, group_colors
		;
		;	for the pathname to the deep coadd image
		;
		deep_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
		deep_path = deep_dir + group_name + '_Deep_Coadd.fits'
		;
		;	skip removal of backgrounds from all of the HWP images, storing them
		;	in the S11 directory
		;
	;	if(status eq 0) then S11_remove_backgrounds, Event, deep_path, group_colors, ig, status
		;
		case meta_flag of
			0:	begin
					print_text, Event, "Finished Skipping Background Removal for Group = "+group_name
				end
			1:	begin
					print_text, Event, "Finished Skipping Background Removal for Meta-Group = "+group_name
				end
		endcase
		;
		wait, 1
		status = 0
		;
		;	evaluate completion status: 0=OK, -1=bad
		;
		case status of
			0: begin
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S11_group_queue[g_ind] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 112
							S11_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S11_meta_group_queue[g_ind+1] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 112
							S11_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
				;
				;	change color of group LED to green
				;
				group_colors[ig] = "green"
				WCT_LED_stack, LED1_wid, group_colors
				;
			  end
			  ;
		  -1: begin
		  		;
				;	change color of group LED to yellow
				;
				group_colors[ig] = "yellow"
				WCT_LED_stack, LED1_wid, group_colors
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S11_group_queue[g_ind] = -2
							;
							;	and don't step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 7
							S11_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S11_meta_group_queue[g_ind+1] = -2
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 7
							S11_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
			 end
		endcase
		MSP_PPOL_Save_Group_File, Event
	endfor
	;
	;	all groups done.
	;
	print_text, Event, "All Queued Groups have had backgrounds found and fit"
	;
	S11_FBS_end:
end
;
;------------------------------------------------------
pro S11_Start_analysis, Event
	;
	;	Begin processing of all enqueued requests for polarimetry processing
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	h_wid = (*w_ptr).S11_history_textbox_wid
	;
	inst_corr_flag = (*w_ptr).S11_Pinst_mode
	;
	meta_flag = (*w_ptr).S11_meta_group_mode
	;
	LED1_wid = (*w_ptr).S11_LED1_groups_wid
	;
	;	if correction with instrumental polarization flag set, reload the images, values
	;
	if(inst_corr_flag eq 1) then begin
		S9_Read_Pinst_Files, Event, 11
		print_text, Event, 'Read in the default PInst correction files'
	endif
	;
	case meta_flag of
		0:	begin
				this_queue = (*g_ptr).S11_group_queue[0:(*g_ptr).n_groups-1]
				these_names = (*g_ptr).group_names[0:(*g_ptr).n_groups-1]
			end
		1:  begin
				this_queue = (*g_ptr).S11_meta_group_queue[1:(*g_ptr).n_meta_groups]
				these_names = (*g_ptr).meta_group_names[1:(*g_ptr).n_meta_groups]
			end
	endcase
	;
	;	how many active items on the queue?
	;
	ind_queue = WHERE(this_queue[*] eq 2,count)
	if(count le 0) then begin
		; if none, quit
		print_text, Event, "WARNING - no items in queue to process."
		goto, S11_SA_end
	endif
	;
	print_text, Event, "Found "+STRING(format='(I)',count)+" groups in the queue. Processing starting"
	;
	;	Clear LED1 to red with the number of groups to process
	;
	WIDGET_CONTROL, LED1_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "red"
	S11_LED_sensitize, Event, 1, 1
	;
	;	for each of the enqueued groups:
	;
	for ig = 0, count-1 do begin
		;
		g_ind = ind_queue[ig]
		group_name = these_names[g_ind]
		;
		;	test that backgrounds have been removed
		;
		case meta_flag of
			0:	begin
					grp_step = (*g_ptr).group_step[g_ind]
				end
			1: 	begin
					grp_step = (*g_ptr).meta_group_step[g_ind+1]
				end
		endcase
		if(grp_step lt 113) then begin
			print_text, Event, 'ERROR - group '+group_name+' not background removed yet - back up'
			goto, S11gp_end
		endif
		;
		case meta_flag of
			0: begin
				print_text, Event, "Starting Polarimetry Analysis for Group = "+group_name
				(*g_ptr).S11_group_queue[g_ind] = 1
				S11_populate_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
			1: begin
				print_text, Event, "Starting Polarimetry Analysis for Meta-Group = "+group_name
				(*g_ptr).S11_meta_group_queue[g_ind+1] = 1
				S11_populate_meta_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
		endcase
		;
		;	change the LED1 bar to blue
		;
		group_colors[ig] = "blue"
		WCT_LED_stack, LED1_wid, group_colors
		;
		deep_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
		deep_path = deep_dir + group_name + '_Deep_Coadd.fits'
		;
		S11_compute_UQ_images, Event, deep_path, group_colors, ig, status
		;
		;	if PInst correction flag set, correct
		;
		if(status eq 0 and inst_corr_flag eq 1) then S11_PInst_correct, Event, deep_path, group_colors, ig, status
		;
		if(status eq 0) then S11_compute_P_PA_images, Event, deep_path, group_colors, ig, status
		;
;		if(status eq 0) then S11_make_DS9_reg_file, Event, deep_path, status
		;
		case meta_flag of
			0:	begin
					print_text, Event, "Finished Polarimetric Analysis for Group = "+group_name
				end
			1:	begin
					print_text, Event, "Finished Polarimetric Analysis for Meta-Group = "+group_name
				end
		endcase
		;
		wait, 1
		;
		;	evaluate completion status: 0=OK, -1=bad
		;
		case status of
			0: begin
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S11_group_queue[g_ind] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 119
							S11_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S11_meta_group_queue[g_ind+1] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 119
							S11_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
				;
				;	change color of group LED to green
				;
				group_colors[ig] = "green"
				WCT_LED_stack, LED1_wid, group_colors
				;
			  end
			  ;
		  -1: begin
		  		;
				;	change color of group LED to yellow
				;
				group_colors[ig] = "yellow"
				WCT_LED_stack, LED1_wid, group_colors
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S11_group_queue[g_ind] = -2
							;
							;	and don't step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 7
							S11_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S11_meta_group_queue[g_ind+1] = -2
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 7
							S11_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
			 end
		endcase
		;
		S11gp_end:
		;
		MSP_PPOL_Save_Group_File, Event
	endfor
	;
	;	all groups done.
	;
	print_text, Event, "All Queued Groups have had Polarimetry Analysis performed"
	;
	;	if instrumental correction was selected, unload all the heap variables
	;
	if(inst_corr_flag eq 1) then begin
		PTR_FREE, (*w_ptr).S9_u_inst_ptr
		PTR_FREE, (*w_ptr).S9_q_inst_ptr
		PTR_FREE, (*w_ptr).S9_u_sig_inst_ptr
		PTR_FREE, (*w_ptr).S9_q_sig_inst_ptr
		PTR_FREE, (*w_ptr).S9_P_inst_values_ptr
	endif
	;
	S11_SA_end:
end
;
;------------------------------------------------------
pro S11_Skip_analysis, Event
	;
	;	skips processing of enqueued request for polarimetry processing
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	h_wid = (*w_ptr).S11_history_textbox_wid
	;
	inst_corr_flag = (*w_ptr).S11_Pinst_mode
	;
	meta_flag = (*w_ptr).S11_meta_group_mode
	;
	LED1_wid = (*w_ptr).S11_LED1_groups_wid
	;
	;	if correction with instrumental polarization flag set, reload the images, values
	;
	if(inst_corr_flag eq 1) then begin
		S9_Read_Pinst_Files, Event, 11
		print_text, Event, 'Read in the default PInst correction files'
	endif
	;
	case meta_flag of
		0:	begin
				this_queue = (*g_ptr).S11_group_queue[0:(*g_ptr).n_groups-1]
				these_names = (*g_ptr).group_names[0:(*g_ptr).n_groups-1]
			end
		1:  begin
				this_queue = (*g_ptr).S11_meta_group_queue[1:(*g_ptr).n_meta_groups]
				these_names = (*g_ptr).meta_group_names[1:(*g_ptr).n_meta_groups]
			end
	endcase
	;
	;	how many active items on the queue?
	;
	ind_queue = WHERE(this_queue[*] eq 2,count)
	if(count le 0) then begin
		; if none, quit
		print_text, Event, "WARNING - no items in queue to process."
		goto, S11_SKA_end
	endif
	;
	print_text, Event, "Found "+STRING(format='(I)',count)+" groups in the queue. Skipping starting"
	;
	;	Clear LED1 to yellow with the number of groups to process
	;
	WIDGET_CONTROL, LED1_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "yellow"
	S11_LED_sensitize, Event, 1, 1
	;
	;	for each of the enqueued groups:
	;
	for ig = 0, count-1 do begin
		;
		g_ind = ind_queue[ig]
		group_name = these_names[g_ind]
		;
		;	test that backgrounds have been removed
		;
		case meta_flag of
			0:	begin
					grp_step = (*g_ptr).group_step[g_ind]
				end
			1: 	begin
					grp_step = (*g_ptr).meta_group_step[g_ind+1]
				end
		endcase
		if(grp_step lt 113) then begin
			print_text, Event, 'ERROR - group '+group_name+' not background removed yet - back up'
			goto, S11gpk_end
		endif
		;
		case meta_flag of
			0: begin
				print_text, Event, "Skipping Polarimetry Analysis for Group = "+group_name
				(*g_ptr).S11_group_queue[g_ind] = 1
				S11_populate_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
			1: begin
				print_text, Event, "Skipping Polarimetry Analysis for Meta-Group = "+group_name
				(*g_ptr).S11_meta_group_queue[g_ind+1] = 1
				S11_populate_meta_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
		endcase
		;
		;	change the LED1 bar to blue
		;
		group_colors[ig] = "blue"
		WCT_LED_stack, LED1_wid, group_colors
		;
		deep_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
		deep_path = deep_dir + group_name + '_Deep_Coadd.fits'
		;
		;	commenting out the following is the "skip" action
		;
		;S11_compute_UQ_images, Event, deep_path, group_colors, ig, status
		;
		;	if PInst correction flag set, correct
		;
		;if(status eq 0 and inst_corr_flag eq 1) then S11_PInst_correct, Event, deep_path, group_colors, ig, status
		;
		;if(status eq 0) then S11_compute_P_PA_images, Event, deep_path, group_colors, ig, status
		;
;		if(status eq 0) then S11_make_DS9_reg_file, Event, deep_path, status
		;
		case meta_flag of
			0:	begin
					print_text, Event, "Finished Skipping Polarimetric Analysis for Group = "+group_name
				end
			1:	begin
					print_text, Event, "Finished Skipping Polarimetric Analysis for Meta-Group = "+group_name
				end
		endcase
		;
		wait, 1
		status = 0
		;
		;	evaluate completion status: 0=OK, -1=bad
		;
		case status of
			0: begin
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S11_group_queue[g_ind] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 118		; 11.8 to identify it as different from 11.9=done
							S11_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S11_meta_group_queue[g_ind+1] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 118
							S11_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
				;
				;	change color of group LED to green
				;
				group_colors[ig] = "green"
				WCT_LED_stack, LED1_wid, group_colors
				;
			  end
			  ;
		  -1: begin
		  		;
				;	change color of group LED to yellow
				;
				group_colors[ig] = "yellow"
				WCT_LED_stack, LED1_wid, group_colors
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S11_group_queue[g_ind] = -2
							;
							;	and don't step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 7
							S11_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S11_meta_group_queue[g_ind+1] = -2
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 7
							S11_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S11_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
			 end
		endcase
		;
		S11gpk_end:
		;
		MSP_PPOL_Save_Group_File, Event
	endfor
	;
	;	all groups done.
	;
	print_text, Event, "All Queued Groups have had Polarimetry Analysis skipped"
	;
	;	if instrumental correction was selected, unload all the heap variables
	;
	if(inst_corr_flag eq 1) then begin
		PTR_FREE, (*w_ptr).S9_u_inst_ptr
		PTR_FREE, (*w_ptr).S9_q_inst_ptr
		PTR_FREE, (*w_ptr).S9_u_sig_inst_ptr
		PTR_FREE, (*w_ptr).S9_q_sig_inst_ptr
		PTR_FREE, (*w_ptr).S9_P_inst_values_ptr
	endif
	;
	S11_SKA_end:
end