;
;	Step 2 Event Handler Routines - Ski Jump Detection, Repair
;
pro S2_Init, Event
	;
	;	initialize S2 stuff when tabbing onto Step 2
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	populate the group table
	;
	n_groups = (*g_ptr).n_groups
	if(n_groups gt 0) then begin
		ind = WHERE((*g_ptr).S2_group_queue[0:n_groups-1] ge 1,count)
		if(count gt 0) then (*g_ptr).S2_group_queue[ind] = -2
	endif
	S2_populate_group_list, Event
	;
	;	clear LEDs to red
	;
	S2_LED_sensitize, Event, 1, 0
	S2_LED_sensitize, Event, 2, 0
	;
	;	and set processing radio buttons to reflect saved status
	;
	S2_Show_Radio_Buttons, Event
	;
	;	set extended mode radio button
	;
	(*w_ptr).S2_extended_mode = 1L - (*w_ptr).S2_extended_mode
	S2_Toggle_extended_mode, Event
	;
	;	populate the history window
	;
	h_wid = (*w_ptr).S2_history_textbox_wid
	SAll_populate_history, Event, h_wid, 2
end
;
;------------------------------------------------------------------
pro S2_Show_Radio_Buttons, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	for idetect = 0, 2 do begin
		case idetect of
			0: wid = (*w_ptr).S2_detection_normal_wid
			1: wid = (*w_ptr).S2_detection_low_wid
			2: wid = (*w_ptr).S2_detection_high_wid
		endcase
		;
		if((*w_ptr).S2_detection_mode eq idetect) then ib = 1 else ib = 0
		WIDGET_CONTROL, wid, SET_BUTTON=ib
	endfor
	;
	for irepair = 0, 1 do begin
		case irepair of
			0: wid = (*w_ptr).S2_repair_subtract_wid
			1: wid = (*w_ptr).S2_repair_divide_wid
		endcase
		;
		if((*w_ptr).S2_repair_mode eq irepair) then ib = 1 else ib = 0
		WIDGET_CONTROL, wid, SET_BUTTON=ib
	endfor
	;
	;	 and save model button
	;
	WIDGET_CONTROL, (*w_ptr).S2_save_models_wid, SET_BUTTON=(*w_ptr).S2_save_models_mode
end
;
;-----------------------------------------------
pro S2_Detection, Event, ibutton
	;
	;	turns on the indicated radio button, turning off the others
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S2_detection_mode = ibutton
	;
	S2_Show_Radio_Buttons, Event
end
;
;-----------------------------------------------
pro S2_Repair, Event, ibutton
	;
	;	turns on the indicated radio button, turning off the others
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S2_repair_mode = ibutton
	;
	S2_Show_Radio_Buttons, Event
end
;
;----------------------------------------------
pro S2_Toggle_save_model, Event
	;
	;	toggles status of save model flag
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_Struct
	w_ptr = PTR_Struct.PTR_wid_id
	;
	mode = (*w_ptr).S2_save_models_mode
	(*w_ptr).S2_save_models_mode = 1L - mode
	;
	case mode of
		1: print_text, Event, "Turning off the Save Background Model Images mode"
		0: print_text, Event, "Turning on the Save Background Model Images mode"
	endcase
	;
	S2_Show_Radio_Buttons, Event
end
;----------------------------------------------
pro S2_Toggle_extended_mode, Event
	;
	;	toggles status of extended mode flag
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_Struct
	w_ptr = PTR_Struct.PTR_wid_id
	;
	mode = (*w_ptr).S2_extended_mode
	(*w_ptr).S2_extended_mode = 1L - mode
	;
	case mode of
		1: print_text, Event, "Turning off the Extended Object mode"
		0: print_text, Event, "Turning on the Extended Object mode"
	endcase
	;
	WIDGET_CONTROL, (*w_ptr).S2_extended_mode_button_wid, SET_BUTTON=(*w_ptr).S2_extended_mode
end
;
;
;-----------------------------------------------
pro S2_Select_Groups, Event
	;
	;	one or more items in the group listbox was/were selected
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	use WIDGET_INFO to determine the number of items selected
	;
	wid = (*w_ptr).S2_group_listbox_wid
	index = WIDGET_INFO(wid, /LIST_SELECT)
	n = N_ELEMENTS(index)
	(*g_ptr).n_groups_selected = n
	;
	print_text, Event, "Number of selected groups = "+STRING(format='(I)',n)
	;
	;
	if(n gt 1) then begin
		in_list = ''
		for in = 0, n-1 do begin
			in_list = in_list + STRING(format='(1x,I5.3,1x)',index[in])
		endfor
	endif else in_list = STRING(format='(1x,F5.3,1x)',index)
	;
	;print_text, Event, "Index of selected items in list = "+in_list
	;
	;	set the index - it gets reset after a button action is handled
	;
	(*g_ptr).selected_groups[0:n-1] = index[*]
	if(n eq 1) then begin
		group = (*g_ptr).group_names[index[0]]
		print_text, Event, "Selected Group "+group+" from Group List"
	endif else begin
		print_text, Event, "Selected "+STRING(format='(I)',n)+" groups from list"
	endelse
	;
end
;
;-----------------------------------------------
pro S2_Add_To_Queue, Event
	;
	;	selected groups are added to the S2 processing queue
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	how many selected?
	;
	n = (*g_ptr).n_groups_selected
	;
	if(n gt 0) then begin
		for ig = 0, n-1 do begin
			j = (*g_ptr).selected_groups[ig]
			;
			;	set queue pointer to 2 (enqueued)
			;
			(*g_ptr).S2_group_queue[j] = 2
		endfor
	endif
	;
	;	reload listbox
	;
	S2_populate_group_list, Event
	(*g_ptr).n_groups_selected = 0
	(*g_ptr).selected_groups[*] = 0
end
;-----------------------------------------------
pro S2_Remove_From_Queue, Event
	;
	;	selected groups are removed (skipped) from the S2 processing queue
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	how many selected?
	;
	n = (*g_ptr).n_groups_selected
	;
	if(n gt 0) then begin
		for ig = 0, n-1 do begin
			j = (*g_ptr).selected_groups[ig]
			;
			;	set queue pointer to 2 (enqueued)
			;
			(*g_ptr).S2_group_queue[j] = -1
		endfor
	endif
	;
	;	reload listbox
	;
	S2_populate_group_list, Event
	(*g_ptr).n_groups_selected = 0
	(*g_ptr).selected_groups[*] = 0
end
;
;-----------------------------------------------
pro S2_LED_sensitize, Event, S2_LED_number, sense
	;
	;	initializes LED stacks
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	if(S2_LED_number eq 1) then begin
		wid = (*w_ptr).S2_LED1_images_wid
	endif else begin
		wid = (*w_ptr).S2_LED2_groups_wid
	endelse
	;
	WIDGET_CONTROL, wid, SENSITIVE=sense
	;
	;	set all colors to red upon sensitization
	;
	WIDGET_CONTROL, wid, GET_UVALUE=nLED
	;
	n_ele = N_ELEMENTS(nLED)
	if(n_ele eq 0) then nLED = 25
	if(nLED eq 0) then nLED = 25
	WIDGET_CONTROL, wid, SET_UVALUE=nLED
	colors = strarr(nLED)
	if(sense eq 1) then colors[*] = "red" else colors[*] = "gray"
	WCT_LED_stack, wid, colors
end
;
;-----------------------------------------------
pro S2_populate_group_list, Event
	;
	;	lists current groups and step status by name on the listbox
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	n_groups = (*g_ptr).n_groups
	if(n_groups gt 0) then begin
		;
		;	load group name,step and meta status, and queue status
		;
		text_vec = strarr(n_groups)
		for ig = 0, n_groups-1 do begin
			i_queue = (*g_ptr).S2_group_queue[ig]
			;
			case i_queue of
				-2:	queue = '?'
				-1: queue = 'Skip'
				 0: queue = 'Done'
				 1: queue = 'Active'
				 2: queue = 'Queued'
				else: queue = '...'
			endcase
			;
			grp_step = (*g_ptr).group_step[ig]
			if((grp_step ge 30 and grp_step le 40) or $
			   (grp_step ge 60 and grp_step le 70) or $
			   (grp_step ge 110 and grp_step le 119)) then begin
			   step_0 = FLOOR(grp_step / 10)
			   step_1 = grp_step - 10 * step_0
			   text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,".",I1.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],step_0,step_1,(*g_ptr).group_meta[ig],queue),1)

			endif else begin
				text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],(*g_ptr).group_step[ig],(*g_ptr).group_meta[ig],queue),1)
			endelse
		endfor
	endif else begin
		text_vec = ['','']
	endelse
	;
	WIDGET_CONTROL, (*w_ptr).S2_group_listbox_wid, SET_VALUE=text_vec
	;
end
;
;------------------------------------------------------
pro S2_Start_Ski_Jump, Event
	;
	;	Begin processing of all enqueued requests for ski jump processing
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	h_wid = (*w_ptr).S2_history_textbox_wid
	;
	case (*w_ptr).S2_detection_mode of
		0: criterion=4.0		; low detection is 4 sigma
		1: criterion=6.0		; normal is 6 sigma
		2: criterion=25.0		; high is 25 sigma
	endcase
	;
	(*w_ptr).S2_mode_FITS_string = 'Jump detection threshold = '+STRING(criterion,format='(f5.1)')+' sigma'
	;
	print_text, Event, "Starting Ski Jump Detection, Repair with threshold = "+STRING(format='(f5.0)',criterion)+" sigma"
	;
	case (*w_ptr).S2_repair_mode of
		0:	repair = 'subtract'
		1:	repair = 'divide'
	endcase
	;
	print_text, Event, "Ski Jump Repair mode = "+repair
	;
	;	how many active items on the queue?
	;
	ind_queue = WHERE((*g_ptr).S2_group_queue[*] eq 2,count)
	if(count le 0) then begin
		; if none, quit
		print_text, Event, "WARNING - no items in queue to process."
		goto, S2_SSJ_end
	endif
	;
	print_text, Event, "Found "+STRING(format='(I)',count)+" groups in the queue. Processing starting"
	;
	;	Clear LED2 to red with the number of groups to process
	;
	LED2_wid = (*w_ptr).S2_LED2_groups_wid
	LED1_wid = (*w_ptr).S2_LED1_images_wid
	;
	WIDGET_CONTROL, LED2_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "red"
	S2_LED_sensitize, Event, 2, 1
	;
	;	clear the text box that counts the number of found ski jumps
	;
	Text_wid = (*w_ptr).S2_ski_jump_count_textbox_wid
	;
	;	for each of the enqueued groups:
	;
	for ig = 0, count-1 do begin
		;
		n_sj = 0L
		WIDGET_CONTROL, Text_wid, SET_VALUE=STRING(format='(I)',n_sj)
		;
		g_ind = ind_queue[ig]
		group_name = (*g_ptr).group_names[g_ind]
		print_text, Event, "Starting Ski Jump Processing for Group = "+group_name
		;
		;	change the list box queue list to active for this group
		;
		(*g_ptr).S2_group_queue[g_ind] = 1
		S2_populate_group_list, Event
		;
		;	and position the list to show this active group
		;
		if( g_ind gt 10) then begin
			WIDGET_CONTROL, (*w_ptr).S2_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
		endif
		;
		;	change the LED2 bar to blue
		;
		group_colors[ig] = "blue"
		WCT_LED_stack, LED2_wid, group_colors
		;
		;	how many images to examine? Reset LED1 to reflect it
		;
		n_image = (*g_ptr).group_numbers[g_ind]
		;
		;	if > 130, use LED scaling
		;
		nLED = 130
		LED_scale = float(n_image) / float(nLED)
		if(LED_scale le 1.0) then LED_scale = 1.0
		;
		WIDGET_CONTROL, LED1_wid, SET_UVALUE=(nLED < n_image)
		image_colors = strarr(nLED < n_image)
		image_colors[*] = "red"
		S2_LED_sensitize, Event, 1, 1
		;
		;	for every image
		;
		for i_image = 0, n_image-1 do begin
			;
			;	set LED1 to blue
			;
			this_LED = ROUND(float(i_image) / LED_scale) & this_LED = 0 > ((nLED-1) < this_LED)
			image_colors[this_LED] = "blue"
			WCT_LED_stack, LED1_wid, image_colors
			;
			;	test for ski jump here
			;
			;	Test_Ski_Jump
			;
			image_path = (*g_ptr).group_images[g_ind,i_image]
			;
			;	test that this file exists - if not, offer user opportunity to reconnect
			;
			test = FILE_INFO(image_path)
			if(test.exists ne 1) then begin
				print_text, Event, 'BDP data not found trying to reconnect to BDP directory'
				reconnect_to_BDP_directory, Event, image_path
			endif
			;
			;	test for LLQ corruption, set flag if so
			;
			img_ptr = PTR_NEW(readfits(image_path,dummy_header, /silent))
			if( test_bad_lower_left_quadrant(img_ptr) eq 0) then begin
				(*g_ptr).group_llq_flags[g_ind,i_image] = 0
			endif else begin
				(*g_ptr).group_llq_flags[g_ind,i_image] = 1
				print_text, Event, 'Found Bad LLQ for image '+FILE_BASENAME(image_path)
			endelse
			PTR_FREE,img_ptr
			;
			;print_text, Event, "Testing for Ski Jump in image "+FILE_BASENAME(image_path)
			;
			S2_ski_jump_detector, Event, image_path, ski_jump_code, CRITERION=criterion
			;
			;	ski_jump_code -
			;		00 = top, bottom OK
			;		10, 01 = top or bottom bad
			;		11 = both bad
			;
			;	change LED to violet
			;
			image_colors[this_LED] = "violet"
			WCT_LED_stack, LED1_wid, image_colors
			;
			;	if test showed no correction needed, set let to green and jump past fix
			;
			if(ski_jump_code eq 0) then ski_jump_flag = 0 else ski_jump_flag = 1
			;
			if(ski_jump_flag eq 0) then begin
				image_colors[this_LED] = "green"
				WCT_LED_stack, LED1_wid, image_colors
				goto, S2_SSJ_image_skip_end
			endif
			;
			;	otherwise, set LED to orange and fix
			;
			print_text, Event, "Significant Ski Jump detected - repairing (code = "+STRING(ski_jump_code,format='(I2)')+")"
			image_colors[this_LED] = "orange"
			WCT_LED_stack, LED1_wid, image_colors
			;
			;	Ski Jump Fixer goes here
			;
			S2_ski_jump_repair, Event, image_path, repair, ski_jump_code
			;
			;	after fix, color LED based on type of jump corrected (pale green for normal, kahki for abnormal)
			;
			;wait, 0.2
			n_sj++
			WIDGET_CONTROL, Text_wid, SET_VALUE=STRTRIM(STRING(format='(I)',n_sj),2)
			;
			if(ski_jump_code lt 20) then this_color = "khaki dark" else this_color = "khaki light"
			image_colors[this_LED] = this_color
			WCT_LED_stack, LED1_wid, image_colors
			;
			;	and add to history
			;
			SAll_add_to_history, Event, h_wid, 2, FILE_BASENAME(image_path) + " fixed ski jump"
			SAll_populate_history, Event, h_wid, 2
			;
			;	this image done, go to next
			;
			S2_SSJ_image_skip_end:
		endfor
		;
		;	all images in this group are done
		;
		print_text, Event, "Finished Ski Jump Test/Repair for "+STRING(format='(I)',n_sj)+" Images in Group = "+group_name
		;
		;	change color of group LED to green
		;
		group_colors[ig] = "green"
		WCT_LED_stack, LED2_wid, group_colors
		;
		;	clear queue tag for this group and rewrite listbox
		;
		(*g_ptr).S2_group_queue[g_ind] = 0
		;
		;	and update step status value for this group
		;
		(*g_ptr).group_step[g_ind] = 2
		S2_populate_group_list, Event
		if( g_ind gt 10) then begin
			WIDGET_CONTROL, (*w_ptr).S2_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
		endif
		MSP_PPOL_Save_Group_File, Event
	endfor
	;
	;	all groups done.
	;
	print_text, Event, "All Queued Groups have been Ski Jump Tested/Repaired"
	;
	S2_SSJ_end:
	MSP_PPOL_Save_Group_File, Event
end