;	Step 5 Event Handler Routines - Matching
;
pro S5_Init, Event
	;
	;	initialize S5 stuff when tabbing onto Step 5
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	populate the group table
	;
	n_groups = (*g_ptr).n_groups
	if(n_groups gt 0) then begin
		ind = WHERE((*g_ptr).S5_group_queue[0:n_groups-1] ge 1,count)
		if(count gt 0) then (*g_ptr).S5_group_queue[ind] = -2
	endif
	S5_populate_group_list, Event
	;
	;	clear LEDs to red
	;
	S5_LED_sensitize, Event, 1, 0
	S5_LED_sensitize, Event, 2, 0
	;
	;	and clear Nbad, other textboxes
	;
	nbad = 0
	WIDGET_CONTROL, (*w_ptr).S5_nbad_textbox_wid, SET_VALUE=STRING(nbad,format='(I)')
	WIDGET_CONTROL, (*w_ptr).S5_textbox_matches_wid, SET_VALUE=' '
	WIDGET_CONTROL, (*w_ptr).S5_textbox_rms_dmag_wid, SET_VALUE=' '
	;
	;	show history
	;
	h_wid = (*w_ptr).S5_history_textbox_wid
	SAll_populate_history, Event, h_wid, 5
end
;
;-----------------------------------------------
pro S5_Select_Groups, Event
	;
	;	one or more items
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	wid = (*w_ptr).S5_group_listbox_wid
	index = WIDGET_INFO( wid, /LIST_SELECT)
	n = N_ELEMENTS(index)
	if(n eq 1 and index[0] eq -1) then goto, S5_SG_end
	;
	;	make sure only selecting groups with step >=4
	;
	ind = WHERE((*g_ptr).group_step[index] ge 4 and $
			    (*g_ptr).group_step[index] ne 31 and $
			    (*g_ptr).group_step[index] ne 32, count)
	if(count gt 0) then begin
		index = index[ind]
		if(count ne n) then print_text, Event, "Warning - some groups not processed through Step 4 - they have been skipped"
		n = count
	endif else begin
		print_text, Event, "Warning - no groups have been processed through Step 4. No groups selected"
		goto, S5_SG_end
	endelse
	;
	(*g_ptr).n_groups_selected = n
	;
	print_text, Event, "Number of selected groups = "+STRING(format='(I)',n)
	;
	;	set the index - it gets reset after a button action is handled
	;
	(*g_ptr).selected_groups[0:n-1] = index[*]
	if(n eq 1) then begin
		group = (*g_ptr).group_names[index[0]]
		print_text, Event, "Selected Group "+group+" from Group List"
	endif else begin
		print_text, Event, "Selected "+STRING(format='(I)',n)+" groups from list"
	endelse
	;
	S5_SG_end:
end
;
;-----------------------------------------------
pro S5_Add_To_Queue, Event
	;
	;	selected groups are added to the S5 processing queue
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	how many selected?
	;
	n = (*g_ptr).n_groups_selected
	;
	if(n gt 0) then begin
		for ig = 0, n-1 do begin
			j = (*g_ptr).selected_groups[ig]
			;
			;	set queue pointer to 2 (enqueued)
			;
			(*g_ptr).S5_group_queue[j] = 2
		endfor
	endif
	;
	;	reload listbox
	;
	S5_populate_group_list, Event
	(*g_ptr).n_groups_selected = 0
	(*g_ptr).selected_groups[*] = 0
end
;-----------------------------------------------
pro S5_Remove_From_Queue, Event
	;
	;	selected groups are removed (skipped) from the S5 processing queue
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	how many selected?
	;
	n = (*g_ptr).n_groups_selected
	;
	if(n gt 0) then begin
		for ig = 0, n-1 do begin
			j = (*g_ptr).selected_groups[ig]
			;
			(*g_ptr).S5_group_queue[j] = -1
		endfor
	endif
	;
	;	reload listbox
	;
	S5_populate_group_list, Event
	(*g_ptr).n_groups_selected = 0
	(*g_ptr).selected_groups[*] = 0
end
;
;-----------------------------------------------
pro S5_LED_sensitize, Event, S5_LED_number, sense
	;
	;	initializes LED stacks
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	if(S5_LED_number eq 1) then begin
		wid = (*w_ptr).S5_LED1_images_wid
	endif else begin
		wid = (*w_ptr).S5_LED2_groups_wid
	endelse
	;
	WIDGET_CONTROL, wid, SENSITIVE=sense
	;
	;	set all colors to red upon sensitization
	;
	WIDGET_CONTROL, wid, GET_UVALUE=nLED
	;
	n_ele = N_ELEMENTS(nLED)
	if(n_ele eq 0) then nLED = 25
	if(nLED eq 0) then nLED = 25
	WIDGET_CONTROL, wid, SET_UVALUE=nLED
	colors = strarr(nLED)
	if(sense eq 1) then colors[*] = "red" else colors[*] = "gray"
	WCT_LED_stack, wid, colors
end
;
;-----------------------------------------------
pro S5_populate_group_list, Event
	;
	;	lists current groups and step status by name on the listbox
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	n_groups = (*g_ptr).n_groups
	if(n_groups gt 0) then begin
		;
		;	load group name,step and meta status, and queue status
		;
		text_vec = strarr(n_groups)
		for ig = 0, n_groups-1 do begin
			if((*g_ptr).group_step[ig] lt 4) then (*g_ptr).S5_group_queue[ig] = -2
			i_queue = (*g_ptr).S5_group_queue[ig]
			;
			case i_queue of
				-2:	queue = '?'
				-1: queue = 'Skip'
				 0: queue = 'Done'
				 1: queue = 'Active'
				 2: queue = 'Queued'
				else: queue = '...'
			endcase
			;
			grp_step = (*g_ptr).group_step[ig]
			if((grp_step ge 30 and grp_step le 40) or $
			   (grp_step ge 60 and grp_step le 70) or $
			   (grp_step ge 110 and grp_step le 119)) then begin
			   step_0 = FLOOR(grp_step / 10)
			   step_1 = grp_step - 10 * step_0
			   text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,".",I1.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],step_0,step_1,(*g_ptr).group_meta[ig],queue),1)

			endif else begin
				text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],(*g_ptr).group_step[ig],(*g_ptr).group_meta[ig],queue),1)
			endelse
		endfor
	endif else begin
		text_vec = ['','']
	endelse
	;
	WIDGET_CONTROL, (*w_ptr).S5_group_listbox_wid, SET_VALUE=text_vec
	;
end
;
;------------------------------------------------------
pro S5_Start_Matching, Event
	;
	;	Begin processing of all enqueued requests for match processing
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	h_ptr = PTR_struct.PTR_history
	;
	h_wid = (*w_ptr).S5_History_textbox_wid
	;
	;	number of bad matches
	;
	nbad = 0
	;
	;	how many active items on the queue?
	;
	ind_queue = WHERE((*g_ptr).S5_group_queue[0:(*g_ptr).n_groups-1] eq 2,count)
	if(count le 0) then begin
		; if none, quit
		print_text, Event, "WARNING - no items in queue to process."
		goto, S5_SM_end
	endif
	;
	print_text, Event, "Found "+STRING(format='(I)',count)+" groups in the queue. Processing starting"
	;
	;	Clear LED2 to red with the number of groups to process
	;
	LED2_wid = (*w_ptr).S5_LED2_groups_wid
	LED1_wid = (*w_ptr).S5_LED1_images_wid
	;
	WIDGET_CONTROL, LED2_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "red"
	S5_LED_sensitize, Event, 2, 1
	;
	;	for each of the enqueued groups:
	;
	for ig = 0, count-1 do begin
		;
		n_m = 0L
		;
		g_ind = ind_queue[ig]
		group_name = (*g_ptr).group_names[g_ind]
		print_text, Event, "Starting Match Processing for Group = "+group_name
		;
		;	change the list box queue list to active for this group
		;
		(*g_ptr).S5_group_queue[g_ind] = 1
		S5_populate_group_list, Event
		;
		;	position list at this group
		;
		if( g_ind gt 10) then begin
			WIDGET_CONTROL, (*w_ptr).S5_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
		endif
		;
		;	change the LED2 bar to blue
		;
		group_colors[ig] = "blue"
		WCT_LED_stack, LED2_wid, group_colors
		;
		;	check that this group has completed PSF fitting (step 4)
		;
		step = (*g_ptr).group_step[g_ind]
		if(step lt 4) then begin
			print_text, Event, "ERROR - this group has not completed PSF fitting (Step 4) - cannot match"
			goto, S5_group_end
		endif
		;
		S5_match_star_lists, Event, g_ind, status
		;
		;	if outcome was positive, say so
		;
		if(status eq 0) then begin
			;
			;	all images in this group are done
			;
			print_text, Event, "Finished Matching Images for Group = "+group_name
			;
			;	change color of group LED to green
			;
			group_colors[ig] = "green"
			(*g_ptr).S5_group_queue[g_ind] = 0
			(*g_ptr).group_step[g_ind] = 5
		endif else begin
			print_text, Event, "Group "+group_name+" failed matching"
			group_colors[ig] = "violet"
			nbad++
			WIDGET_CONTROL, (*w_ptr).S5_nbad_textbox_wid, SET_VALUE=STRING(nbad,format='(I)')
			;
			SAll_add_to_history, Event, h_wid, 5, group_name + "failed full matching - status " + STRING(status,format='(I)')
			;
			(*g_ptr).group_step[g_ind] = 4
			(*g_ptr).S5_group_queue[g_ind] = -2
	  	endelse
		;
		WCT_LED_stack, LED2_wid, group_colors
		;
		;	and update step status value for this group
		;
		S5_populate_group_list, Event
		if( g_ind gt 10) then begin
			WIDGET_CONTROL, (*w_ptr).S5_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
		endif
;		MSP_PPOL_Save_Group_File, Event
		;
		S5_group_end:
	endfor
	;
	;	all groups done.
	;
	print_text, Event, "All Queued Groups have been Matched"
	if(nbad ne 0) then print_text, Event, " But, with "+STRING(nbad, format='(I)')+" match failures - warning..."
	;
	S5_SM_end:
	MSP_PPOL_Save_Group_File, Event
end