;
;	Step 8 Event Handler Routines - Deep Photometry
;
pro S8_Init, Event
	;
	;	initialize S8 stuff when tabbing onto Step 8
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	populate the group, metagroup table based on current
	;		meta-group mode
	;
	h_wid = (*w_ptr).S8_history_textbox_wid
	SAll_populate_history, Event, h_wid, 8
	;
	mg_mode = (*w_ptr).S8_meta_group_mode
	case mg_mode of
		0:	begin
				S8_Select_group_mode, Event, 0
				n_groups = (*g_ptr).n_groups
				if(n_groups gt 0) then begin
					ind = WHERE((*g_ptr).S8_group_queue[0:n_groups-1] ge 1,count)
					if(count gt 0) then (*g_ptr).S8_group_queue[ind] = -2
				endif
				S8_populate_group_list, Event
			end
		1:	begin
				S8_Select_group_mode, Event, 1
				n_meta_groups = (*g_ptr).n_meta_groups
				if(n_meta_groups gt 0) then begin
					ind = WHERE((*g_ptr).S8_meta_group_queue[1:n_meta_groups] ge 1,count)
					if(count gt 0) then (*g_ptr).S8_meta_group_queue[ind+1] = -2
				endif
				S8_populate_meta_group_list, Event
			end
		else: print_text, Event, "ERROR - illegal meta-group mode number for Step 8"
	endcase
	;
	;	clear LEDs to red
	;
	S8_LED_sensitize, Event, 1, 0
	S8_LED_sensitize, Event, 2, 0
	;
	;	initialize options buttons and buffers
	;
	(*w_ptr).S8_flatten_images_flag = 1 - (*w_ptr).S8_flatten_images_flag
	S8_Toggle_flatten_images, Event
	;
	(*w_ptr).S8_calibrator_flag = 1 - (*w_ptr).S8_calibrator_flag
	S8_Toggle_calibrator, Event
	;
	(*w_ptr).S8_save_regions_flag = 1 - (*w_ptr).S8_save_regions_flag
	S8_Toggle_regions, Event
	;
	S8_Show_Options_Textbox, Event, 0	; N_sig
	S8_Show_Options_Textbox, Event, 1	; NG
	S8_Show_Options_Textbox, Event, 2   ; 2MASS offset
	;
;	(*w_ptr).S8_deep_mode = 1 - (*w_ptr).S8_deep_mode
;	S8_Use_deep_star_list, Event
	;
	(*w_ptr).S8_residuals_mode = 1 - (*w_ptr).S8_residuals_mode
	S8_Save_residual_images, Event
	;
	(*w_ptr).S8_PHOTCAT_mode = 1 - (*w_ptr).S8_PHOTCAT_mode
	S8_Deposit_into_photcat, Event
	;
	(*w_ptr).S8_phot_only_mode = 1 - (*w_ptr).S8_phot_only_mode
	S8_Phot_only_processing, Event
	;
	(*w_ptr).S8_save_PSF_models_flag = 1 - (*w_ptr).S8_save_PSF_models_flag
	S8_Toggle_Save_PSF_Models_Mode, Event
	;
	(*w_ptr).S8_fix_2MASS_flag = 1 - (*w_ptr).S8_fix_2MASS_flag
	S8_Toggle_fix_2MASS_Mode, Event
	;
	S8_Show_Values_Textbox, Event, 0, 0			; group
	S8_Show_Values_Textbox, Event, 1, 'none'	; image
	S8_Show_Values_Textbox, Event, 2, 0			; N_stars
	;
end
;
;-----------------------------------------------
;
pro S8_Show_Options_Textbox, Event, ibox, extra
	;
	;	if extra argument is present, override lookup of common blocks
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	narg = N_PARAMS()
	if(narg eq 3) then flag = 1 else flag = 0
	;
	case ibox of
		0:	begin
				wid = (*w_ptr).S8_nsig_textbox_wid
				if(flag eq 0) then nsig = (*w_ptr).S8_textbox_value[0] else nsig = extra
				str = STRTRIM(STRING(nsig,format='(f5.1)'))+'  '
			end
		1:  begin
				wid = (*w_ptr).S8_ng_textbox_wid
				if(flag eq 0) then ng = (*w_ptr).S8_textbox_value[1] else ng = extra
				str = STRTRIM(STRING(ng,format='(I)'))+'  '
			end
		2:  begin
				wid = (*w_ptr).S8_2MASS_offset_textbox_wid
				if(flag eq 0) then two_mass = (*w_ptr).S8_textbox_value[2] else two_mass = extra
				str = STRTRIM(STRING(two_mass,format='(f6.4)'))+' '
			end
	endcase
	;
	center_in_textbox, wid, str
end
;----------------------------------------------
pro S8_Show_Values_Textbox, Event, ibox, value
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	case ibox of
		0:	wid = (*w_ptr).S8_group_textbox_wid
		1:  wid = (*w_ptr).S8_image_textbox_wid
		2:  wid = (*w_ptr).S8_nstars_textbox_wid
	endcase
	;
	type = SIZE(value,/TYPE)
	if(type eq 2 or type eq 3) then begin
		str = STRTRIM(STRING(value,format='(I)'),2)
	endif else begin
		if(type eq 7) then str = value else str = ''
	endelse
	;
	center_in_textbox, wid, str
end
;----------------------------------------------
pro S8_Select_group_mode, Event, meta_group_mode
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S8_meta_group_mode = meta_group_mode
	;
	WIDGET_CONTROL, (*w_ptr).S8_show_groups_wid, SET_BUTTON=(1-meta_group_mode)
	WIDGET_CONTROL, (*w_ptr).S8_show_meta_groups_wid, SET_BUTTON=meta_group_mode
	;
	case meta_group_mode of
		0: begin
			print_text, Event, "Selecting Single Group Mode"
			S8_populate_group_list, Event
		   end
		1: begin
			print_text, Event, "Selecting Meta-Group Mode"
			S8_populate_meta_group_list, Event
		   end
	endcase
end
;------------------------------------------------
pro S8_Toggle_fix_2MASS_mode, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S8_fix_2MASS_flag = 1 - (*w_ptr).S8_fix_2MASS_flag
	;
	WIDGET_CONTROL, (*w_ptr).S8_fix_2MASS_checkbox_wid, SET_BUTTON=(*w_ptr).S8_fix_2MASS_flag
	;
	case (*w_ptr).S8_fix_2MASS_flag of
		0: print_text, Event, "Turning OFF option to fix 2MASS-Mimir offset (normal operation)"
		1: print_text, Event, "Turning ON option to fix 2MASS-Mimir offset (mock star modeling)"
	endcase
end
;
;------------------------------------------------
pro S8_Toggle_flatten_images, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S8_flatten_images_flag = 1 - (*w_ptr).S8_flatten_images_flag
	;
	WIDGET_CONTROL, (*w_ptr).S8_flatten_images_button_wid, SET_BUTTON=(*w_ptr).S8_flatten_images_flag
	;
	case (*w_ptr).S8_flatten_images_flag of
		0: print_text, Event, "Turning OFF option to flatten images before fitting"
		1: print_text, Event, "Turning ON option to flatten images before fitting"
	endcase
end
;
;------------------------------------------------
pro S8_Toggle_Save_PSF_Models_Mode, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S8_save_PSF_models_flag = 1 - (*w_ptr).S8_save_PSF_models_flag
	;
	WIDGET_CONTROL, (*w_ptr).S8_save_PSF_models_checkbox_wid, SET_BUTTON=(*w_ptr).S8_save_PSF_models_flag
	;
	case (*w_ptr).S8_save_PSF_models_flag of
		0: print_text, Event, "Turning OFF option to save PSF models to disk"
		1: print_text, Event, "Turning ON option to save PSF models to disk"
	endcase
end
;
;------------------------------------------------
pro S8_Toggle_calibrator, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S8_calibrator_flag = 1 - (*w_ptr).S8_calibrator_flag
	;
	WIDGET_CONTROL, (*w_ptr).S8_calibrator_button_wid, SET_BUTTON=(*w_ptr).S8_calibrator_flag
	;
	case (*w_ptr).S8_calibrator_flag of
		0: print_text, Event, "Turning OFF option to process as calibrator"
		1: print_text, Event, "Turning ON option to process as calibrator"
	endcase
end
;
;------------------------------------------------
pro S8_Toggle_regions, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S8_save_regions_flag = 1 - (*w_ptr).S8_save_regions_flag
	;
	WIDGET_CONTROL, (*w_ptr).S8_save_regions_checkbox_wid, SET_BUTTON=(*w_ptr).S8_save_regions_flag
	;
	case (*w_ptr).S8_save_regions_flag of
		0: print_text, Event, "Turning OFF option to save DS9 region files"
		1: print_text, Event, "Turning ON option to save DS9 region files"
	endcase
end
;
;-----------------------------------------------
pro S8_Save_residual_images, Event
	;
	;	toggle this mode
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S8_residuals_mode = 1 - (*w_ptr).S8_residuals_mode
	;
	WIDGET_CONTROL, (*w_ptr).S8_save_residuals_wid, SET_BUTTON=(*w_ptr).S8_residuals_mode
	;
	case (*w_ptr).S8_residuals_mode of
		0: print_text, Event, "Changing mode to NOT save residual (star subtracted) images"
		1: print_text, Event, "Changing mode to SAVE residual (star subtracted) images"
	endcase
	;
end
;
;-----------------------------------------------
pro S8_Deposit_into_photcat, Event
	;
	;	toggle this mode
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S8_PHOTCAT_mode = 1 - (*w_ptr).S8_PHOTCAT_mode
	;
	WIDGET_CONTROL, (*w_ptr).S8_deposit_into_photcat_wid, SET_BUTTON=(*w_ptr).S8_PHOTCAT_mode
	;
	case (*w_ptr).S8_PHOTCAT_mode of
		0: print_text, Event, "Changing mode to NOT save results in PHOTCAT file form"
		1: print_text, Event, "Changing mode to SAVE results in PHOTCAT file form"
	endcase
	;
end
;
;-----------------------------------------------
pro S8_Phot_only_processing, Event
	;
	;	toggle this mode
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S8_phot_only_mode = 1 - (*w_ptr).S8_phot_only_mode
	;
	WIDGET_CONTROL, (*w_ptr).S8_phot_only_option_wid, SET_BUTTON=(*w_ptr).S8_phot_only_mode
	;
	case (*w_ptr).S8_phot_only_mode of
		0: print_text, Event, "Changing mode to perform PSF fitting of Deep and HWP images"
		1: print_text, Event, "Changing mode to NOT perform PSF fitting of HWP images - Deep images (PHOT) only"
	endcase
	;
	S8_LED_sensitize, Event, 1, 0
end
;
;-----------------------------------------------
pro S8_Modify_textbox, Event, ibox
	;
	;	handle insert character events NSIG and NG
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	first, fetch current pointer into text box buffer
	;
	case ibox of
		0:	wid = (*w_ptr).S8_nsig_textbox_wid
		1:	wid = (*w_ptr).S8_ng_textbox_wid
		2:  wid = (*w_ptr).S8_2MASS_offset_textbox_wid
	endcase
	;
	nch = (*w_ptr).S8_textbox_nch[ibox]
	;
	;	if it is zero, check whether something is already there
	;
	if(nch eq 0) then begin
		mch = WIDGET_INFO(wid,/TEXT_NUMBER)
		if(mch gt 1) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			mch = STRLEN(existing_text[0])
			;
			;	pack into character buffer
			;
			(*w_ptr).S8_textbox_buffer[ibox,*] = ''
			;
			for im = 0, mch-1 do begin
				(*w_ptr).S8_textbox_buffer[ibox,im] = STRMID(existing_text, im, 1)
			endfor
			(*w_ptr).S8_textbox_nch[ibox] = mch
			nch = mch
		endif
	endif
	;
	if(Event.type eq 0) then begin
		;
		;	if character not CR, put it into buffer
		;
		ch = Event.ch
		if(ch ne 10B) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			(*w_ptr).S8_textbox_nch[ibox] = STRLEN(existing_text)
			for il = 0, STRLEN(existing_text[0])-1 do (*w_ptr).S8_textbox_buffer[ibox,il] = STRMID(existing_text, il, 1)
			WIDGET_CONTROL, wid, SET_TEXT_SELECT=[Event.offset,0]
		endif else begin
			;
			;	was a character return, interpret and load value buffer
			;
			;		- first, check for non-zero buffer length
			;
			if((*w_ptr).S8_textbox_nch[ibox] gt 0) then begin
				;
				;		- load the buffer into a single string
				;
				name = ''
				for ich = 0, (*w_ptr).S8_textbox_nch[ibox]-1 do begin
					name = name + (*w_ptr).S8_textbox_buffer[ibox,ich]
				endfor
				;
				name = STRTRIM(name,2)
				;
				;	new name available - read it into value buffer
				;
				reads, name, format='(f)',value
				(*w_ptr).S8_textbox_value[ibox] = value
				(*w_ptr).S8_textbox_group_value[ibox] = value	; save in the group buffer, also
				;
				case ibox of
					0: begin
							print_text, Event, "Setting NSIG = "+name
							S8_Show_Options_Textbox, Event, 0
					   end
					1: begin
							print_text, Event, "Setting NG = "+name
							S8_Show_Options_Textbox, Event, 1
					   end
					2: begin
							print_text, Event, "Setting 2MASS-Mimir offset mag = "+name
							(*w_ptr).S8_2MASS_offset_value = value
							S8_Show_Options_Textbox, Event, 2
					   end
				endcase
				;
			endif else begin
				print_text, Event, "Warning - no characters in textbox"
			endelse
			;
		endelse
		;
		;print_text, Event, "Insert Character Event - Character = "+STRING(format='(a)',Event.ch)
	endif else begin
		if(Event.type eq 1) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			;
			name = STRTRIM(Event.str,2)
			length = STRLEN(name)
			if(length gt 1) then begin
				this_name = strarr(length)
				for il=0, length-1 do begin
					this_name[il] = STRMID(name,il,1)
				endfor
			endif else begin
				if(length eq 0) then goto, S8MT_end
				;
				this_name = strarr(1)
				this_name[0] = name
			endelse
		endif else begin
			if(Event.type eq 2) then begin
				;
				;	delete type event, go to offset, delete length
				;
				WIDGET_CONTROL, wid, GET_VALUE=existing_text
				if(Event.length eq 0) then goto, S8MT_end
			endif else begin
				print_text, Event, "Event not recognized"
			endelse
			new_name = 0
		endelse
		(*w_ptr).S8_textbox_nch[ibox] = STRLEN(existing_text)
		for il = 0, STRLEN(existing_text[0])-1 do (*w_ptr).S8_textbox_buffer[ibox,il] = STRMID(existing_text, il, 1)
		WIDGET_CONTROL, wid, SET_TEXT_SELECT=[Event.offset,0]
	endelse
	;
	S8MT_end:
end
;-----------------------------------------------
pro S8_Select_Groups, Event
	;
	;	select groups or metagroups
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	get group/metagroup mode
	;
	tab_no = (*w_ptr).S8_meta_group_mode
	;
	wid = (*w_ptr).S8_group_listbox_wid
	;
	index = WIDGET_INFO( wid, /LIST_SELECT)
    n = N_ELEMENTS(index)
    skip = 0
	if(n eq 0 and index[0] eq -1) then skip=1
	if(skip eq 0) then begin
		;
		case tab_no of
			0:	begin
					ind = WHERE((*g_ptr).group_step[index] ge 7 and $
			    				(*g_ptr).group_step[index] ne 31 and $
			    				(*g_ptr).group_step[index] ne 32, count)
					if(count gt 0) then begin
						index = index[ind]
						if(n ne count) then print_text, Event, "Warning - some groups not processed through Step 7 - skipping those"
						n = count
					endif else begin
						print_text, Event, "Warning - no groups processed through Step 7. Go back."
						goto, S8_SG_end
					endelse
					;
					(*g_ptr).n_groups_selected = n
					(*g_ptr).selected_groups[0:n-1] = index[*]
					if(n eq 1) then begin
						group = (*g_ptr).group_names[index[0]]
						print_text, Event, "Selected Group "+group+" from Group List"
					endif else begin
						print_text, Event, "Selected "+STRING(format='(I)',n)+" groups from list"
					endelse
				end
			1:	begin
					ind = WHERE((*g_ptr).meta_group_step[index+1] ge 7, count)
					if(count gt 0) then begin
						index = index[ind]
						if(n ne count) then print_text, Event, "Warning - some meta-groups not processed through Step 7 - skipping those"
						n = count
					endif else begin
						print_text, Event, "Warning - no meta-groups processed through Step 7. Go back."
						goto, S8_SG_end
					endelse
					(*g_ptr).n_meta_groups_selected = n
					(*g_ptr).selected_meta_groups[1:n] = index[*]+1
					if(n eq 1) then begin
						group = (*g_ptr).meta_group_names[index[0]+1]
						print_text, Event, "Selected Group "+group+" from Group List"
					endif else begin
						print_text, Event, "Selected "+STRING(format='(I)',n)+" groups from list"
					endelse
				end
		endcase
	   ;
	endif else print_text, Event, "Warning - no groups selected"
	S8_SG_end:
end
;
;-----------------------------------------------
pro S8_Add_groups, Event
	;
	;	selected groups are added to the group or metagroup queues
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	get mode
	;
	tab_no = (*w_ptr).S8_meta_group_mode
	;
	case tab_no of
		0:	begin
				;
				;	how many selected?
				;
				n = (*g_ptr).n_groups_selected
				;
				if(n gt 0) then begin
					for ig = 0, n-1 do begin
						j = (*g_ptr).selected_groups[ig]
						;
						;	set queue pointer to 2 (enqueued)
						;
						(*g_ptr).S8_group_queue[j] = 2
					endfor
				endif
				;
				;	reload listbox
				;
				S8_populate_group_list, Event
				;
				;	clear selections
				;
				(*g_ptr).n_groups_selected = 0
				if(n gt 0) then (*g_ptr).selected_groups[0:n-1] = 0
			end
		1: 	begin
				;
				n = (*g_ptr).n_meta_groups_selected
				;
				if(n gt 0) then begin
					(*g_ptr).S8_meta_group_queue[0] = -1
					for ig = 1, n do begin
						j = (*g_ptr).selected_meta_groups[ig]
						;
						;	set queue pointer to 2 (enqueued)
						;
						(*g_ptr).S8_meta_group_queue[j] = 2
					endfor
					print_text, Event, "Added "+STRING(n,format='(I)')+" meta-groups to queue"
				endif else begin
					print_text, Event, "No meta-groups were selected"
				endelse
				;
				;	reload listbox
				;
				S8_populate_meta_group_list, Event
				;
				(*g_ptr).n_meta_groups_selected = 0
				if(n gt 0) then (*g_ptr).selected_meta_groups[1:n] = 0
			end
	endcase

end
;-----------------------------------------------
pro S8_Remove_groups, Event
	;
	;	selected groups are removed (skipped) indicated queue
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	get mode
	;
	tab_no = (*w_ptr).S8_meta_group_mode
	;
	case tab_no of
		0:	begin
				;
				;	how many selected?
				;
				n = (*g_ptr).n_groups_selected
				;
				if(n gt 0) then begin
					for ig = 0, n-1 do begin
						j = (*g_ptr).selected_groups[ig]
						;
						(*g_ptr).S8_group_queue[j] = -1
					endfor
				endif
				;
				;	reload listbox
				;
				S8_populate_group_list, Event
				;
				;	clear selections
				;
				(*g_ptr).n_groups_selected = 0
				if(n gt 0) then (*g_ptr).selected_groups[0:n-1] = 0
			end
		1:	begin
				;
				;	how many selected?
				;
				n = (*g_ptr).n_meta_groups_selected
				;
				if(n gt 0) then begin
					for ig = 1, n do begin
						j = (*g_ptr).selected_meta_groups[ig]
						;
						(*g_ptr).S8_meta_group_queue[j] = -1
					endfor
				endif
				;
				S8_populate_meta_group_list, Event
				;
				(*g_ptr).n_meta_groups_selected = 0
				if(n gt 0) then (*g_ptr).selected_meta_groups[1:n] = 0
			end
	endcase
end
;
;-----------------------------------------------
pro S8_LED_sensitize, Event, S8_LED_number, sense
	;
	;	initializes LED stacks
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	if(S8_LED_number eq 1) then begin
		wid = (*w_ptr).S8_LED1_images_wid
		case (*w_ptr).S8_phot_only_mode of
			0: max_LED = 17
			1: max_LED = 1
		endcase
		WIDGET_CONTROL, wid, SET_UVALUE=0
	endif else begin
		wid = (*w_ptr).S8_LED2_groups_wid
		max_LED = 25
	endelse
	;
	WIDGET_CONTROL, wid, SENSITIVE=sense
	;
	;	set all colors to red upon sensitization
	;
	WIDGET_CONTROL, wid, GET_UVALUE=nLED
	;
	n_ele = N_ELEMENTS(nLED)
	if(n_ele eq 0) then nLED = max_LED
	if(nLED eq 0) then nLED = max_LED
	WIDGET_CONTROL, wid, SET_UVALUE=nLED
	colors = strarr(nLED)
	if(sense eq 1) then colors[*] = "red" else colors[*] = "gray"
	WCT_LED_stack, wid, colors
end
;
;-----------------------------------------------
pro S8_populate_group_list, Event
	;
	;	lists current groups and step status by name on the listbox
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	n_groups = (*g_ptr).n_groups
	if(n_groups gt 0) then begin
		;
		;	load group name,step and meta status, and queue status
		;
		text_vec = strarr(n_groups)
		for ig = 0, n_groups-1 do begin
			if((*g_ptr).group_step[ig] lt 7) then (*g_ptr).S8_group_queue[ig] = -2
			i_queue = (*g_ptr).S8_group_queue[ig]
			;
			case i_queue of
				-3: queue = "to Meta"
				-2:	queue = '?'
				-1: queue = 'Skip'
				 0: queue = 'Done'
				 1: queue = 'Active'
				 2: queue = 'Queued'
				else: queue = '...'
			endcase
			;
			grp_step = (*g_ptr).group_step[ig]
			if((grp_step ge 30 and grp_step le 40) or $
			   (grp_step ge 60 and grp_step le 70) or $
			   (grp_step ge 110 and grp_step le 119)) then begin
			   step_0 = FLOOR(grp_step / 10)
			   step_1 = grp_step - 10 * step_0
			   text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,".",I1.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],step_0,step_1,(*g_ptr).group_meta[ig],queue),1)

			endif else begin
				text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],(*g_ptr).group_step[ig],(*g_ptr).group_meta[ig],queue),1)
			endelse
			;
			;text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,", Meta=",I2.1,", Action= ",A)',$
			;		(*g_ptr).group_names[ig],(*g_ptr).group_step[ig],(*g_ptr).group_meta[ig],queue),1)
		endfor
	endif else begin
		text_vec = ['','']
	endelse
	;
	WIDGET_CONTROL, (*w_ptr).S8_group_listbox_wid, SET_VALUE=text_vec
	;
	(*g_ptr).n_groups_selected = 0
	(*g_ptr).selected_groups[*] = 0
end
;
;------------------------------------------------------------------
pro S8_populate_meta_group_list, Event
	;
	;	loads the meta-name listbox on the indicated tab_number
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	wid = (*w_ptr).S8_group_listbox_wid
	;
	n_meta = (*g_ptr).n_meta_groups
	if(n_meta gt 0) then begin
		;
		text_vec = strarr(n_meta)
		for img = 1, n_meta do begin
			;
			(*g_ptr).S8_meta_group_queue[0] = -1
			if((*g_ptr).meta_group_step[img] lt 7) then (*g_ptr).S8_meta_group_queue[img] = -2
			;
			case (*g_ptr).S8_meta_group_queue[img] of
				-2:	queue = '?'
				-1: queue = 'Skip'
				 0: queue = 'Done'
				 1: queue = 'Active'
				 2: queue = 'Queued'
				else: queue = '...'
			endcase
			;
			text_vec[img-1] = STRTRIM(STRING(format='(A," Step=",I2.1," Action= ",A)',$
				(*g_ptr).meta_group_names[img],(*g_ptr).meta_group_step[img],queue),1)
		endfor
	endif else begin
		text_vec = ['','']
	endelse
	;
	WIDGET_CONTROL, wid, SET_VALUE=text_vec
	;
	(*g_ptr).n_meta_groups_selected = 0
	(*g_ptr).selected_meta_groups[*] = 0
end
;------------------------------------------------------
pro S8_Start_psf_fitting, Event
	;
	;	Begin psf processing of all enqueued requests
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	h_wid = (*w_ptr).S8_history_textbox_wid
	;
	LED1_wid = (*w_ptr).S8_LED1_images_wid
	LED2_wid = (*w_ptr).S8_LED2_groups_wid
	;
	;	group or meta-group mode?
	;
	meta_flag = (*w_ptr).S8_meta_group_mode
	;
	case meta_flag of
		0: begin
				this_queue = (*g_ptr).S8_group_queue[0:(*g_ptr).n_groups-1]
				these_names = (*g_ptr).group_names[0:(*g_ptr).n_groups-1]
			end
		1:  begin
				this_queue = (*g_ptr).S8_meta_group_queue[1:(*g_ptr).n_meta_groups]
				these_names = (*g_ptr).meta_group_names[1:(*g_ptr).n_meta_groups]
			end
	endcase
	;
	;	how many active items on the queue?
	;
	ind_queue = WHERE(this_queue eq 2,count)
	if(count le 0) then begin
		; if none, quit
		print_text, Event, "WARNING - no items in queue to process."
		goto, S8_SP_end
	endif
	;
	print_text, Event, "Found "+STRING(format='(I)',count)+" groups in the queue. Processing starting"
	;
	;	Clear LED2 to red with the number of groups to process
	;
	WIDGET_CONTROL, LED2_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "red"
	S8_LED_sensitize, Event, 2, 1
	;
	;	for each of the enqueued groups:
	;
	for ig = 0, count-1 do begin
		;
		g_ind = ind_queue[ig]
		group_name = these_names[g_ind]
		;
		;	echo group/meta-group name to textbox
		;
		center_in_textbox, (*w_ptr).S8_group_textbox_wid, group_name
		;
		case meta_flag of
			0: begin
				print_text, Event, "Starting PSF-fitting Photometry for Group = "+group_name
				(*g_ptr).S8_group_queue[g_ind] = 1
				S8_populate_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S8_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
			1: begin
				print_text, Event, "Starting PSF-fitting Photometry for Meta-Group = "+group_name
				(*g_ptr).S8_meta_group_queue[g_ind+1] = 1
				S8_populate_meta_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S8_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
		endcase
		;
		;	change the LED2 bar to blue
		;
		group_colors[ig] = "blue"

		WCT_LED_stack, LED2_wid, group_colors
		;
		;	do the PSF fitting
		;
		S8_PSF_fitting, Event, meta_flag, g_ind, group_name, LED1_wid, status
		;status=0
		;
		case meta_flag of
			0:  begin
					print_text, Event, "Finished PSF-Fitting for Group = "+group_name
				end
			1:	begin
					print_text, Event, "Finished PSF-Fitting for Meta-Group = "+group_name
				end
		endcase
		;
		wait, 1
		;
		;	evaluate completion status: 0=OK, -1=bad
		;
		case status of
			0: begin
				;
				;	change color of group LED to green
				;
				group_colors[ig] = "green"
				WCT_LED_stack, LED2_wid, group_colors
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S8_group_queue[g_ind] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 8
							S8_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S8_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S8_meta_group_queue[g_ind+1] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 8
							S8_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S8_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
			  end
			  ;
		  -1: begin
		  		;
				;	change color of group LED to yellow
				;
				group_colors[ig] = "yellow"
				WCT_LED_stack, LED2_wid, group_colors
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S8_group_queue[g_ind] = -2
							;
							;	and don't step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 7
							S8_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S8_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S8_meta_group_queue[g_ind+1] = -2
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 7
							S8_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S8_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
			 end
		endcase
		MSP_PPOL_Save_Group_File, Event
	endfor
	;
	;	all groups done.
	;
	print_text, Event, "All Queued Groups have had PSF-fitting Photometry applied"
	;
	S8_SP_end:
end
