pro S3_flatten_image_background, Event, input_image, output_image, RADIUS=this_radius, $
              ORDER=this_order, MODE=this_mode, MAX_PTS=this_max_pts, TPS = this_TPS, $
              MCS = this_MCS
;
; based on 'S7_fit_using_S11_points.pro' - this version flattens individual images
;   (esp K-band images) separately for the top and bottom halves and returns
;    images with constant backgrounds
;    
;    DPC  20140717
;         20140801  found that some meta coadds couldn't be corrected with polynomials, so
;                   added the TPS (Thin Plate Spline) background modeling option
;
debug = 0
;
if(N_ELEMENTS(this_radius) eq 0) then radius = 13 else radius=this_radius
if(N_ELEMENTS(this_order) eq 0) then order = 4 else order=this_order
if(N_ELEMENTS(this_mode) eq 0) then fit_option_mode = 2 else fit_option_mode=this_mode
if(N_ELEMENTS(this_max_pts) eq 0) then maxpts = 25 else maxpts = this_max_pts
if(N_ELEMENTS(this_tps) eq 0) then do_tps = 0 else do_tps = 1
if(N_ELEMENTS(this_mcs) eq 0) then do_mcs = 0 else do_mcs = 1
;
if(debug ne 0) then print, 'Echo: radius=',radius,' order=',order,' fit_option_mode=',fit_option_mode,' maxpts=',maxpts, $
     ', do_TPS=',do_tps
;
isize = SIZE( input_image )
ncol = isize[1]
ncol2 = FLOOR(ncol / 2)
nrow = isize[2]
nrow2 = FLOOR(nrow / 2)
;
npts = maxpts * maxpts
xy_points = fltarr(npts,2)
ipt = 0L
for ix = 0, maxpts-1 do begin
    x = (float(ncol) / float(maxpts)) * (float(ix)+0.5)
    for iy = 0, maxpts-1 do begin
        y = (float(nrow) / float(maxpts)) * (float(iy)+0.5)
        ;
        xy_points[ipt,0] = x
        xy_points[ipt,1] = y
        ipt++
    endfor
endfor
;
;	use mode to determine how many loops and loop ends
;
;fit_option_mode = 2       ; top, bottom separately
;
case fit_option_mode of
	0: begin
		;
		;	all 4 Quads together, so one zone
		;
		mx = 1
		my = 1
		x0 = [0]
		x1 = [ncol-1]
		y0 = [0]
		y1 = [nrow-1]
	   end
	1: begin
		;
		;	all 4 Quads independent, so 4 Zones
		;
		mx = 2
		my = 2
		x0 = [0, ncol2]	; LL, UL, LR, UR
		x1 = [ncol2-1, ncol-1]
		y0 = [0, nrow2-1]
		y1 = [nrow2, nrow-1]
	   end
	2: begin
		;
		;	top and bottom independently
		;
		mx = 1
		my = 2
		x0 = [0, 0]
		x1 = [ncol-1, ncol-1]
		y0 = [0, nrow2-1]
		y1 = [nrow2, nrow-1]
	   end
endcase
;
;	create the model image (from original, times zero)
;
ptr_model = PTR_NEW( input_image )
;
;	for the indicated number of zones
;
zone_sky = fltarr(mx,my)
zone_sigma = fltarr(mx,my)
;
for jx = 0, mx-1 do begin
;	print, 'starting x loop with jx = ',jx
	this_x0 = x0[jx]
	this_x1 = x1[jx]
;	print, 'this_x0, _x1 = ',this_x0, this_x1
	;
	for jy = 0, my-1 do begin
;		print, 'starting y loop with jy = ',jy
		this_y0 = y0[jy]
		this_y1 = y1[jy]
;		print, 'this y0, _y1 = ',this_y0, this_y1
		;
		;	fit background
		;
		;		for each point, compute mfm value of the zone
		;
		mean_xy = fltarr(npts,3)*0.0
		rms_xy = fltarr(npts)*0.0
		nmean=0L
		;
		for i = 0, npts-1 do begin
			;
			mean_xy[nmean,*] = 0.0
			;
			cx = xy_points[i,0]
			cy = xy_points[i,1]
			;
			;	compare to current zone - if point center in zone, keep
			;
			if((cx le this_x1) and (cx ge this_x0) and $
				(cy le this_y1) and (cy ge this_y0)) then begin
				;
				;	if debug set, mark these centers on reg file
				;
;				if(debug eq 1) then begin
;					printf, lu_reg, cx, cy, radius, format='("circle ",f7.1,1x,f7.1,1x,I2,1x, "# color= white width=  1")'
;				endif
				;
				;	collect all the image pixels within radius of this point
				;
				xr0 = FLOOR(this_x0 > (cx - radius))
				xr1 = CEIL((cx + radius) < this_x1)
				yr0 = FLOOR(this_y0 > (cy - radius))
				yr1 = CEIL((cy + radius) < this_y1)
				;
				vec = fltarr( (xr1 - xr0 + 1) * (yr1 - yr0 + 1) )
				nvec = 0L
				;
				for ix = xr0, xr1 do begin
					for iy = yr0, yr1 do begin
						;
						dx = float(ix - cx)
						dy = float(iy - cy)
						dr = sqrt(dx * dx + dy * dy)
						if(dr le radius) then begin
							vec[nvec] = input_image[ix,iy]
							;
							mean_xy[nmean,0] += float(ix)
							mean_xy[nmean,1] += float(iy)
							nvec++
						endif
					endfor
				endfor
				;
				if(nvec gt 0) then begin
  				mean_xy[nmean,0] /= float(nvec)
  				mean_xy[nmean,1] /= float(nvec)
  				;
  				vec = vec[0:nvec-1]
  				ind = WHERE(FINITE(vec) and (vec gt -1.0e5),nfinite)
  				;
  				if(nfinite gt 0) then begin
  					zz = median_filtered_mean(vec[ind])
  					if(FINITE(zz[0]) and FINITE(zz[1])) then begin
  					 ;
  					 if(zz[0] lt -100.0) then stop
  					 ;
  						mean_xy[nmean,2] = zz[0]
  						rms_xy[nmean] = zz[1]
  						nmean++
  					endif
  				endif 
  			endif
			endif
		endfor
		;
		;
		if(debug ne 0) then print, 'for this x,y zone, nmean = ',nmean
		;
		;
		;	mean sky and mean rms
		;
		zz_sky = median_filtered_mean(REFORM( mean_xy[0:nmean-1,2] ))
		zz_rms = median_filtered_mean(rms_xy[0:nmean-1])
		;
		mean_sky = zz_sky[0]
		sky_noise = zz_rms[0]
		;
		if(debug ne 0) then print, 'Mean sky = ',zz_sky[0],'+/-',zz_sky[1],' rms = ',zz_rms[0],'+/-',zz_rms[1]
		;
		zone_sky[jx,jy] = mean_sky
		zone_sigma[jx,jy] = sky_noise
		;
		;	clean out baddies
		;
		test_xy = REFORM(mean_xy[0:nmean-1,2])
		;
		ind_good = WHERE( abs(test_xy - mean_sky)/sky_noise lt 5.0,ngood,COMPLEMENT=ind_bad,NCOMPLEMENT=nbad)
		;
		if(debug ne 0) then print, '   ngood pts = ',ngood
		;
		if(nbad gt 0 and debug ne 0) then begin
		    print, 'Rejected points:'
		    for ibad = 0, nbad-1 do begin
		        print, 'X, Y, Sky = ',mean_xy[ind_bad[ibad],0],mean_xy[ind_bad[ibad],1],mean_xy[ind_bad[ibad],2]
		    endfor
		endif
		;
		if(ngood ge 0) then begin
			mean_xy = mean_xy[ind_good,*]
		endif else begin
			if(debug ne 0) then print_text, Event, 'Too few good points for background fit'
			(*ptr_model)[this_x0:this_x1,this_y0:this_y1] = 0.0
			goto, y_loop_end
		endelse
		;
		;	have all the points. Next, select either polynomial or TPS or MCS background modeling
		;	
		if(do_tps eq 1 or do_mcs eq 1) then begin
		  ;
		  ; prepare three vectors...
		  ;
		  zvec = reform(mean_xy[*,2])
		  xvec = reform(mean_xy[*,0])
		  yvec = reform(mean_xy[*,1])
		  ;
;		   = MIN_CURVE_SURF(zvec, xvec, yvec, /TPS, GS = [1, 1],$ 
      ;
      ; MCS takes precident
      ;
      if(do_mcs eq 1) then begin
  		  ptr_sub_image = PTR_NEW(MIN_CURVE_SURF(zvec, xvec, yvec, GS=[6,6],$
                BOUNDS=[this_x0,this_y0,this_x1,this_y1]))
      endif else begin
          ;
          ; TPS
          ;
          ptr_sub_image = PTR_NEW(MIN_CURVE_SURF(zvec, xvec, yvec, /TPS, GS=[6,6],$
                BOUNDS=[this_x0,this_y0,this_x1,this_y1]))
      endelse
      ;
      (*ptr_model)[this_x0:this_x1,this_y0:this_y1] = CONGRID((*ptr_sub_image), this_x1 - this_x0 + 1, $
            this_y1 - this_y0 + 1)
  PTR_FREE, ptr_sub_image;
		endif else begin
		  ;
		  ; polynomial background model
  		;	establish maximum fitting order
  		;	supported by the data
  		;
  		order_i = order
  		;
  		order_loop:
  		min_pts = CEIL(float(order_i) * (float(order_i - 1 ) * 2.5))
  		if(ngood lt min_pts) then begin
  			order_i--
  			if(order_i le 0) then stop
  			goto, order_loop
  		endif
  		;
  		degree = order_i < ngood
  		;
  		if(debug ne 0) then print, 'Fitting with degree = ',degree
  		;
  		Result = SFIT( TRANSPOSE( mean_xy), degree, KX=coeffs, /IRREGULAR, /MAX_DEGREE )
  		;
  		this_y = findgen(this_y1 - this_y0 + 1)+0.5+this_y0
  		;
  		for ix = this_x0, this_x1 do begin
  			this_x = float(ix)+0.5
  			;
  			(*ptr_model)[ix,this_y0:this_y1] = 0.0
  			;
  			ik = 0
  			for kx = 0, degree do begin
  				for ky = 0, degree do begin
  					if(kx+ky le degree) then begin
  ;					  for iy = this_y0, this_y1 do begin
  ;					     this_yy = float(iy) + 0.5
  ;					     (*ptr_model)[ix,iy] += coeffs[ik] * (this_yy^ky) * (this_x^kx)
  ;					  endfor
  						(*ptr_model)[ix,this_y0:this_y1] += coeffs[ik] * (this_y^ky) * (this_x^kx)
  						ik++
  					endif
  				endfor
  			endfor
  		endfor
  	endelse
		;
		;	save the difference image, except for deleted data
		;
		(*ptr_model)[this_x0:this_x1,this_y0:this_y1] *= -1.0
		(*ptr_model)[this_x0:this_x1,this_y0:this_y1] += input_image[this_x0:this_x1,this_y0:this_y1]
		;
		; find deleted data
		; 
		ptr_mask = PTR_NEW((*ptr_model)[this_x0:this_x1,this_y0:this_y1])
		(*ptr_mask) *= -1.0
		(*ptr_mask) -= 1.0e5
		(*ptr_mask) <= 1.0    ; all deleted pixels
		(*ptr_mask) >= 0.0    ; all good pixels
		;
		(*ptr_model)[this_x0:this_x1,this_y0:this_y1] = -1.0e6 * (*ptr_mask) + $
		          (1.0 - (*ptr_mask))*(*ptr_model)[this_x0:this_x1,this_y0:this_y1]
		;
		PTR_FREE, ptr_mask
		;
		;	next Y zone
		;
;		print, 'S7 fit with S11 points for image ',FILE_BASENAME(file_path),' quad mode = ',fit_option_mode,$
;		 	' quad being fit (jx,jy) = ',jx,jy,' degree = ',degree,' points = ',ngood
		;
		y_loop_end:
	endfor
	;
	;	next X zone
	;
endfor
;
;	compute the mean background and sigma
;
vec = REFORM( zone_sky[*,*] )
zz_sky = median_filtered_mean(vec)
vec_sig = REFORM( zone_sigma[*,*] )
zz_sig = median_filtered_mean(vec_sig)
;
mean_full_sky = zz_sky[0]
mean_full_sigma = zz_sig[0]
;
;	save result in output image and return
;
output_image = (*ptr_model)
output_image += zz_sky[0]					; put mean sky back into image; respecting bad pixels, again
;
ptr_mask = PTR_NEW(output_image)
(*ptr_mask) *= -1.0   ; invert
(*ptr_mask) -= 1.0e5  ; pull down to save all good negative values under threshold
(*ptr_mask) <= 1.0    ; all deleted pixels were more negative than -1e5
(*ptr_mask) >= 0.0    ; all good pixels
;
output_image = -1.0e6 * (*ptr_mask) + $
          (1.0 - (*ptr_mask))*output_image
;
PTR_FREE, ptr_mask
;
if(debug eq 1) then begin
	FREE_LUN, lu_reg
endif
;
;	free heap
;
PTR_FREE, ptr_model
;
end