pro S4_PSF_fit, Event, image_data, image_header, image_pathname, status, residual_image, NG=ng, SAVE_RESIDUALS=isave_f, $
				SIGMA=sigma, SAT_LIMIT=sat_limit, LLQ_FLAG=llq_flag
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_Struct
	d_ptr = PTR_Struct.PTR_Data
	s_ptr = PTR_Struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	;
	;	perform psf fitting on one image
	;
	;	DPC	20080625
	;	DPC 20081205	imported into PPOL from g_PSF_fit, modified for GUI use
	;	DPC 20081226    modified to run faster
	;	DPC 20090225    modified to do PSF iterative cleaning
	;	DPC 20090917	added multiple apertures to final aper phot
	;
	;---------------------------------------------------------------------------------
	image = image_data					; save copy of original image with bad pixels in place
	;
	if(KEYWORD_SET(llq_flag)) then llq_flag = 1 else llq_flag = 0
	;**
	;**
	;	20120603 - force bad pixel fix for testing
	;
	;image = fix_bad_pixels(image)
	;
	;**
	;**

	start_time = systime(/seconds)
	;
	;	create PSFs in grid (here ng = 1)
	;--------------------------------------------------------------------------------------
	;
	;		constants to pass
	;
	ng 					= 1								; number of zones in each of x and y
	if(N_ELEMENTS(SAT_LIMIT) eq 0) then sat_limit = 9000L	; a reasonable value for Mimir images.
	;
	pass2_sigma 		= sigma
	save_region_files 	= 0									; 0 - none, 1 - all
	;
	flatten_pre_flag 	= (*w_ptr).S4_flatten_image_flag
	fix_bad_pixels_flag = (*w_ptr).S4_fix_bad_pixels_flag
	;
	ap_rad 				= 2.80						; times FWHM
	clean_rad 			= 3.6						; times ap_rad times FWHM
	ap_vec 				= [0.4,0.475,0.55,0.625,$
						   0.7,0.775,0.85,0.925,$
						   1.0,1.05,1.1,1.15]			; times FWHM times ap_rad
	n_ap 				= N_ELEMENTS(ap_vec)
	;
	;------
	fit_scale 			= 1.75					; times fwhm - sets fitting zone ; 20120603 - note these are NOT
																				 ; the 'fudge' factors - those
																				 ; are in the g_create_PSF_Grid routine
	ref_ap 				= [1.2, 1.7]				; times ap_rad times FWHM
	;
	ref_ap *= 1.6		; fudge factor - 20120603 - DPC - for better bright star results
	;
	;-------
	rad_scale 			= 3.00					; times fwhm - sets PSF size
	;
	rad_scale			*= 1.25					; 20120603 - DPC - fudge factor for better PSF modeling
	;
	ronois				= 3.1					; single image readout noise in ADU
	;
	step_dir_text 		= 'S4_First_PSF_Fits'
	ng_textbox_wid 		= (*w_ptr).S4_ng_textbox_wid
	nsig_textbox_wid 	= (*w_ptr).S4_nsig_textbox_wid
	analysis_dir		= (*s_ptr).analysis_dir
	;
	;----------------------------------------------------------------------------------------------
	g_create_PSF_grid, Event, image_data, image_header, image_pathname, sigma, status, $
				NG=ng, SAT_LIMIT=sat_limit, skysig, mean_FWHM, find_image, xborder, $
				yborder, sky, clean_rad, ap_rad, phpadu, apr, skyrad, badpix, $
				bright_limit, rad_PSF, fit_rad, PSF_model, PSF_gauss, PSF_mag, $
				ronois, this_ng, n_ap, ap_vec, ref_ap, rad_scale, n_total_psf_stars, $
				x_psf, y_psf, mag_psf, save_region_files, flatten_pre_flag, $
				fix_bad_pixels_flag, fit_scale, ng_textbox_wid, nsig_textbox_wid, $
				step_dir_text, analysis_dir, LLQ=llq_flag
	;
	if(status lt 0) then begin
		status = -1
		goto, S4PSFF_end
	endif
	;
	this_sky_save = median_filtered_mean(sky)
	this_sky_save = this_sky_save[0]
	this_sky_sigma_save = skysig
	;-----------------------------------------------------------------------||
	;
	bright_limit = 2.0 * sat_limit		; sat_limit is the limit for PSF building stars, bright_limit for unsat. stars
	;
	;	STEP 8: group all found stars into groups
	;
	;	first, find, then do aperture phot on all stars again, both bright and faint
	;
	nsig = sigma
	faint_sigma = 4.75
	max_stars = 500
	max_group = 75
	bad_pix_value = -1.0e6
	do_phot = 'Aper'
	;
	;	get astrometry from header
	;
	header = image_header
	extast,header,astr,noparams
	;
	debug = 0
	;
	image_size = SIZE(find_image)
	xsize = image_size[1]
	ysize = image_size[2]
	;
	hmin = nsig * skysig
	hmin4 = faint_sigma * skysig
	;
	roundlim = [-1.0,0.85]
	sharplim = [0.1,0.85]
	;
	if(mean_FWHM lt 3.0) then begin
		roundlim = [-1.1,0.9]
		sharplim = [0.05,0.95]
	endif
	;
	fwhm = mean_FWHM
	;
;	debug=0
	g_find, find_image, x, y, flux, sharp, roundness, hmin, fwhm, roundlim, sharplim, $
			SILENT=(1-debug), /NO_BAD_PIX_FIX
	;
	ind = WHERE(x gt xborder and x lt (xsize-xborder) and y gt yborder and y lt (ysize-yborder), good_stars)
	;
	if(good_stars gt 0) then begin
		x = x[ind]
		y = y[ind]
		flux = flux[ind]
	endif
	;
	nbright_stars = good_stars
	;
	g_find, find_image, x_f, y_f, flux_f, sharp_f, roundness_f, hmin4, fwhm, roundlim, $
		sharplim, SILENT=(1-debug), /NO_BAD_PIX_FIX
	;
	ind_f = WHERE(x_f gt xborder and x_f lt (xsize-xborder) and y_f gt yborder and y_f lt (ysize-yborder), good_stars_f)
	;
	if(good_stars_f gt 0) then begin
		x_f = x_f[ind_f]
		y_f = y_f[ind_f]
		flux_f = flux_f[ind_f]
	endif
	;
	nfaint_stars = good_stars_f
	;
	;	if necessary, limit to max_stars
	;
	if(nbright_stars gt max_stars) then begin
		ind_sort = REVERSE(SORT(flux))
		max_ind = ind_sort[0:(max_stars-1)]
		x_can = x[max_ind]
		y_can = y[max_ind]
		flux_can = flux[max_ind]
		sky_can = fltarr(max_stars)
		sky_can += sky
		nbright_stars = max_stars
	endif else begin
		x_can = x
		y_can = y
		sky_can = fltarr(nbright_stars)
		sky_can += sky
	endelse
	;
	;	aperture phot the bright stars
	;
	g_aper,find_image,x_can,y_can,mags_can,errap_can,sky_can,skyerr_can,phpadu,apr,skyrad,badpix,bright_limit,$
			SILENT=(1-debug), /NO_BAD_PIX_FIX
	;
	mags_can = REFORM( mags_can[0,*] )
	errap_can = REFORM( errap_can[0,*] )
	;
	ind_good = WHERE( FINITE(mags_can) and errap_can lt 0.3 and mags_can gt -10. and mags_can lt 25., ngood)
	if(ngood gt 0) then begin
		mags_can = mags_can[ind_good]
		x_can = x_can[ind_good]
		y_can = y_can[ind_good]
		errap_can = errap_can[ind_good]
		sky_can = sky_can[ind_good]
		skyerr_can = skyerr_can[ind_good]
		nbright_stars = ngood
	endif
	;
	print_text, Event, 'PASS 1: using '+STRING(nbright_stars,format='(I)')+' bright stars '
	;
	;	associate faint stars within clean_rad of the surviving bright stars
	;
	save_all = intarr(nfaint_stars)
	;
	rad_test = clean_rad * ap_rad * mean_FWHM
	;
	n_can = nbright_stars
	n_f = 0
	if(n_can gt 0) then begin
		n_can1 = n_can-1
		for is = 0, n_can1 do begin
			dx = x_f - x_can[is]
			dy = y_f - y_can[is]
			rad = sqrt(dx * dx + dy * dy)
			ind = SORT(rad)
			save_all[ind[0]] = -1			; index of the current bright star
			jnd = WHERE(rad lt rad_test and save_all ne -1,count_faint)
			if(count_faint gt 0) then begin
				save_all[jnd] = 1			; turn on save flag
			endif
		endfor
		;
		ind_faint = WHERE(save_all eq 1,faint_count)
		if(faint_count ne 0) then begin
			x_f = x_f[ind_faint]				; these are the x, y, flux of faint stars near the bright stars
			y_f = y_f[ind_faint]
			flux_f = flux_f[ind_faint]
			n_f = faint_count
		endif else n_f = 0
	endif
	;
;	print_text, Event, "PASS 1: Number of faint stars near bright ones = "+STRING(n_f,format='(I)')
	;
	n_new_f = 0
	if(n_f ne 0) then begin
		;
		;	aperture phot faint stars
		;
		g_aper,find_image,x_f,y_f,mags_f,errap_f,sky_f,skyerr_f,phpadu,apr,skyrad,badpix,bright_limit,$
			SILENT=(1-debug), /NO_BAD_PIX_FIX
		;
		mags_f = REFORM( mags_f[0,*])
		errap_f = REFORM( errap_f[0,*])
		;
		ind_good_f = WHERE( FINITE(mags_f) and errap_f lt 0.5 and mags_f gt -10. and mags_f lt 25., ngood_f)
		if(ngood_f gt 0) then begin
			mags_f = mags_f[ind_good_f]
			x_f = x_f[ind_good_f]
			y_f = y_f[ind_good_f]
			errap_f = errap_f[ind_good_f]
			sky_f = sky_f[ind_good_f]
			skyerr_f = skyerr_f[ind_good_f]
			n_new_f = ngood_f
		endif else n_new_f = 0
		;
		print_text, Event, "PASS 1: Number of faint stars with good photometry = "+STRING(n_f,format='(I)')
		;
;		if(n_f_good lt n_f) then stop
		;
	endif
	n_f = n_new_f
	;
	if(n_f gt 0) then begin
		;
		;	merge lists
		;
		x_both = [x_can,x_f]
		y_both = [y_can,y_f]
		mags_both = [mags_can, mags_f]
		sky_both = [sky_can, sky_f]
		errap_both = [errap_can, errap_f]
	endif else begin
		;
		;	merge, but no faint stars
		;
		x_both = x_can
		y_both = y_can
		mags_both = mags_can
		sky_both = sky_can
		errap_both = errap_can
	endelse
	;
	nstars1 = n_can + n_f
	;
;	debug = 1
	;
	print_text,Event, "PASS 1: grouping "+STRING(format='(I)',nstars1)+" bright + faint neighbor stars"
	;
	rcrit = rad_PSF + fit_rad
	;
	;group, Event, x, y, rcrit, group_vector
	;
	rcrit0 = rcrit
	;
	group_9_loop = 0
	;
	loop_group_9:
	;
	if(N_ELEMENTS(x_both) eq 1) then begin
		group_vector = intarr(1)
	endif else begin
		group, Event, x_both, y_both, rcrit0, group_vector
	endelse
	;
	;	check the groups -- if max has more than 100 stars, reset rcrit
	;
	this_max_stars = MAX(HISTOGRAM(group_vector,MIN=0))
	if(this_max_stars gt max_group) then begin
		print_text,Event, "WARNING: PASS1 grouping found a group with "+STRING(format='(I)',max_stars)+" stars."
		rcrit0 = 0.9*rcrit0
		group_9_loop++
		;
		if(group_9_loop le 5) then begin
			print_text,Event, "   reducing rcrit to "+STRING(rcrit0)+" pixels and retrying. Loop number "+STRING(group_9_loop)
			goto, loop_group_9
		endif else begin
			print_text, Event, "   still too many stars in largest group at max loop - WARNING"
		endelse
	endif
	;
	;	STEP 9: For all groups, form PSFs, fit group stars, remove from image
	;
	star_id = indgen(nstars1)
	iplot=0
	isave=0
	print=0
	message1 = 'file to save stellar print info'
	message2 = 'image for 1st order star removal'
	idpc=0
;
;stop
;
	g_fit_and_remove_stars, this_ng, find_image, resid_image, x_both, y_both, mags_both, errap_both, sky_both, $
			star_id, group_vector, PSF_model, PSF_gauss, PSF_rad, PSF_mag, sum_resid, $
			phpadu, ronois, fit_rad, PRINT=print, MESSAGE1=message1, IPLOT=iplot, $
			ISAVE=isave, MESSAGE2=message2, idpc, status
	;
	;	check run status
	;
	if(status eq -1) then begin
		print_text, Event, "ERROR - PSF computation fails"
		stop
	endif
	;
	;	look for case of no good stars fit
	;
	test = N_ELEMENTS(group_vector)
	if(test eq 1) then begin
		if(group_vector eq -1) then begin
			;
			;	no stars were fit - quit remaining processing
			;
			print_text, Event, "group_vector empty - setting status = -1 and quitting"
			status = -1
			goto, S4PSFF_end
		endif else begin
			print_text, Event, '1 group only'
		endelse
	endif
	;
	ind_good_fit = WHERE( FINITE(mags_both) and mags_both lt 25. and mags_both gt -10. and errap_both lt 0.5, n_good_fit)
	;
;	stop
	;
	if(n_good_fit gt 0) then begin
		;
		xmerge = x_both[ind_good_fit]
		ymerge = y_both[ind_good_fit]
		magmerge = mags_both[ind_good_fit]
		errmagmerge = errap_both[ind_good_fit]
		skymerge = sky_both[ind_good_fit]
		nstars = N_ELEMENTS(xmerge)
	endif else begin
		print_text, Event, "S4_PSF_FIT: ERROR - no stars survived PSF fitting."
		;
;		debug = 0
		;
		if(debug gt 0) then begin
			nfit = N_ELEMENTS(x)
			print, "Star   X    Y    mag    errmag"
			for ifit = 0, nfit-1 do begin
				print, x[ifit], y[ifit], mags[ifit], errmag[ifit]
			endfor
			;
			stop
			;
		endif
		status = -1
		goto, S4PSFF_end
	endelse
	;
	;	check for stars with good photometry
	;
	nstars = N_ELEMENTS(xmerge)
	;
	;print, "Tried to fit and remove a total of ",nstars," stars"
	;
	good_ind = WHERE(abs(abs(magmerge) - 99.99) gt 1.0 and FINITE(magmerge),mcount)
	;
	;print, "...of those, ",mcount," had good fitted magnitudes."
	;
	xmerge = xmerge[good_ind]
	ymerge = ymerge[good_ind]
	magmerge = magmerge[good_ind]
	errmagmerge = errmagmerge[good_ind]
	skymerge = skymerge[good_ind]
	;
	star_id = indgen(mcount)
	;
	;	at this point, have list of stars and provisional magnitudes
	;
	;	next, implement PSF-fitting assisted aperture photometry - 20090402 - DPC
	;
	;	1. regroup remaining stars
	;	2. for each star in a group
	;		A. fit and remove all other stars
	;		B. do PSF fitting of single star on residual image
	;
	rcrit0 = rcrit
	iloop_final = 0L
	;
	loop_group_final:
	;
	print_text, Event, "Loop :"+STRING(iloop_final+1,format='(I)')+" Rad_test = "+STRING(rad_test)+" Rcrit = "+STRING(rcrit0)
	;
	if(N_ELEMENTS(xmerge) eq 1) then begin
		group_vector_final = intarr(1)
	endif else begin
		group, Event, xmerge, ymerge, rcrit0, group_vector_final
	endelse
	;
	max_group_final = MAX(HISTOGRAM(group_vector_final,MIN=0))
	iloop_final = 0L
	if(max_group_final gt max_group) then begin
		print_text, Event, "WARNING: Final grouping found too many stars in one group. Max found = "+STRING(max_group_final)
		print_text, Event, "   Loop "+STRING(iloop_final+1)+"... reducing rcrit from "+STRING(rcrit0)+" to "+STRING(0.9*rcrit0)
		rcrit0 = 0.9* rcrit0
		iloop_final++
		if(iloop_final le 19) then goto, loop_group_final
		print_text, Event,"  Have reduced rcrit as much as is reasonable, trying to fit as-is"
	endif
	;
	n_groups = N_ELEMENTS(UNIQ(group_vector_final,SORT(group_vector_final)))
	;
	xfinal = 		dblarr(mcount)
	yfinal = 		dblarr(mcount)
	magfinal = 		dblarr(mcount,n_ap)
	errmagfinal = 	dblarr(mcount,n_ap)
	skyfinal = 		dblarr(mcount)
	errskyfinal = 	dblarr(mcount)
	groupfinal = 	lonarr(mcount)
	;
	istar_final = 0L
	;
	print_text, Event, "Final star list has "+STRING(mcount)+" stars spread over "+STRING(n_groups)+" groups"
	;
	n_groups1 = n_groups-1
	for ig = 0, n_groups1 do begin
		;
		;	how many stars in this group?
		;
		ind_group = WHERE(group_vector_final eq ig, n_g_stars)
		;
;		print_text, Event, "..group "+STRING(ig+1)+" has "+STRING(n_g_stars)+" stars "
		;
		;start_group_time = SYSTIME(/SECONDS)
		;
		;	if 1 star perform aperture photometry on it
		;
		if(n_g_stars eq 1) then begin
			xf = [xmerge[ind_group[0]]]
			yf = [ymerge[ind_group[0]]]
			;
			;	check that star isn't within border region
			;
			if(xf lt xborder or xf gt (xsize - xborder)) then goto, in_border
			if(yf lt yborder or yf gt (ysize - yborder)) then goto, in_border
			;
			;	first, test to be sure there aren't deleted pixels within
			;	closest aperture radius
			;
			xmin1 = xborder > FLOOR(xf - apr[0])
			xmax1 = (xsize - xborder) < CEIL(xf + apr[0])
			ymin1 = yborder > FLOOR(yf - apr[0])
			ymax1 = (ysize - yborder) < CEIL(yf + apr[0])
			;
			ind_bad = WHERE(image[xmin1:xmax1, ymin1:ymax1] eq -1.0e6,bad_count)
			;
			if(bad_count le 1) then begin			; allow up to 1 bad pixel
;				debug = 0
				case do_phot of
					'Aper': begin
								this_apr = ap_vec * ap_rad * mean_FWHM
								skyrad = ref_ap * ap_rad * mean_FWHM
								;
								g_aper,find_image,xf,yf,magf,errf,skyf,skyerrf,phpadu,this_apr,skyrad,$
									badpix,bright_limit,SILENT=(1-debug),/NO_BAD_PIX_FIX
									;
							end
					'Gaus':	begin
								g_gaussian_fit_one_star, find_image, mean_fwhm, xf, yf, mag, mag_unc, sky, sky_unc, debug
								xf = [xf]
								yf = [yf]
								magf = [mag]
								errf = [mag_unc]
								skyf = [sky]
								skyerrf = [sky_unc]
							end
					'PSF':	begin
								x = [xf]
								y = [yf]
								mags = [magmerge[ind_group[0]]]
								errmag = [errmagmerge[ind_group[0]]]
								sky = [skymerge[ind_group[0]]]
								n_this_g = N_ELEMENTS(ind_group[0])
								this_group = intarr(n_this_g)
								fake_star_id = indgen(n_this_g)
								no_nstar = 0						; fit this star
								;
								g_fit_and_remove_stars, ng, find_image, resid_image, x, y, mags, errmag, sky, $
									fake_star_id, this_group, PSF_model, PSF_gauss, PSF_rad, PSF_mag, sum_resid, $
									phpadu, ronois, fit_rad, PRINT=iprint, MESSAGE1=message1, IPLOT=iplot, $
									ISAVE=isave, MESSAGE2=message2, idpc, status, NO_NSTAR=no_nstar
								;
								magf = mags
								errf = errmag
								skyf = sky
								skyerrf = 0.0		; oops...
							end
				endcase
				;
				;	save results
				;
				xfinal[istar_final] = xf[0]
				yfinal[istar_final] = yf[0]
				magfinal[istar_final,*] = magf[*]
				errmagfinal[istar_final,*] = errf[*]
				skyfinal[istar_final] = skyf
				errskyfinal[istar_final] = skyerrf
				groupfinal[istar_final] = ig
				;
				istar_final++
			endif else begin
				if(debug gt 0) then print, "Skipping single star at X,Y = ",xf, yf," as having ",bad_count," nearby bad pixels"
			endelse
			;
			;print, "processed with g_aper directly to save star number ",istar_final
			;
			in_border:
			;
		endif else begin
			;
			;	more than one star. cycle through the stars
			;
			;print, "need to process the ",n_g_stars," stars in this group with PSF-assistance..."
			;
			n_g_stars1 = n_g_stars-1
			for i_g_star = 0, n_g_stars1 do begin
				;
				this_star_ind = ind_group[i_g_star]
				;
				;	if this star is *NOT* on the candidate list, dump this loop iteration
				;
				this_x = xmerge[this_star_ind]
				this_y = ymerge[this_star_ind]
				;
				dx_can = this_x - x_can
				dy_can = this_y - y_can
				;
				dr_can = sqrt(dx_can * dx_can + dy_can * dy_can)
				ind_can = WHERE(dr_can le fwhm, n_match_can)
				if(n_match_can eq 0) then goto, skip_this_star

				;print, "... processing star ",i_g_star," in the group, which is STAR_ID number ",this_star_ind
				;
				;	create a modified group vector with this star removed
				;
				new_group_ind = ind_group
				;
				remove, i_g_star, new_group_ind			; removes the selected element from the ind_vec
				;
				;print, "... plan to remove, via PSF fitting STAR_IDs = ",new_group_ind
				;
				;	load vectors for the surviving stars
				;
				x = [xmerge[new_group_ind]]
				y = [ymerge[new_group_ind]]
				mags = [magmerge[new_group_ind]]
				errmag = [errmagmerge[new_group_ind]]
				sky = [skymerge[new_group_ind]]
				n_this_g = N_ELEMENTS(new_group_ind)
				this_group = intarr(n_this_g)
				fake_star_id = indgen(n_this_g)
				;
				;print, "...... preparing to remove stars. star_id vec = ",star_id," this group = ",this_group
				;
				;	next, remove (without new nstar fitting) all the surviving stars from the image
				;
				iplot=0
				isave=0
				iprint=0
				message1 = 'file to save stellar print info'
				message2 = 'image for 1st order star removal'
				idpc=0
;				mags = REFORM(mags[0,*])
;				errmag = REFORM(errap[0,*])
				no_nstar = 1
				;
				;print, "Calling G_FIT_AND_REMOVE_STARS with iprint, iplot, isave = ",iprint, iplot, isave
				;
				g_fit_and_remove_stars, ng, find_image, resid_image, x, y, mags, errmag, sky, $
						fake_star_id, this_group, PSF_model, PSF_gauss, PSF_rad, PSF_mag, sum_resid, $
						phpadu, ronois, fit_rad, PRINT=iprint, MESSAGE1=message1, IPLOT=iplot, $
						ISAVE=isave, MESSAGE2=message2, idpc, status, NO_NSTAR=no_nstar
				;
				;	and then prepare to do aperture photometry for the designated star on the
				;	residual image
				;
				xmin1 = xborder > FLOOR(this_x - apr[0])
				xmax1 = (xsize - xborder) < CEIL(this_x + apr[0])
				ymin1 = yborder > FLOOR(this_y - apr[0])
				ymax1 = (ysize - yborder) < CEIL(this_y + apr[0])
				;
				ind_bad = WHERE(image[xmin1:xmax1, ymin1:ymax1] eq -1.0e6,bad_count)
				;
				if(bad_count le 1) then begin		; can have 1 bad pixel
;					debug = 0
					case do_phot of
						'Aper': begin
									this_apr = ap_vec * ap_rad * mean_FWHM
									skyrad = ref_ap * ap_rad * mean_FWHM
									;
									g_aper,resid_image,this_x,this_y,magf,errf,skyf,skyerrf,phpadu,this_apr,$
										skyrad,badpix,bright_limit,$
										SILENT=(1-debug),/NO_BAD_PIX_FIX
									;
								end
						'Gaus':	begin
									g_gaussian_fit_one_star, resid_image, mean_fwhm, this_x, this_y, mag, mag_unc, sky, sky_unc, debug
									this_x = [this_x]
									this_y = [this_y]
									magf = [mag]
									errf = [mag_unc]
									skyf = [sky]
									skyerrf = [sky_unc]
								end
						'PSF':	begin
									x = [this_x]
									y = [this_y]
									mags = [magmerge[this_star_ind]]
									errmag = [errmagmerge[this_star_ind]]
									sky = [skymerge[this_star_ind]]
									n_this_g = N_ELEMENTS(this_star_ind)
									this_group = intarr(n_this_g)
									fake_star_id = indgen(n_this_g)
									no_nstar = 0						; this time, fit the star
									;
									g_fit_and_remove_stars, ng, resid_image, final_resid_image, x, y, mags, errmag, sky, $
										fake_star_id, this_group, PSF_model, PSF_gauss, PSF_rad, PSF_mag, sum_resid, $
										phpadu, ronois, fit_rad, PRINT=iprint, MESSAGE1=message1, IPLOT=iplot, $
										ISAVE=isave, MESSAGE2=message2, idpc, status, NO_NSTAR=no_nstar
									;
									magf = mags
									errf = errmag
									skyf = sky
									skyerrf = 0.0		; oops...
								end
					endcase
					;
					;print,"...created resid_image for aperture fitting at X,Y = ",this_x[0], this_y[0]
					;
;					g_aper,resid_image,this_x,this_y,magf,errf,skyf,skyerrf,phpadu,apr,skyrad,badpix,bright_limit,$
;						SILENT=(1-debug),NO_BAD_PIXEL_FIX=1
					;
					;	save results
					;
					xfinal[istar_final] = this_x[0]
					yfinal[istar_final] = this_y[0]
					magfinal[istar_final,*] = magf[*]
					errmagfinal[istar_final,*] = errf[*]
					skyfinal[istar_final] = skyf
					errskyfinal[istar_final] = skyerrf
					groupfinal[istar_final] = ig
					;
					istar_final++
					;
					;print,"...using PSF-assisted aperture phot, measured and saved star number ",istar_final
				endif else begin
					if(debug gt 0) then print, "Skipping grouped star at X,Y = ",this_x, this_y," as having ",bad_count," nearby bad pixels"
				endelse
				;
				in_border_2:
				;
				skip_this_star:
			endfor										; process the next star in group
		endelse											; this group processed
		;
;		end_group_time = SYSTIME(/SECONDS)
;		print, "Group ",ig," took ",end_group_time - start_group_time," seconds to process"
	endfor												; all groups processed
	;
;	assist_fit_time = SYSTIME(/SECONDS)
;	assist_time = assist_fit_time - fit_time
;	print_text, Event, "Time to compute PSF-fitting-assisted aperture photometry = "+STRING(assist_time)+" seconds"
	;
	;	compress result vectors
	;
	nstars = istar_final
	;
	ns1 = nstars-1
	;
	xfinal = xfinal[0:ns1]
	yfinal = yfinal[0:ns1]
	magfinal = magfinal[0:ns1,*]
	errmagfinal = errmagfinal[0:ns1,*]
	;
	mag_best = dblarr(nstars)
	errmag_best = dblarr(nstars)
	;
	;	preload best arrays if not aperture phot
;	;
	skyfinal = skyfinal[0:ns1]
	errskyfinal = errskyfinal[0:ns1]
	groupfinal = groupfinal[0:ns1]
	;
	;print_text, Event, "Completed photometry for "+STRING(nstars)+" stars"
	;
	;	next, load PSF-fitting-assisted aperture phot values
	;
	xmerge = xfinal[0:ns1]
	ymerge = yfinal[0:ns1]
	magmerge = magfinal[0:ns1,*]
	errmagmerge = errmagfinal[0:ns1,*]
;	magmerge = mag_best[0:ns1]
;	errmagmerge = errmag_best[0:ns1]
	skymerge = skyfinal[0:ns1]
	groupmerge = groupfinal[0:ns1]
	;
;	stop
	;
	;	trim to good photometry
	;
	ind_good_phot = WHERE(errmagmerge[*,3] le 0.25 and FINITE(magmerge[*,3]), n_good_phot)
	if(n_good_phot gt 0) then begin
		xmerge = xmerge[ind_good_phot]
		ymerge = ymerge[ind_good_phot]
		magmerge = magmerge[ind_good_phot,*]
		errmagmerge = errmagmerge[ind_good_phot,*]
		skymerge = skymerge[ind_good_phot]
		groupmerge = groupmerge[ind_good_phot]
	endif
	;
	;	then, if > max_stars, sort by brightness
	;
	if(n_good_phot gt max_stars) then begin
		ind_brite = SORT(magmerge[*,3])
		ind_save = ind_brite[0:(max_stars-1)]
		xmerge = xmerge[ind_save]
		ymerge = ymerge[ind_save]
		magmerge = magmerge[ind_save,*]
		errmagmerge = errmagmerge[ind_save,*]
		skymerge = skymerge[ind_save]
		groupmerge = groupmerge[ind_save]
	endif
	;
	nstars = N_ELEMENTS(xmerge)
	;
	;	dump values
	;
;	if(debug eq 1) then begin
;		for ic = 0, count-1 do begin
;			print_text, Event, "x,y,mag,sky = "+STRING(xp[ic],yp[ic],magp[ic],skyp[ic])
;		endfor
;	endif
	;
	FXADDPAR, header, 'HISTORY', "PPOL-Step 4-version "+(*s_ptr).version_short_string
	SXADDPAR, header, 'PSF_NG', ng, "Number of X, Y grid zones in psf build"
	SXADDPAR, header, 'PSF_SIG', pass2_sigma, "Detection threshold in sigma of sky"
	SXADDPAR, header, 'PSF_SAT', sat_limit, "Saturation Limit for allowed stars"
	SXADDPAR, header, 'PSF_FWHM', fwhm,"FWHM in pixels"
	SXADDPAR, header, 'PSF_FRAD', fit_rad, "PSF fitting radius, in pixels"
	SXADDPAR, header, 'PSF_RSCL', rad_scale, "PSF model size, in FWHM units"
	;
	SXADDPAR, header, 'PSFMAG', PSF_mag,"nominal PSF magnitude generated internally"
	;
	SXADDPAR, header, 'nPSF', n_total_PSF_stars
	nPSF = n_total_PSF_stars
	for inPSF = 0, nPSF-1 do begin
		PSFnum = STRING(inPSF+1, format='("PSF_",i3.3)')
		SXADDPAR, header, PSFnum+"X",x_psf[inPSF],"X location of this PSF star"
		SXADDPAR, header, PSFnum+"Y",y_psf[inPSF],"Y location of this PSF star"
		SXADDPAR, header, PSFnum+"M",mag_psf[inPSF],"Magnitude of this PSF star"
	endfor
	;
	;	add aperture phot info
	;
	SXADDPAR, header, 'PP4_FWHM', fwhm, "FWHM used in aperture phot, in pixels"
	SXADDPAR, header, 'PP4_ARAD', ap_rad, "Maximum aperture radius in FWHM units"
	SXADDPAR, header, 'PP4_REF0', ref_ap[0],"Reference aperture inner radius in FWHM units"
	SXADDPAR, header, 'PP4_REF1', ref_ap[1],"Reference aperture outer radius in FWHM units"
	SXADDPAR, header, 'PP4_CLEN', clean_rad, "Radius to PSF clean to, in FWHM * ARAD units"
	SXADDPAR, header, 'PP4_NAP', n_ap,"Number of apertures used in Ap Phot"
	for iap = 0, n_ap-1 do begin
		name = 'PP4_AP'+STRING(iap,format='(I2.2)')
		SXADDPAR, header, name, ap_vec[iap], "Aperture "+STRING(iap+1,format='(I2)')+" in FWHM * ARAD units"
	endfor
	;
	;	and sky background and sigma info
	;
	SXADDPAR, header, 'PP4_SKY', this_sky_save, 'Mean sky value'
	SXADDPAR, header, 'PP4_SSKY', this_sky_sigma_save, 'Sky standard deviation'
	SXADDPAR, header, 'PP4_LLQ', llq_flag, 'LLQ bad flag (1=bad; 0=good)'
	print_text, Event, 'Mean, Stdev Sky = '+STRING(this_sky_save,this_sky_sigma_save,format='(f9.1,2x,f9.2)')
	print_text, Event, 'LLQ bad flag = '+STRING(llq_flag,format='(i1)')
	;
	image_header = header
	;
	;print_text,Event, "Finished fitting "+STRING(format='(I)',nstars)+" stars. Ready to save final photometry"
	;
	;	sort by RA (reverse X)
	;
	igood = 0L
	;
	if(nstars gt 0) then begin
		ind = REVERSE(SORT(xmerge))
		;
		nstars1 = nstars-1
		for is = 0, nstars1 do begin
			;
			;	only keep stars with uncertainties under 0.25 mag	(20090720 DPC)
			;
			(*d_ptr).x[igood] = xmerge[ind[is]]
			(*d_ptr).y[igood] = ymerge[ind[is]]
			(*d_ptr).ra[igood] = 0.0
			(*d_ptr).dec[igood] = 0.0
			for iap = 0, n_ap-1 do begin
				(*d_ptr).mag[igood,iap] = (magmerge[ind[is],iap] > (-9.999)) < 25.0	 ; trap bad at -10 mag and +25 mag
				(*d_ptr).errmag[igood,iap] = errmagmerge[ind[is],iap] < 9.999  ; errors, too
			endfor
			(*d_ptr).sky[igood] = skymerge[ind[is]]
			(*d_ptr).psf_group[igood] = groupmerge[ind[is]]
			igood++
			;
		endfor
		nstars_good = igood
		(*d_ptr).nstars = nstars_good
		;
		;	if astrometry present, include RA, Dec output info
		;
		if(noparams eq -1) then begin
			;
			;	no astrometry
			;
		endif else begin
			;
			;xy2ad, xmerge[ind[0:nstars-1]], ymerge[ind[0:nstars-1]], astr, a, d
			ns1 = nstars_good-1
			if(ns1 gt 0) then begin
				xvec = (*d_ptr).x[0:ns1]
				yvec = (*d_ptr).y[0:ns1]
				xy2ad, xvec, yvec, astr, a, d
				for i = 0, ns1 do begin
						(*d_ptr).ra[i] = a[i]
						(*d_ptr).dec[i] = d[i]
				endfor
			endif
		endelse
		;
		end_time = SYSTIME(/SECONDS)
		print_text,Event, "** Completed photometry for "+STRING(nstars_good)+" stars in "+STRING(format='(I)',end_time - start_time)+" sec."
		print_text, Event, "**------------------------------------**"
		;
		;
		status = 0			; all worked !
		;
	endif else status = -1
	;
	S4PSFF_end:
	;
;	stop
	;
end
