pro S6_ap_correction_factors, Event, group_name, group_summary_file
	;
	;  10/30/2008 -- original code -- ap
	;  12/3/2008 -- sorts by julian date,
	;               keeps only ones place and decimal of julian date -- afp
	;
	;	DPC	20081221	imported into MSP_PPOL
	;	DPC 20090920	added multi-aperture loop and selection of best aperture
	;	DPC 20091214	added number of points to calc of merit function, bumped dither threshold
	;	DPC 20091228	added reference image and mis-order capability to work with zero hops
	;	DPC 20170814  added amplitude display, kept scaling values even if USE=0 (for photometry overrides later)
	;---------------------------------------------------
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	;
	;	thresholds for detection, mitigation
	;
	fwhm_threshold = 5.0		; FWHM
	flux_threshold = 5.0		; FWHM
	dither_noise_threshold = 7.0	; FWHM
	max_fit_degree = 5			; maximum secular fitting polynomial degree
	;
	plot_wid = (*w_ptr).S6A_plot_window_wid
	(*w_ptr).S6A_group_name = group_name
	S6_populate_textbox, Event, 0
	;
	sky_plot_wid = (*w_ptr).S6A_sky_plot_window_wid
	fp_plot_wid = (*w_ptr).S6A_fp_plot_window_wid
	;
	device,decompose=0
	color_names = ["black", "white", "red", "orange", "yellow", "green", "blue", "violet", "gray"]
	color_values= [10,11,12,13,14,15,16,17,18,19]
	black = 10 & white = 11 & red = 12 & orange = 13 & yellow = 14 & green = 15 & blue = 16 & violet = 17 & gray = 18
	device, decomposed=0
	loadct, 13, /silent
	tvlct,[  0,255,255,255,255, 50,  0,148,233], $
	      [  0,255,  0,165,200,185,  0,  0,233], $
	      [  0,255,  0,  0,  0, 50,255,211,233], 10
	;---------------------------------------------------------------------------------
	;
	;	clear the plots
	;
	WIDGET_CONTROL, plot_wid, GET_VALUE=win_no
	wset, win_no
	erase, color=18
	WIDGET_CONTROL, sky_plot_wid, GET_VALUE=win_no
	wset, win_no
	erase, color=18
	WIDGET_CONTROL, fp_plot_wid, GET_VALUE=win_no
	wset, win_no
	erase, color=18
	;
	;	read in summary list
	;
	;file_in = dialog_pickfile(Title="Select post-psf match list (from dan's program)",path="E:\Pinnick\")
	file_in = group_summary_file
	;
	print_text, Event, "Reading match summary file "+FILE_BASENAME(file_in)
	;
	;	reads in list from file, saving into a header any line beginning with semicolon (;)
	;
	header = strarr(100)
	nheader = 0L
	openr, in_lun, file_in, /GET_LUN
	info_list = strarr(1000)
	num=0
	line_in=''
	while (~EOF(in_lun)) do begin
	    readf, in_lun, line_in, format='(a)'
	    ;
	    ;	test for semicolon in any column
	    ;
	    s_test = STRPOS(line_in, ';')
	    if(s_test eq -1) then begin
	    	;
	    	;	no, so was a summary line, save it
	    	;
		    info_list[num]=line_in
			num++
		endif else begin
			;
			;	yes, is a header line, save it
			;
			header[nheader] = STRMID(line_in,s_test+1)
			nheader++
		endelse
	endwhile
	FREE_LUN, in_lun
	info_in_array=info_list[0:num-1]
	;
	;	how many apertures represented?
	;
	n_ap = SXPAR(header, "PP4_NAP")
	;
	if(n_ap eq 0) then begin
		print_text, Event, 'ERROR - Number of apertures not found in summary file'
		stop
	endif else begin
		print_text, Event, 'Match summary file contains data for '+STRING(n_ap,format='(I2)')+' apertures'
	endelse
	;
	;	get key info to pass on to output file
	;
	ref_image = SXPAR(header, "PP5_RFIM")		; which image is the reference one?
	save_PP5_RFIM = ref_image					       ; save reference image
	;
	print, "reference image number = ",ref_image
	;
	save_PP4_ARAD = SXPAR(header, "PP4_ARAD")
	save_PP4_REF0 = SXPAR(header, "PP4_REF0")
	save_PP4_REF1 = SXPAR(header, "PP4_REF1")
	save_PP4_CLEN = SXPAR(header, "PP4_CLEN")
	;
	save_camera = SXPAR(header, "MB_CAMRA",COUNT=ncam)
	save_filter = SXPAR(header, "MB_FILTR",COUNT=nfilt)
	if(ncam eq 0 or nfilt eq 0) then stop
	;
	;
	foo_info = {	file_num:				0L, $
					file_name:				'', $
					julian_date:		 0.0D0, $
					hwp:				   0.0, $
					n_stars:				0L, $
					del_mag:		 dblarr(n_ap), $
					mag_uncer:		 dblarr(n_ap), $
					x_offset:			 0.0d0, $		; X offset to add to this image to match reference
					y_offset:			 0.0d0, $
					fwhm:				 0.0d0, $
					use_flag:		 lonarr(n_ap), $
					dither:					0L, $		; dither sequence number
					basis:			 strarr(n_ap), $
					ff_num:					0L  $
				}
	;
	cf_data=replicate(foo_info,num)
	;
	;	set up the nominal hwp values and fold angles
	;
	nom_hwp_angle = [0.0, 22.3, 45.2, 67.5, 89.8, 112.7, 135.0, 157.3, $
					 180.2, 202.5, 224.8, 247.7, 270.0, 292.3, 315.2, 337.5]
	nom_fold_angle = [0, 45, 90, 135, 0, 45, 90, 135, 0, 45, 90, 135, 0, 45, 90, 135]
	;
	;	put each line into the structure
	;
	for i=0, num-1 do begin
		next_line=info_in_array[i]
		next_line_split=strsplit(next_line,' ',/regex,/extract)
		n_parts = N_ELEMENTS(next_line_split)
		;
		cf_data[i].file_num=fix(next_line_split[0])
		;
		;	windows allows blanks in filenames and directories. have to
		;	locate that and patch it here
		;
		;	look forward in the parts until "phot.dat" is found - this is the
		;		last part making up the filename
		;
		last_name_part = 1
		for ip = 1, n_parts-1 do begin
			test = STRPOS(next_line_split[ip], "phot.dat")
			if(test ne -1) then begin
				last_name_part = ip
				goto, out_loop
			endif
		endfor
		out_loop:
		;
		;	reconstruct the name
		;
		name = ''
		for ip = 1, last_name_part do begin
			name = name + next_line_split[ip]
		endfor
		;
		cf_data[i].file_name=name
		;
		;	all others are offset from the last name part
		;
		lnp = last_name_part - 1
		;
		cf_data[i].julian_date=double(next_line_split[lnp+2])-2000000.0d0
		cf_data[i].hwp=float(next_line_split[lnp+3])
		cf_data[i].n_stars=fix(next_line_split[lnp+4])
		;
		cf_data[i].x_offset = double(next_line_split[lnp+7])
		cf_data[i].y_offset = double(next_line_split[lnp+8])
		;
		cf_data[i].fwhm = double(next_line_split[lnp+12])
		;
		;	get the multi-aperture dmag and sdmag values
		;
		for iap = 0, n_ap-1 do begin
			cf_data[i].del_mag[iap] =double(next_line_split[lnp+13+2*iap])
			cf_data[i].mag_uncer[iap]=double(next_line_split[lnp+14+2*iap])
			cf_data[i].use_flag[iap] = 1L
		endfor
		;
		;
		current_hwp=cf_data[i].hwp
		;
		;	match to nominal HWP and fold angle
		;
		d_hwp = abs(abs(current_hwp) - nom_hwp_angle)
		d_hwp_min = MIN(d_hwp,ind_match)
;		ind_match = WHERE(d_hwp eq d_hwp_min)
		;
		;	load
		;
		cf_data[i].hwp = nom_hwp_angle[ind_match]
		cf_data[i].ff_num = nom_fold_angle[ind_match]
		;
;		print, 'image ',i,' has hwp ',current_hwp,' and ind_match ',ind_match[0],' and fold_ang ',cf_data[i].ff_num
		;
	endfor
	;
	;	sort data by julian date
	;
	j_date_order=sort(cf_data.julian_date)
	cf_data=cf_data(j_date_order)
	;
;	print, 'jd sort order',j_date_order
	;
	;	remove julian date offset and reset to minutes basis, time referenced to reference image
	;
	;julian_extra = floor(cf_data[0].julian_date*1.0d1) / 10.0d0
	cf_data.julian_date = (cf_data.julian_date - cf_data[ref_image].julian_date)*24.0d0*60.0d0
	;
	;	flag each image by its dither number
	;
	Set_Dither, Event, cf_data
	;
	;print,"dither assignment by djt"
	n_d = N_ELEMENTS(cf_data)
;	for idt = 0, n_d-1 do begin
;		print, "iDt, d_time, dither = ",idt,cf_data[idt].julian_date,cf_data[idt].dither
;	endfor
;	stop
	max_dither = cf_data[n_d-1].dither
	;
	;	first test: compute FWHM mean, sigma for this set of images, flag and turn off
	;		any deviating high by more than 5 sigma - only do once for entire set of apertures
	;
	fwhm = cf_data.fwhm
	zz_fwhm = median_filtered_mean(fwhm)
	;
	fwhm_sigma_test = (zz_fwhm[1] > (0.2 * zz_fwhm[0])) > 0.2  ; raised to 20% of mean 20170814 from 5%
	;
	ind_bad = WHERE((fwhm - zz_fwhm[0])/ fwhm_sigma_test gt fwhm_threshold, nbad)
	;
	;	set the bad ones to 'do not use'
	;
	if(nbad gt 0) then begin
		cf_data[ind_bad].use_flag[*] = 0L
		cf_data[ind_bad].basis[*] = 'FWHM'
	endif
	;
	;==================================================================
	;
	;	BIG LOOP on APERTURE Number - do the following for each of the apertures:
	;
	;==================================================================
	;
	save_cf_mult = dblarr(2,n_ap,num)
	save_cf_secular = dblarr(2,n_ap,num)
	save_cf_intensity = dblarr(2,n_ap,num)
	save_cf_pol = dblarr(2,n_ap,num)
	save_cf_use = dblarr(2,n_ap,num)
	save_cf_sky = dblarr(2,n_ap,num)
	;
	save_fluxes = fltarr(n_ap,num)
	save_fluxerr = fltarr(n_ap,num)
	save_secular_cf = fltarr(n_ap,num)
	save_intensity_cf = fltarr(n_ap,num)
	save_pol_cf = fltarr(n_ap,num)
	save_sky_cf = fltarr(n_ap,num)
	save_fp = fltarr(n_ap,2)
	save_ind_bad = lonarr(n_ap,num)
	;
	;
	sky_noise_ap = dblarr(n_ap,2)
	fp_uncert_ap = dblarr(n_ap,2)
	ngood_ap = dblarr(n_ap, 2)			; number of good values used
	nbad_ap = dblarr(n_ap)				; number of bad values turned off
	best_aper_no = 0L
	;
	best_sky_noise = 0.0d0
	best_fp_uncert = 0.0d0
	best_nbad = 0L
	;
	this_aperture_use = intarr(n_ap,num)
	this_aperture_use[*,*] = 1					; to start
	;
	for iap = 0, n_ap-1 do begin
		;
		;	put into April's variables - note vector nature of these, though (DPC 20090920)
		;
		mags       = cf_data.del_mag[iap]
		mags_uncer = cf_data.mag_uncer[iap]
		;
		;	convert to flux units, for a convenient reference point
		;
		fluxes = 10.0d0^(-0.40d0 * mags)
		;
		;	--- two pass loop - first pass determines properties and turns off use flags for bad dithers
		;	                  - second pass redetermines properties with bad dithers absent
		;
		for ipass = 0, 1 do begin
			;
			;	and determine mean properties
			;
			ind_good = WHERE(cf_data.use_flag[iap] eq 1 and REFORM(this_aperture_use[iap,*]) eq 1, ngood)
			if(ngood gt 0) then zz_flux = median_filtered_mean(fluxes[ind_good])
			;
			;		flux uncertainty
			;
			;dm_df = (mags)*(-1.0d0)*(ALOG(10))/2.5
			;
			fluxerr = mags_uncer * fluxes * (1./1.086)
			zz_fluxerr = median_filtered_mean(fluxerr)
			ind_fe_bad = WHERE((fluxerr - zz_fluxerr[0])/zz_fluxerr[1] lt -3.5, n_fe_bad)
			if(n_fe_bad gt 0) then begin
				fluxerr[ind_fe_bad] = zz_fluxerr[0] ;	+ 2.0*zz_fluxerr[1]
			endif
			;
			;	check for fluxes that are out by more than threshold and kill them
			;
			flux_test = fluxerr > zz_flux[1]
			ind_fbad = WHERE(abs(fluxes[ind_good] - zz_flux[0])/flux_test[ind_good] gt flux_threshold, nfbad)
			if(nfbad gt 0) then begin
				cf_data[ind_good[ind_fbad]].use_flag[iap] = 0L
				cf_data[ind_good[ind_fbad]].basis[iap] = 'flux'
			endif
			ind_good = WHERE(cf_data.use_flag[iap] eq 1,ngood)
			ind_bad = WHERE(cf_data.use_flag[iap] eq 0,nbad)
			;
			if(nbad gt 0) then begin
				save_ind_bad[iap,0:nbad-1] = ind_bad[0:nbad-1]
			endif
			;
;			nbad_ap[iap] = nbad		; save the number of bad images for merit function calc, later
			;
			;	identify which aperture is being considered here
			;
			(*w_ptr).S6A_aper_no_value = iap+1
			(*w_ptr).S6A_bad_image_count = nbad
			;
			;	print aper no and nbad to textboxes
			;
			S6_populate_textbox, Event, 0
			;
			;	save fluxes, fluxerr
			;
			save_fluxes[iap,*] = fluxes[*]
			save_fluxerr[iap,*] = fluxerr[*]
			;
			;	plot fluxes
			;
			WIDGET_CONTROL, plot_wid, GET_VALUE=win_no
			wset, win_no
			erase
			;
			;	reset margins a bit
			;
			!X.MARGIN = [7,1]
			!Y.MARGIN = [4,1]
			;
			ymax = MAX(fluxes+fluxerr)
			ymin = MIN(fluxes-fluxerr)
			max_deviation = (ymax-1.) > (1.-ymin)
			n_deci = CEIL(max_deviation * 50.0)
			;
			yrange = [1-0.02*n_deci,1+0.02*n_deci]
			dy = yrange[1]-yrange[0]
			;
			xrange = [MIN(cf_data.julian_date), MAX(cf_data.julian_date)]
			dx = xrange[1] - xrange[0]
			xrange[0] = xrange[0]-0.02*dx
			xrange[1] = xrange[1]+0.02*dx
			dx = xrange[1] - xrange[0]
			plot, cf_data.julian_date, fluxes, yrange=yrange, xrange=xrange, xstyle=1 , ystyle=1, $
					xtitle='Time offset from reference image [minutes]', ytitle='Scale Factor',$
					color=black, background=gray, /NODATA
			oplot, cf_data.julian_date, fluxes, color=black, psym=-4, thick=1.5
			oplot, cf_data.julian_date, fluxes, color=black, psym=0
			x = xrange[0]+0.05*dx
			y = yrange[0]+0.1*dy
			XYOuts, x, y, "Data", color=black, charsize=1.5
			;
			x = xrange[0] + 0.1375*dx
			y = yrange[0] + 0.1*dy
			XYOuts, x, y, "Dither", color=green, charsize=1.5, charthick=1.5
			;
			; 	add error bars
			;
			yerrx = dblarr(num)
			yerry1 = dblarr(num)
			yerry2 = dblarr(num)
			for i=0,num-1 do begin
				;plot yerror bar
				yerrx=cf_data[i].julian_date
				yerry1=(fluxes[i]-fluxerr[i]) > yrange[0]
				yerry2=(fluxes[i]+fluxerr[i]) < yrange[1]
				plots,[yerrx,yerrx],[yerry1,yerry2],color=black
			endfor
			;
			;		determine the correction factors to fit and remove, so as to uncover
			;			any remaining sky variations
			;
			;		in order, they are:
			;					1. secular variation
			;					2. intensity variation (ie, I_u, I_q must match for every quad of obs)
			;					3. field polarization
			;					4. sky variation
			;
			;		the final scale factor to report will be the product of
			;					1 * 2 * 4
			;
			;			but, the field polarization will *not* be removed.
			;
			;-------------  part 1 - secular variation --------------------
			;	 up to a 6th order baseline fit, using F-tests to guide polynomical order selection
			;
			;	this correction does not ignore dither-group deletions
			;
			Degree_max = max_fit_degree 
			;
			last_chisq = 1.0e6
			ndata_pts = N_ELEMENTS(cf_data[ind_good])
			dof = ndata_pts - 1
			;
			; update degree_max, if too few data points
			; 
			Degree_max <= (ndata_pts - 4)
			;
			keep = intarr(Degree_max + 1)
			;
			for iorder = 0, degree_max do begin
				;
				;	compute poly fit for this order
				;
				Result = POLY_FIT(cf_data[ind_good].julian_date,fluxes[ind_good], iorder , CHISQ=chisq , /DOUBLE, $
						MEASURE_ERRORS= fluxerr[ind_good], SIGMA=sigma , STATUS=fit_status )
				;
				;	compute f-test statistic
				;
				F_x = (last_chisq - chisq) / (chisq / float(ndata_pts - iorder))
				;
				;	and probability that this term is significant
				;
				F_prob = F_PDF(F_x, 1, (ndata_pts - iorder - 1))
				;
				F_prob_1 = 1.0 - F_prob
				if(F_prob gt (1.0 - 0.05)) then ikeep = 1 else ikeep = 0
				keep[iorder] = ikeep
				;
				; but, always keep the zero order term
				; 
				if(iorder eq 0) then keep[0] = 1
				;
				;print, "iorder = ",iorder," chisq = ",chisq," DoF = ",(ndata_pts - iorder)," F_x = ",F_x," F_prob = ",F_prob, " 1-F_prob = ",F_prob_1," keep? = ",ikeep
				;
				last_chisq = chisq
			endfor
			;
			;	what is the highest order term that needs to be kept?
			;
			ind = WHERE(keep eq 1,nkeep)
			if(nkeep gt 0) then begin
				last_term = ind[nkeep-1]
				;
				;	refit to this order
				;
				Result = POLY_FIT(cf_data[ind_good].julian_date,fluxes[ind_good], last_term , CHISQ=chisq , /DOUBLE, $
						MEASURE_ERRORS= fluxerr[ind_good], SIGMA=sigma , STATUS=fit_status )
				;
				degree = last_term
			endif else begin
				result = fltarr(Degree_max + 1)		; to zero all coeffs
				degree = 1
			endelse
			;
			x = double(cf_data[*].julian_date)
			secular_cf = 0.0d0 * x + result[0]
			for ideg = 1, degree do begin
				secular_cf += result[ideg] * (x ^ double(ideg))
			endfor
			;
			;	save secular_cf
			;
			save_secular_cf[iap,*] = secular_cf[*]
			;
			oplot,cf_data.julian_date, secular_cf, psym=0, color=blue, thick=1.5
			x = xrange[0]+0.25*dx
			y = yrange[0]+0.1*dy
			XYOuts, x, y, "Secular CF", color=blue, charsize=1.5
			;
			;  compute secular amplitude and display it
			;  
			(*w_ptr).S6A_scaling_amplitude = 100.0 * (max(secular_cf[*]) - min(secular_cf[*]))
			;
			S6_populate_textbox, Event, 0
			;
			;	apply correction
			;
			secular_corrected_fluxes = fluxes / secular_cf
			;
			;------------- Part 2 - intensity corrections ---------------------------
			;
			foo = cf_data.ff_num
			use = cf_data.use_flag[iap] and REFORM(this_aperture_use[iap,*])	; respect dither-use flags
			;
			ind_0_u   = where(foo eq 0 and use)
			ind_45_u  = where(foo eq 45 and use)

			ind_90_u  = where(foo eq 90 and use)
			ind_135_u = where(foo eq 135 and use)
			;
			ind_0   = where(foo eq 0)
			ind_45  = where(foo eq 45)

			ind_90  = where(foo eq 90)
			ind_135 = where(foo eq 135)
			;
			ff_mfm = dblarr(4,2)
			;
			for x = 0, 3 do begin
				case x of
					0: current_ind = ind_0_u
					1: current_ind = ind_45_u
					2: current_ind = ind_90_u
					3: current_ind = ind_135_u
				endcase
				;
;				print, 'x = ',x,' inds = ',current_ind
				;
				;	if photometric, non HWP=0 cases will be empty
				;
				normal_flag = 1				; polarimetric
				if(N_ELEMENTS(current_ind) eq 1) then begin
					if(current_ind eq -1) then normal_flag = 0 ; photometric
				endif
				;
				if(normal_flag eq 0) then begin
						result = 0.0
						ff_mfm[x,*] = 0.0
						ff_mfm[x,1] = 0.0
				endif else begin
					result = median_filtered_mean(secular_corrected_fluxes[current_ind])
					ff_mfm[x,*] = result
					ff_mfm[x,1] = ff_mfm[x,1] / sqrt(n_elements(current_ind)-1.)	
				endelse
			endfor
			;
			;intensity correction factor insures overall intensity is not incremented or decremented
			;
			;	april's original set
			;	intensity_q_cf	=	(ff_mfm(0,0)+ff_mfm(1,0))/2.
			;	intensity_u_cf	=	(ff_mfm(2,0)+ff_mfm(3,0))/2.
			;
			;	my set
			;
			if(normal_flag eq 1) then begin
				intensity_q_cf	=	(ff_mfm[0,0] + ff_mfm[2,0]) / 2.
				intensity_u_cf	=	(ff_mfm[1,0] + ff_mfm[3,0]) / 2.
				;
				intensity_cf = dblarr(num)
				;
				intensity_cf[ind_0]  = intensity_q_cf
				intensity_cf[ind_45] = intensity_u_cf
				intensity_cf[ind_90] = intensity_q_cf
				intensity_cf[ind_135]= intensity_u_cf
				;
				;	correct for intensity before finding field polarization
				;
				secular_intensity_corrected_fluxes = secular_corrected_fluxes / intensity_cf
			endif else begin
				secular_intensity_corrected_fluxes = secular_corrected_fluxes
				intensity_cf = secular_intensity_corrected_fluxes * 0.0 + 1.0
			endelse
			;
			save_intensity_cf[iap,*] = intensity_cf[*]
			;
			oplot,cf_data.julian_date,intensity_cf,color=green, psym=0, thick=1.5
			x = xrange[0]+0.45*dx
			y = yrange[0]+0.1*dy
			XYOuts, x, y, "UQInten CF", color=green, charsize=1.5
			;
;			for img = 0, num-1 do begin
;				print, 'img ',img,' HWP ',cf_data[img].hwp,' jd ',cf_data[img].julian_date,' UQ_corr ',intensity_cf[img]
;			endfor
			;
			;---------------------- Part 3 - Field polarization
			;
			if(normal_flag eq 1) then begin
				for x = 0, 3 do begin
					case x of
						0: current_ind = ind_0_u
						1: current_ind = ind_45_u
						2: current_ind = ind_90_u
						3: current_ind = ind_135_u
					endcase
					result = median_filtered_mean(secular_intensity_corrected_fluxes[current_ind])
					ff_mfm[x,*] = result
					ff_mfm[x,1] = ff_mfm[x,1] / sqrt(n_elements(current_ind)-1.)
				endfor
				;
				;	mfm is pol correction factor
				;
				pol_cf = dblarr(num)
				;
				pol_cf[ind_0]   = ff_mfm[0,0]
				pol_cf[ind_45]  = ff_mfm[1,0]		; changed 90->45
				pol_cf[ind_90]  = ff_mfm[2,0]		; changed 45-> 90
				pol_cf[ind_135] = ff_mfm[3,0]
				;
				;	calculate field polarization
				;
				field_q = (ff_mfm(0,0)-ff_mfm(2,0))/(ff_mfm(0,0)+ff_mfm(2,0))
				field_u = (ff_mfm(1,0)-ff_mfm(3,0))/(ff_mfm(1,0)+ff_mfm(3,0))
				;
				field_q_sig = sqrt(  ((ff_mfm(0,0))^2*(ff_mfm(2,1))^2+(ff_mfm(2,0))^2*(ff_mfm(0,1))^2)  $
										*  4./( (ff_mfm(0,0)+ff_mfm(2,0))^4)  )
				field_u_sig = sqrt(  ((ff_mfm(1,0))^2*(ff_mfm(3,1))^2+(ff_mfm(3,0))^2*(ff_mfm(1,1))^2)  $
										*  4./( (ff_mfm(1,0)+ff_mfm(3,0))^4)  )
				;
				ap_uq2polpa,[field_u,field_u_sig],[field_q,field_q_sig],pol,pa
				;
				;print_text, Event,"Field polarization = "+STRING(pol[0]*100.0,pol[1]*100.0,format='(F8.3,"+/-",F6.3," percent")')
				;
				;	save fp uncert
				;
				fp_uncert_ap[iap,ipass] = pol[1]*100.0
				;
				save_fp[iap,0] = pol[0] * 100.0
				save_fp[iap,1] = pol[1] * 100.0
				;
				(*w_ptr).S6A_field_pol_value = pol[0] * 100.0
				(*w_ptr).S6A_field_pol_sigma = pol[1]*100.0
				;
				S6_populate_textbox, Event, 0
				;
				secular_intensity_pol_corrected_fluxes = secular_intensity_corrected_fluxes / pol_cf
				;
				save_pol_cf[iap,*] = pol_cf[*]
				;
				oplot,cf_data.julian_date,pol_cf,psym=-1,color=yellow, thick=1.5
			endif else begin
				fp_uncert_ap[iap,ipass] = 10.0
				;
				save_fp[iap,0] = 0.0
				save_fp[iap,1] = 10.0
				;
				(*w_ptr).S6A_field_pol_value = 0.0
				(*w_ptr).S6A_field_pol_sigma = 0.0
				;
				S6_populate_textbox, Event, 0
				;
				secular_intensity_pol_corrected_fluxes = secular_intensity_corrected_fluxes
				;
				save_pol_cf[iap,*] = secular_intensity_pol_corrected_fluxes * 0.0 + 1.0
				pol_cf = save_pol_cf
			endelse
			;
			x = xrange[0]+0.65*dx
			y = yrange[0]+0.1*dy
			XYOuts, x, y, "Pol CF", color=yellow, charsize=1.5,charthick=1.5
			;
			;---------------------- Part 4 - sky correction -----------------------
			;
			;	sky correction factor is the result - this is what remains after all
			;	other corrections have been factored in
			;
			sky_cf = secular_intensity_pol_corrected_fluxes
			;
			;	compute the power in the sky term
			;
			flg_good_and_use = lonarr(num)
			flg_good_and_use[ind_good] = 1L
			flg_good_and_use = flg_good_and_use and REFORM(this_aperture_use[iap,*]) ; logical and of ind_good and ap use
			;
			ind_good_and_use = WHERE(flg_good_and_use)
			zz = median_filtered_mean(sky_cf[ind_good_and_use])
			;
;			print_text, Event, "Sky correction power (rms, as %) = "+STRING(100.0*zz[1],format='(F12.3)')
			;
			;	save sky noise value
			;
			sky_noise_ap[iap,ipass] = zz[1]*100.0
			;
			(*w_ptr).S6A_sky_noise_value = zz[1]*100.0
			S6_populate_textbox, Event, 0
			;
			save_sky_cf[iap,*] = sky_cf[*]
			;
			oplot,cf_data.julian_date,sky_cf,psym=-1,color=violet, thick=1.5	; this is just pol
			x = xrange[0]+0.80*dx
			y = yrange[0]+0.1*dy
			XYOuts, x, y, "Sky CF", color=violet, charsize=1.5, charthick=1.375
			;
			;	look at sky noise as a function of dither position for first pass only
			;
			if(ipass eq 0) then begin
				sky_noise_dither = fltarr(max_dither)
				for idither = 0, max_dither-1 do begin
					ind_dither = WHERE(cf_data.dither eq idither+1 and cf_data.dither gt 0, n_dither)   ;  and cf_data.dither lt 8
					if(n_dither gt 0) then begin
						zz_dither = median_filtered_mean(sky_cf[ind_dither])
						;
						sky_noise_dither[idither] = zz_dither[1]		; the dispersion is saved
					endif
				endfor
				;
				ind_ok = WHERE(sky_noise_dither ne 0.0,n_ok)
				if(n_ok gt 0) then begin
					zz_dither_noise = median_filtered_mean(sky_noise_dither[ind_ok])
				endif else begin
					print, "Error - no good dither positions in Step 6A"
					zz_dither_noise = [0.0,33.0]			; %
				endelse
				;
				;	any dither positions show up as outliers?
				;
				;	trap uncertainty to no lower than 0.2% and mean to no lower than 0.5% to
				;		prevent rejection of otherwise good data - DPC 20110315
				;
				zz_dither_noise[0] >= 0.005
				zz_dither_noise[1] >= 0.002
				;
				ind_bad_dither = WHERE((sky_noise_dither - zz_dither_noise[0])/zz_dither_noise[1] gt dither_noise_threshold,nbad_dither)    ; signed to save small noise
				if(nbad_dither gt 0) then begin
					print_text, Event, "WARNING - high sky noise found for "+STRING(nbad_dither,format='(i1)')+" dither positions"
					for ibad_dither = 0, nbad_dither-1 do begin
						;
						ind_this_dither_bad = ind_bad_dither[ibad_dither]+1
						;
						print, "turning off use flags for dither position ",ind_this_dither_bad
						;
						;	clear this aperture use flag for the bad ones
						;
						ind_bad_da = WHERE(cf_data.dither eq ind_this_dither_bad,nind_bad)
						if(nind_bad gt 0) then begin
							;
							this_aperture_use[iap,ind_bad_da] = 0
							;
							;	and mark all data points with a red triangle
							;
							plots, cf_data[ind_bad_da].julian_date, sky_cf[ind_bad_da], symsize=1.5, psym=5, thick=3, color=red
						endif
					endfor

				endif
			endif
			;
			;-----------------------------
			;	next, flag all the bad images with red boxes
			;
			if(nbad gt 0) then begin
				for ibad = 0, nbad-1 do begin
					this_bad = ind_bad[ibad]
					xbad = cf_data[this_bad].julian_date
					ybad = sky_cf[this_bad]
					plots, xbad, ybad, symsize=1.25, psym=4, thick=4, color=red
				endfor
				x = xrange[0]+0.13*dx
				y = yrange[0]+0.0375*dy
				XYOuts, x, y, "Deleted", color=red, charsize=1.5
			endif
			;
			;	flag dither positions
			;
			xdither = cf_data[*].julian_date
			dy = yrange[1] - yrange[0]
			ydither = yrange[1]-0.015*dy*(cf_data[*].dither + 2)
			this_dith_color = intarr(N_ELEMENTS(cf_data))
			this_dith_color[*] = green
			ind_bad_dith = WHERE(REFORM(this_aperture_use[iap,*]) eq 0,nbd)
			if(nbd gt 0) then this_dith_color[ind_bad_dith] = red
			if(nbad gt 0) then this_dith_color[ind_bad] = red
;			for idd=0,N_ELEMENTS(cf_data)-1 do
			plots, xdither, ydither, thick=2, color=this_dith_color, psym=1
			;
			;-----------------------------------------------------------------------
			;
			;			compute final scale factor for each image
			;
			; 	the final correction factor is sky_cf * intensity_cf * secular_cf
			;			(ie, leave the field polarization intact)
			;
			final_correction_factor = sky_cf * intensity_cf * secular_cf
			;
			;oplot,cf_data.julian_date,fluxes/(final_correction_factor),psym=-1,color=10
			;
			multiplicative_cf = 1.0 / final_correction_factor
			;
;			if(ipass eq 0 and iap eq 8) then stop
			;
			;	and zero any with use flags set to 0 - [20170814 changed to non-zero]
			;
;			if(nbad gt 0) then multiplicative_cf[ind_bad] = 0.0
			ind_ap_bad = WHERE(REFORM(this_aperture_use[iap,*]) eq 0,napbad)
			if(napbad gt 0) then begin
;				multiplicative_cf[ind_ap_bad] = 0.0      ; 20170814
			endif
			;
			nch_file = STRLEN(FILE_BASENAME(cf_data[0].file_name))
			nbuf2 = (nch_file - 8) / 2
			chbuf = ''
			if(nbuf2 gt 0) then begin
				for ib = 0, nbuf2-1 do begin
					chbuf = chbuf + ' '
				endfor
			endif
			;
;			if(ipass eq 1) then begin
				for i = 0, num-1 do begin

					;
					;	save for later file writing
					;
					save_cf_mult[ipass,iap,i] = multiplicative_cf[i]
					save_cf_secular[ipass,iap,i] = secular_cf[i]
					save_cf_intensity[ipass,iap,i] = intensity_cf[i]
					save_cf_pol[ipass,iap,i] = pol_cf[i]
					save_cf_sky[ipass,iap,i] = sky_cf[i]
					save_cf_use[ipass,iap,i] = this_aperture_use[iap,i]
					;
;					printf, lu_out, FILE_BASENAME(cf_data[i].file_name), cf_data[i].hwp, multiplicative_cf[i], $
;									cf_data[i].x_offset, cf_data[i].y_offset, cf_data[i].use_flag[iap], secular_cf[i], $
;									intensity_cf[i], pol_cf[i], sky_cf[i], cf_data[i].basis[iap], $
;									format='(a,2x,f5.1,1x,F11.8,1x,f7.2,1x,f7.2,4x,i1,2x,F11.8,1x,F11.8,1x,F11.8,1x,F11.8,1x,a4)'
				endfor
				;
				print_text, Event,'Finished scaling group '+group_name+' for aperture number '+STRING(iap,format='(I2)')
			;
			;cursor, xxx, yyy, /wait		; for debug only
;			endif
			;
			;	compute number of values kept
			;
			ind_good = WHERE(cf_data.use_flag[iap] eq 1 and REFORM(this_aperture_use[iap,*]) eq 1, ngood)
			if(ngood gt 0) then ngood_ap[iap,ipass] = ngood else ngood_ap[iap,ipass] = 0
			;
			WAIT, 0.2				; so plot is persistent for a short time
			;
			;	end of two-pass loop of checking for high sky noise vs dither position
			;
		endfor
		;
		;	end of aperture loop
		;
	endfor
	;
	;	with all apertures finished, do plots of sky noise vs aperture and fp uncert vs aperture
	;
	WIDGET_CONTROL, sky_plot_wid, GET_VALUE=win_no
	wset, win_no
	erase
	;
	;	reset margins a bit
	;
	!X.MARGIN = [7,1]
	!Y.MARGIN = [3,1]
	ymax = MAX(sky_noise_ap)
	ymin = MIN(sky_noise_ap)
	dy = ymax - ymin
	;
	yrange = [ymin - 0.05*dy, ymax + 0.05*dy]
	;
	x_ap = indgen(n_ap)+1
	;
	xrange = [0,n_ap+1]
	;
	plot, x_ap, sky_noise_ap[*,0], yrange=yrange, xrange=xrange, xstyle=1 , ystyle=1, $
			xtitle='Aper. No.', ytitle='Sky Noise [%]', charsize=0.75, $
			color=black, background=gray, /NODATA
	oplot, x_ap, sky_noise_ap[*,0], color=blue, psym=-4, thick=1.5
	oplot, x_ap, sky_noise_ap[*,0], color=blue, psym=0
	;
	oplot, x_ap, sky_noise_ap[*,1], color=orange, psym=-4, thick=1.5
	oplot, x_ap, sky_noise_ap[*,1], color=orange, psym=0
	;
	;	identify and mark minimum
	;
;	x = xrange[0]+0.05*dx
;	y = yrange[0]+0.1*dy
;	XYOuts, x, y, "Data", color=black, charsize=1.5
;
	;
	;	same, for fp uncert
	;
	WIDGET_CONTROL, fp_plot_wid, GET_VALUE=win_no
	wset, win_no
	erase
	;
	;	reset margins a bit
	;
	!X.MARGIN = [7,1]
	!Y.MARGIN = [3,1]
	;
	ymax = MAX(fp_uncert_ap)
	ymin = MIN(fp_uncert_ap)
	dy = ymax - ymin
	;
	yrange = [ymin - 0.05*dy, ymax + 0.05*dy]
	;
	plot, x_ap, fp_uncert_ap[*,0], yrange=yrange, xrange=xrange, xstyle=1 , ystyle=1, $
			xtitle='Aper. No.', ytitle='FP Uncer. [%]', charsize=0.75, $
			color=black, background=gray, /NODATA
	oplot, x_ap, fp_uncert_ap[*,0], color=blue, psym=-4, thick=1.5
	oplot, x_ap, fp_uncert_ap[*,0], color=blue, psym=0
	;
	oplot, x_ap, fp_uncert_ap[*,1], color=orange, psym=-4, thick=1.5
	oplot, x_ap, fp_uncert_ap[*,1], color=orange, psym=0
	;
	;
	;	identify and mark minimum
	;
;	x = xrange[0]+0.05*dx
;	y = yrange[0]+0.1*dy
;	XYOuts, x, y, "Data", color=black, charsize=1.5
	;
	;	compute merit function to select best aperture
	;
	;		start by finding minima of key variables
	;
	;			- separate by inclusion, exclusion of dither removal
	;
	sky_noise_0 = REFORM(sky_noise_ap[*,0])
	sky_noise_1 = REFORM(sky_noise_ap[*,1])
	;
	fp_uncert_0 = REFORM(fp_uncert_ap[*,0])
	fp_uncert_1 = REFORM(fp_uncert_ap[*,1])
	;
	n_used_0 = REFORM(ngood_ap[*,0])
	n_used_1 = REFORM(ngood_ap[*,1])
		;
		full_merit_0 = sky_noise_0 * fp_uncert_0 * (1./sqrt(float(n_used_0)))
		full_merit_1 = sky_noise_1 * fp_uncert_1 * (1./sqrt(float(n_used_1)))
		;
;		print, "merit fn(0) : ",full_merit_0
;		print, " n_used_(0) : ",n_used_0
;		print, "merit fn(1) : ",full_merit_1
;		print, " n_used_(1) : ",n_used_1
		;
		best_merit_0 = MIN(full_merit_0)
		best_merit_1 = MIN(full_merit_1)
		;
		;	test - dropped dither must improve merit by at least 1%
		;
		d_merit = (best_merit_0 - best_merit_1) / best_merit_0
		if(d_merit le 0.01) then begin
			best_pass = 0
			full_merit = full_merit_0
			best_merit = best_merit_0
		endif else begin
			best_pass = 1
			full_merit = full_merit_1
			best_merit = best_merit_1
		endelse
		;
;		print, "best merit = ",best_merit," for pass = ",best_pass
		;
		ind_merit = WHERE(full_merit eq best_merit,nmerit)
		if(nmerit ge 1) then begin
			best_ap = ind_merit[0];
			print_text, Event, "Used merit function method to find best aperture = "+STRING(best_ap+1,format='(I2)')
		endif else begin
			print_text, Event, 'ERROR - merit function computation failed. Stopping.'
			stop
		endelse
;	endelse
	;
	;	plot vertical lines at best aperture on both plots
	;
	found_best:
	WIDGET_CONTROL, sky_plot_wid, GET_VALUE=win_no
	wset, win_no
	oplot, [best_ap+1, best_ap+1], [-10,10], color=red, psym=0, thick=2, linestyle=2
	;
	WIDGET_CONTROL, fp_plot_wid, GET_VALUE=win_no
	wset, win_no
	oplot, [best_ap+1, best_ap+1], [-10,10], color=red, psym=0, thick=2, linestyle=2
	;
	;
;	WAIT, 1
	;
	;	return plot margins to default
	;
	!X.MARGIN = [10,3]
	!Y.MARGIN = [4,2]
	;
	;	plot best aperture values
	;
	WIDGET_CONTROL, plot_wid, GET_VALUE=win_no
	wset, win_no
	erase, color=18
	;
	;	reset margins a bit
	;
	!X.MARGIN = [7,1]
	!Y.MARGIN = [4,1]
	;
	fluxes = REFORM(save_fluxes[best_ap,*])
	fluxerr = REFORM(save_fluxerr[best_ap,*])
	;
	ymax = MAX(fluxes+fluxerr)
	ymin = MIN(fluxes-fluxerr)
	max_deviation = (ymax-1.) > (1.-ymin)
	n_deci = CEIL(max_deviation * 50.0)
	;
	yrange = [1-0.02*n_deci,1+0.02*n_deci]
	dy = yrange[1]-yrange[0]
	;
	xrange = [MIN(cf_data.julian_date), MAX(cf_data.julian_date)]
	dx = xrange[1] - xrange[0]
	xrange[0] = xrange[0]-0.02*dx
	xrange[1] = xrange[1]+0.02*dx
	dx = xrange[1] - xrange[0]
	plot, cf_data.julian_date, fluxes, yrange=yrange, xrange=xrange, xstyle=1 , ystyle=1, $
			xtitle='Time offset from reference image [minutes]', ytitle='Scale Factor',$
			color=black, background=gray, /NODATA
	oplot, cf_data.julian_date, fluxes, color=black, psym=-4, thick=1.5
	oplot, cf_data.julian_date, fluxes, color=black, psym=0
	x = xrange[0]+0.05*dx
	y = yrange[0]+0.1*dy
	XYOuts, x, y, "Data", color=black, charsize=1.5
	;
	x = xrange[0] + 0.1375*dx
	y = yrange[0] + 0.1*dy
	XYOuts, x, y, "Dither", color=green, charsize=1.5, charthick=1.5
	;
	; 	add error bars
	;
	yerrx = dblarr(num)
	yerry1 = dblarr(num)
	yerry2 = dblarr(num)
	for i=0,num-1 do begin
		;plot yerror bar
		yerrx=cf_data[i].julian_date
		yerry1=(fluxes[i]-fluxerr[i]) > yrange[0]
		yerry2=(fluxes[i]+fluxerr[i]) < yrange[1]
		plots,[yerrx,yerrx],[yerry1,yerry2],color=black
	endfor
	;
	secular_cf = REFORM(save_secular_cf[best_ap,*])
	;
	oplot,cf_data.julian_date, secular_cf, psym=0, color=blue, thick=1.5
	x = xrange[0]+0.25*dx
	y = yrange[0]+0.1*dy
	XYOuts, x, y, "Secular CF", color=blue, charsize=1.5
	;
	intensity_cf = REFORM(save_intensity_cf[best_ap,*])
	;
	oplot,cf_data.julian_date,intensity_cf,color=green, psym=0, thick=1.5
	x = xrange[0]+0.45*dx
	y = yrange[0]+0.1*dy
	XYOuts, x, y, "UQInten CF", color=green, charsize=1.5
	;
	pol_cf = REFORM(save_pol_cf[best_ap,*])
	;
	oplot,cf_data.julian_date,pol_cf,psym=-1,color=yellow, thick=1.5
	x = xrange[0]+0.65*dx
	y = yrange[0]+0.1*dy
	XYOuts, x, y, "Pol CF", color=yellow, charsize=1.5,charthick=1.5
	;
	sky_cf = REFORM(save_sky_cf[best_ap,*])
	;
	oplot,cf_data.julian_date,sky_cf,psym=-1,color=violet, thick=1.5	; this is just pol
	x = xrange[0]+0.80*dx
	y = yrange[0]+0.1*dy
	XYOuts, x, y, "Sky CF", color=violet, charsize=1.5, charthick=1.375
	;
	;	flag dither positions
	;
	xdither = cf_data[*].julian_date
	dy = yrange[1] - yrange[0]
	ydither = yrange[1]-0.015*dy*(cf_data[*].dither + 2)
	this_dith_color = intarr(N_ELEMENTS(cf_data))
	this_dith_color[*] = green
;	ind_bad_dith = WHERE(REFORM(this_aperture_use[best_ap,*]) eq 0,nbd)
	ind_bad_dith = WHERE((REFORM(cf_data[*].use_flag[best_ap]) and REFORM(save_cf_use[best_pass,best_ap,*])) eq 0, nbd)
	if(nbd gt 0) then begin
		this_dith_color[ind_bad_dith] = red
		for ibad = 0, nbd-1 do begin
			this_bad = ind_bad_dith[ibad]
			xbad = cf_data[this_bad].julian_date
			ybad = save_sky_cf[best_ap,this_bad]
			plots, xbad, ybad, symsize=1.25, psym=4, thick=4, color=red
		endfor
	endif
	;
	if(nbd gt 0) then this_dith_color[save_ind_bad[best_ap,0:nbd-1]] = red	; nbad
	nbad = 0
	plots, xdither, ydither, thick=2, color=this_dith_color, psym=1
	if(nbd + nbad gt 0) then show_del = 1 else show_del = 0
	;
	if(show_del) then begin
		x = xrange[0]+0.13*dx
		y = yrange[0]+0.0375*dy
		XYOuts, x, y, "Deleted", color=red, charsize=1.5
	endif
	;
	;	return plot margins to default
	;
	!X.MARGIN = [10,3]
	!Y.MARGIN = [4,2]
	;
	;	update textboxes with best aperture values
	;
	;	identify which aperture is being considered here
	;
	(*w_ptr).S6A_aper_no_value = best_ap + 1
	(*w_ptr).S6A_bad_image_count = nbd+nbad
	;
	;	print aper no and nbad to textboxes
	;
;	S6_populate_textbox, Event, 0
	;
	(*w_ptr).S6A_field_pol_value = save_fp[best_ap,0]
	(*w_ptr).S6A_field_pol_sigma = save_fp[best_ap,1]
	;
;	S6_populate_textbox, Event, 0
	;
	(*w_ptr).S6A_sky_noise_value = sky_noise_ap[best_ap]
	;
	;  and secular amplitude of correction (as percentage)
	;
	(*w_ptr).S6A_scaling_amplitude = 100.0*(max(save_cf_secular[best_pass,best_ap,*]) - $
	                                 min(save_cf_secular[best_pass,best_ap,*]))
	;
	S6_populate_textbox, Event, 0
	;
	;	write results
	;
	;	save correction factors in group named files in Scaling directory
	;
	dir = (*s_ptr).analysis_dir + 'S6_Scaling' + PATH_SEP()
	path_out = dir + replace_underscores(group_name) + '_group_scale_factors.dat'
	openw, lu_out, path_out, /GET_LUN
	;
	printf, lu_out, ';PPOL-Step 6-version '+(*s_ptr).version_short_string
	printf, lu_out, format='(";Scaling factors (+ X, Y offsets) for images in ",a)',group_name
	;
	;	identify aperture
	;
	str_ap = "PP4_AP"+STRING(best_ap,format='(I2.2)')
	save_PP6_APRD = SXPAR(header, str_ap)
	;
	printf, lu_out, '; --------------------------------------------------------------------------------------'
	printf, lu_out, '; corrections follow for aperture number '+STRING(best_ap+1,format='(I2)')+' with properties:'
	printf, lu_out, ';PP4_ARAD=            '+STRING(save_PP4_ARAD,format='(f5.3)')+'/ Maximum Aper Size, in FWHM units'
	printf, lu_out, ';PP4_REF0=            '+STRING(save_PP4_REF0,format='(f5.3)')+'/ Inner ref radius, in FHWM * ARAD units'
	printf, lu_out, ';PP4_REF1=            '+STRING(save_PP4_REF1,format='(f5.3)')+'/ Outer ref radius, in FHWM * ARAD units'
	printf, lu_out, ';PP4_CLEN=            '+STRING(save_PP4_CLEN,format='(f5.3)')+'/ Outer cleaning radius, in FHWM * ARAD units'
	printf, lu_out, ';PP5_RFIM=            '+STRING(save_PP5_RFIM,format='(I5)'  )+'/ Reference Image number for matching'
	printf, lu_out, ';PP6_APRD=         '+STRING(save_PP6_APRD,format='(f8.6)')+'/ Aperture size in FWHM * ARAD units'
	printf, lu_out, ';'
	printf, lu_out, '; summary fit value follow'
	printf, lu_out, ';PP6_APNO=               '+STRING(best_ap+1,format='(I2.1)')+'/ Aperture number with best merit value'
	printf, lu_out, ';PP6_FPOL=        '+STRING(save_fp[best_ap,0],format='(f9.5)')+'/ Mean Field Polarization [in %]'
	printf, lu_out, ';PP6_SFPL=        '+STRING(save_fp[best_ap,1],format='(f9.5)')+'/ Uncertainty in Field Polarization [in %]'
	printf, lu_out, ';PP6_SNOI=        '+STRING(sky_noise_ap[best_ap],format='(f9.5)')+'/ Sky Noise [in %]'
	printf, lu_out, ';PP6_AMPL=        '+STRING((*w_ptr).S6A_scaling_amplitude ,format='(f9.5)')+'/ Secular Correction Amplitude [in %]'
	printf, lu_out, ';PP6_NBAD=              '+STRING(nbd+nbad,format='(I3.1)')+'/ Number of frames removed from final scaling'
	printf, lu_out, ';'
	printf, lu_out, '; key observation information'
	printf, lu_out, ';MB_CAMRA=        '+save_camera+'/ Mimir Camera Optics Used'
	printf, lu_out, ';MB_FILTR=        '+save_filter+'/ Mimir Filter Option Used'
	printf, lu_out, ';'
	;
	long_string = ';'+chbuf+'filename'+chbuf+'   HWPA '+'    Total CF '+'XOffset '+'YOffset  '+'Dith. '+'UseFlg '+' Sec CF   '+ $
					'   UQ CF   '+'   Field Pol   '+'  Sky CF    UseFlagBasis'
 	printf, lu_out, format='(a)',long_string

	for i = 0, num-1 do begin
		;
		multiplicative_cf[i] = save_cf_mult[best_pass,best_ap,i]
		secular_cf[i] = save_cf_secular[best_pass,best_ap,i]
		intensity_cf[i] = save_cf_intensity[best_pass,best_ap,i]
		pol_cf[i] = save_cf_pol[best_pass,best_ap,i]
		sky_cf[i] = save_cf_sky[best_pass,best_ap,i]
		;
		this_use_flag = cf_data[i].use_flag[best_ap] and save_cf_use[best_pass,best_ap,i]
;		if(this_use_flag eq 0) then multiplicative_cf[i] = 0.0 ; 20170814 turned off - USE Flag sets processing
		;
		if(save_cf_use[best_pass,best_ap,i] eq 0) then begin
			cf_data[i].basis[best_ap] = cf_data[i].basis[best_ap] + ' dither'
		endif
		;
		;	if scale factor = 0., then kill offsets, too  (DPC 20090702)
		;	    *** undone DPC  20170809 for Tabby star phot
		;
;		if(abs(multiplicative_cf[i]) lt 0.01) then begin
;			cf_data[i].x_offset = 0.0
;			cf_data[i].y_offset = 0.0
;		endif
		;
		printf, lu_out, FILE_BASENAME(cf_data[i].file_name), cf_data[i].hwp, multiplicative_cf[i], $
									cf_data[i].x_offset, cf_data[i].y_offset, cf_data[i].dither, this_use_flag, $
									secular_cf[i], $
									intensity_cf[i], pol_cf[i], sky_cf[i], cf_data[i].basis[best_ap], $
									format='(a,2x,f5.1,1x,F11.8,1x,f7.2,1x,f7.2,4x,i2,6x,i1,2x,F11.8,1x,F11.8,1x,F11.8,1x,F11.8,1x,a)'
	endfor
	;
	FREE_LUN, lu_out
end
