pro S6_meta_correction_factors, Event, meta_group_number, meta_group_name, output_summary_path
;
;	for designated meta-group:
;		1. access and ingest group-level scaling factors and filenames
;		2. access and ingest the relative scale factors (group-to-group)
;		3. output full list of images with input * relative scale factors
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
w_ptr = PTR_struct.PTR_wid_id
s_ptr = PTR_struct.PTR_summary
g_ptr = PTR_struct.PTR_group
;
plot_wid = (*w_ptr).S6D_relative_scales_draw_wid
;
device,decompose=0
color_names = ["black", "white", "red", "orange", "yellow", "green", "blue", "violet", "gray"]
color_values= [10,11,12,13,14,15,16,17,18,19]
device, decomposed=0
loadct, 13, /silent
tvlct,        [0,        255,     255,     255,    255,        50,      0,       148,    190], $
              [0,        255,       0,     165,    255,       205,      0,         0,    190], $
              [0,        255,       0,       0,      0,        50,    255,       211,    190], 10
black = 10
white = 11
red = 12
;
;----------------------------------------------------------------
;
;	reserve room for up to 64 groups in this meta-group, each of which can have up to 210 images
;
file_list = strarr(64,210)
file_list_num = lonarr(64)	; number of images in each group
group_scale_factors = dblarr(64,210)
group_x_offsets = fltarr(64,210)
group_y_offsets = fltarr(64,210)
group_HWP_angles = fltarr(64,210)
group_use_flag = intarr(64,210) 		; use flag for plotting
;
;	get the group names from the meta-group buffer
;
ngroups = (*g_ptr).meta_group_numbers[meta_group_number]				; number of groups in this meta
group_ids = (*g_ptr).meta_group_groups[meta_group_number,0:ngroups-1]	; group numbers in this meta
group_names = (*g_ptr).group_names[group_ids]
;
;print, "group names in this meta = ",group_names
;
;	names of the summary scale files
;
group_paths = strarr(ngroups)
for ig = 0, ngroups-1 do begin
	group_paths[ig] = replace_underscores(FILE_BASENAME(group_names[ig]))
endfor
;
group_dir = (*s_ptr).analysis_dir + "S6_Scaling" + PATH_SEP()
group_paths = group_dir + group_paths + '_group_scale_factors.dat'
;
;	for each, open, read
;
print_text, Event, "Number of groups in this meta-group = "+STRING(ngroups,format='(I)')
;
n_total_images = 0L
header = strarr(ngroups,200)
nheader = lonarr(ngroups)
;
pp4_arad = dblarr(ngroups)
pp4_ref0 = dblarr(ngroups)
pp4_ref1 = dblarr(ngroups)
pp4_clen = dblarr(ngroups)
pp5_rfim = intarr(ngroups)
pp6_aprd = dblarr(ngroups)
;
save_camera = strarr(ngroups)
save_filter = strarr(ngroups)
;
;
for ig = 0, ngroups-1 do begin
	print_text, Event, "Reading Scale factors for group "+FILE_BASENAME(group_paths[ig])
	openr, lu_in, group_paths[ig], /GET_LUN
	;
	;print_text, Event, "Opened group scaling file "+FILE_BASENAME(group_paths[ig])
	;
	line = ''
;	readf, lu_in, line						; 1st line
	;
	nline = 0L
	WHILE (~EOF(lu_in)) do begin
		readf, lu_in, line
		;
		;	test for semicolon in any column
		;
		s_test = STRPOS(line, ';')
		if(s_test eq -1) then begin
			;
			parts = STRSPLIT(line, ' ', /REGEX, /EXTRACT)
			;
			;	check that the file name has a "phot" in it, skip line if not
			;
			test = STRPOS(parts[0],"phot")
			if(test eq -1) then goto, skip_a_line
			;
			file_list[ig,nline] = STRTRIM(parts[0],2)
			sf = 0.0d0
			reads, parts[2], sf
			group_scale_factors[ig,nline] = sf
			;
			;  crud, in not zeroing the scale factors, I lost the use flags. 20180518
			;
			;  to recover the use flags, must read each one and then carry it forward for overriding in S7 
			;    if desired
			;
			;if(sf eq 0) then group_use_flag[ig,nline] = 0 else group_use_flag[ig,nline] = 1
			;
			reads, parts[6], this_use_flag, format='(I)'
			group_use_flag[ig,nline] = this_use_flag
			;
			;------------------------------------------------------ 20180518
			;
			file_list_num[ig] = nline
			;
			reads, parts[1], hwp
			group_HWP_angles[ig,nline] = hwp
			reads, parts[3], xoff
			group_x_offsets[ig,nline] = xoff
			reads, parts[4], yoff
			group_y_offsets[ig,nline] = yoff
			;
			nline++
			n_total_images++
		endif else begin
			;
			;	header line, save it, if it begins with PP or MB and isn't PPOL
			;
			p_test = STRPOS(line, 'PP')
			ppol_test = STRPOS(line, 'PPOL')
			mb_test = STRPOS(line, 'MB_')
			if((p_test ne -1 or mb_test ne -1) and ppol_test eq -1) then begin
				header[ig,nheader[ig]] = STRMID(line, s_test + 1)
				nheader[ig]++
			endif
		endelse
		;
		skip_a_line:
	ENDWHILE
	n_total_images--
	;
	FREE_LUN, lu_in
endfor
;
;	all group files read -
;
;		get key word info about apertures for each group
;
for ig = 0, ngroups-1 do begin
	this_header = REFORM(header[ig,0:nheader[ig]-1])
	;
	pp4_arad[ig] = SXPAR(this_header, "PP4_ARAD")
	pp4_ref0[ig] = SXPAR(this_header, "PP4_REF0")
	pp4_ref1[ig] = SXPAR(this_header, "PP4_REF1")
	pp4_clen[ig] = SXPAR(this_header, "PP4_CLEN")
	pp5_rfim[ig] = SXPAR(this_header, "PP5_RFIM")
	pp6_aprd[ig] = SXPAR(this_header, "PP6_APRD")
	;
	save_camera[ig] = SXPAR(this_header, "MB_CAMRA")
	save_filter[ig] = SXPAR(this_header, "MB_FILTR")
endfor
;
;	compute average properties of most (but not the reference image index)
;
ave_arad = TOTAL(pp4_arad) / float(ngroups)
ave_ref0 = TOTAL(pp4_ref0) / float(ngroups)
ave_ref1 = TOTAL(pp4_ref1) / float(ngroups)
ave_clen = TOTAL(pp4_clen) / float(ngroups)
ave_aprd = TOTAL(pp6_aprd) / float(ngroups)
;
;print_text, Event, "Ave ARAD, REF0, REF1, CLEN, APRD = "+STRING(ave_arad, ave_ref0, ave_ref1, ave_clen, ave_aprd)
;
;	group-to-group relative scaling from match
;	file of reference images
;
input_dir = (*s_ptr).analysis_dir + 'S6_Scaling' + PATH_SEP()
new_group_name = replace_underscores(meta_group_name)
input_path = input_dir + new_group_name + '_meta_match.dat'
;
print_text, Event, "Opening file of group-relative scale factors = "+FILE_BASENAME(input_path)
openr, lu_scale, input_path, /GET_LUN
;
;	read
;
rel_grp_id = lonarr(ngroups)
rel_grp_name = strarr(ngroups)
rel_grp_first_file = strarr(ngroups)
rel_grp_dmag = dblarr(ngroups)
rel_grp_flux = dblarr(ngroups)
rel_grp_x_offset = fltarr(ngroups)
rel_grp_y_offset = fltarr(ngroups)
;
line = ''
irg = 0
WHILE ~EOF(lu_scale) do begin
	line = ''
	readf, lu_scale, line, format='(a)'		; because it is long
	;
;	print, line
	;
	;	get past header
	;
	ind = STRPOS(line,';')
;	print, "ind = ",ind
	;
	if(ind eq -1) then begin
		;
;		print, line
		;
		;	parse
		;
		parts = STRSPLIT(line,' ',/extract)
		;
		reads, parts[1], id, format='(I)'
		rel_grp_id[irg] = id
		rel_grp_name[irg] = STRTRIM(parts[2],2)
		rel_grp_first_file[irg] = STRTRIM(parts[3],2)
		reads, parts[8], dmag, format='(F)'
		rel_grp_dmag[irg] = dmag
		;
		reads, parts[12], xoff
		reads, parts[13], yoff
		rel_grp_x_offset[irg] = xoff
		rel_grp_y_offset[irg] = yoff
		;
		;	flux of this file
		;
		rel_grp_flux[irg] = 10.0d0^(-0.40d0 * dmag)
		;
		print, "irg = ",irg," group ID = ", rel_grp_id[irg]," group name = ",rel_grp_name[irg],$
				" first file name = ",FILE_BASENAME(rel_grp_first_file[irg]),$
				" difference mag = ",rel_grp_dmag[irg]," flux equiv. = ",rel_grp_flux[irg]
		;
		irg++
	endif
ENDWHILE
;
FREE_LUN, lu_scale
;
for irg = 0, ngroups-1 do begin
	print_text, Event, "Scale factor for group "+STRING(irg+1,format='(I)')+" is "+STRING(rel_grp_flux[irg],$
					format='(F)')
endfor
;
;	match the orders of the groups
;
ind = lonarr(ngroups)
for ig = 0, ngroups-1 do begin
	jnd = WHERE(rel_grp_id[*] eq group_ids[ig],count)
	if(count eq 1) then begin
		ind[ig] = jnd[0]
	endif
endfor
;
;print, "group matched order = ",ind
;
;	put rel arrays into that order
;
rel_grp_dmag = rel_grp_dmag[ind]
rel_grp_x_offset = rel_grp_x_offset[ind]
rel_grp_y_offset = rel_grp_y_offset[ind]
;
relative_scale = dblarr(ngroups)
relative_scale[0] = 1.0d0
if(ngroups gt 2) then begin
	for ig = 1, ngroups-1 do begin					; in group_ids order
		relative_scale[ig] = rel_grp_flux[0] / rel_grp_flux[ig]
		;
		;print_text, Event, "Relative scale factor for group "+STRING(ig+1,format='(I)')+" relative to first = "+$
		;					STRING(relative_scale[ig],format='(F12.8)')
	endfor
endif else begin
	if(ngroups eq 2) then begin
		relative_scale[1] = rel_grp_flux[0] / rel_grp_flux[1]
	endif
endelse
;
;	open output file and start writing files and scale products
;
openw, lu_out, output_summary_path, /GET_LUN
;
printf, lu_out, format='(";;Meta-Group Scale factors (and X,Y offsets) for meta-group ",a)',meta_group_name
;
scale = dblarr(n_total_images)
use_flag = intarr(n_total_images)
c_scale = intarr(ngroups)
i_images = 0L
;
for ig = 0, ngroups-1 do begin
	;
	x_g_offset = rel_grp_x_offset[ig]
	y_g_offset = rel_grp_y_offset[ig]
	;
	nimages = file_list_num[ig]
	;
	printf, lu_out, ';  '
	printf, lu_out, '; --------------------------------------------------------------------------------------'
	printf, lu_out, format='(";;Images in ",a," had gain ",f12.8," and X,Y offsets ",f7.2,1x,f7.2," applied")',$
						group_names[ig],relative_scale[ig],x_g_offset, y_g_offset
	;
	;	save aperture phot info
	;
	printf, lu_out, '; --------------------------------------------------------------------------------------'
;	printf, lu_out, '; corrections follow for aperture number '+STRING(best_ap+1,format='(I2)')+' with properties:'
	printf, lu_out, ';PP4_ARAD=            '+STRING(pp4_arad[ig],format='(f5.3)')+'/ Maximum Aper Size, in FWHM units'
	printf, lu_out, ';PP4_REF0=            '+STRING(pp4_ref0[ig],format='(f5.3)')+'/ Inner ref radius, in FHWM * ARAD units'
	printf, lu_out, ';PP4_REF1=            '+STRING(pp4_ref1[ig],format='(f5.3)')+'/ Outer ref radius, in FHWM * ARAD units'
	printf, lu_out, ';PP4_CLEN=            '+STRING(pp4_clen[ig],format='(f5.3)')+'/ Outer cleaning radius, in FHWM * ARAD units'
; p5_rfim ?
	printf, lu_out, ';PP6_APRD=         '+STRING(pp6_aprd[ig],format='(f8.6)')+'/ Aperture size in FWHM * ARAD units'
	printf, lu_out, ';'
	printf, lu_out, ';MB_CAMRA=               '+save_camera[ig]+'/ Mimir Camera Optics Used'
	printf, lu_out, ';MB_FILTR=             '+save_filter[ig]+'/ Mimir Filter Option Used'
	printf, lu_out, ';'
	;
	for ii = 0, nimages-1 do begin
		;
		s_all = relative_scale[ig] * group_scale_factors[ig,ii]
		scale[i_images] = s_all
		;---------------------------20180518
		if(group_use_flag[ig,ii] eq 0) then use_flag[i_images] = 0 else use_flag[i_images] = 1
		;
		;if(s_all eq 0) then use_flag[i_images] = 0 else use_flag[i_images] = 1
		;---------------------------20180518
		total_x_offset = x_g_offset + group_x_offsets[ig,ii]
		total_y_offset = y_g_offset + group_y_offsets[ig,ii]
		;
		;	if gain = 0, then zero its offsets, too   (DPC 20090702)
		;
		if(abs(s_all) lt 0.01) then begin
			total_x_offset = 0.0
			total_y_offset = 0.0
		endif
		;
		printf, lu_out, file_list[ig,ii], group_HWP_angles[ig,ii], s_all, $
					total_x_offset, total_y_offset, use_flag[i_images], $
					format='(a,2x,f5.1,1x,f12.8,1x,f7.2,1x,f7.2,1x,i2)'
		;
		i_images++
		;
		if(ii eq nimages-1) then c_scale[ig] = i_images-1				; last image of this color
	endfor
endfor
;
FREE_LUN, lu_out
;
;print, "total images = ",n_total_images
;ind = WHERE(scale eq 0,count)
;if(count gt 0) then begin
;	print, "ERROR - scale factors zero at ind = ",ind
;endif
;
;	plot
;
ind_good = WHERE(use_flag eq 1, ngood)
;
if(ngood gt 0) then begin
	ymax = MAX(scale[ind_good])
	ymin = MIN(scale[ind_good])
	dy = ymax - ymin
	;
	x = indgen(n_total_images)
	xrange = [0, n_total_images+1]
	;
	WIDGET_CONTROL, plot_wid, GET_VALUE=win_no
	wset, win_no
	erase
	;
	max_deviation = (ymax-1.) > (1.-ymin)
	n_deci = ROUND(max_deviation * 100.0)
	;
	yrange = [1-0.015*n_deci,1+0.015*n_deci]
	dy = yrange[1]-yrange[0]
	;
	dx = xrange[1] - xrange[0]
	xrange[0] = xrange[0]-0.02*dx
	xrange[1] = xrange[1]+0.02*dx
	dx = xrange[1] - xrange[0]
	;
	plot, x, scale, yrange=yrange, xrange=xrange, xstyle=1 , ystyle=1, $
			xtitle='Image Number (arb)', ytitle='Scale Factor',$
			color=10, background=18, /NODATA
	;
	iend = -1
	for ig = 0, ngroups-1 do begin
		color = (ig MOD 6) + 12
		istart = iend+1
		iend = c_scale[ig]
		;
		npts = iend - istart + 1
		this_x = x[istart:iend]
		this_scale = scale[istart:iend]
		this_use = use_flag[istart:iend]
		this_ind_good = WHERE(this_use eq 1,this_ngood)
		if(this_ngood gt 0) then begin
			good_x = this_x[this_ind_good]
			good_y = this_scale[this_ind_good]
			;
			oplot, good_x, good_y, color = color
		endif
		;
		;	mark bad data as red diamond with vertical dashed line of group color
		;
		if(npts ne this_ngood) then begin
			this_ind_bad = WHERE(this_use eq 0, this_nbad)
			if(this_nbad gt 0) then begin
				bad_x = this_x[this_ind_bad]
				bad_y = fltarr(this_nbad) + 1.0
				top_y = fltarr(this_nbad) + yrange[0]
				bot_y = fltarr(this_nbad) + yrange[1]
				;
				for ib = 0, this_nbad-1 do begin
					oplot, [bad_x[ib],bad_x[ib]], [bot_y[ib],top_y[ib]], color=color, linestyle=2
				endfor
				;
				plotS, bad_x, bad_y, psym=4, color=red, thick=2
			endif
		endif
		;
		xo = xrange[0] + dx*(float(ig)/ngroups+0.05)
		yo = yrange[0] + 0.1 * dy
		XYOutS, xo, yo, FILE_BASENAME(group_names[ig]), color=color, charsize=1.25
	endfor
	;
	print_text, Event, "Completed Meta-group scale factor determination for meta-group "+meta_group_name
endif else begin
	print_text, Event, "ERROR - no images with non-zero scale factors - no plot generated"
endelse
;
end