pro S7_group_coadding, Event, group_id, group_name, LED_wid, status
	;
	;	manages coadding for one group, including LED progress
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	o_s_f = (*w_ptr).S7_override_scaling_mode 	; = 0 normal, = 1 override scaling
	o_use_0 = (*w_ptr).S7_USE_0_override_flag   ; 0 = use normal USE=0 rejection; 1 = ignore USE=0 rejections
	ss_sub_div = (*w_ptr).S7_subtract_ss_flag   ; 1 = subtract SS; 0 = divide by SS
	;
	max_hwp = 15				; normal is 15 - test mode is 0; photometric mode sets it to 0 below
	;
	status = -1
	bad_pixel_value = -1.0e6
	;
	row513_flag = (*w_ptr).S7_mask_row513_mode					; kill row 513 if set
	flatten_pre_flag = (*w_ptr).S7_flatten_images_pre_flag
	flatten_post_flag = (*w_ptr).S7_flatten_images_post_flag
	fix_bad_pixels_flag = (*w_ptr).S7_fix_bad_pixels_flag
	;
	if((*w_ptr).S7_skip_RC_flag eq 1) then begin		; skip row, column flattening in model_background, if set
		rowf = 1
		colf = 1
	endif else begin
		rowf = 0
		colf = 0
	endelse
	;
	;	background removal modes
	;
	if( (*w_ptr).S7_options_mode_and_tab eq 0) then begin
		print_text, Event, 'Starting Coadding with Star-Field type background removal'
		;
		;	Star Field type backgrounds
		;
		if( (*w_ptr).S7_stars_mode eq 0) then begin
			print_text, Event, '   and using SuperSky method'
			supersky_flag = 1
			apply_illumination_flag = 0
			low_order_flag = 0
			use_step11_points_flag = 0
			save_supersky_flag = (*w_ptr).S7_save_supersky_mode
			smooth_supersky_flag = (*w_ptr).S7_smooth_supersky_mode
			supersky_boxcar = (*w_ptr).S7_box_width_value
		endif else begin
			print_text, Event, '    and using Modeling of sky method'
			supersky_flag = 0
			apply_illumination_flag = 1	; if set, compute illumination function and scale by it
			low_order_flag = 0
			save_supersky_flag = 0
			smooth_supersky_flag = 0
			supersky_boxcar = 0
			use_step11_points_flag = 0
		endelse
	endif else begin
		print_text, Event, 'Starting Coadding with Extended Object(s) type background removal'
		;
		;	Extended Objects type backgrounds
		;
		supersky_boxcar = 0
		if( (*w_ptr).S7_extended_mode eq 0) then begin
			print_text, Event, '     and using Constant sky method'
			low_order_flag = 1
			use_step11_points_flag = 0
			fit_option_mode = -1
			supersky_flag = 0
			apply_illumination_flag = 0
		endif else begin
			print_text, Event, '     and using S11 grid points fitting'
			low_order_flag = 0
			use_step11_points_flag = 1
			supersky_flag = 0
			apply_illumination_flag = 0
			fit_option_mode = (*w_ptr).S7_fit_options_mode	; 0 = 4Q together; 1 = 4Q independent; 2 = T/B indep.
			case fit_option_mode of
				0: print_text, Event,'     with 4 Quads fit together'
				1: print_text, Event,'     with 4 Quads fit independently'
				2: print_text, Event,'     with Top and Bottom pairs of Quads fit independently'
				-1: print_text, Event,'ERROR - should not have gotten here. Quad-mode = -1'
			endcase
		endelse
	endelse
	;
	star_sky_flag = (*w_ptr).S7_star_sky_flag			; 1 = distinguish stars from sky in mfm, 0 = same (for comets)
	ss_star_sky_flag = (*w_ptr).S7_ss_star_sky_flag  ; same, but for constructing superskies
	;
	if(ss_sub_div eq 1) then supersky_mode = 'subtract'	else supersky_mode = 'divide'	; subtract (sky) or divide (2nd order FF)
  ;
	supersky_kernel = supersky_boxcar												; * FWHM = pixels for median filtering
	;
	double_precision_flag = 0
	print_text, Event,"Starting Coadding with DoublePrecisionFlag = "+STRING(double_precision_flag,format='(I1)')
	;
	h_wid = (*w_ptr).S7_history_textbox_wid
	;
	;	any comments to add?
	;
	comment_flag = (*w_ptr).S7_comments_mode
	if(comment_flag) then comment_str = (*w_ptr).S7_comments_string
	;
	astro_dir = (*s_ptr).analysis_dir + 'S3_Astrometry' + PATH_SEP()
	scale_dir = (*s_ptr).analysis_dir + 'S6_Scaling' + PATH_SEP()
	psf_dir = (*s_ptr).analysis_dir + 'S4_First_PSF_Fits' + PATH_SEP()
	s11_dir = (*s_ptr).analysis_dir + 'S11_Full_Field_Polarimetry' + PATH_SEP()
	;
	;	create name of summary scaling file for this group
	;
	this_group = replace_underscores(group_name)
	;
	;	if extended and s11 points method selected, create path to points file, test for it, and read it
	;
	if( use_step11_points_flag eq 1) then begin
		s11_file_path = s11_dir + this_group + '_Deep_Coadd_bg_points.dat'
		s11_ptr = s7_read_points_file(Event, s11_file_path)
		if(NOT(PTR_VALID(s11_ptr))) then begin
			str = "Could not read S11 points file"
			print_text, Event, "ERROR - "+str+" - ending"
			SAll_add_to_history, Event, h_wid, 7, str
			status = -1
			goto, S7_GC_end
		endif
	endif
	;
	scale_path = scale_dir + this_group + '_group_scale_factors.dat'
	;
	;	check that it exists
	;
	test = FILE_INFO(scale_path)
	if(NOT(test.exists)) then begin
		str = group_name+" has no scaling summary file"
		print_text, Event, "ERROR - group "+str+" - ending"
		SAll_add_to_history, Event, h_wid, 7, str
		status = -1
		goto, S7_GC_end
	endif
	;
	;	prepare variables
	;
	max_files 		= 350
	file_names 		= strarr(max_files)
	hwp_angles 		= fltarr(max_files)
	scale_factors 	= dblarr(max_files)
	x_offsets 		= intarr(max_files)
	y_offsets 		= intarr(max_files)
	;
	n_llq_flags		= 0L
	n_files 		= 0L
	;
	openr, lu_scale, scale_path, /GET_LUN
	line = ''
	;
	header = strarr(1)
	nheader = 0
	;
	WHILE ~EOF(lu_scale) do begin
		readf, lu_scale, line
		;
		;	test for "phot" in line - skip if missing
		;
		test = STRPOS(line,"phot")
		if(test ne -1) then begin
			;
			parts = STRSPLIT(line,' ',/extract)
			file_names[n_files] = psf_dir + parts[0]
			;
			reads, parts[1], hwp, 		format='(F)'
			reads, parts[2], scale, 	format='(D)'
			reads, parts[3], x_off, 	format='(F)'
			reads, parts[4], y_off, 	format='(F)'
			reads, parts[5], dither, 	format='(I)'
			reads, parts[6], use_flag, 	format='(I)'
			;
			;	cleck for reasonableness and limits (and deleted images)
			;
			if(scale lt 0.3 or scale gt 3.0) then begin
				str = FILE_BASENAME(file_names[n_files])+" has scale = "+parts[2]+' so skipping'
				print_text, Event, "Warning - Image "+str
				SAll_add_to_history, Event, h_wid, 7, str
				goto, skip_this
			endif
			;
			;	similarly, check use flag
			;
			if(use_flag eq 0 and o_USE_0 eq 0) then begin
				str = FILE_BASENAME(file_names[n_files])+" has use_flag = "+parts[6]+' so skipping'
				print_text, Event, "Warning - Image "+str
				SAll_add_to_history, Event, h_wid, 7, str
				goto, skip_this
			endif
			;
			;	all others, add to vectors
			;
			hwp_angles[n_files] = hwp
			;
			scale_factors[n_files] = scale
			x_offsets[n_files] = ROUND(x_off)
			y_offsets[n_files] = ROUND(y_off)
			;
			n_files++
			;
			skip_this:
		endif else begin
			;
			;	all other lines with = signs go into header, after stripping leading semicolon
			;
			test_eq = STRPOS(line, '=')
			if(test_eq ne -1) then begin
				len = STRLEN(line)
				header[nheader] = STRMID(line,1,len-1)
				header = [header, '']
				nheader++
			endif
		endelse
	ENDWHILE
	FREE_LUN, lu_scale
	;
	header = header[0:nheader-1]
	save_PP4_ARAD = SXPAR(header, "PP4_ARAD")
	save_PP4_REF0 = SXPAR(header, "PP4_REF0")
	save_PP4_REF1 = SXPAR(header, "PP4_REF1")
	save_PP4_CLEN = SXPAR(header, "PP4_CLEN")
	save_PP6_APRD = SXPAR(header, "PP6_APRD")
	save_PP6_AMPL = SXPAR(header, "PP6_AMPL",count=acount)
	if(acount eq 0) then save_PP6_AMPL = -1.0d0    ; new keyword - old files won't have it
	;
	;	get camera type
	;
	this_camera_type = STRTRIM(SXPAR(header, "MB_CAMRA"),2)
	if(this_camera_type eq 'F5') then f17 = 0 else f17 = 1
	;
	file_names = file_names[0:n_files-1]
	hwp_angles = hwp_angles[0:n_files-1]
	scale_factors = scale_factors[0:n_files-1]
	;
	;	evaluate against scaling override
	;
	ind_so = WHERE(scale_factors ne 0.0,n_scale)
	if(n_scale gt 0 and o_s_f eq 1) then begin
		scale_factors[ind_so] = 1.0
	endif
	;
	x_offsets = x_offsets[0:n_files-1]
	y_offsets = y_offsets[0:n_files-1]

	xmin = MIN(x_offsets)
	xmax = MAX(x_offsets)
	ymin = MIN(y_offsets)
	ymax = MAX(y_offsets)
	;
	dx = xmax - xmin
	dy = ymax - ymin
	;
	;	compute final image dimensions and astrometry information
	;
	first_file_name = file_names[0]
	ref_header = S7_read_phot_file_header(first_file_name)
	;
	crpix1 = sxpar(ref_header, "CRPIX1")
	crpix2 = sxpar(ref_header, "CRPIX2")
	naxis1 = sxpar(ref_header, "NAXIS1")
	naxis2 = sxpar(ref_header, "NAXIS2")
	crval1 = sxpar(ref_header, "CRVAL1")
	crval2 = sxpar(ref_header, "CRVAL2")
	fwhm = sxpar(ref_header, "PSF_FWHM")
	;
	new_naxis1 = naxis1 + dx
	new_naxis2 = naxis2 + dy
	;
	new_crpix1 = crpix1 + (x_offsets[0] - xmin)		; this is where the old crpix1,2 will go in the
	new_crpix2 = crpix2 + (y_offsets[0] - ymin)		;    new image, if we keep the same old CRVAL1,2
	;
	new_x_center = float(new_naxis1) / 2.			; assuming IDL-like pixel coordinates...
	new_y_center = float(new_naxis2) / 2.			;   ("WWDS9T" - what would DS9 think???)
	;
	;	update header for new crpix1,2 values
	;
	SXADDPAR, ref_header, 'CRPIX1', new_crpix1
	SXADDPAR, ref_header, 'CRPIX2', new_crpix2
	;
	extast, ref_header, ref_astr_struct, noparams
	;
	;	compute ra, dec of new center
	;
	xy2ad, new_x_center, new_y_center, ref_astr_struct, ra_cen, dec_cen
	;
;	print, "old ra, dec center = ",crval1, crval2
;	print, "old crpix1, 2 = ",crpix1, crpix1
;	print, "xmin, xmax, dx = ", xmin, xmax, dx
;	print, "ymin, ymax, dy = ", ymin, ymax, dy
;	print, "new image size = ",new_naxis1, new_naxis2
;	print, "new crpix1, 2 = ",new_crpix1, new_crpix2
;	print, "final crpix1, 2 = ",new_x_center, new_y_center
;	print, "final ra, dec center = ",ra_cen, dec_cen
	;
	;	save new center ra, dec and new ref pixels
	;
	SXADDPAR, ref_header, 'CRPIX1', new_x_center
	SXADDPAR, ref_header, 'CRPIX2', new_y_center
	SXADDPAR, ref_header, 'CRVAL1', ra_cen
	SXADDPAR, ref_header, 'CRVAL2', dec_cen
	SXADDPAR, ref_header, 'NAXIS1', new_naxis1
	SXADDPAR, ref_header, 'NAXIS2', new_naxis2
	;
	;	amount by which a star's position should be incremented in X and Y
	;	from the pixel values in this image to find the approximate mean
	;	detector X,Y pixel values when detected
	;
	x_pix_offset = xmin - (float(TOTAL(x_offsets))/float(n_files))
	y_pix_offset = ymin - (float(TOTAL(y_offsets))/float(n_files))
	;
	SXADDPAR, ref_header, 'MPP_XOFF', x_pix_offset, 'Add this to X pix to get detector X'
	SXADDPAR, ref_header, 'MPP_YOFF', y_pix_offset, 'Add this to Y pix to get detector Y'
	SXADDPAR, ref_header, 'HISTORY', 'Created in MSP_PPOL Step 7 (group coadding) on '+SYSTIME()
	;
	;	if comment flag set, add comments
	;
	if(comment_flag) then begin
		SXADDPAR, ref_header, 'HISTORY', comment_str
	endif
	;
	;	add info from Step 6 to pass onto Step 8
	;
	SXADDPAR, ref_header, 'PP4_ARAD', save_PP4_ARAD,'Maximum Aper Size, in FWHM units'
	SXADDPAR, ref_header, 'PP4_REF0', save_PP4_REF0,'Inner ref radius, in FHWM * ARAD units'
	SXADDPAR, ref_header, 'PP4_REF1', save_PP4_REF1,'Outer ref radius, in FHWM * ARAD units'
	SXADDPAR, ref_header, 'PP4_CLEN', save_PP4_CLEN,'Outer cleaning radius, in FHWM * ARAD units'
	SXADDPAR, ref_header, 'PP6_APRD', save_PP6_APRD,'Aperture size in FWHM * ARAD units'
	SXADDPAR, ref_header, 'PP6_AMPL', save_PP6_APRD,'Secular Amplitude (in %)'
	;
	;	and reset X, Y image offsets to be referenced to 0,0
	;
	x_offsets = x_offsets - xmin
	y_offsets = y_offsets - ymin
	;
	;	parse into the 16 lists
	;
	hwp_values = [0.0,22.5,45.0,67.5,90.0,112.5,135.0,157.5,180.0,202.5,225.0,247.5,270.0,$
					292.5,315.0,337.5]
	hwp_values = hwp_values[0:max_hwp]
	;
	hwp_file_list = lonarr(max_hwp+1,n_files)
	hwp_list_n = lonarr(max_hwp+1)
	;
	;	Reset LED1 to reflect up to 17 sublists
	;
	WIDGET_CONTROL, LED_wid, SET_UVALUE=max_hwp+2
	image_colors = strarr(max_hwp+2)
	image_colors[*] = "red"
	S7_LED_sensitize, Event, 1, 1
	;
	;	compare each HWP to the list, finding the one closest (in case steps are
	;			bumped in the LOIS code...)
	;
	for ifile = 0, n_files-1 do begin
		dhwp = abs(hwp_angles[ifile] - hwp_values[*])
		min_dhwp = MIN(dhwp)
		ind = WHERE(dhwp eq min_dhwp and dhwp lt 5.0,count)		; must find one within 5 deg
		if(count eq 1) then begin
			hwp_file_list[ind,hwp_list_n[ind]] = ifile		; as a reference number
			hwp_list_n[ind]++
		endif else begin
			if(count ne 0) then begin
				str = "Multiple HWP angle matches for "+file_names[ifile]
				print_text, Event, "ERROR - "+str
				SAll_add_to_history, Event, h_wid, 7, str
			endif else begin
				str = "No HWP angle match for "+file_names[ifile]
				print_text, Event, "ERROR - "+str
				SAll_add_to_history, Event, h_wid, 7, str
			endelse
		endelse
	endfor
	;
	fn_mean_images = double(CEIL(MEDIAN(hwp_list_n)))
	;
	if(fn_mean_images eq 0) then begin
		;
		;	likely this is a photometric group - only images in HWP=00 are included
		;
		fn_mean_images = hwp_list_n[0]
		max_hwp = 0							; to terminate the loops early
	endif
	;
	SXADDPAR, ref_header, 'MPP_NSCL', fn_mean_images, 'Number of images this one is scaled to'
	;
;	for ihwp = 0, 15 do begin
;		print, "Found ",hwp_list_n[ihwp]," files with HWP = ",hwp_values[ihwp]
;	endfor
	;
	print_text, Event, "All HPW angles parsed"
	;
	hwp_coadd_paths = strarr(max_hwp+1)
	;
	;----------------------------- if switch set, compute supersky image ------------
	;
	if(supersky_flag gt 0) then begin
		;
		disk_mode = save_supersky_flag							; save results to disk
		memory_mode = 1											; return with supersky cube pointed to in memory
		;
		group_name_vector = [group_name]	; save group name in vector
		image_group_number = intarr(N_ELEMENTS(file_names)) ; all group "0"
		;
		S7_create_supersky, Event, group_name_vector, hwp_list_n, hwp_file_list, $
						file_names, scale_factors, image_group_number, $
						LED_wid, image_colors, $
						double_precision_flag, disk_mode, memory_mode, supersky, max_hwp
		;
	endif
	;
	;----------------------------- coadd 16 HWP images --------------------------------
	;
	;	for every HWP case
	;
	for ihwp = 0, max_hwp do begin
	;for ihwp = 1, 1 do begin			; test case
		;
		;	set LED1 to blue
		;
		this_LED = ihwp
		image_colors[this_LED] = "blue"
		WCT_LED_stack, LED_wid, image_colors
		;
		;	if supersky flag set, recover the supersky image for this hwp
		;
		if(supersky_flag eq 1) then this_supersky = PTR_NEW( REFORM( (*supersky)[*,*,ihwp] ) )
		;
		ind_list = REFORM(hwp_file_list[ihwp,0:hwp_list_n[ihwp]-1])
		;
		x_off = x_offsets[ind_list]
		y_off = y_offsets[ind_list]
		scale = scale_factors[ind_list]
		;
		;	get the vector of names
		;
		image_names = FILE_BASENAME(file_names[ind_list])
		;
		;	and convert back to FITS image names
		;
		n_images = N_ELEMENTS(image_names)
		;
		for i_image = 0, n_images-1 do begin
			parts = STRSPLIT(image_names[i_image],"_phot.dat",/REGEX,/EXTRACT)
			image_names[i_image] = parts[0] + '.fits'
			;
			;	set up its FITS header
			;
			if(i_image eq 0) then begin
				first_image_path = astro_dir + image_names[0]
				this_header = ref_header							; DPC 20090702
			endif
			;
			FXADDPAR, this_header, 'HISTORY', "PPOL-Step 7-version "+(*s_ptr).version_short_string
			SXADDPAR, this_header, 'HISTORY', 'includes '+image_names[i_image]
			SXADDPAR, this_header, 'HISTORY', '  with X,Y offsets '+STRING(x_off[i_image],y_off[i_image],$
										format='(I3,1x,I3)')+" pixels"
			SXADDPAR, this_header, 'HISTORY', '  and multiplied by scale factor '+STRING(scale[i_image],$
										format='(F13.8)')
			SXADDPAR, this_header, 'MPP_NADD', n_images, 'Number of images coadded into this one'
		endfor
		;
		image_path = astro_dir + image_names
		;
		if(double_precision_flag) then raw_images = dblarr(n_images, naxis1, naxis2) else $
									   raw_images = fltarr(n_images, naxis1, naxis2)
		;
		;	fetch the raw images
		;
		all_file_names = ''
		sky_image = dblarr(n_images)
		sky_sigma = dblarr(n_images)
		for i_image = 0, n_images-1 do begin
			if(i_image eq 2*fix(i_image/2)) then image_colors[this_LED] = "orange" else $
							image_colors[this_LED] = "blue"
			;
			WCT_LED_stack, LED_wid, image_colors
			;
			;	compute pathnames to Astrometry directory
			;
			all_file_names = all_file_names + FILE_BASENAME(image_path[i_image]) + ', '
			;
			;	flatten input image, if flatten_pre_flag set
			;
			sky_image[i_image] = 0.0
			sky_sigma[i_image] = 100.0
			;
			if(flatten_pre_flag eq 0) then begin
					input_image = fix_high_pixels(readfits(image_path[i_image], header, /SILENT))
					;
					;	if LLQ flag set for this image, clear LLQ to mean background of other quads
					;
					input_image = llq_clear(input_image,header)
					;
					;	fix low pixels w/TPS
					;
					input_image = fix_low_pixels(input_image, /TPS)
					;
					;	if extended and using S11 grid points to background remove, go do so
					;
					if( use_step11_points_flag eq 1 ) then begin
						S7_fit_using_S11_points, Event, input_image, fit_option_mode, s11_ptr, this_sky, $
												this_sigma, x_off[i_image], y_off[i_image], $
												image_path[i_image]
					endif else begin
						;
						;	if not, get constant background
						;
						input_image = model_background(input_image, SKY=this_sky, SIGMA=this_sigma, $
										NOROWF=rowf, NOCOLF=colf, /LOW_ORDER)
					endelse
					;
					print_text, Event, "Image "+FILE_BASENAME(image_path[i_image])+" has sky = "+STRING(this_sky)
					sky_image[i_image] = this_sky
					sky_sigma[i_image] = this_sigma
;				endelse
			endif else begin
				input_image = fix_high_pixels(readfits(image_path[i_image], header, /SILENT))
				;
				;	if LLQ flag set for this image, clear LLQ to mean background of other quads
				;
				input_image = llq_clear(input_image,header)
				;
				input_image = fix_low_pixels(input_image, /TPS)
				;
				input_image = model_background(input_image, SKY=this_sky, SIGMA=this_sigma, /MCS,$
												NOROWF=rowf,NOCOLF=colf)
				print_text, Event, "Image "+FILE_BASENAME(image_path[i_image])+" has sky = "+STRING(this_sky)
				sky_image[i_image] = this_sky
				sky_sigma[i_image] = this_sigma
			endelse
			;
			;	test for LLQ flag, while header still present
			;
			llq_f = SXPAR(header, 'PP3_LLQF', count=kwcount)
			if(kwcount eq 1) then begin
				if(llq_f eq 1) then n_llq_flags++
			endif
			;
			;	if supersky selected, scale supersky, subtract it and replace it with the mean sky
			;
			if(supersky_flag eq 1) then begin
				case supersky_mode of
					'subtract': input_image += (1.0 -  (*this_supersky)) * sky_image[i_image]
					'divide'  : input_image /= (*this_supersky)
				endcase
			endif
			;
			input_size = SIZE(input_image)
			xis = input_size[1]
			yis = input_size[2]
			;
			if(yis eq 1026 and row513_flag eq 1) then input_image[*,513] = bad_pixel_value
			;
			input_image[0:2,*] 				= bad_pixel_value
			input_image[*,0:8] 				= bad_pixel_value	; gets worst of ski jump
			input_image[xis-3:xis-1,*] 		= bad_pixel_value
			input_image[*,yis-14:yis-1] 	= bad_pixel_value	; ski jump again
			;
			raw_images[i_image,*,*] 		= input_image[*,*]
		endfor
		;
		print_text, Event, "Done Reading images "+all_file_names
		;
		ind_sky = WHERE(scale ne 0.0,ngoodsky)
		if(ngoodsky gt 0) then begin
			scaled_mean_sky = TOTAL(sky_image[ind_sky] * scale[ind_sky]) / float(ngoodsky)
			mean_sky_sigma = TOTAL(sky_sigma[ind_sky] * scale[ind_sky]) / float(ngoodsky)
		endif else mean_sky = 0.0
		;
		;------------------ compute output image --------------------------
		;
		print_text, Event, "Beginning computation of HWP_"+STRING(ihwp,format='(I2.2)')+" image"
		;
		;	load structure for passing to coadder
		;
		;if(ihwp eq 0) then debug = 1 else debug = 0
		debug = 0
		;
		;	check star_sky flag - if 1 use normal mean_sky_sigma, else boost to high value
		;
		if(star_sky_flag eq 0) then mean_sky_sigma = 1.0e5
		;
		coadd_struct = { new_naxis1:		new_naxis1,		$	x-axis of output image
						 new_naxis2: 		new_naxis2, 	$	y-axis
						 naxis11:			naxis1-1, 		$
						 naxis21:			naxis2-1, 		$
						 n_images:			n_images, 		$	number of constituent images
						 x_off:				x_off,			$
						 y_off:				y_off, 			$
						 raw_images:		raw_images,		$	the input data cube
						 sky_image:			sky_image,		$
						 scale:				scale,			$
						 double_precision_flag: double_precision_flag, $
						 fn_mean_images: 	fn_mean_images, $
						 mean_sky_sigma: 	mean_sky_sigma, $
						 debug: 			debug, 			$
						 scaled_mean_sky:	scaled_mean_sky }

		output_image = S7_coadd(coadd_struct)
		;
		;	create file name and write to disk
		;
		output_name = replace_underscores(group_name) + "_HWP_" + STRTRIM(STRING(ihwp,format='(I2.2)'),2) + '.fits'
		output_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
		output_path = output_dir + output_name
		;
		hwp_coadd_paths[ihwp] = output_path
		;
		FXADDPAR, this_header, 'HISTORY', "PPOL-Step 7-version "+(*s_ptr).version_short_string
		SXADDPAR, this_header, 'FILENAME', output_name, 'Created by MSP_PPOL Step 7'
		SXADDPAR, this_header, 'PPOL_HWP', hwp_values[ihwp], 'HWP angle in degrees, from PPOL S7'
		SXADDPAR, this_header, 'MB_HWP', hwp_values[ihwp],'Updated HWP angle from PPOL S7'
		;
		;	get current JD and add one min for each HWP angle
		;
		date_obs = FXPAR(this_header, "DATE-OBS")
		reads, date_obs, year, month, day, hour, minute, second, format='(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f4.2)
		jd = JULDAY(Month, Day, Year, 0, ihwp, 0)
		SXADDPAR, this_header, 'MB_JD', jd, "JD manufactured by PPOL S7"
		;
		;	if apply_illumination flag set, find model background and divide by scaled background shape
		;		(DPC 20091128)
		;
		if(apply_illumination_flag eq 1) then begin
			flat_image = model_background(output_image,SKY=sky,/MCS,NOROWF=rowf,NOCOLF=colf)
			model_image = (*flat_image) * 0.0
			;
			out_good_flag = (*output_image) + 0.5e6
			out_good_flag >= 0.0
			out_good_flag <= 1.0
			out_bad_flag = 1.0 - out_good_flag
			;
			model_image = (((*output_image) - (*flat_image)) + sky) * out_good_flag + out_bad_flag
			;
			;	scale the model to its mean sky value and then divide (correct) output image for this
			;	illumination pattern
			;
			(*output_image) /=  (model_image / sky)
			(*output_image) += (1.0 - 1.0e6) * out_bad_flag
			SXADDPAR, this_header, 'HISTORY', 'Corrected for illumination, with mean sky = '+STRING(sky)
		endif
		;
		;	if flatten_post flag set, flatten image
		;
		if(flatten_post_flag eq 1) then begin
			output_image = model_background(output_image, SKY=sky, STEP_SIZE=30, /MCS, NOROWF=rowf,NOCOLF=colf)
			SXADDPAR, this_header, 'HISTORY', 'Post Coadded TPS Flattened, with mean sky = '+STRING(sky)
		endif
		;
		;	if flag set, fix bad pixels remaining in image
		;
		if(fix_bad_pixels_flag eq 1) then begin
			output_image = fix_bad_pixels(output_image)
			SXADDPAR, this_header, 'HISTORY', 'Post coadding, fixed remaining bad pixels'
		endif
		;
		;	and low pixels remaining
		;
		output_image = fix_low_pixels(output_image, /TPS)
		;
		test_output_image = SIZE(output_image,/TYPE)
		if(test_output_image eq 10) then begin
			;
			;	was a pointer, dereference it
			;
			writefits, output_path, (*output_image), this_header
		endif else begin
			writefits, output_path, output_image, this_header
		endelse
		;
		print_text, Event, "Wrote coadded image "+output_name
		;
		image_colors[this_LED] = "green"
		WCT_LED_stack, LED_wid, image_colors
		;
		if(supersky_flag eq 1) then PTR_FREE, this_supersky
	endfor
	;
	;	if supersky was selected, release the heap images of the supersky images
	;
	if(supersky_flag eq 1) then PTR_FREE, supersky
	;
	;	be sure to close supersky_image from heap - try second test
	;
	type = SIZE(supersky, /TYPE)
	if(type eq 10) then PTR_FREE, supersky
	;
	type = SIZE(this_supersky, /TYPE)
	if(type eq 10) then PTR_FREE, this_supersky
	;
	;-------------------------- deep coadd ---------------------------------------------
	;
	;if(max_hwp eq 0) then stop
	;
	;	reserve room, then read images
	;
	this_LED = max_hwp+1
	image_colors[this_LED] = "blue"
	WCT_LED_stack, LED_wid, image_colors
	;
	if(double_precision_flag) then begin
		raw_images = dblarr(max_hwp+1,new_naxis1, new_naxis2)
		output_image = dblarr(new_naxis1, new_naxis2)
	endif else begin
		raw_images = fltarr(max_hwp+1,new_naxis1, new_naxis2)
		output_image = fltarr(new_naxis1, new_naxis2)
	endelse
	;
	n_total_files = 0L
	n_total_scaled = 0L
	sky_image = dblarr(max_hwp+1)
	sky_sigma = dblarr(max_hwp+1)
	for ihwp = 0, max_hwp do begin
		;
		;	flatten background with low-order to pick up sky levels
		;
		sky_image[ihwp] = 0.0d0
		sky_sigma[ihwp] = 100.0
		;
		if(flatten_pre_flag eq 0) then begin
			raw_images[ihwp,*,*] = fix_high_pixels(readfits(hwp_coadd_paths[ihwp],header,/SILENT))
		endif else begin
			raw_images[ihwp,*,*] = model_background(readfits(hwp_coadd_paths[ihwp],header,/SILENT),$
										SKY=this_sky,SIGMA=this_sigma,/LOW_ORDER,NOROWF=rowf,NOCOLF=colf)
			sky_image[ihwp] = this_sky
			sky_sigma[ihwp] = this_sigma
		endelse
		;
		n_this_files = sxpar(header, 'MPP_NADD')
		n_total_files += ROUND(n_this_files)					; update total file count
		n_scaled = sxpar(header, 'MPP_NSCL')
		n_total_scaled += ROUND(n_scaled)
		if(ihwp eq 0) then first_header = header
		SXADDPAR, first_header, 'HISTORY', 'deep coadd includes '+FILE_BASENAME(hwp_coadd_paths[ihwp])
	endfor
	;
	print_text, Event, "Beginning coadding of Deep image"
	;
	zz_sky = median_filtered_mean(sky_image)
	mean_sky = zz_sky[0]
	ss = median_filtered_mean(sky_sigma)
	mean_sky_sigma = ss[0]
	;
	;---------------------------------------------------------
	x_off = fltarr(max_hwp+1)
	y_off = fltarr(max_hwp+1)
	naxis1 = new_naxis1
	naxis2 = new_naxis2
	;
	n_images = max_hwp+1
	scale = fltarr(max_hwp+1)
	scale[*] = 1.0
	;
	fn_mean_images = float(max_hwp+1)
	scaled_mean_sky = mean_sky
	debug = 0
	;
	if(star_sky_flag eq 0) then mean_sky_sigma = 1.0e6
	;
	coadd_struct = { new_naxis1:	new_naxis1,	$	x-axis of output image
					 new_naxis2: 	new_naxis2, 	$	y-axis
					 naxis11:		naxis1-1, $
					 naxis21:		naxis2-1, $
					 n_images:		n_images, 	$	number of constituent images
					 x_off:			x_off,		$
					 y_off:			y_off, 		$
					 raw_images:	raw_images,	$	the input data cube
					 sky_image:		sky_image,		$
					 scale:			scale,		$
					 double_precision_flag: double_precision_flag, $
					 fn_mean_images: fn_mean_images, $
					 mean_sky_sigma: mean_sky_sigma, $
					 debug: 		debug, $
					 scaled_mean_sky:scaled_mean_sky }

	output_image = S7_coadd(coadd_struct)
	;
	;	for each pixel in the final image, watching for bad pixels...(and zero pixels)
	;
	;	output name
	;
	output_name = replace_underscores(group_name) + '_Deep_Coadd.fits'
	output_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
	output_path = output_dir + output_name
	;
	FXADDPAR, this_header, 'HISTORY', "PPOL-Step 7-version "+(*s_ptr).version_short_string
	SXADDPAR, first_header, 'FILENAME', output_name, 'Created by MSP_PPOL Step 7'
	SXADDPAR, first_header, 'MPP_DADD', n_total_files, 'total number of files in this deep coadd'
	SXADDPAR, first_header, 'MPP_NSCL', n_total_scaled, 'Number of images this sum is scaled to'
	;
	;	post flatten and bad pixel fix if flags set
	;
	if(flatten_post_flag eq 1) then output_image = model_background(output_image,STEP_SIZE=25,/MCS,$
														NOROWF=rowf,NOCOLF=colf)
	if(fix_bad_pixels_flag eq 1) then output_image = fix_bad_pixels(output_image)
	;
	output_image = fix_low_pixels(output_image, /TPS)
	;
	test_output_image = SIZE(output_image, /TYPE)
	if(test_output_image eq 10) then begin
		writefits, output_path, (*output_image), first_header
	endif else begin
		writefits, output_path, output_image, first_header
	endelse
	;
	print_text, Event, "Wrote deep coadded image "+output_name
	;
	image_colors[this_LED] = "plum"
	WCT_LED_stack, LED_wid, image_colors
	;------------------------------------------- redo astrometry --------------------
	deep_image = readfits(output_path, header)
	;
	;	updated for use with F17 images...
	;
;	f17 = 0							; uses camera designation discovered above
	tolerance = 0.0015
	platescale_est = 0.579
	if(f17 eq 1) then platescale_est = 0.179
	;
	print_text, Event, "Starting revised astrometry determination"
	deep_header = ppol_astrometry(Event, output_path, deep_image, header, return_flag, platescale_est, tolerance,$
							find_scale,F17=f17, ARRAY_2MASS=array_2mass, TWO_MASS=two_mass, $
							FILE_2MASS=mass_array)
	;
	SXADDPAR, deep_header, 'PP7_NLLQ', n_llq_flags, ' Number of images with bad LLQs in this average'
	;
	;  measure FWHM in this image
	;  
	fwhm_debug = 0
	sat_limit = 6000.0 * n_total_scaled
	;
	g_sky, deep_image, skymode, sky_sigma, mfm_mean, mfm_stdev, /SILENT
	;
	g_get_FWHM, deep_image, deep_image, skymode, skysig, mfm_mean, mfm_stdev, $
      mean_FWHM, sig_FWHM, sat_limit, fwhm_debug, F17=f17
  ;
  SXADDPAR, deep_header, 'PP7_MSKY',mfm_mean/n_total_scaled,' Mean Sky (in ADU) per Image'
  SXADDPAR, deep_header, 'PP7_FWHM',mean_FWHM,' FWHM [in pixels] of coadded image'
  ;
  ; seeing, corrected for pixelation
  ; 
  seeing = sqrt(mean_FWHM^2 - 1.0)
  if(f17 eq 1) then seeing *= 0.179 else seeing *= 0.58
  SXADDPAR, deep_header, 'PP7_SEE',seeing,' Seeing, FWHM in arcsec, corrected for pixelation'
	;
	;	rewrite deep image with updated header
	;
	print_text, Event, "Resaving deep image with updated astrometry header"
	writefits, output_path, deep_image, deep_header
	;
	;-------------------------------------------------------------------------------
	image_colors[this_LED] = "green"
	WCT_LED_stack, LED_wid, image_colors
	;
	status = 0
	S7_GC_end:
	;
	if(PTR_VALID(s11_ptr)) then begin
		PTR_FREE, s11_ptr
	endif
end
