function S7_new_coadd_with_median_filtering, coadd_struct
;
;	performs coaddition on images (the new way)
;	using vectorization and MEDIAN filtering
;
;	20110318 DPC	returns pointer to output image instead of image itself
;
start_memory = MEMORY()		; trap current memory value
start_memory = start_memory[0]
;
nx = coadd_struct.new_naxis1
ny = coadd_struct.new_naxis2
;
;	check that scale = 0 images aren't included
;
ind_good = WHERE(coadd_struct.scale ne 0.0,nimages)
;
;	double precision flag
;
dp_flag = coadd_struct.double_precision_flag
;
if(dp_flag) then begin
	image_data_ptr 			= PTR_NEW(dblarr(nimages,nx,ny))
	product_data_mask_ptr 	= PTR_NEW(dblarr(nimages,nx,ny))
	sky_image 				= coadd_struct.sky_image
	scale 					= coadd_struct.scale
	scaled_mean_sky 		= coadd_struct.scaled_mean_sky
	fn_mean_images 			= coadd_struct.fn_mean_images
	accum_ptr 				= PTR_NEW(dblarr(nx,ny))
	square_ptr				= PTR_NEW(dblarr(nx,ny))
	sigma_ptr				= PTR_NEW(dblarr(nx,ny))
	median_ptr				= PTR_NEW(dblarr(nx,ny))
endif else begin
	image_data_ptr 			= PTR_NEW(fltarr(nimages,nx,ny))
	product_data_mask_ptr 	= PTR_NEW(fltarr(nimages,nx,ny))
	sky_image 				= FLOAT(coadd_struct.sky_image)
	scale 					= FLOAT(coadd_struct.scale)
	scaled_mean_sky 		= FLOAT(coadd_struct.scaled_mean_sky)
	fn_mean_images 			= FLOAT(coadd_struct.fn_mean_images)
	accum_ptr 				= PTR_NEW(fltarr(nx,ny))
	square_ptr				= PTR_NEW(fltarr(nx,ny))
	sigma_ptr				= PTR_NEW(fltarr(nx,ny))
	median_ptr				= PTR_NEW(fltarr(nx,ny))
endelse
;
;	compute the sky sigma from the incoming sky values (not great, but inexpensive)
;
sky_cut = ROUND(5.0 *  coadd_struct.mean_sky_sigma)
;
image_count_ptr 			= PTR_NEW(intarr(nx,ny))
image_mask_ptr 				= PTR_NEW(intarr(nimages,nx,ny))
good_mask_ptr				= PTR_NEW(intarr(nimages,nx,ny))
is_sky_ptr					= PTR_NEW(intarr(nimages,nx,ny))
sky_ptr						= PTR_NEW(intarr(nx,ny))
star_ptr 					= PTR_NEW(intarr(nx,ny))
final_mask_ptr				= PTR_NEW(intarr(nx, ny))
;
(*image_data_ptr) += -1.0e6		; clear to deleted
;
;	check variable type of raw_images - whether pointer or not.
;
this_type = SIZE( coadd_struct.raw_images, /type)
if(this_type eq 10) then is_pointer = 1 else is_pointer = 0
;
for ii = 0, nimages-1 do begin
	;
	;	load scaled raw image data into data cube
	;
	x0 = coadd_struct.x_off[ind_good[ii]]
	x1 = x0+coadd_struct.naxis11
	y0 = coadd_struct.y_off[ind_good[ii]]
	y1 = y0+coadd_struct.naxis21
	;
	if(is_pointer) then begin
		(*image_data_ptr)[ii,x0:x1,y0:y1] = ( (*coadd_struct.raw_images)[ind_good[ii],*,*] - sky_image[ind_good[ii]]) * scale[ind_good[ii]]
	endif else begin
		(*image_data_ptr)[ii,x0:x1,y0:y1] = (coadd_struct.raw_images[ind_good[ii],*,*] - sky_image[ind_good[ii]]) * scale[ind_good[ii]]
	endelse
	;
endfor
;
;	and compute whether each pixel is looking at "sky" or not
;
(*is_sky_ptr) = (*image_data_ptr)
(*is_sky_ptr) *= -1
(*is_sky_ptr) += sky_cut
(*is_sky_ptr) = FLOOR( TEMPORARY ( (*is_sky_ptr) ) )
(*is_sky_ptr) += 1
(*is_sky_ptr) >= 0
(*is_sky_ptr) <= 1
;
;	old, memory hog version
;(*is_sky_ptr) = ((1 + FLOOR( sky_cut - (*image_data_ptr) ) ) > 0) < 1		; 1 is sky, 0 is non-sky
;
;	and test its neighbors, borrowing the good_mask_ptr array for now
;
xy_shift = [[0, -1, 0, 1, -2, -1, 1, 2, -1, 0, 1, 0], $		; x offsets
			[2, 1, 1, 1, 0, 0, 0, 0, -1, -1, -1, -2]]			; y offsets
for ixy = 0, 11 do begin
	(*good_mask_ptr) += SHIFT( (*is_sky_ptr), 0, xy_shift[ixy,0], xy_shift[ixy,1] )
endfor
;
if(coadd_struct.debug) then stop
;
;	keep as sky any pixel with surrounding count > 10
;
(*good_mask_ptr) -= 10
(*good_mask_ptr) <= 1
(*good_mask_ptr) >= 0				; 0 is not-sky, 1 is sky
;
;	final "sky" mask is the truncated good_mask_ptr
;
(*is_sky_ptr) = (*good_mask_ptr)
;
;	now that each image has "voted", compile the results
;
(*sky_ptr) = TOTAL( (*is_sky_ptr), 1)
threshold = FLOOR( 0.75 * nimages )
(*sky_ptr) -= threshold
(*sky_ptr) >= 0
(*sky_ptr) <= 1
;
(*star_ptr) = -1 * (*sky_ptr)
(*star_ptr) += 1
;
PTR_FREE, is_sky_ptr
;
if(coadd_struct.debug) then stop
;
;	compute the deleted pixel masks (original = "good", computed = "image")
;
(*good_mask_ptr) = (*image_data_ptr)
(*good_mask_ptr) += 50000
(*good_mask_ptr) >= 0
(*good_mask_ptr) <= 1
;
(*image_mask_ptr) += 0
(*image_mask_ptr) += (*good_mask_ptr)
;
;	do the median filtering (2.5 passes with 6 sigma cutoff)
;
sky_vec_scale = [2.25, 3.0, 3.5]
star_vec_scale = [3.0, 6.0, 35.0]
;
for ipass = 0, N_ELEMENTS(sky_vec_scale)-1 do begin
	;
;	this_scale = scale_start + step * float(ipass)
	this_sky_scale = sky_vec_scale[ipass]
	this_star_scale = star_vec_scale[ipass]
	;
	;	compute mean
	;
	(*product_data_mask_ptr) = (*image_data_ptr)
	(*product_data_mask_ptr) *= (*image_mask_ptr)
	;
	(*accum_ptr) 		= TOTAL( (*product_data_mask_ptr) ,1, DOUBLE=dp_flag, /NAN)
	(*image_count_ptr) 	= TOTAL( (*image_mask_ptr) ,1, DOUBLE=dp_flag, /NAN)
	(*accum_ptr) 		/= FLOAT( (*image_count_ptr) ) ; average
	;
	if(dp_flag) then begin
		(*square_ptr) 		= TOTAL(DOUBLE( (*image_data_ptr) ) * (*product_data_mask_ptr) ,1, /DOUBLE, /NAN)
		(*square_ptr)		/= FLOAT( (*image_count_ptr) )
		(*sigma_ptr) 		= SQRT( (*square_ptr) - DOUBLE ((*accum_ptr)) * (*accum_ptr) )
	endif else begin
		(*square_ptr) 		= TOTAL((*image_data_ptr) * (*product_data_mask_ptr) ,1, /NAN)
		(*square_ptr)		/= FLOAT( (*image_count_ptr) )
		(*sigma_ptr) 		= SQRT( (*square_ptr) - (*accum_ptr) * (*accum_ptr) )
	endelse
	(*sigma_ptr) 		*= ( this_star_scale * (*star_ptr) + this_sky_scale * (*sky_ptr) )
	(*sigma_ptr)		>= 1.0		; don't let fall below unity
	;
;	if(ipass eq npass-1) then stop
	;
	;	compute Median of non-deleted values at each pixel
	;
	for iy = 0, ny-1 do begin
		for ix = 0, nx-1 do begin
			;
			;	get vector of indices of non-deleted data
			;
			ind = WHERE( (*image_mask_ptr)[*,ix,iy], count)
			case count of
				0:		(*median_ptr)[ix,iy] = -1.0e6
				1:		(*median_ptr)[ix,iy] = (*image_data_ptr)[ind, ix, iy]
				else:	begin
							vec = (*image_data_ptr)[ind, ix, iy]
							(*median_ptr)[ix,iy] = d_median(vec)
						end
			endcase
		endfor
	endfor
	;
	;	recompute image mask (all within scale * sigma of median get 1, else get 0)
	;
	for i = 0, nimages-1 do (*image_mask_ptr)[i,*,*] = FLOOR( (abs( (*image_data_ptr)[i,*,*] - (*median_ptr))/(*sigma_ptr) ) )
	(*image_mask_ptr) <= 1
	(*image_mask_ptr) *= -1
	(*image_mask_ptr) += 1
	;
	;	remask with original deleted pixels that were already turned off
	;
	(*image_mask_ptr) *= (*good_mask_ptr)
endfor
;
;	final half pass
;
(*accum_ptr) 		= TOTAL( (*image_data_ptr) * (*image_mask_ptr),1, DOUBLE=dp_flag, /NAN)
(*image_count_ptr) 	= TOTAL( (*image_mask_ptr) ,1, DOUBLE=dp_flag, /NAN)
;
;	compute mask
;
(*final_mask_ptr) 	= (*image_count_ptr) < 1
;
(*accum_ptr) 		/= FLOAT( (*image_count_ptr) + 1 - (*final_mask_ptr) )
;
;					^-- average -------^         ^ deleted pixels divided by unity -^
;
PTR_FREE, image_data_ptr, product_data_mask_ptr, square_ptr
;
;	prepare output image
;
if(dp_flag) then begin
	output_image 			= dblarr(nx,ny)
endif else begin
	output_image 			= fltarr(nx,ny)
endelse
;
output_image = PTR_NEW( (*accum_ptr) )
;
;	add sky back in
;
(*output_image) += scaled_mean_sky
;
;	scale by number of mean images
;
(*output_image) *= fn_mean_images
;
;	mask out bad
;
(*output_image) *= (*final_mask_ptr)
(*output_image) += (1 - (*final_mask_ptr)) * (-1.0e6)
;
if(coadd_struct.debug) then stop
;
PTR_FREE, image_count_ptr, image_mask_ptr, good_mask_ptr, accum_ptr
PTR_FREE, sigma_ptr, median_ptr, final_mask_ptr
PTR_FREE, star_ptr, sky_ptr
;
;	check memory on exit
;
end_memory = MEMORY()
end_memory = end_memory[0]
dif_memory = end_memory - start_memory
if(dif_memory gt nx * ny * 12L ) then begin	; 3 single precion images, 1.5 double precision
	print, 'Memory leak - stopping'
	stop
endif
;
return, output_image
;
end

