pro S7_old_meta_group_coadding, Event, g_ind, group_name, LED_wid, status
	;
	;	handles coadding of meta_groups, using sub-image parsing if needed
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	o_s_f = (*w_ptr).S7_override_scaling_mode 	; = 0 normal, = 1 override scaling
	;
	status = -1
	bad_pixel_value = -1.0e6
	;
	flatten_pre_flag = (*w_ptr).S7_flatten_images_pre_flag
	flatten_post_flag = (*w_ptr).S7_flatten_images_post_flag
	fix_bad_pixels_flag = (*w_ptr).S7_fix_bad_pixels_flag
	low_order_flag = (*w_ptr).S7_low_order_flag
	;
	apply_illumination_flag = (*w_ptr).S7_apply_illumination_flag	; if set, compute illumination function and scale by it
	;
	double_precision_flag = 0
	print_text, Event,"Starting Coadding with DoublePrecisionFlag = "+STRING(double_precision_flag,format='(I1)')
	;
	h_wid = (*w_ptr).S7_history_textbox_wid
	;
	;	any comments to add?
	;
	comment_flag = (*w_ptr).S7_comments_mode
	if(comment_flag) then comment_str = (*w_ptr).S7_comments_string
	;
	astro_dir = (*s_ptr).analysis_dir + 'S3_Astrometry' + PATH_SEP()
	scale_dir = (*s_ptr).analysis_dir + 'S6_Scaling' + PATH_SEP()
	psf_dir = (*s_ptr).analysis_dir + 'S4_First_PSF_Fits' + PATH_SEP()
	;
	;	create name of summary scaling file for this meta-group
	;
	this_group = replace_underscores(group_name)
	;
	scale_path = scale_dir + this_group + '_meta_scale.dat'
	;
	;	check that it exists
	;
	test = FILE_INFO(scale_path)
	if(NOT(test.exists)) then begin
		str = group_name+" has no scaling summary file"
		print_text, Event, "ERROR - group "+str+" - ending"
		SAll_add_to_history, Event, h_wid, 7, str
		status = -1
		goto, S7_MGC_end
	endif
	;
	;	first time accessing file, determine how many lines in it
	;
	n_lines = FILE_LINES(scale_path)
	print_text, Event, "Opened Meta-group scale file "+FILE_BASENAME(scale_path)
	print_text, Event, " and found "+STRING(n_lines,format='(I)')+" lines (files+comments)"
	;
	;	prepare variables
	;
	file_names = strarr(n_lines)
	hwp_angles = fltarr(n_lines)
	scale_factors = dblarr(n_lines)
	x_offsets = intarr(n_lines)
	y_offsets = intarr(n_lines)
	;
	n_files = 0L
	;
	openr, lu_scale, scale_path, /GET_LUN
	line = ''
	;
	header = strarr(64,200)
	nheader = lonarr(64)
	;
	ngroups= -1L
	;
	WHILE ~EOF(lu_scale) do begin
		readf, lu_scale, line
		;
		;	pull off any line starting with semicolon into header
		;
		s_test = STRPOS(line, ';')
		if(s_test ne -1) then begin
			;
			;	if the word "gain" is present, increment the groups counter
			;
			if(STRPOS(line,'gain') ne -1) then begin
				ngroups++
				nheader[ngroups] = 0L
			endif
			;
			;	but skip lines with two semicolons or "; -"
			;
			two_semi = STRPOS(line, ';;')
			semi_dash = STRPOS(line, '; -')
			if(two_semi eq -1 and semi_dash eq -1) then begin
				if(ngroups ge 0) then begin
					header[ngroups,nheader[ngroups]] = STRMID(line, s_test + 1)
					nheader[ngroups]++
				;
				endif
			endif
			;
		endif else begin
			;
			;	test for "phot" in line - skip if missing
			;
			test = STRPOS(line,"phot")
			if(test ne -1) then begin
				;
				parts = STRSPLIT(line,' ',/extract)
				file_names[n_files] = psf_dir + parts[0]
				;
				reads, parts[1], hwp, format='(F)'
				hwp_angles[n_files] = hwp
				reads, parts[2], scale, format='(D)'
				scale_factors[n_files] = scale
				reads, parts[3], x_off, format='(F)'
				x_offsets[n_files] = ROUND(x_off)
				reads, parts[4], y_off, format='(F)'
				y_offsets[n_files] = ROUND(y_off)
				;
				;	cleck for reasonableness and limits
				;
				if(scale lt 0.6 or scale gt 1.4) then begin
					str = FILE_BASENAME(file_names[n_files])+" has scale = "+parts[2]
					print_text, Event, "Warning - Image "+str
					SAll_add_to_history, Event, h_wid, 7, str
				endif
				;
				n_files++
			endif
		endelse
	ENDWHILE
	FREE_LUN, lu_scale
	;
	ngroups++
	;
	print_text, Event, "Read in "+STRING(ngroups,format='(I2)')+" groups"
	;
	pp4_arad = dblarr(ngroups)
	pp4_ref0 = dblarr(ngroups)
	pp4_ref1 = dblarr(ngroups)
	pp4_clen = dblarr(ngroups)
	pp6_aprd = dblarr(ngroups)
	;
	for ig = 0, ngroups-1 do begin
		this_header = REFORM(header[ig,0:nheader[ig]-1])
		;
		pp4_arad[ig] = SXPAR(this_header, "PP4_ARAD")
		pp4_ref0[ig] = SXPAR(this_header, "PP4_REF0")
		pp4_ref1[ig] = SXPAR(this_header, "PP4_REF1")
		pp4_clen[ig] = SXPAR(this_header, "PP4_CLEN")
		pp6_aprd[ig] = SXPAR(this_header, "PP6_APRD")
	endfor
	;
	;	compute average properties of each
	;
	ave_arad = TOTAL(pp4_arad) / float(ngroups)
	ave_ref0 = TOTAL(pp4_ref0) / float(ngroups)
	ave_ref1 = TOTAL(pp4_ref1) / float(ngroups)
	ave_clen = TOTAL(pp4_clen) / float(ngroups)
	ave_aprd = TOTAL(pp6_aprd) / float(ngroups)
	;
	print_text, Event, "Ave ARAD, REF0, REF1, CLEN, APRD = "+$
		STRING(ave_arad, ave_ref0, ave_ref1, ave_clen, ave_aprd,format='(5(f7.3,1x))')
	;
	file_names = file_names[0:n_files-1]
	hwp_angles = hwp_angles[0:n_files-1]
	scale_factors = scale_factors[0:n_files-1]
	;
	;	evaluate against scaling override
	;
	ind_so = WHERE(scale_factors ne 0.0,n_scale)
	if(n_scale gt 0 and o_s_f eq 1) then begin
		scale_factors[ind_so] = 1.0
	endif else begin
		;
		;	otherwise, push all other lists down to match good scaling factors
		;
		n_to_skip = n_files - n_scale
		if(n_to_skip gt 0) then	print_text, Event,'Skipping '+STRING(n_to_skip)+' images with UseFlag = 0'
		x_offsets = x_offsets[ind_so]
		y_offsets = y_offsets[ind_so]
		file_names = file_names[ind_so]
		hwp_angles = hwp_angles[ind_so]
		scale_factors = scale_factors[ind_so]
		n_files = n_scale
	endelse
	;
	x_offsets = x_offsets[0:n_files-1]
	y_offsets = y_offsets[0:n_files-1]

	x_o_min = FLOOR(MIN(x_offsets))
	x_o_max = CEIL(MAX(x_offsets))
	y_o_min = FLOOR(MIN(y_offsets))
	y_o_max = CEIL(MAX(y_offsets))
	;
	d_o_x = x_o_max - x_o_min
	d_o_y = y_o_max - y_o_min
	;
	;	compute final image dimensions and astrometry information
	;
	first_file_name = file_names[0]
	ref_header = S7_read_phot_file_header(first_file_name)
	;
	crpix1 = sxpar(ref_header, "CRPIX1")
	crpix2 = sxpar(ref_header, "CRPIX2")
	naxis1 = sxpar(ref_header, "NAXIS1")
	naxis2 = sxpar(ref_header, "NAXIS2")
	crval1 = sxpar(ref_header, "CRVAL1")
	crval2 = sxpar(ref_header, "CRVAL2")
	;
	new_naxis1 = naxis1 + d_o_x
	new_naxis2 = naxis2 + d_o_y
	;
	new_crpix1 = crpix1 + (ROUND(x_offsets[0]) - x_o_min)		; this is where the old crpix1,2 will go in the
	new_crpix2 = crpix2 + (ROUND(y_offsets[0]) - y_o_min)		;    new image, if we keep the same old CRVAL1,2
	;
	new_x_center = float(new_naxis1) / 2.			; assuming IDL-like pixel coordinates...
	new_y_center = float(new_naxis2) / 2.			;   ("WWDS9T" - what would DS9 think???)
	;
	;	update header for new crpix1,2 values
	;
	SXADDPAR, ref_header, 'CRPIX1', new_crpix1
	SXADDPAR, ref_header, 'CRPIX2', new_crpix2
	;
	extast, ref_header, ref_astr_struct, noparams
	;
	;	compute ra, dec of new center
	;
	xy2ad, new_x_center, new_y_center, ref_astr_struct, ra_cen, dec_cen
	;
;	print, "old ra, dec center = ",crval1, crval2
;	print, "old crpix1, 2 = ",crpix1, crpix1
;	print, "xmin, xmax, dx = ", xmin, xmax, dx
;	print, "ymin, ymax, dy = ", ymin, ymax, dy
;	print, "new image size = ",new_naxis1, new_naxis2
;	print, "new crpix1, 2 = ",new_crpix1, new_crpix2
;	print, "final crpix1, 2 = ",new_x_center, new_y_center
;	print, "final ra, dec center = ",ra_cen, dec_cen
	;
	;	save new center ra, dec and new ref pixels
	;
	SXADDPAR, ref_header, 'CRPIX1', new_x_center
	SXADDPAR, ref_header, 'CRPIX2', new_y_center
	SXADDPAR, ref_header, 'CRVAL1', ra_cen
	SXADDPAR, ref_header, 'CRVAL2', dec_cen
	SXADDPAR, ref_header, 'NAXIS1', new_naxis1
	SXADDPAR, ref_header, 'NAXIS2', new_naxis2
	;
	;	amount by which a star's position should be incremented in X and Y
	;	from the pixel values in this image to find the approximate mean
	;	detector X,Y pixel values when detected
	;
	x_pix_offset = x_o_min - (float(TOTAL(x_offsets))/float(n_files))
	y_pix_offset = y_o_min - (float(TOTAL(y_offsets))/float(n_files))
	;
	SXADDPAR, ref_header, 'MPP_XOFF', x_pix_offset, 'Add this to X pix to get detector X'
	SXADDPAR, ref_header, 'MPP_YOFF', y_pix_offset, 'Add this to Y pix to get detector Y'
	SXADDPAR, ref_header, 'HISTORY', 'Created in MSP_PPOL Step 7 (meta-group coadding) on '+SYSTIME()
	;
	;	if comment flag set, add comments
	;
	if(comment_flag) then begin
		SXADDPAR, ref_header, 'HISTORY', comment_str
	endif
	;
	;	add info from Step 6 to pass onto Step 8
	;
	SXADDPAR, ref_header, 'PP4_ARAD', ave_arad,'Maximum Aper Size, in FWHM units'
	SXADDPAR, ref_header, 'PP4_REF0', ave_ref0,'Inner ref radius, in FHWM * ARAD units'
	SXADDPAR, ref_header, 'PP4_REF1', ave_ref1,'Outer ref radius, in FHWM * ARAD units'
	SXADDPAR, ref_header, 'PP4_CLEN', ave_clen,'Outer cleaning radius, in FHWM * ARAD units'
	SXADDPAR, ref_header, 'PP6_APRD', ave_aprd,'Aperture size in FWHM * ARAD units'
	;
	;	and reset X, Y image offsets to be referenced to 0,0
	;
	x_offsets -=  x_o_min
	y_offsets -=  y_o_min
	;
	;	Reset LED to reflect 17 sublists
	;
	WIDGET_CONTROL, LED_wid, SET_UVALUE=17
	image_colors = strarr(17)
	image_colors[*] = "red"
	S7_LED_sensitize, Event, 1, 1
	;
	;	parse into the 16 lists
	;
	hwp_values = [0,22.3,45.2,67.5,89.8,112.7,135.0,157.3,180.2,202.5,224.8,247.7,270.0,$
					292.3,315.2,337.5]
	;
	hwp_file_list = lonarr(16,n_files)
	hwp_list_n = lonarr(16)
	;
	;	compare each HWP to the list, finding the one closest (in case steps are
	;			bumped in the LOIS code...)
	;
	for ifile = 0, n_files-1 do begin
		dhwp = abs(hwp_angles[ifile] - hwp_values[*])
		min_dhwp = MIN(dhwp)
		ind = WHERE(dhwp eq min_dhwp and dhwp lt 5.0,count)		; must find one within 2 deg
		if(count eq 1) then begin
			hwp_file_list[ind,hwp_list_n[ind]] = ifile		; as a reference number
			hwp_list_n[ind]++
		endif else begin
			if(count ne 0) then begin
				str = "Multiple HWP angle matches for "+file_names[ifile]
				print_text, Event, "ERROR - "+str
				SAll_add_to_history, Event, h_wid, 7, str
			endif else begin
				str = "No HWP angle match for "+file_names[ifile]
				print_text, Event, "ERROR - "+str
				SAll_add_to_history, Event, h_wid, 7, str
			endelse
		endelse
	endfor
	;
	fn_mean_images = DOUBLE(CEIL(MEDIAN(hwp_list_n)))
	;
	print_text,Event, "fn_mean_images = "+STRING(fn_mean_images,format='(F6.1)')
	;
	SXADDPAR, ref_header, 'MPP_NSCL', fn_mean_images, 'Number of images this one is scaled to'
	;
;	for ihwp = 0, 15 do begin
;		print, "Found ",hwp_list_n[ihwp]," files with HWP = ",hwp_values[ihwp]
;	endfor
	;
	print_text, Event, "All HPW angles parsed"
	;
	hwp_coadd_paths = strarr(16)
	;
	;----------------------------- coadd 16 HWP images --------------------------------
	;
	;
	nchunks = 1
	dy_chunk = new_naxis2
	old_new_naxis2 = new_naxis2
	;
	hwp_nchunks = intarr(16)
	hwp_dy_chunk = intarr(16)
	;
	if(double_precision_flag eq 1) then nsplit = 4 else nsplit = 8
	;
	for ihwp = 0, 15 do begin
		ind_list = REFORM(hwp_file_list[ihwp,0:hwp_list_n[ihwp]-1])
		;
		n_images = N_ELEMENTS(ind_list)
		;
		;	get the nominal nsplit value, then decrement it for large numbers of images
		;
		this_nsplit = nsplit
		for ispl = 0, 5 do begin
			if(n_images gt 2^(4+ispl)) then this_nsplit = ((this_nsplit - 1) > 1)
		endfor
		;
		;	test for whether we need to deal with memory management
		;
		if(n_images gt this_nsplit) then begin
			;
			;	must parse calculation into multiple chunks of the output image
			;
			nchunks = CEIL(float(n_images) / float(this_nsplit))
;			dy_chunk = CEIL(float(new_naxis2)/float(nchunks))
			;
;;			new_naxis2 >= (dy_chunk * nchunks ) 	; take the larger - DPC 20100516 turned off - worried about round off errors
			;
		endif else begin
			nchunks = 1
;			dy_chunk = new_naxis2
		endelse
		;
		hwp_nchunks[ihwp] = nchunks
	endfor
	;
	for ihwp = 0, 15 do hwp_dy_chunk[ihwp] = CEIL(float(new_naxis2) / float(hwp_nchunks[ihwp]))
	;
	if(old_new_naxis2 ne new_naxis2) then SXADDPAR, ref_header, 'NAXIS2', new_naxis2
	;
;	stop
	;
	;	for every HWP case
	;
	for ihwp = 0, 15 do begin
;	for ihwp = 0, 0 do begin			; test case
		;
		nchunks = hwp_nchunks[ihwp]
		dy_chunk = hwp_dy_chunk[ihwp]
		;
		;	set LED1 to blue
		;
		this_LED = ihwp
		image_colors[this_LED] = "blue"
		WCT_LED_stack, LED_wid, image_colors
		;
		ind_list = REFORM(hwp_file_list[ihwp,0:hwp_list_n[ihwp]-1])
		;
		x_off = x_offsets[ind_list]
		y_off = y_offsets[ind_list]
		scale = scale_factors[ind_list]
		;
;		print, "Scale Factors = ",scale
		;
		;	get the vector of names
		;
		image_names = FILE_BASENAME(file_names[ind_list])
		;
		;	and convert back to FITS image names
		;
		n_images = N_ELEMENTS(image_names)
		;
		print_text, Event, "Beginning Coadding (Meta) of "+STRING(n_images,format='(I3)')+" images for HWP position "+STRING(ihwp+1,format='(I2)')
		;
		for i_image = 0, n_images-1 do begin
			parts = STRSPLIT(image_names[i_image],"_phot.dat",/REGEX,/EXTRACT)
			image_names[i_image] = parts[0] + '.fits'
			;
			;	set up its FITS header
			;
			if(i_image eq 0) then begin
				first_image_path = astro_dir + image_names[0]
				this_header = ref_header							; DPC 20090702
			endif
			;
			SXADDPAR, this_header, 'HISTORY', 'includes '+image_names[i_image]
			SXADDPAR, this_header, 'HISTORY', '  with X,Y offsets '+STRING(x_off[i_image],y_off[i_image],$
										format='(I3,1x,I3)')+" pixels"
			SXADDPAR, this_header, 'HISTORY', '  and multiplied by scale factor '+STRING(scale[i_image],$
										format='(F13.8)')
			SXADDPAR, this_header, 'MPP_NADD', n_images, 'Number of images coadded into this one'
		endfor
		;
		image_path = astro_dir + image_names
		;
		print_text, Event, "Computing final image as "+STRING(nchunks,format='(I)')+" sections"
		;
		yborder = new_naxis2 - naxis2
		xborder = new_naxis1 - naxis1
		;
		if(double_precision_flag) then output_image = PTR_NEW(dblarr(new_naxis1, new_naxis2)) else $
									   output_image = PTR_NEW(fltarr(new_naxis1, new_naxis2))
		;
		naxis11 = naxis1-1
		naxis21 = naxis2-1
		;
		;	for nchunks repeats through the data...
		;
		for ichunk = 0, nchunks-1 do begin
			;
			if(nchunks gt 1) then print_text, Event, "Meta Coadding - Image Section number "+STRING(ichunk+1,format='(I)') $
				else print_text, Event, "Meta Coadding - full image without sectioning"
			;
			;	starting, ending output Y values
			;
			ymin_out = dy_chunk * ichunk
			ymax_out = (new_naxis2-1) < (ymin_out + dy_chunk - 1)
			;
			;	where to find the data on raw images (same y's + up to yborder)
			;		- note these turn into y values of 0->ny_chunk-1 when mapped into raw_images array
			;
			ymin_chunk = (ymin_out - yborder) > 0
			ymax_chunk = (naxis2-1) < (ymax_out)
			ny_chunk = ymax_chunk - ymin_chunk + 1
			;
			;	where to copy the data from in the coadded image to the output image
			;	(up yborder from ends), if yborder fully used
			;
			if(ichunk eq 0) then begin
				ymin = 0
				ymax = ny_chunk - 1
			endif else begin
				if(ichunk ne (nchunks-1)) then begin
					ymin = yborder
					ymax = ny_chunk - 1
				endif else begin
					ymin = yborder
					ymax = ny_chunk - 1 + ymin
				endelse
			endelse
			;						; y limits for reading out of this data chunk
			ny = ymax - ymin + 1
			;
;			if(ny_chunk ne ny) then stop
			;
			if(double_precision_flag) then raw_images = PTR_NEW(dblarr(n_images, naxis1, ny_chunk)) else $
									       raw_images = PTR_NEW(fltarr(n_images, naxis1, ny_chunk))
			(*raw_images) += -1.0e6
			;

			;
;			if(double_precision_flag) then output_chunk = dblarr(new_naxis1, ny+yborder) else $
;									       output_chunk = fltarr(new_naxis1, ny+yborder)
			;
			print_text, Event, "Ymin_chunk, Ymax_chunk, ny_chunk, Ymin, Ymax, Ny, Ymin_out, Ymax_out = "+$
					STRING(ymin_chunk, ymax_chunk, ny_chunk, ymin, ymax, ny, ymin_out, ymax_out, format='(8(I5,2x))')
			;
			;	fetch the raw images, shift by their offsets
			;
			sky_image = dblarr(n_images)
			sky_sigma = dblarr(n_images)
			;
			n_images_read = 0
			for i_image = 0, n_images-1 do begin
				if(i_image eq 2*fix(i_image/2)) then image_colors[this_LED] = "orange" $
												else image_colors[this_LED] = "blue"
				;
				WCT_LED_stack, LED_wid, image_colors
				;
				;	compute pathnames to Astrometry directory
				;
				;	flatten if flag set
				;
				sky_image[i_image] = 0.0d0
				sky_sigma[i_image] = 0.0d0
				;
				if(flatten_pre_flag eq 0) then begin
					if(low_order_flag eq 0) then begin
						raw_chunk = readfits(image_path[i_image], header, /SILENT)
;						print_text, Event, "Read image "+FILE_BASENAME(image_path[i_image])
						n_images_read++
					endif else begin
						raw_chunk = readfits(image_path[i_image], header, /SILENT)
						raw_chunk = model_background(raw_chunk,SKY=this_sky, SIGMA=this_sigma, /LOW_ORDER)
;						print_text, Event, "Image "+FILE_BASENAME(image_path[i_image])+" has sky = "+STRING(this_sky)
						sky_image[i_image] = this_sky
						sky_sigma[i_image] = this_sigma
						n_images_read++
					endelse
				endif else begin
					raw_chunk = readfits(image_path[i_image], header, /SILENT)
					raw_chunk = model_background(raw_chunk,SKY=this_sky,SIGMA=this_sigma)
;					print_text, Event, "Image "+FILE_BASENAME(image_path[i_image])+" has sky = "+STRING(this_sky)
					sky_image[i_image] = this_sky
					sky_sigma[i_image] = this_sigma
					n_images_read++
				endelse
				;
				input_size = SIZE(raw_chunk)
				xis = input_size[1]
				yis = input_size[2]
				raw_chunk[0:2,*] = -1.0e6
				raw_chunk[*,0:6] = -1.0e6
				raw_chunk[xis-3:xis-1,*] = -1.0e6
				raw_chunk[*,yis-14:yis-1] = -1.0e6
				;
				(*raw_images)[i_image,*,*] = -1.0e6
				(*raw_images)[i_image,*,*] = raw_chunk[*,ymin_chunk:ymax_chunk]			; select this portion of the input image
			endfor
			;
			print_text, Event, "Read in "+STRING(n_images_read,format='(I3)')+" images"
			;
			;	at this point, the input images should be lined up with output image
			;
			;	compute mean sky value for ALL images with non-zero scale factors
			;
			ind_sky = WHERE(scale ne 0.0,ngoodsky)
			if(ngoodsky gt 0) then begin
;				zz = median_filtered_mean(sky_image[ind_sky]*scale[ind_sky])
;				scaled_mean_sky = zz[0]
				scaled_mean_sky = TOTAL(sky_image[ind_sky] * scale[ind_sky]) / float(ngoodsky)
				mean_sky_sigma = TOTAL(sky_sigma[ind_sky] * scale[ind_sky]) / float(ngoodsky)
			endif else mean_sky = 0.0
			;
;			print, "For ",ngoodsky," non-zero scaled skies out of ",n_images," possible, scaled_mean_sky = ",scaled_mean_sky
			;------------------ compute output image --------------------------
			;
			;	load structure for passing to coadder
			;
			;if(ihwp eq 0) then debug = 1 else debug = 0
			coadd_debug = 0
			;
			;	replace full Y-axis extent with chunk extent
			;
			naxis2_chunk = ny_chunk
			new_naxis2_chunk = ny_chunk+yborder
			;
			print_text,Event,"Naxis21, New_naxis2 = "+STRING(naxis2_chunk, new_naxis2_chunk)
			;
;			stop
			;
			coadd_struct = { new_naxis1:	new_naxis1,		$	x-axis of output image
;							 new_naxis2: 	new_naxis2, 	$	y-axis
							 new_naxis2:	new_naxis2_chunk, $
							 naxis11:		naxis1-1, 		$
;							 naxis21:		naxis2-1, 		$
							 naxis21:		naxis2_chunk-1, $
							 n_images:	n_images, 			$	number of constituent images
							 x_off:		x_off,				$
							 y_off:		y_off, 				$
							 raw_images:	(*raw_images),			$	the input data cube
							 sky_image:		sky_image,		$
							 scale:		scale,				$
							 double_precision_flag: double_precision_flag, $
							 fn_mean_images: fn_mean_images, $
							 mean_sky_sigma: mean_sky_sigma, $
							 debug: coadd_debug, 			$
							 scaled_mean_sky:scaled_mean_sky }
			;
			this_output_chunk = PTR_NEW(S7_coadd(coadd_struct))
			;
			;	load into output image
			;
			(*output_image)[*,ymin_out:ymax_out] = (*this_output_chunk)[*,ymin:ymax]
			;
			PTR_FREE, this_output_chunk
			PTR_FREE, raw_images
			;
		endfor
		;
		;	create file name and write to disk
		;
		output_name = replace_underscores(group_name) + "_HWP_" + STRTRIM(STRING(ihwp,format='(I2.2)'),2) + '.fits'
		output_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
		output_path = output_dir + output_name
		;
		hwp_coadd_paths[ihwp] = output_path
		;
		SXADDPAR, this_header, 'FILENAME', output_name, 'Created by MSP_PPOL Step 7'
		SXADDPAR, this_header, 'PPOL_HWP', hwp_values[ihwp], 'HWP angle in degrees, from PPOL'
		SXADDPAR, this_header, 'MB_HWP', hwp_values[ihwp],'Updated HWP angle from PPOL S7'
		;
		;	get current JD and add one min for each HWP angle
		;
		date_obs = FXPAR(this_header, "DATE-OBS")
		reads, date_obs, year, month, day, hour, minute, second, format='(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f4.2)
		jd = JULDAY(Month, Day, Year, 0, ihwp, 0)
		SXADDPAR, this_header, 'MB_JD', jd, "JD manufactured by PPOL S7"
		;
		;	if apply_illumination flag set, find model background and divide by scaled background shape
		;		(DPC 20091128)
		;
		if(apply_illumination_flag eq 1) then begin
			flat_image = PTR_NEW(model_background( (*output_image) ,SKY=sky))
			model_image = PTR_NEW(flat_image * 0.0)
			;
			out_good_flag = PTR_NEW( (*output_image) + 0.5e6 )
			(*out_good_flag) >= 0.0
			(*out_good_flag) <= 1.0
			out_bad_flag = PTR_NEW(1.0 - (*out_good_flag))
			;
			(*model_image) = (( (*output_image) - (*flat_image)) + sky) * (*out_good_flag) + (*out_bad_flag)
			;
			;	scale the model to its mean sky value and then divide (correct) output image for this
			;	illumination pattern
			;
			(*output_image) /=  ( (*model_image) / sky)
			(*output_image) += (1.0 - 1.0e6) * (*out_bad_flag)
			SXADDPAR, this_header, 'HISTORY', 'Corrected for illumination, with mean sky = '+STRING(sky)
			;
			PTR_FREE, flat_image, model_image, out_good_flag, out_bad_flag
		endif
		;
		;	if flatten_post flag set, flatten image
		;
		if(flatten_post_flag eq 1) then begin
			(*output_image) = model_background( (*output_image), SKY=sky)
			SXADDPAR, this_header, 'HISTORY', 'Post Coadded Flattened, with mean sky = '+STRING(sky)
		endif
		;
		;	if flag set, fix bad pixels remaining in image
		;
		if(fix_bad_pixels_flag eq 1) then begin
			(*output_image) = fix_bad_pixels( (*output_image) )
			SXADDPAR, this_header, 'HISTORY', 'Post coadding, fixed remaining bad pixels'
		endif
		;
		writefits, output_path, (*output_image), this_header
		;
		print_text, Event, "Wrote coadded image "+output_name
		;
		image_colors[this_LED] = "green"
		WCT_LED_stack, LED_wid, image_colors
		;
		PTR_FREE, output_image
	endfor
	;
;	stop
	;
;	if(ihwp eq 1) then goto, S7_MGC_end
	;
	;-------------------------- deep coadd ---------------------------------------------
	;
	;	reserve room, then read images
	;
	this_LED = 16
	image_colors[this_LED] = "blue"
	WCT_LED_stack, LED_wid, image_colors
	;
	if(double_precision_flag) then begin
		raw_images = PTR_NEW(dblarr(16,new_naxis1, new_naxis2))
		(*raw_images) += -1.0d6
		output_image = PTR_NEW(dblarr(new_naxis1, new_naxis2))
		(*output_image) += -1.0d6
	endif else begin
		raw_images = PTR_NEW(fltarr(16,new_naxis1, new_naxis2))
		(*raw_images) += -1.0e6
		output_image = PTR_NEW(fltarr(new_naxis1, new_naxis2))
		(*output_image) += -1.0e6
	endelse
	;
	n_total_files = 0L
	n_total_scaled = 0L
	sky_image = dblarr(16)
	sky_sigma = dblarr(16)
	for ihwp = 0, 15 do begin
		;
		;	flatten if flag set
		;
		sky_image[ihwp] = 0.0d0
		sky_sigma[ihwp] = 100.0
		;
		if(flatten_pre_flag eq 0) then begin
			(*raw_images)[ihwp,*,*] = readfits(hwp_coadd_paths[ihwp],header,/SILENT)
		endif else begin
			(*raw_images)[ihwp,*,*] = model_background(readfits(hwp_coadd_paths[ihwp],header,/SILENT),SKY=this_sky,SIGMA=this_sigma,/LOW_ORDER)
			sky_image[ihwp] = this_sky
			sky_sigma[ihwp] = this_sigma
		endelse
		;
		n_this_files = sxpar(header, 'MPP_NADD')
		n_total_files = n_total_files + n_this_files		; update total file count
		n_scaled = sxpar(header, 'MPP_NSCL')
		n_total_scaled += ROUND(n_scaled)
		if(ihwp eq 0) then first_header = header
		SXADDPAR, first_header, 'HISTORY', 'deep coadd includes '+FILE_BASENAME(hwp_coadd_paths[ihwp])
	endfor
	;
	print_text, Event, "Beginning coadding of Deep image"
	;
	zz_sky = median_filtered_mean(sky_image)
	mean_sky = zz_sky[0]
	;
	ss = median_filtered_mean(sky_sigma)
	mean_sky_sigma = ss[0]
	;
	;---------------------------------------------------------
	x_off = fltarr(16)
	y_off = fltarr(16)
	naxis1 = new_naxis1
	naxis2 = new_naxis2
	;
	n_images = 16
	scale = fltarr(16)
	scale[*] = 1.0
	;
	fn_mean_images = 16.0
	scaled_mean_sky = mean_sky
	debug = 0
	;
	coadd_struct = { new_naxis1:	new_naxis1,	$	x-axis of output image
					 new_naxis2: 	new_naxis2, 	$	y-axis
					 naxis11:		naxis1-1, $
					 naxis21:		naxis2-1, $
					 n_images:	n_images, 	$	number of constituent images
					 x_off:		x_off,		$
					 y_off:		y_off, 		$
					 raw_images: (*raw_images),	$	the input data cube
					 sky_image:		sky_image,		$
					 scale:		scale,		$
					 double_precision_flag: double_precision_flag, $
					 fn_mean_images: fn_mean_images, $
					 mean_sky_sigma: mean_sky_sigma, $
					 debug: debug, $
					 scaled_mean_sky:scaled_mean_sky }

	(*output_image) = S7_coadd(coadd_struct)
	;
	PTR_FREE, raw_images
	;
	;	output name
	;
	output_name = replace_underscores(group_name) + '_Deep_Coadd.fits'
	output_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
	output_path = output_dir + output_name
	;
	SXADDPAR, first_header, 'FILENAME', output_name, 'Created by MSP_PPOL Step 7'
	SXADDPAR, first_header, 'MPP_DADD', n_total_files, 'total number of files in this deep coadd'
	SXADDPAR, first_header, 'MPP_NSCL', n_total_scaled, 'Number of images this sum is scaled to'
	;
	;	flatten and fix bad pixels, if flags set
	;
	if(flatten_post_flag eq 1) then (*output_image) = model_background( (*output_image) )
	if(fix_bad_pixels_flag eq 1) then (*output_image) = fix_bad_pixels( (*output_image) )
	;
	writefits, output_path, (*output_image), first_header
	;
	print_text, Event, "Wrote deep coadded image "+output_name
	;
	PTR_FREE, output_image
	;
	image_colors[this_LED] = "plum"
	WCT_LED_stack, LED_wid, image_colors
	;------------------------------------------- redo astrometry --------------------
	deep_image = readfits(output_path, header)
	;
	;	this needs to be updated for use with F17 images...
	;
	f17 = 0
	tolerance = 0.0005
	platescale_est = 0.579
	;
	print_text, Event, "Starting revised astrometry determination"
	deep_header = ppol_astrometry(Event, output_path, deep_image, header, return_flag, platescale_est, tolerance,$
							F17=f17, ARRAY_2MASS=array_2mass, TWO_MASS=two_mass, $
							FILE_2MASS=mass_array)
	;
	;	rewrite deep image with updated header
	;
	print_text, Event, "Resaving deep image with updated astrometry header"
	writefits, output_path, deep_image, deep_header
	;
	;-------------------------------------------------------------------------------
	;
	image_colors[this_LED] = "green"
	WCT_LED_stack, LED_wid, image_colors
	;
	status = 0
	S7_MGC_end:
end