pro S8_Deep_PSF_photometry, Event, deep_path, LED1_wid, group_colors, n_stars, phot_struct, $
							x_psf, y_psf, n_xy_psf, rad_PSF, status, phot_header, $
							PSF_model														; added 20150812 DPC
	;
	;	performs variable PSF-fitting photometry of one Deep coadded image
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_Struct
	d_ptr = PTR_Struct.PTR_Data
	s_ptr = PTR_Struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	;
	;
	extra_noise = 0.045							; photometric uncertainty floor - 20120502
	;
	;
	region_flag = (*w_ptr).S8_save_regions_flag
	analysis_dir = (*s_ptr).analysis_dir
	step_dir_text = 'S8_Deep_PSF_Fitting'
	;
	h_wid = (*w_ptr).S8_history_textbox_wid
	nstar_wid = (*w_ptr).S8_nstars_textbox_wid
	;
	method = 'Aper'							; method of final photometry: "Aper" or "PSF"
	;
	flatten_pre_flag = (*w_ptr).S8_flatten_images_flag
	calibrator_flag = (*w_ptr).S8_calibrator_flag	; if set, shut down extra bump for merging stars
	;
	group_colors[0] = 'pink'
	WCT_LED_stack, LED1_wid, group_colors
	;
	;	initial number of grid zones, sigma level
	;
	ng_start = (*w_ptr).S8_textbox_value[1]		; original global values
	ng = ng_start
	nsig_start = (*w_ptr).S8_textbox_value[0]
	nsig = float(nsig_start)
	;
	;	load the "group" buffers and "start" buffers
	;
	(*w_ptr).S8_textbox_group_value[*] = (*w_ptr).S8_textbox_value[*]
	(*w_ptr).S8_textbox_group_start[*] = (*w_ptr).S8_textbox_value[*]
	;
;	phot_region_file = 1			; 1 to save a region file for all found stars; 0 to skip
	;
	;	for these coadded images, the PSF-creation sat limit is 9000 ADU per image in the coadd,
	;		and for final fitting = 11000 ADU per image
	;
	PSF_sat_limit =  7500L			; 8000 led to ring of death around psfs
	;
	;PSF_sat_limit = 13000L			; test for bright star problems - 20120531 DPC
	;
	FIT_sat_limit = 13000L
	;
	;	save residual image?
	;
	resid_flag = (*w_ptr).S8_residuals_mode
	save_intermediate_images = resid_flag			; 1 to save some debugging images; 0 for normal operations
	;
	simple_PSF_assist = 1							; 1 for no tight-groups, 0 to enable tight-group simultaneous fitting
	;
	debug = 0
	;
	start_of_time = SYSTIME(/SECONDS)
	;
	;	also, no maximum number of stars
	;
	max_stars = 1.0e10
	max_in_group = 150			; maximum number of stars in one group
	;
	;	read in the image
	;
	image_data = readfits(deep_path, image_header, /SILENT)
	image = image_data										; leave "image" alone to retain bad pixels
	;
	header = image_header
	;
	cd, current=old_directory
	if(save_intermediate_images eq 1) then begin
		file_out_image = dialog_pickfile(title='Original Image before star removal')
		writefits, file_out_image, image_data, image_header
		starting_directory = FILE_DIRNAME(file_out_image)
		cd, starting_directory
	endif
	;
	if(flatten_pre_flag ne 0) then begin
		image_data = model_background(image_data)
		if(save_intermediate_images eq 1) then begin
			file_out_flat = dialog_pickfile(title='Image after background flattening')
			writefits, file_out_flat,  find_image, image_header
		endif
	endif
	;
	;	get image size
	;
	image_size = SIZE(image_data)
	xsize = image_size[1]
	ysize = image_size[2]
	;
	;	and offsets relative to the detector
	;
	xoff = abs(sxpar(image_header, 'MPP_XOFF'))
	yoff = abs(sxpar(image_header, 'MPP_YOFF'))
	;
	;	and number of images that made up this coadd
	;
	n_total_images = long(sxpar(image_header,"MPP_NSCL"))
	;
	;	if none, try another variable
	;
	if(n_total_images le 0) then n_total_images = long(sxpar(image_header,"MPP_DADD"))
	;
	if(n_total_images le 0) then begin
		status = -1
		str = "Cannot determine N_TOTAL_IMAGES for "+FILE_BASENAME(deep_path)
		print_text, Event, "ERROR - "+str
		SAll_add_to_history, Event, h_wid, 8, str
		goto, S8PSFF_end
	endif
	;
	;	also, look to see if this is a set of images taken with COADDS non unity
	;
	coadds = SXPAR(image_header, 'NUMCOADD')
	;
	if(coadds eq 0) then coadds = 1
	;
	PSF_sat_limit_total = double(PSF_sat_limit * n_total_images * coadds)	; because coadds come in as summed images
	FIT_sat_limit_total = double(FIT_sat_limit * n_total_images * coadds)
	;
	;	recover aperture photometry constants
	;
	PP4_ARAD = SXPAR(header, "PP4_ARAD")
	PP4_REF0 = SXPAR(header, "PP4_REF0")
	PP4_REF1 = SXPAR(header, "PP4_REF1")
	PP4_CLEN = SXPAR(header, "PP4_CLEN")
	PP6_APRD = SXPAR(header, "PP6_APRD")
	;
	ap_rad = PP4_ARAD * PP6_APRD			; times FWHM
	clean_rad = PP4_CLEN					; times ap_rad times FWHM
	ref_ap = [PP4_REF0,PP4_REF1]			; times ap_rad times FWHM
	;
	;	we'll skip half the overlap regions for star fitting
	;
	if(xsize gt 1024) then xborder = CEIL((xsize - 1024)/1.5) else xborder = 4
	if(ysize gt 1026) then yborder = CEIL((ysize - 1026)/1.5) else yborder = 9
	;
	;	extract astrometry info, if present
	;
	extast,header,astr,noparams			;  if noparams = -1, no astrometry present
	;
	center_in_textbox, nstar_wid, 'Build PSF grid'
	group_colors[0] = 'plum'
	WCT_LED_stack, LED1_wid, group_colors
	;
	;-------------------------------------------------------------------------------------------------
	;		constants to pass
	;
	sat_limit = PSF_sat_limit_total
	;
	sigma 				= 8.0								; set high threshold for PSF star selection
	save_region_files 	= region_flag						; 0 - none, 1 - all
	;
	fix_bad_pixels_flag = 0									; don't fix coadded S8 images
	;
	ap_vec 				= [0.6,0.65,0.7,0.75,$
						   0.8,0.85,0.9,0.95,$
						   1.0,1.05,1.1,1.15]			; times FWHM times ap_rad
	n_ap 				= N_ELEMENTS(ap_vec)
	;
	fit_scale 			= 1.75					; times fwhm
	rad_scale 			= 3.00					; times fwhm + 3
	;
	ronois				= 3.1 * sqrt(n_total_images)	; effective readout noise (in ADU)
	;
	step_dir_text 		= 'S8_Deep_PSF_Fitting'
	ng_textbox_wid 		= (*w_ptr).S8_ng_textbox_wid
	nsig_textbox_wid 	= (*w_ptr).S8_nsig_textbox_wid
	analysis_dir		= (*s_ptr).analysis_dir
	image_pathname		= deep_path
	;
	this_sig_mag_cut			= 0.75					; keep only mags with uncertainties smaller
	;
	;----------------------------------------------------------------------------------------------
	g_create_PSF_grid, Event, image_data, image_header, image_pathname, sigma, status, $
				NG=ng, SAT_LIMIT=sat_limit, skysig, mean_FWHM, find_image, xborder, $
				yborder, sky, clean_rad, ap_rad, phpadu, apr, skyrad, badpix, $
				bright_limit, rad_PSF, fit_rad, PSF_model, PSF_gauss, PSF_mag, $
				ronois, this_ng, n_ap, ap_vec, ref_ap, rad_scale, n_total_psf_stars, $
				x_psf, y_psf, mag_psf, save_region_files, flatten_pre_flag, $
				fix_bad_pixels_flag, fit_scale, ng_textbox_wid, nsig_textbox_wid, $
				step_dir_text, analysis_dir
				;
	;---------------------------------------------------------------------------------------------
	;
	find_image = image								; recover original image
	;
	;	override ng (but not nsig) with return value
	;
	ng = this_ng
	;
	(*w_ptr).S8_textbox_group_value[1] = ng				; updated "group" values here for passing to HWP fits
	(*w_ptr).S8_textbox_group_value[0] = nsig_start		;
	;
	;	also update the "start" values with the result of this deep coadd photometry
	;
	(*w_ptr).S8_textbox_group_start[1] = ng
	(*w_ptr).S8_textbox_group_start[0] = nsig_start
	;
	nsig = float(nsig_start)
	pass2_sigma 		= nsig
	short_ap_vec		= [0.5,0.6,0.7]* mean_FWHM * ap_rad		; for aperture phot later - these are good, central values
;	skyrad 				= ref_ap * mean_FWHM * ap_rad
	skyrad 				= [0.8,2.1] * mean_FWHM * ap_rad
	;
	;	load the "group" buffers
	;
	(*w_ptr).S8_textbox_group_value[*] = (*w_ptr).S8_textbox_value[*]
	;
	S8_Show_Options_Textbox, Event, 1			; optional final argument overrides common block look up
	S8_Show_Options_Textbox, Event, 0
	;
	;-----------------------------------------------------------------------||
	;
	center_in_textbox, nstar_wid, 'PSF grid ready'
	group_colors[0] = 'skyblue'
	WCT_LED_stack, LED1_wid, group_colors
	;
	psf_build_time = SYSTIME(/seconds) - start_of_time
	;
;	stop
	;
	;---------------------------------------------------------------------------------------------------
	;
	;	at this point, the PSFs have been built and their 2D functional forms determined
	;
	;	now, build the full star list using PSF fitting photometry:
	;
	;		1. find all stars to indicated limit
	;		2. using DAO grouping, group stars
	;		3. fit and remove stars (moving fitted stars to star list)
	;		3' smooth the residual image and subtract the smoothed from the unsmoothed
	;		4. look at the difference image, and do another find to the indicated sigma limit
	;		5. using DAO grouping, group stars
	;		6. fit and remove stars (move to star list)
	;		7. sort star list, remove dupes
	;
	;
	;		8. for each star on the star list:
	;			8.1. select all stars in list within 10 FWHMs, call these a group
	;			8.2. fit and remove all these stars (including central star) to get better photometry
	;			8.3. for image, fit and remove all *except* the target star
	;			8.4. do PSF fitting photometry on the target star
	;			8.5. move the target star photometry to the final phot list
	;			8.6. go on to next star on list
	;		9. when done, write to PHOTCAT and pass star list up for HWP fitting
	;
	;--------------------------------------------------------------------------------------------------
	;
	badpix = [-1.0e4,1.4*FIT_sat_limit_total]
	bright_limit = 1.4*FIT_sat_limit_total						; to halt bright/saturated stars from being fit
	;
	start_list_time = SYSTIME(/seconds)
	;
	;	Step 1: Find all stars below the FIT sat limit
	;-----------------
	g_sky, find_image, skymode, skysig, mfm_mean, mfm_stdev, SILENT=(1-debug)
	;
	print_text, Event, 'PASS 1: sky level = '+STRING(mfm_mean,format='(F9.1)')+' +/- '+STRING(mfm_stdev,format='(F7.1)')
	;
	hmin = nsig * mfm_stdev
	;
	roundlim = [-1.0,0.85]
	sharplim = [0.1,0.85]
	;
	if(mean_FWHM lt 3.0) then begin
		roundlim = [-1.1,0.9]
		sharplim = [0.05,0.95]
	endif
	;
	fwhm = mean_FWHM
	;
;	debug=0
	g_find, find_image, x, y, flux, sharp, roundness, hmin, fwhm, roundlim, sharplim, $
		SILENT=(1-debug), /NO_BAD_PIX_FIX
	;
	nstars = N_ELEMENTS(x)
	print_text, Event, 'PASS 1: found '+STRING(nstars)+' stars down to '+STRING(nsig,format='(F6.1)')+' sigma over sky'
	;
	;	count stars within non-border region only
	;
	ind = WHERE(x gt xborder and x lt (xsize-xborder) and y gt yborder and y lt (ysize-yborder), $
				good_stars, COMPLEMENT=ind_bad, NCOMPLEMENT=nbad)
	;
	print_text,Event, "PASS 1: number of non-border stars found = "+STRING(format='(I)',good_stars)
	;
	;	identify in a reg file all the good and bad found stars
	;
	if(region_flag gt 0) then begin
		;
		region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		;
		file_reg = FILE_BASENAME(deep_path,".fits") + '_A_Pass_1_found_stars_green_inside_red_outside.reg'
		region_path = region_dir + file_reg
		;
		openw, lu_reg, region_path, /GET_LUN
		;
		;	put good stars in green, border in red
		;
		if(good_stars gt 0) then begin
			for igood = 0, good_stars-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x[ind[igood]]+1,$
												y[ind[igood]]+1,1.5*mean_FWHM, 'green'
			endfor
		endif
		;
		if(nbad gt 0) then begin
			for ibad = 0, nbad-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x[ind_bad[ibad]]+1,$
												y[ind_bad[ibad]]+1,1.5*mean_FWHM, 'red'
			endfor
		endif
		;
		FREE_LUN, lu_reg
		;
	endif
	;
	if(good_stars gt 0) then begin
		x = x[ind]
		y = y[ind]
		flux = flux[ind]
		sharp = sharp[ind]
		roundness = roundness[ind]
	endif
	;
	;	aperture photometry on all the good stars
	;
	phpadu = 8.21
	;
	center_in_textbox, nstar_wid, 'PASS 1 Aper Ph.'
	group_colors[0] = 'blue'
	WCT_LED_stack, LED1_wid, group_colors
	;
;	debug = 0
	g_aper,find_image,x,y,mags,errap,sky,skyerr,phpadu,short_ap_vec,skyrad,badpix,bright_limit,$
				SILENT=(1-debug),/NO_BAD_PIX_FIX,/EXACT
	;
	;	STEP 2: group all found stars into groups
	;
	nstars1 = N_ELEMENTS(x)
	;
	;	for each star, find which aperture is the first with good uncertainties and push
	;			uncertainties and magnitudes down into first list item
	;
	for ist = 0, nstars1-1 do begin
		ind_ok = WHERE(abs(errap[*,ist]) lt this_sig_mag_cut and FINITE(mags[*,ist]) and $
			mags[*,ist] gt -10 and mags[*,ist] lt 25.0,n_ok)
		if(n_ok gt 0) then begin
			errap[0,ist] = errap[ind_ok[0],ist]
			mags[0,ist] = mags[ind_ok[0],ist]
		endif else begin
			errap[0,ist] = 99.0
		endelse
	endfor
	;
	ind_good_ap = WHERE(abs(errap[0,*]) lt this_sig_mag_cut, n_good_ap, COMPLEMENT=ind_bad_ap, NCOMPLEMENT=nbad_ap)
	;
	if(nbad_ap gt 0) then begin
		print_text, Event,'PASS 1: rejected '+STRING(nbad_ap)+' star candidates in aperture photometry'
		;
		if(nbad_ap gt 0 and debug gt 0) then begin
			for ibad = 0, nbad_ap-1 do begin
				print_text, Event, '   rejected star at (x,y,mag) = '+STRING(x[ind_bad_ap[ibad]],y[ind_bad_ap[ibad]],$
								mags[0,ind_bad_ap[ibad]])
			endfor
		endif
	endif
	;
	if(region_flag gt 0) then begin
		region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		file_reg = FILE_BASENAME(deep_path,'.fits') + '_B_Pass_1_Ap_Phot_stars_good_green_bad_red.reg'
		region_path = region_dir + file_reg
		openw, lu_reg, region_path, /GET_LUN
		;
		;	put good stars in green, border in red
		;
		if(n_good_ap gt 0) then begin
			for igood = 0, n_good_ap-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x[ind_good_ap[igood]]+1,$
												y[ind_good_ap[igood]]+1,1.5*mean_FWHM, 'green'
			endfor
		endif
		;
		if(nbad_ap gt 0) then begin
			for ibad = 0, nbad_ap-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x[ind_bad_ap[ibad]]+1,$
												y[ind_bad_ap[ibad]]+1,1.5*mean_FWHM, 'red'
			endfor
		endif
		;
		FREE_LUN, lu_reg
	endif
	;
	x = x[ind_good_ap]
	y = y[ind_good_ap]
	mags = REFORM(mags[0,ind_good_ap])
	sky = sky[ind_good_ap]
	errmag = REFORM(errap[0,ind_good_ap])
	nstars1 = n_good_ap
	;
	print_text,Event, "PASS 1: grouping "+STRING(format='(I)',nstars1)+" stars"
	rcrit0 = rad_PSF + fit_rad
	;
	;	if ng is small, increase rcrit somewhat to account for varying psfs that are not caught
	;	due to small star numbers
	;
	if(ng le 2 and calibrator_flag ne 1) then rcrit0 += fit_rad * (3 - ng)
	;
	rcrit = rcrit0
	;
	pass1_group_loop = 0L
	star_id = indgen(nstars1)
	;
	regroup_pass1:
	;
	group, Event, x, y, rcrit, group_vector
	;
	;	find the largest group. If its size is > max_in_group stars, reduce rcrit and retry
	;
	ngroups = N_ELEMENTS(group_vector)
	this_max_stars = MAX(HISTOGRAM(group_vector,MIN=0))
	if(this_max_stars gt max_in_group) then begin
		print_text,Event, "WARNING: PASS1 had a group with "+STRING(format='(I)',this_max_stars)+" stars, where only "+STRING(max_in_group, format='(I4)')+" allowed"
		;print, "   reducing rcrit and retrying"
		if(pass1_group_loop lt 20) then begin
			rcrit = 0.95 * rcrit
			pass1_group_loop++
			print_text, Event, "  new rcrit = "+STRING(format='(F6.1)',rcrit)+"  Restarting PASS1 grouping"
			goto, regroup_pass1
		endif else print_text, Event, "WARNING: PASS1 grouping radius decremented 20 times, but still found large groups"
	endif
	;
	n_groups = N_ELEMENTS(UNIQ(group_vector,SORT(group_vector)))
	print_text, Event, "PASS 1: star list has "+STRING(ngroups)+" stars spread over "+STRING(n_groups)+" groups"
	;
	;
	;	STEP 3: For all groups, form PSFs, fit group stars, remove from image
	;
	print_text, Event, 'PASS 1: starting PSF fitting and removing stars'
	;
	ind_x = WHERE(mags lt -10.0,n_neg)
	if(n_neg gt 0) then begin
		print,"ERROR - mags brighter than -10 in Deep PSF Fitting - stopping'
		stop
	end
	;
	center_in_textbox, nstar_wid, 'PASS 1 PSF Ph.'
	group_colors[0] = 'skyblue'
	WCT_LED_stack, LED1_wid, group_colors
	;
	iplot=0
	isave=save_intermediate_images
	print=0
	message1 = 'file to save stellar print info'
	message2 = 'Image after PASS 1 star removal'
	idpc=0
	;
	;	save x, y positions of stars going in
	;
	x_pos_save = x
	y_pos_save = y
	n_before_fit = N_ELEMENTS(x_pos_save)
	;
	g_fit_and_remove_stars, this_ng, find_image, resid_image, x, y, mags, errmag, sky, $
			star_id, group_vector, PSF_model, PSF_gauss, PSF_rad, PSF_mag, sum_resid, $
			phpadu, ronois, fit_rad, PRINT=print, MESSAGE1=message1, IPLOT=iplot, $
			ISAVE=isave, MESSAGE2=message2, idpc, status
	;
	;	check run status
	;
	if(status eq -1) then begin
		str = "PSF computation failed for "+FILE_BASENAME(deep_path)
		print_text, Event, "ERROR - "+str
		status = -99
		SAll_add_to_history, Event, h_wid, 8, str
		goto, S8PSFF_end
	endif
	;
	;	look for case of no good stars fit
	;
	test = N_ELEMENTS(group_vector)
	if(test eq 1) then begin
		if(group_vector eq -1) then begin
			;
			;	no stars were fit - quit remaining processing
			;
			status = -1
			str = "No stars fit (empty group vector) for "+FILE_BASENAME(deep_path)
			print_text, Event, "ERROR - "+str
			SAll_add_to_history, Event, h_wid, 8, str
			goto, S8PSFF_end
		endif
	endif
	;
	ind_PSF_good = WHERE(errmag le this_sig_mag_cut and FINITE(mags) and $
					mags gt -10. and mags lt 25.0, n_good_PSF, $
					COMPLEMENT=ind_bad_ap, NCOMPLEMENT=nbad_ap)
	;
	print_text, Event, "PASS 1: after PSF fitting, good, bad star count = "+STRING(n_good_PSF, nbad_ap)
	;
	if(nbad_ap gt 0) then begin
		print_text, Event, 'PASS 1: lost '+STRING(nbad_ap,format='(I4)')+' stars in PSF fitting'
;
		if(nbad_ap gt 0 and debug gt 0) then begin
			for ibad = 0, nbad_ap-1 do begin
				print_text, Event, '   rejected star at (x,y,mag) = '+STRING(x[ind_bad_ap[ibad]],y[ind_bad_ap[ibad]],$
								mags[ind_bad_ap[ibad]])
			endfor
		endif
	endif
	;
	if(region_flag gt 0) then begin
		region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		file_reg = FILE_BASENAME(deep_path,'.fits') + '_C_Pass_1_PSF_fit_stars_good_green_bad-ap_yellow_bad_red.reg'
		region_path = region_dir + file_reg
		openw, lu_reg, region_path, /GET_LUN
		;
		;	put good stars in green, border in red
		;
		if(n_good_PSF gt 0) then begin
			for igood = 0, n_good_PSF-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x[ind_PSF_good[igood]]+1,$
												y[ind_PSF_good[igood]]+1,1.5*mean_FWHM, 'green'
			endfor
		endif
		;
		if(nbad_ap gt 0) then begin
			for ibad = 0, nbad_ap-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x[ind_bad_ap[ibad]]+1,$
												y[ind_bad_ap[ibad]]+1,1.5*mean_FWHM, 'yellow'
			endfor
		endif
		;
		;	and any that didn't survive fitting
		;
		if(n_good_PSF ne n_before_fit) then begin
			print_text, Event, "PASS 1: Lost "+STRING(n_before_fit - n_good_PSF)+" stars in the PSF fitting"
			;
			;	find the ones that differ.
			;
			for i_start = 0, n_before_fit-1 do begin
				dx = x - x_pos_save[i_start]
				dy = y - y_pos_save[i_start]
				dr = dx * dx + dy * dy
				ind = SORT(dr)
				if(sqrt(dr[ind[0]]) gt 0.5*mean_FWHM) then begin	; found one that didn't match, put to reg file
					printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x_pos_save[i_start]+1,$
												y_pos_save[i_start]+1,1.5*mean_FWHM, 'red'
				endif
			endfor
		endif
		;
		FREE_LUN, lu_reg
	endif
	;
	x = x[ind_PSF_good]
	y = y[ind_PSF_good]
	mags = mags[ind_PSF_good]
	errmag = errmag[ind_PSF_good]
	sky = sky[ind_PSF_good]
	group_vector = group_vector[ind_PSF_good]
	;
	center_in_textbox, nstar_wid, STRING(n_good_PSF,format='("P1:",i4)')
	group_colors[0] = 'slateblue'
	WCT_LED_stack, LED1_wid, group_colors
	;
	;------------------------------- PASS 2, 3 -------------------------------------
	;
	;	load results into merge variables
	;
	xmerge = x
	ymerge = y
	magmerge = mags
	errmagmerge = errmag
	skymerge = sky
	group_vector3 = group_vector
	nstars = N_ELEMENTS(group_vector)
	;
	;	STEP 4: Find stars in residual image, perform aperture photometry, group them
	;
;	print_text, Event, "PASS 2: redetermining sky, refinding stars"
	;
;	debug = 0
	g_sky, resid_image, skymode, skysig, mfm_mean, mfm_stdev, SILENT=(1-debug)
	;
	print_text, Event, 'PASS 2: new sky level = '+STRING(mfm_mean,format='(F9.1)')+'+/-'+STRING(mfm_stdev,format='(F7.1)')
	;
	;	find, fill deep negative holes from poor PSF subtraction
	;
	del_image = ((resid_image + 0.5e6) < 1) > 0
	mask_image = (resid_image - mfm_mean) lt -5.0*mfm_stdev
	;
	resid_image *= (1 - mask_image)
	resid_image += mask_image * mfm_mean
	resid_image *= del_image
	resid_image += (1 - del_image) * (-1.0e6)
	;
;	stop
	;
	;	then compute a smoothed version of the residual image
	;
	smooth_image = SMOOTH(resid_image, CEIL(10.0*mean_FWHM) )
	dif_image = resid_image - smooth_image
	;
	g_sky, dif_image, skymode, skysig, mfm_mean, mfm_stdev, SILENT=(1-debug)
	;
	print_text, Event, 'PASS 2: smoothed, difference sky level = '+STRING(mfm_mean,format='(F9.1)')+$
					'+/-'+STRING(mfm_stdev,format='(F7.1)')
	;
	;	find stars in the difference image, but fit them in the non-differenced image
	;
	hmin2 = pass2_sigma * mfm_stdev
	sig0 = hmin2
	;
	roundlim = [-1.0,0.85]
	sharplim = [0.1,0.85]
	;
	if(mean_FWHM lt 3.0) then begin
		roundlim = [-1.1,0.9]
		sharplim = [0.05,0.95]
	endif
	;
;	debug=0
	loop_find:
	;
	g_find, dif_image, x2, y2, flux2, sharp2, roundness2, hmin2, fwhm, roundlim, $
		sharplim, SILENT=(1-debug), /NO_BAD_PIX_FIX
	;
	nstars2 = N_ELEMENTS(x2)
	;
	print_text, Event, 'PASS 2: number of new stars found in residual difference image = '+$
		STRING(nstars2)+' down to '+STRING(pass2_sigma,format='(F5.1)')+' sigma over sky'
		;
	ind2 = WHERE(x2 gt xborder and x2 lt (xsize-xborder) and y2 gt yborder and y2 lt (ysize-yborder), $
				good_stars2, COMPLEMENT=ind_bad, NCOMPLEMENT=nbad)
				;
	print_text,Event, 'PASS 2: number of non-border stars found in residual difference image = '+STRING(format='(I)',good_stars2)
	;
	if(region_flag gt 0) then begin
		;
		region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		;
		file_reg = FILE_BASENAME(deep_path,'.fits') + '_D_Pass_2_found_stars_inside_green_outside_red.reg'
		region_path = region_dir + file_reg
		;
		openw, lu_reg, region_path, /GET_LUN
		;
		;	put good stars in green, border in red
		;
		if(good_stars2 gt 0) then begin
			for igood = 0, good_stars2-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x2[ind2[igood]]+1,$
												y2[ind2[igood]]+1,1.5*mean_FWHM, 'green'
			endfor
		endif
		;
		if(nbad gt 0) then begin
			for ibad = 0, nbad-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x2[ind_bad[ibad]]+1,$
												y2[ind_bad[ibad]]+1,1.5*mean_FWHM, 'red'
			endfor
		endif
		;
		FREE_LUN, lu_reg
		;
	endif
	;
	if(good_stars2 gt 0) then begin
		x2 = x2[ind2]
		y2 = y2[ind2]
		flux2 = flux2[ind2]
		sharp2 = sharp2[ind2]
		roundness2 = roundness2[ind2]
	endif
	;
	star_id2 = indgen(good_stars2)
	nstars2 = good_stars2
	;
	center_in_textbox, nstar_wid, 'PASS 2 Aper Ph.'
	group_colors[0] = 'skyblue'
	WCT_LED_stack, LED1_wid, group_colors
	;
	debug = 0
	g_aper,resid_image,x2,y2,mags2,errap2,sky2,skyerr2,phpadu,short_ap_vec,skyrad,badpix,bright_limit,$
			SILENT=(1-debug),/NO_BAD_PIX_FIX,/EXACT
	;
	;	again, check for good photometric uncertainties
	;
	nst2 = N_ELEMENTS(x2)
	for ist = 0, nst2-1 do begin
		ind_ok = WHERE(abs(errap2[*,ist]) le this_sig_mag_cut and FINITE(mags2[*,ist]) $
			and mags2[*,ist] gt -10 and mags2[*,ist] lt 25.0, n_ok)
		if(n_ok gt 0) then begin
			errap2[0,ist] = errap2[ind_ok[0],ist]
			mags2[0,ist] = mags2[ind_ok[0],ist]
		endif else begin
			errap2[0,ist] = 99.0
		endelse
	endfor
	;
	ind_good_ap2 = WHERE(abs(errap2[0,*]) le this_sig_mag_cut, n_good_ap2, $
			COMPLEMENT=ind_bad_ap, NCOMPLEMENT=nbad_ap)
	;
	if(nbad_ap gt 0) then begin
		print_text, Event, 'PASS 2: lost '+STRING(nbad_ap)+' stars in aperture phot.'
;
		if(nbad_ap gt 0 and debug gt 0) then begin
			for ibad = 0, nbad_ap-1 do begin
				print_text, Event, '   rejected star at (x,y,mag) = '+STRING(x2[ind_bad_ap[ibad]]+1,y2[ind_bad_ap[ibad]]+1,$
								mags2[0,ind_bad_ap[ibad]])
			endfor
			;
		endif
	endif
	;
	if(region_flag gt 0) then begin
		region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		;
		file_reg = FILE_BASENAME(deep_path,'.fits') + '_E_Pass_2_Aper_phot_stars_good_green_bad_red.reg'
		region_path = region_dir + file_reg
		;
		openw, lu_reg, region_path, /GET_LUN
		;
		;	put good stars in green, border in red
		;
		if(n_good_ap2 gt 0) then begin
			for igood = 0, n_good_ap2-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x2[ind_good_ap2[igood]]+1,$
												y2[ind_good_ap2[igood]]+1,1.5*mean_FWHM, 'green'
			endfor
		endif
		;
		if(nbad_ap gt 0) then begin
			for ibad = 0, nbad_ap-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x2[ind_bad_ap[ibad]]+1,$
												y2[ind_bad_ap[ibad]]+1,1.5*mean_FWHM, 'red'
			endfor
		endif
		;
		FREE_LUN, lu_reg
	endif
	;
	x2 = x2[ind_good_ap2]
	y2 = y2[ind_good_ap2]
	mags2 = REFORM(mags2[0,ind_good_ap2])
	errap2 = REFORM(errap2[0,ind_good_ap2])
	sky2 = sky2[ind_good_ap2]
	;
	nstars2 = n_good_ap2
	;
	rcrit = rcrit0				; start with the old, original grouping radius
	;
	iloop2=0L
	;
	;	Step 5: Group stars, or, if fewer than 2 stars, don't group
	;
	if(nstars2 ge 2) then begin
		;
		print_text, Event, 'PASS 2: Grouping '+STRING(nstars2)+' new stars'
		loop_group2:
		group, Event, x2, y2, rcrit, group_vector2
		;
		;	check the groups -- if max has more than max_in_group stars, reset rcrit and/or sigma cut
		;
		ngroups = N_ELEMENTS(group_vector2)
		this_max_stars = MAX(HISTOGRAM(group_vector2,MIN=0))
		if(this_max_stars gt max_in_group) then begin
			print_text,Event, "WARNING: PASS2 has a group with "+STRING(format='(I)',this_max_stars)+" stars."
			;print, "   reducing rcrit and retrying"
			if(iloop2 lt 20) then begin
				rcrit = 0.95 * rcrit
				iloop2++
				print_text, Event, "  new rcrit = "+STRING(rcrit)+". restarting grouping"
				goto, loop_group2
			endif else print_text, Event, "WARNING: PASS2 grouping radius decremented 20 times, but still found large groups"
		endif
	endif else begin
		group_vector2 = indgen(1)
	endelse
	;
	;	STEP 6: PSF fit them (and remove from image)
	;
	ind_x = WHERE(mags2 lt -10.0,n_neg)
	if(n_neg gt 0) then begin
		print,"ERROR - mag brighter than -10 in Deep'
		stop
	end
	;
	center_in_textbox, nstar_wid, 'PASS 2 PSF Ph.'
	group_colors[0] = 'blue'
	WCT_LED_stack, LED1_wid, group_colors
	;
	iplot = 0
	isave = save_intermediate_images
	print=0
	message1 = 'file for print save of 2nd order info'
	message2 = 'Image after PASS 2 star removal'
	idpc=0
	mags2 = REFORM(mags2[0,*])
	errmag2 = REFORM(errap2[0,*])
	;
	x_pos_save = x2
	y_pos_save = y2
	n_before_fit = N_ELEMENTS(x_pos_save)
	;
	print_text, Event, 'PASS 2: starting PSF fitting and removal with '+STRING(nstars2)+' stars'
	g_fit_and_remove_stars, this_ng, resid_image, resid_image2, x2, y2, mags2, errmag2, sky2, $
			star_id2, group_vector2, PSF_model, PSF_gauss, PSF_rad, PSF_mag, sum_resid, $
			phpadu, ronois, fit_rad, PRINT=print, MESSAGE1=message1, IPLOT=iplot, $
			ISAVE=isave, MESSAGE2=message2,idpc, status
	;
	;	look for case of no good stars fit
	;
	test = N_ELEMENTS(group_vector2)
	if(test eq 1) then begin
		if(group_vector2 eq -1) then goto, S8_P_reenter
	endif
	;
	;	stars were fit, evaluate them
	;
	ind_good_PSF_2 = WHERE(abs(errmag2) le this_sig_mag_cut and FINITE(mags2) and $
					mags2 gt -10.0 and mags2 lt 25.0, $
					n_good_PSF_2, COMPLEMENT=ind_bad_ap, NCOMPLEMENT=nbad_ap)
	;
	if(nbad_ap gt 0) then begin
		print_text, Event, 'PASS 2: lost '+STRING(nbad_ap)+' stars in PSF fitting'
		;
		if(nbad_ap gt 0 and debug gt 0) then begin
			for ibad = 0, nbad_ap-1 do begin
				print_text, Event, '   rejected star at (x,y,mag) = '+STRING(x2[ind_bad_ap[ibad]]+1,y2[ind_bad_ap[ibad]]+1,$
								mags2[ind_bad_ap[ibad]])
			endfor
			;
		endif
	endif
	;
	if(region_flag gt 0) then begin
		region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		;
		file_reg = FILE_BASENAME(deep_path,'.fits') + '_F_Pass_2_PSF_fit_stars_good_green_bad-ap_yellow_bad_red.reg'
		region_path = region_dir + file_reg
		;
		openw, lu_reg, region_path, /GET_LUN
		;
		;	put good stars in green, border in red
		;
		if(n_good_PSF_2 gt 0) then begin
			for igood = 0, n_good_PSF_2-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x2[ind_good_PSF_2[igood]]+1,$
												y2[ind_good_PSF_2[igood]]+1,1.5*mean_FWHM, 'green'
			endfor
		endif
		;
		if(nbad_ap gt 0) then begin
			for ibad = 0, nbad_ap-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x2[ind_bad_ap[ibad]]+1,$
												y2[ind_bad_ap[ibad]]+1,1.5*mean_FWHM, 'yellow'
			endfor
		endif
		;
		;	and any that didn't survive fitting
		;
		if( (n_good_PSF_2 + nbad_ap) ne n_before_fit) then begin
			print_text, Event, "PASS 2: Lost "+STRING(n_before_fit - (n_good_PSF_2 + nbad_ap) )+" stars in the PSF fitting"
			;
			;	find the ones that differ.
			;
			for i_start = 0, n_before_fit-1 do begin
				dx = x2 - x_pos_save[i_start]
				dy = y2 - y_pos_save[i_start]
				dr = dx * dx + dy * dy
				ind = SORT(dr)
				if(sqrt(dr[ind[0]]) gt 0.5*mean_FWHM) then begin	; found one that didn't match, put to reg file
					printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x_pos_save[i_start]+1,$
												y_pos_save[i_start]+1,1.5*mean_FWHM, 'red'
				endif
			endfor
		endif
		;
		FREE_LUN, lu_reg
	endif
	;
	x2 = x2[ind_good_PSF_2]
	y2 = y2[ind_good_PSF_2]
	mags2 = mags2[ind_good_PSF_2]
	errmag2 = errmag2[ind_good_PSF_2]
	sky2 = sky2[ind_good_PSF_2]
	group_vector2 = group_vector2[ind_good_PSF_2]
	;
	nstars2 = n_good_PSF_2
	;
	group_colors[0] = 'rosybrown light'
	WCT_LED_stack, LED1_wid, group_colors
	center_in_textbox, nstar_wid, STRING(test,format='("P2:",I4)')
	;
	;	STEP 7: Merge, sort the star lists from both passes. Remove dupes
	;
	xmerge = [x, x2]
	ymerge = [y, y2]
	magmerge = [mags, mags2]
	skymerge = [sky, sky2]
	errmagmerge = [errmag, errmag2]
	;
	S8_P_reenter:
	;
	;		sort by -RA (=+x)
	;
	ind_ra = SORT(xmerge)
	xmerge = xmerge[ind_ra]
	ymerge = ymerge[ind_ra]
	magmerge = magmerge[ind_ra]
	skymerge = skymerge[ind_ra]
	errmagmerge = errmagmerge[ind_ra]
	n_merge = N_ELEMENTS(xmerge)
	;
	;		find, remove dupes - any stars w/i 0.25 FWHM, or
	;							 stars with mag dif > 2 within 0.5 FWHM, or
	;							 stars with mag dif > 4 within FWHM, or
	;							 stars with mag dif > 5 within fit_rad ; turned this one off 20100826
	;
	;		this leaves bright, blended stars in the list and they
	;			will have to be fit simultaneously
	;
	;	-- ng bump for small ng values
	;
	if(ng le 2 and calibrator_flag ne 1) then bump = 1.5 * (3 - ng) else bump = 1.0
	;
	fwhm_test = mean_FWHM * bump
	rad1 = (0.5*mean_FWHM * bump) > 1.5
	rad2 = (0.75*mean_FWHM * bump) > 2.5
	rad3 = mean_FWHM * bump
	rad4 = 2.5 * mean_FWHM * bump
	rad5 = rad_scale * mean_FWHM * bump
	;
	for istar = 0, n_merge-2 do begin
		;
		restart_jstar_loop:
		;
		for jstar = istar+1, n_merge-1 do begin
			dx = xmerge[jstar] - xmerge[istar]
			if(dx gt (rad5+3)) then goto, dupe_i_end
			dy = ymerge[istar] - ymerge[jstar]
			if(abs(dy) gt (rad5+3)) then goto, dupe_j_end
			if(magmerge[jstar] lt -40.0) then goto, dupe_j_end
			if(magmerge[istar] lt -40.0) then goto, dupe_i_end
			;
			dm = abs(magmerge[istar] - magmerge[jstar])
			;
			rad = sqrt(dx * dx + dy * dy)
			;
			merge = 0
			if(rad le rad5 and dm gt 7.0) then merge = 1
			if(rad le rad4 and dm gt 5.0) then merge = 1
			if(rad le rad3 and dm gt 3.5) then merge = 1
			if(rad le rad2 and dm gt 2.0) then merge = 1
			if(rad le rad1) then merge = 1
			;
			if(merge eq 1) then begin
				;
				;	found a dupe, merge the stars
				;
				F_i = 10.0^(-0.4*magmerge[istar])
				F_j = 10.0^(-0.4*magmerge[jstar])
				F_total = F_i + F_j
				mag_total = -2.5d0*alog10(F_total)
				;
				;	weight x, y by fluxes
				;
				mean_x = (xmerge[istar]*F_i + xmerge[jstar]*F_j)/F_total
				mean_y = (ymerge[istar]*F_i + ymerge[jstar]*F_j)/F_total
				;
				if(magmerge[istar] lt magmerge[jstar]) then begin
					magmerge[istar] = mag_total
					xmerge[istar] = mean_x
					ymerge[istar] = mean_y
					magmerge[jstar] = -49.0d0
				endif else begin
					magmerge[jstar] = mag_total
					xmerge[jstar] = mean_x
					ymerge[jstar] = mean_y
					magmerge[istar] = -49.0d0
				endelse
				goto, restart_jstar_loop
				;
;				stop
			endif
			;
			dupe_j_end:
		endfor
		dupe_i_end:
	endfor
	;
	;		remove any dupes found
	;
	ind_nodupe = WHERE(abs(magmerge + 49.0d0) gt 1.0,nodupes, COMPLEMENT=ind_dupe, NCOMPLEMENT=ndupes)
	;
	if(ndupes ne 0) then begin
		print_text, Event, 'PASS 3: Removing '+STRING(ndupes)+' duplicate stars from list'
		;
		if(ndupes gt 0 and debug gt 0) then begin
			for id = 0, ndupes-1 do begin
				print_text, Event, '  removing dupe at (x,y,mag) = '+STRING(xmerge[ind_dupe[id]],ymerge[ind_dupe[id]],$
									magmerge[ind_dupe[id]])
			endfor
		endif
		;
		xmerge = xmerge[ind_nodupe]
		ymerge = ymerge[ind_nodupe]
		magmerge = magmerge[ind_nodupe]
		skymerge = skymerge[ind_nodupe]
		errmagmerge = errmagmerge[ind_nodupe]
	endif
	;
	nstars3 = N_ELEMENTS(xmerge)
	print_text,Event, "PASS 3: in combining PASS 1 & 2 lists, "+STRING(format='(I)',nstars3)+" stars survived to be fit"
	star_id = indgen(nstars3)
	;
	group_colors[0] = 'khaki light'
	WCT_LED_stack, LED1_wid, group_colors
	;
	;	build groups
	;
	rcrit = rcrit0
	;
	iloop3 =0L
;
	loop_group_3:
	group, Event, xmerge, ymerge, rcrit, group_vector3
	;
	;	check the groups -- if max has more than max_in_group stars, reset rcrit and/or sigma cut
	;
	ngroups = N_ELEMENTS(group_vector3)
	this_max_stars = MAX(HISTOGRAM(group_vector3,MIN=0))
	if(this_max_stars gt max_in_group) then begin
		print_text,Event, "WARNING: PASS3 has a group with "+STRING(format='(I)',this_max_stars)+" stars."
		;print, "   reducing rcrit and retrying"
		if(iloop3 lt 20) then begin
			rcrit = 0.95 * rcrit
			iloop3++
			print_text, Event, "  new rcrit = "+STRING(rcrit)+". restarting grouping"
			goto, loop_group_3
		endif else print_text, Event, "WARNING: PASS3 grouping radius decremented 20 times, but still found large groups"
	endif
	;
	ind_x = WHERE(magmerge lt -10.0,n_neg)
	if(n_neg gt 0) then begin
		print,"ERROR - mag brighter than -10 in PASS2 Deep PSF Fitting. Stopping'
		stop
	end
	;
	;	do normal PSF fitting first, and again remove dupes
	;
	group_colors[0] = 'yellow'
	WCT_LED_stack, LED1_wid, group_colors
	center_in_textbox, nstar_wid, STRING(ngroups,format='("P3:",I4)')
	;
	iplot=0
	isave=save_intermediate_images
	print=0
	message1 = 'file for print save of 2nd order info'
	message2 = 'Image after PASS 3 (=1+2) star removal'
	idpc=0
	;
	x_pos_save = xmerge
	y_pos_save = ymerge
	n_before_fit = N_ELEMENTS(x_pos_save)
	;
	print_text, Event, 'PASS 3: starting PSF fitting and removal with '+STRING(nstars3)+' stars'
	g_fit_and_remove_stars, this_ng, find_image, resid_image3, xmerge, ymerge, magmerge, errmagmerge, skymerge, $
			star_id, group_vector3, PSF_model, PSF_gauss, PSF_rad, PSF_mag, sum_resid, $
			phpadu, ronois, fit_rad, PRINT=print, MESSAGE1=message1, IPLOT=iplot, $
			ISAVE=isave, MESSAGE2=message2,idpc, status
	;
	;	look for case of no good stars fit
	;
	test = N_ELEMENTS(group_vector3)
	;
	ind_good_PSF_3= WHERE(errmagmerge lt this_sig_mag_cut and FINITE(magmerge) and $
					magmerge gt -10. and magmerge lt 25., $
					n_good_PSF_3, COMPLEMENT=ind_bad, NCOMPLEMENT=nbad)
	;
	if(nbad gt 0) then begin
		print_text, Event, 'PASS 3: lost '+STRING(nbad)+' stars in PSF fitting'
		;
		if(nbad gt 0 and debug gt 0) then begin
			for id = 0, nbad-1 do begin
				print_text, Event, '  removing bad star at (x,y,mag) = '+STRING(xmerge[ind_bad[id]],ymerge[ind_bad[id]],$
									magmerge[ind_bad[id]])
			endfor
			;
		endif
	endif
	;
	if(region_flag gt 0) then begin
		region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		;
		file_reg = FILE_BASENAME(deep_path,'.fits') + '_G_Pass_3_PSF_fit_stars_good_green_bad-ap_yellow_bad_red.reg'
		region_path = region_dir + file_reg
		;
		openw, lu_reg, region_path, /GET_LUN
		;
		;	put good stars in green, border in red
		;
		if(n_good_PSF_3 gt 0) then begin
			for igood = 0, n_good_PSF_3-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',xmerge[ind_good_PSF_3[igood]]+1,$
												ymerge[ind_good_PSF_3[igood]]+1,1.5*mean_FWHM, 'green'
			endfor
		endif
		;
		if(nbad_ap gt 0) then begin
			for ibad = 0, nbad_ap-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',xmerge[ind_bad_ap[ibad]]+1,$
												ymerge[ind_bad_ap[ibad]]+1,1.5*mean_FWHM, 'yellow'
			endfor
		endif
		;
		if( (n_good_PSF_3 + nbad) ne n_before_fit) then begin
			print_text, Event, "PASS 3: Lost "+STRING(n_before_fit - (n_good_PSF_3 + nbad))+" stars in the PSF fitting"
			;
			;	find the ones that differ.
			;
			for i_start = 0, n_before_fit-1 do begin
				dx = xmerge - x_pos_save[i_start]
				dy = ymerge - y_pos_save[i_start]
				dr = dx * dx + dy * dy
				ind = SORT(dr)
				if(sqrt(dr[ind[0]]) gt 0.5*mean_FWHM) then begin	; found one that didn't match, put to reg file
					printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x_pos_save[i_start]+1,$
												y_pos_save[i_start]+1,1.5*mean_FWHM, 'red'
				endif
			endfor
		endif
		;
		FREE_LUN, lu_reg
	endif
	;
	xmerge = xmerge[ind_good_PSF_3]
	ind_sort = SORT(xmerge)
	;
	full_ind = ind_good_PSF_3[ind_sort]
	;
	xmerge = xmerge[ind_sort]
	;
	ymerge = ymerge[full_ind]
	magmerge = magmerge[full_ind]
	errmagmerge = errmagmerge[full_ind]
	skymerge = skymerge[full_ind]
	;
	n_merge = N_ELEMENTS(xmerge)
	;
	;		find, remove dupes - any stars w/i 0.25 FWHM, or
	;							 stars with mag dif > 2 within 0.5 FWHM, or
	;							 stars with mag dif > 4 within FWHM, or
	;							 stars with mag dif > 5 within fit_rad
	;
	;		this leaves bright, blended stars in the list and they
	;			will have to be fit simultaneously
	;
	if(ng le 2 and calibrator_flag ne 1) then bump = 1.5 * (3 - ng) else bump = 1.0
	;
	fwhm_test = mean_FWHM * bump
	rad1 = (0.5*mean_FWHM * bump) > 1.5
	rad2 = (0.75*mean_FWHM * bump) > 2.5
	rad3 = mean_FWHM * bump
	rad4 = 2.5 * mean_FWHM * bump
	rad5 = rad_scale * mean_FWHM * bump
	;
	for istar = 0, n_merge-2 do begin
		;
		restart_jstar3_loop:
		;
		for jstar = istar+1, n_merge-1 do begin
			dx = xmerge[jstar] - xmerge[istar]
			if(dx gt (rad5+3)) then goto, dupe_i_end_3
			dy = ymerge[istar] - ymerge[jstar]
			if(abs(dy) gt (rad5+3)) then goto, dupe_j_end_3
			if(magmerge[jstar] lt -40.0) then goto, dupe_j_end_3
			if(magmerge[istar] lt -40.0) then goto, dupe_i_end_3
			;
			dm = abs(magmerge[istar] - magmerge[jstar])
			;
			rad = sqrt(dx * dx + dy * dy)
			;
			merge = 0
			if(rad le rad5 and dm gt 7.0) then merge = 1
			if(rad le rad4 and dm gt 5.0) then merge = 1
			if(rad le rad3 and dm gt 3.5) then merge = 1
			if(rad le rad2 and dm gt 2.0) then merge = 1
			if(rad le rad1) then merge = 1
			;
			if(merge eq 1) then begin
				;
				;	found a dupe, merge the stars
				;
				F_i = 10^(-0.4*magmerge[istar])
				F_j = 10^(-0.4*magmerge[jstar])
				F_total = F_i + F_j
				mag_total = -2.5*alog10(f_total)
				;
				;	weight x, y by fluxes
				;
				mean_x = (xmerge[istar]*F_i + xmerge[jstar]*F_j)/F_total
				mean_y = (ymerge[istar]*F_i + ymerge[jstar]*F_j)/F_total
				;
				if(magmerge[istar] lt magmerge[jstar]) then begin
					magmerge[istar] = mag_total
					xmerge[istar] = mean_x
					ymerge[istar] = mean_y
					magmerge[jstar] = -49.0d0
				endif else begin
					magmerge[jstar] = mag_total
					xmerge[jstar] = mean_x
					ymerge[jstar] = mean_y
					magmerge[istar] = -49.0d0
				endelse
				goto, restart_jstar3_loop
				;
;				stop
			endif
			;
			dupe_j_end_3:
		endfor
		dupe_i_end_3:
	endfor
	;
	;		remove any dupes found
	;
	ind_nodupe = WHERE(abs(magmerge + 49.0d0) gt 1.0,nodupes, COMPLEMENT=ind_dupe, NCOMPLEMENT=ndupes)
	;
	if(ndupes gt 0) then begin
		print_text, Event, 'PASS 3: Removing '+STRING(ndupes)+' duplicate stars from list'
		;
		if(ndupes gt 0 and debug gt 0) then begin
			for id = 0, ndupes-1 do begin
				print_text, Event, '  removing dupe at (x,y,mag) = '+STRING(xmerge[ind_dupe[id]],ymerge[ind_dupe[id]],$
									magmerge[ind_dupe[id]])
			endfor
		endif
		;
		xmerge = xmerge[ind_nodupe]
		ymerge = ymerge[ind_nodupe]
		magmerge = magmerge[ind_nodupe]
		skymerge = skymerge[ind_nodupe]
		errmagmerge = errmagmerge[ind_nodupe]
		n_merge = N_ELEMENTS(xmerge)
	endif
	;
	;	finally, ignore all stars in the coadding fringe
	;
	ind_inside = WHERE((xmerge gt xborder) and (xmerge lt (xsize-xborder)) and (ymerge gt yborder) and $
						(ymerge lt (ysize-yborder)), n_inside, COMPLEMENT=ind_outside, NCOMPLEMENT=noutside)
	;
	if(region_flag gt 0) then begin
		region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		;
		file_reg = FILE_BASENAME(deep_path,'.fits') + '_H_Pass_3_Final_stars_inside_green_outside_red.reg'
		region_path = region_dir + file_reg
		;
		openw, lu_reg, region_path, /GET_LUN
		;
		;	put good stars in green, border in red
		;
		if(n_inside gt 0) then begin
			for igood = 0, n_inside-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',xmerge[ind_inside[igood]]+1,$
												ymerge[ind_inside[igood]]+1,1.5*mean_FWHM, 'green'
			endfor
		endif
		;
		if(noutside gt 0) then begin
			for ibad = 0, noutside-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',xmerge[ind_outside[ibad]]+1,$
												ymerge[ind_outside[ibad]]+1,1.5*mean_FWHM, 'red'
			endfor
		endif
		;
		FREE_LUN, lu_reg
	endif
	;
	if(n_inside gt 0) then begin
		if(noutside gt 0) then print_text, Event, "Rejecting "+STRING(noutside,format='(I)')+" border stars"
		;
		xmerge = xmerge[ind_inside]
		ymerge = ymerge[ind_inside]
		magmerge = magmerge[ind_inside]
		skymerge = skymerge[ind_inside]
		errmagmerge = errmagmerge[ind_inside]
		n_merge = N_ELEMENTS(xmerge)
	endif else begin
		print_text, Event, "ERROR - No stars within coadded boundary"
		goto, S8PSFF_end
	endelse
	;
	;	save a copy of the star-subtracted image, if the save intermediate flag is set
	;
	if(save_intermediate_images gt 0) then begin
		resid_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Residual_Images' + PATH_SEP()
		test_dir = FILE_INFO(resid_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, resid_dir
		;
		file_res = FILE_BASENAME(deep_path,'.fits') + '_Residuals.fits'
		resid_path = resid_dir + file_res
		;
		resid_header = image_header
		SXADDPAR, resid_header, 'HISTORY', 'Residual Image computed '+SYSTIME()
		writefits, resid_path, resid_image3, resid_header
		print_text, Event, "Saved residual image file to "+resid_dir
	endif
	;
	nstars3 = N_ELEMENTS(xmerge)
;	print_text,Event, "PASS 4: Starting PSF-Assisted PSF-fitting for "+STRING(format='(I)',nstars3)+" stars"
	star_id = indgen(nstars3)
	;
	end_list_time = SYSTIME(/seconds)
	list_time = end_list_time - start_list_time
	print_text, Event, "Time to form full star list = "+STRING(list_time)+" seconds after PSF build time"
	print_text, Event, "Total time to here = "+STRING(list_time + psf_build_time, format='(F9.1)')+" seconds"
	;
;	stop
	;
	;	regroup the final star list, looking for very tight doubles (rcrit ~< fit_rad)
	;
	if(simple_PSF_assist eq 0) then begin
		rcrit = fit_rad
		group, Event, xmerge, ymerge, rcrit, group_vector4
;		save_group4 = group_vector4
		ngroups = N_ELEMENTS(UNIQ(group_vector4,SORT(group_vector4)))
		print_text,Event,'PASS 4: Found '+STRING(ngroups)+' tight groups among '+STRING(nstars3)+' stars'
	endif else begin
		;
		;	each star its own group
		;
		group_vector4 = indgen(N_ELEMENTS(xmerge))
		ngroups = nstars3
		print_text, Event, 'PASS 4: Doing Simple PSF-Assist w/o tight groups on '+STRING(nstars3)+' stars. '
	endelse
	;
	;	STEP 8: For each tight group, collect nearby stars, fit and remove
	;
	dx10 = (clean_rad * ap_rad * mean_FWHM > (rad_scale * mean_FWHM)) / 2.0
	dx10 >= 1.25*skyrad[1]	; at least as large as outer limit of sky radius
	dy10 = dx10
	dx10_2 = (dx10) * (dx10)
	;
	x_final = dblarr(20000)
	y_final = dblarr(20000)
	mag_final = dblarr(20000)
	sky_final = dblarr(20000)
	errmag_final = dblarr(20000)
	grp_final = intarr(20000)
	;
	nstar = 0L
	;
;	resid_save_image = find_image
	;
	test_grp = CEIL(ngroups/2)					; set > 1 for saving images
	;
	;		5 percent steps
	;
	n10 = CEIL(ngroups / 10.0)
	color = 'goldenrod light'
	;
	n_gauss_good = 0L
	n_gauss_binary = 0L
	n_gauss_bad = 0L
	;
	if(region_flag gt 0) then begin
			;
			region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
			test_dir = FILE_INFO(region_dir)
			if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
			;
			file_reg = FILE_BASENAME(deep_path,".fits") + '_I_Gaussfit_Classify_green_good_yellow_double_red_bad.reg'
			region_path = region_dir + file_reg
			bad_star_path = region_dir + FILE_BASENAME(deep_path,".fits") + '_bad_pass_4_stars.dat'
			;
			openw, lu_reg, region_path, /GET_LUN
			openw, lu_bad_stars, bad_star_path, /GET_LUN
	endif
	;
	print_text, Event, 'PASS 4: NGROUPS = '+STRING(ngroups,format='(i5)')
	;
	for igrp = 0, ngroups-1 do begin
		;
		if((igrp+1) eq n10 * fix((igrp+1) / n10)) then begin
			fraction = 10 * ((igrp+1) / n10)
			center_in_textbox, nstar_wid, 'P4: '+STRING(fraction,format='(I3)')+'%'
			if(group_colors[0] eq 'goldenrod light') then begin
				group_colors[0] = 'goldenrod dark'
			endif else group_colors[0] = 'goldenrod light'
			WCT_LED_stack, LED1_wid, group_colors
		endif
		;
		;	find mean X, Y centers of this group
		;
		ind_strs = WHERE(group_vector4 eq igrp,nstrs)
		;
		this_x = [TOTAL(xmerge[ind_strs])/float(nstrs)]
		this_y = [TOTAL(ymerge[ind_strs])/float(nstrs)]
		this_sky = [TOTAL(skymerge[ind_strs])/float(nstrs)]
		this_mag = [magmerge[ind_strs[0]]]
		;
;		print, 'Grp ',igrp,' mean X,Y = ',this_x, this_y,' for ',nstrs,' stars'
		;
		;	STEP 8.1. collect all stars within clean-size of center position
		;
		n_10 = 0L
		;
		xfit = dblarr(1000)
		yfit = dblarr(1000)
		magfit = dblarr(1000)
		skyfit = dblarr(1000)
		errmagfit = dblarr(1000)
		;
		if(igrp eq test_grp) then start_time = SYSTIME(/SECONDS)
		;
		dx = xmerge - this_x[0]
		dy = ymerge - this_y[0]
		dr = dx * dx + dy * dy
		ind_dr = SORT(dr)
		ind_close = WHERE(dr[ind_dr] lt dx10_2,nclose)
		;
		n_10 = nclose - 1					; skip self
		ind_keep = ind_dr[ind_close]
		;
		;	if not close groups, then the fitting star is the first one on the
		;	list. If close groups, will have to revisit this...
		;
		x_10 = xmerge[ind_keep]
		y_10 = ymerge[ind_keep]
		sky_10 = skymerge[ind_keep]
		;
		xmin = ((FLOOR(MIN(x_10) - rad_PSF)) < (FLOOR(x_10[0] - dx10))) > 0
		xmax = ((CEIL(MAX(x_10) + rad_PSF)) > (CEIL(x_10[0] + dx10))) < (xsize-1)
		ymin = ((FLOOR(MIN(y_10) - rad_PSF)) < (FLOOR(y_10[0] - dy10))) > 0
		ymax = ((CEIL(MAX(y_10) + rad_PSF)) > (CEIL(y_10[0] + dy10))) < (ysize-1)
		;
		nx_phot = xmax - xmin + 1
		ny_phot = ymax - ymin + 1
		;
		;	select this portion of the original image, only
		;
		sub_image = find_image[xmin:xmax, ymin:ymax]
		;
		;	and load up the surrounding stars into the star subtraction arrays
		;
		if(n_10 gt 0) then begin
			x_sub = x_10[1:nclose-1] - xmin
			y_sub = y_10[1:nclose-1] - ymin
			mag_sub = magmerge[ind_keep[1:nclose-1]]
			sky_sub = skymerge[ind_keep[1:nclose-1]]
			errmag_sub = errmagmerge[ind_keep[1:nclose-1]]
		endif
		;
		;	and central star's coordinates in the sub-image
		;
		x_cen = x_10[0] - xmin
		y_cen = y_10[0] - ymin
		;
		;	STEP 8.3.: rebuild the PSF for this portion of the image
		;
		PSF_residuals = g_make_PSF(ng, this_x, this_y, PSF_model, $
						PSF_gauss, PSF_rad, PSF_mag, sum_resid, status)
		;
		;	test for bad PSF
		;
		if(status eq -1) then begin
			print, "ERROR in Step 8: call to g_make_PSF returned with status=-1"
			goto, S8PSFF_end
		endif
		;
		mkhdr, hdr, PSF_residuals         ;Create a minimal FITS header
		sxaddpar, hdr, 'PHPADU', phpadu, 'Photons per Analog Digital Unit'
		sxaddpar, hdr, 'RONOIS', ronois, 'Readout Noise'
		sxaddpar, hdr, 'PSFRAD', PSF_rad, 'Radius where PSF is defined (pixels)'
		sxaddpar, hdr, 'FITRAD', fit_rad, 'Fitting Radius'
		sxaddpar, hdr, 'PSFMAG', PSF_mag, 'PSF Magnitude'
	 	sxaddpar, hdr, 'GAUSS1', PSF_gauss[0], 'Gaussian Scale Factor'
		sxaddpar, hdr, 'GAUSS2', PSF_gauss[1], 'Gaussian X Position'
		sxaddpar, hdr, 'GAUSS3', PSF_gauss[2], 'Gaussian Y Position'
		sxaddpar, hdr, 'GAUSS4', PSF_gauss[3], 'Gaussian Sigma: X Direction'
		sxaddpar, hdr, 'GAUSS5', PSF_gauss[4], 'Gaussian Sigma: Y Direction'
		;
		;	report gaussians that are too narrow - DPC 20090307
		;
		if(PSF_gauss[3] lt 0.35) then begin
		 	print, "G_FIT_AND_REMOVE_STARS: Warning - sigx = ",PSF_gauss[3]," for group ",igrp
		endif
		if(PSF_gauss[4] lt 0.35) then begin
		 	print, "G_FIT_AND_REMOVE_STARS: Warning - sigy = ",PSF_gauss[4]," for group ",igrp
		endif
		;
		sxaddpar, hdr, 'SUMRESID', sum_resid, 'Flux sum of resid array inside PSFRAD'
		;
		x_w_psf = this_x[0]
		y_w_psf = this_y[0]
		sxaddpar, hdr, 'X_W_PSF', x_w_psf, 'Flux-Weighted X position'
		sxaddpar, hdr, 'Y_W_PSF', y_w_psf, 'Flux-Weighted Y position'
		idsub = -1
		verbose = 0
		;
		;	if special star, stop to allow saving sub_image and/or PSF model
		;
;		if(abs(x_10[0] - 79.0) lt 3.0 and abs(y_10[0] - 758.0) lt 3.0) then begin
;			stop
;		endif
		;
		;	remove non-central stars from the image, using their PSFs
		;
		if(n_10 gt 0) then begin
;			id_sub = intarr(N_ELEMENTS(x_sub))
			id_sub = -1							; says subtract all the stars on the list
			;
			g_substar, sub_image, x_sub, y_sub, mag_sub,id_sub,PSF_residuals, hdr,VERBOSE = verbose
			;
			if(igrp eq test_grp and save_intermediate_images eq 1) then begin
				file_out = dialog_pickfile(Title='Image of removed neighbors, only')
				writefits, file_out, sub_image, header
			endif
  		endif
		;
		;  STEP 8.4.: Fit central stars to locally cleaned image
		;
		group_vector = intarr(nstrs)
		star_id = indgen(nstrs)
		;
;		that_x = xmerge[ind_strs]
;		that_y = ymerge[ind_strs]
;		that_mag = magmerge[ind_strs]
;		that_sky = skymerge[ind_strs]
;		that_errmag = errmagmerge[ind_strs]
		;
		that_x = x_cen
		that_y = y_cen
		;
;		iplot=0
;		if(igrp eq test_grp) then isave = save_intermediate_images else isave=0
;		print=0
;		message1 = 'image after-central'
;		message2 = 'Image after central star removed'
;		idpc=0
;		;
;		if(method eq 'PSF') then begin
;			g_fit_and_remove_stars, this_ng, this_image, that_image, that_x, that_y, that_mag, $
;				that_errmag, that_sky, $
;				star_id, group_vector, PSF_model, PSF_gauss, PSF_rad, PSF_mag, sum_resid, $
;				phpadu, ronois, fit_rad, PRINT=print, MESSAGE1=message1, IPLOT=iplot, $
;				ISAVE=isave, MESSAGE2=message2, RESID_HEADER=resid_header, $
;				RESID_FILENAME=resid_filename, idpc, status
;		endif
		;
		if(method eq 'Aper') then begin
			;
			;	fit this star to be sure that its A/B and FWHM are similar to the image averages
			;		- if not, then this is probably a double and needs to be rejected somehow
			;
			if(n_gauss_good eq 0) then a_debug = 1 else a_debug = 0
			;
			;	kill everything outside of skyrad[1] (set to sky)
			;
			xp = findgen(nx_phot)
			yp = findgen(ny_phot)
			;
			gsub_image = sub_image
			;
			r2_test = (0.5*skyrad[1])^2
			r3_test = (skyrad[1])^2
			for ixx = 0, nx_phot-1 do begin
				for iyy = 0, ny_phot-1 do begin
					dr2 = (that_x - xp[ixx])^2 + (that_y - yp[iyy])^2
					if(dr2 gt r2_test) then gsub_image[ixx,iyy] = sky_10[0]
					if(dr2 gt r3_test) then sub_image[ixx,iyy] = sky_10[0]
				endfor
			endfor
			;
			;	also, backfill pixels far below sky
			;
			ind_zz = WHERE(gsub_image ne sky_10[0],nzz)
			if(nzz gt 0) then begin
				zz = median_filtered_mean(gsub_image[ind_zz])
				ind_low = WHERE((zz[0] - gsub_image)/zz[1] gt 4.0,nlow)
				if(nlow gt 0) then gsub_image[ind_low] = zz[0]
			endif
			;
			ind_yy = WHERE(sub_image ne sky_10[0],nyy)
			if(nyy gt 0) then begin
				yy = median_filtered_mean(sub_image[ind_yy])
				ind_ylow = WHERE((yy[0] - sub_image)/yy[1] gt 4.0, nylow)
				if(nylow gt 0) then sub_image[ind_ylow] = yy[0]
			endif
			;
			;	and remove high pixels in the outer 1/4 of the zone for the gaussian test
			;
			r4_test = (0.75*0.5*skyrad[1])^2
			for ixx = 0, nx_phot-1 do begin
				for iyy = 0, ny_phot-1 do begin
					dr3 = (that_x - xp[ixx])^2 + (that_y - yp[iyy])^2
					if(dr3 lt r2_test and dr3 ge r4_test) then begin
						if((gsub_image[ixx,iyy]-zz[0])/zz[1] gt 4.0) then begin
							gsub_image[ixx,iyy] = zz[0]
						endif
					endif
				endfor
			endfor
			;
			;	turn on debug in gaussian fitting for any one particular star or non-star
			;
			a_debug = 0
;			if(abs(x_10[0] - 79.0) lt 3.0 and abs(y_10[0] - 758.0) lt 3.0) then a_debug = 1 else a_debug = 0
			;
			threshold = hmin
			status = S8_gaussian_test_one_star(gsub_image, nx_phot, ny_phot, that_x, that_y, mean_FWHM, $
						a_debug, AXIS=axis, SIGMA=threshold)
			;
			if(status eq 0) then begin
				if(region_flag gt 0) then printf, lu_reg, $
							FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x_10[0]+1, y_10[0]+1,1.75*mean_FWHM, 'green'

				n_gauss_good++
				if(N_ELEMENTS(axis_save) eq 0) then axis_save = [axis] else axis_save = [axis_save, axis]
				;
				this_apr = ap_rad * mean_FWHM
				skyrad = ref_ap * this_apr
				g_aper,sub_image,that_x,that_y,that_mag,that_errmag,that_sky,skyerr,phpadu,this_apr,$
					skyrad,badpix,bright_limit,$
					SILENT=(1-debug),/NO_BAD_PIX_FIX, /EXACT
			endif
			if(status eq 1) then begin
				if(region_flag gt 0) then printf, lu_reg, $
							FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x_10[0]+1, y_10[0]+1,1.75*mean_FWHM, 'yellow'
				n_gauss_binary++
			endif
			if(status eq -1) then begin
				if(region_flag gt 0) then printf, lu_reg, $
							FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x_10[0]+1, y_10[0]+1,1.75*mean_FWHM, 'red'
				n_gauss_bad++
			endif
			;
		endif
		;
		if(igrp eq test_grp and simple_PSF_assist eq 0) then begin
			end_fit_time = SYSTIME(/seconds)
			print_text, Event, 'time to fit neighbors and one star = '+STRING(end_fit_time - start_fit_time)+' seconds'
			print_text, Event, 'time to find neighbors and fit them = '+STRING(end_fit_time - start_time)+' seconds'
			project_time = n_merge * (end_fit_time - start_time)
			print_text, Event, 'project completion of all stars in '+STRING(project_time)+' seconds'
		endif
		;
		;	STEP 8.5: Save the photometry to the final list, if it is good
		;
		if(status eq 0) then begin
			jnd_good = WHERE(that_errmag lt this_sig_mag_cut and FINITE(that_mag) $
					and that_mag gt -10. and that_mag lt 25.,ngood,COMPLEMENT=jnd_bad,NCOMPLEMENT=n_jnd_bad)		; better than 1.0 mag one sigma
			;
			;	save the rejected ones onto a file
			;
			if(region_flag eq 1 and n_jnd_bad gt 0) then begin
				for jbad = 0, n_jnd_bad-1 do begin
					x_bad = that_x[jnd_bad[jbad]] + xmin
					y_bad = that_y[jnd_bad[jbad]] + ymin
					mag_bad = that_mag[jnd_bad[jbad]]
					s_mag_bad = that_errmag[jnd_bad[jbad]]
					;
					printf, lu_reg, $
							FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x_bad+1, y_bad+1,2.25*mean_FWHM, 'magenta'
					printf, lu_bad_stars, x_bad, y_bad, mag_bad, s_mag_bad, $
							FORMAT='(f8.1,1x,f8.1,f7.2,1x,f6.3)'
				endfor
			endif
			;
			if(ngood gt 0) then begin
				for igood = 0, ngood-1 do begin
					x_final[nstar] = that_x[jnd_good[igood]] + xmin
					y_final[nstar] = that_y[jnd_good[igood]] + ymin
					mag_final[nstar] = that_mag[jnd_good[igood]]
					;
					if(mag_final[nstar] lt -10.0) then begin
						print, 'ERROR - mag brighter than -10. Stopping '
						stop
					endif
					;
					sky_final[nstar] = that_sky[jnd_good[igood]]
					errmag_final[nstar] = that_errmag[jnd_good[igood]]
					grp_final[nstar] = igrp
					nstar++
				endfor
				;
				;	if save resid image is selected, remove these stars
			endif else begin
				if(N_ELEMENTS(that_x) gt 1) then that_x = REFORM(that_x[0])
				if(N_ELEMENTS(that_y) gt 1) then that_y = REFORM(that_y[0])
				if(N_ELEMENTS(that_mag) gt 1) then that_mag = REFORM(that_mag[0]) else that_mag = that_mag[0]
				if(N_ELEMENTS(that_errmag) gt 1) then that_errmag = REFORM(that_errmag[0]) else that_errmag = that_errmag[0]
				;
				if(a_debug gt 0) then begin
					print_text, Event, 'PASS 4: lost star at (x,y) = '+STRING(that_x, that_y)
					print_text, Event, "                     (mag, errmag) = "+STRING(that_mag, that_errmag)+' in final Aperture Photometry'
					print_text, Event, '   mag, errmag, sky = '+STRING(that_mag, that_errmag, that_sky, format='(3(F12.3,2x))')
					stop
				endif
			endelse
		endif
		;
		;	STEP 8.6.: Go on to next star
		;
	endfor
	;
	if(region_flag gt 0) then FREE_LUN, lu_reg, lu_bad_stars
	;
	if(nstar ne nstars3) then print_text, Event, 'PASS 4: rejected '+STRING(nstars3 - nstar)+' star candidates in PSF-assisted Aperture Photometry'
	;
	print_text, Event, 'PASS 4: gaussian test of cleaned zones found '+$
		STRING(n_gauss_good, n_gauss_binary, n_gauss_bad, format='(I5.1,",",I5.1,",",I5.1)')+$
		" good, binary, bad stars"
	;
	print_text, Event, 'PSF assisted Aperture photometry finished for '+STRING(nstar)+' stars'
	;
;	stop
	;
	;	modify FITS header to save NG, sig, a/b values - does the FITS header go anywhere????
	;
	zz_axis = median_filtered_mean(axis_save)
	;
	FXADDPAR, header, 'HISTORY', "PPOL-Step 8-version "+(*s_ptr).version_short_string
	SXADDPAR, header, 'PSF_NG', ng, "Number of X, Y grid zones in psf build"
	SXADDPAR, header, 'PSF_SIG', pass2_sigma, "Detection threshold in sigma of sky"
	SXADDPAR, header, 'PSF_SAT', FIT_sat_limit_total, "Saturation Limit for allowed stars"
	SXADDPAR, header, 'AXIS_ABR',zz_axis[0],"Mean PSF A/B axis ratio"
	;
	;	if save residuals set, load header and filename
	;
	image_header = header
	phot_header = header
	;
	group_colors[0] = 'palegreen dark'
	WCT_LED_stack, LED1_wid, group_colors
	center_in_textbox, nstar_wid, STRING(nstar,format='(I)')
	;
;	no_pass23:
;	;
;	SXADDPAR, header, 'PSF_NG', ng, "Number of X, Y grid zones in psf build"
;	SXADDPAR, header, 'PSF_SIG', pass2_sigma, "Detection threshold in sigma of sky"
;	SXADDPAR, header, 'PSF_SAT', FIT_sat_limit_total, "Saturation Limit for allowed stars"
;	;
;	image_header = header
	;
	print_text,Event, "Finished fitting "+STRING(format='(I)',nstar)+" stars. Saving photometry in phot_structure"
	;
	hwp_angle = [0,22.5,45.0,67.5,90.0,112.5,135.0,157.5,180.0,202.5,225.0,247.5,270.0,$
					292.5,315.0,337.5]
	;
	;	get X, Y offsets to detector position out of the header
	;
	x_det_off = sxpar(header, 'MPP_XOFF', count=xcount)
	y_det_off = sxpar(header, 'MPP_YOFF', count=ycount)
	;
	if(xcount ne 1 or ycount ne 1) then begin
		print_text, Event, 'ERROR - X,Y detector offsets not found in FITS header. Stopping'
		print, 'ERROR - X,Y detector offsets not found in FITS header. Stopping'
		stop
	endif
	;
	;	sort by RA (reverse X)
	;
	if(nstar gt 0) then begin
		ind = REVERSE(SORT(x_final))
		;
		star_struct = {	id:					    	0L, $
							x:						0.0d0, $	; x position in coadded image
							y:						0.0d0, $
							detector_x:				0L,    $	; star location on Mimir detector
							detector_y:				0L,    $
							ra:						0.0d0, $	; J2000
							dec:					0.0d0, $
							mag:					0.0d0, $
							errmag:					0.0d0, $
							ierrmag:				0.0d0, $	; space for Stokes I way to estimate uncerts 20120321 DPC
							sky:					0.0d0, $
							psf_group:					0L, $
							;
							;	polarimetry data
							;
							pol_flag:			        0L, $	if zero, no pol present
							hwp_angle:			fltarr(16), $	; cuz I'm lazy...
							hwp_mag:			dblarr(16), $
							hwp_magerr:			dblarr(16)  $
						}
		phot_struct = REPLICATE(star_struct,nstar)
		;
		;	pull astro information
		;
		extast, header, astr_struct, noparams
		;
		;	compute ra, dec for each star
		;
		xy2ad, x_final, y_final, astr_struct, ra, dec
		;
		;	load structure
		;
		for is = 0, nstar-1 do begin
			phot_struct[is].id = is
			phot_struct[is].x = x_final[ind[is]]
			phot_struct[is].y = y_final[ind[is]]
			phot_struct[is].detector_x = x_final[ind[is]] + x_det_off
			phot_struct[is].detector_y = y_final[ind[is]] + y_det_off
			phot_struct[is].ra = ra[ind[is]]
			phot_struct[is].dec = dec[ind[is]]
			phot_struct[is].mag = mag_final[ind[is]]
			phot_struct[is].errmag = errmag_final[ind[is]]
			phot_struct[is].ierrmag = sqrt(extra_noise^2 + 	errmag_final[ind[is]]^2)  ; RSS of phot with noise floor
			phot_struct[is].sky = sky_final[ind[is]]
			phot_struct[is].psf_group = grp_final[ind[is]]
			;
			;	and polarimetry
			;
			phot_struct[is].pol_flag = 0
			for ihwp = 0, 15 do begin
				phot_struct[is].hwp_mag[ihwp] = -99.99
				phot_struct[is].hwp_magerr[ihwp] = 99.99
				phot_struct[is].hwp_angle[ihwp] = hwp_angle[ihwp]
			endfor
		endfor
		n_stars = nstar
		;
		end_of_time = SYSTIME(/SECONDS)
		print_text,Event, "Done processing  in "+STRING(format='(I)',end_of_time - start_of_time)+" seconds"
		;
		status = 0			; all worked !
		;
		group_colors[0] = 'palegreen light'
		WCT_LED_stack, LED1_wid, group_colors
		;
	endif else begin
		status = -1
		str = "No stars fit (empty group3 vector) for "+FILE_BASENAME(deep_path)
		print_text, Event, "ERROR - "+str
		SAll_add_to_history, Event, h_wid, 8, str
		goto, S8PSFF_end
	endelse
	;
	;-----------------------------------------------------------------------||
	;	save region file of all Aperture-fit star locations for this image
	;
	if(region_flag eq 1) then begin
		region_dir = (*s_ptr).analysis_dir + 'S8_Deep_PSF_Fitting' + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		;
		file_reg = FILE_BASENAME(deep_path,".fits") + '_J_final_Aper_phot.reg'
		region_path = region_dir + file_reg
		;
		n_xy_psf = intarr(1,1)
		n_xy_psf[0,0] = nstar
		x_psf = fltarr(1,1,nstar)
		y_psf = fltarr(1,1,nstar)
		;
		x_psf[0,0,0:(nstar-1)] = phot_struct[0:(nstar-1)].x
		y_psf[0,0,0:(nstar-1)] = phot_struct[0:(nstar-1)].y
		;
		S8_Save_PSF_stars_region_file, Event, region_path, 2.5*mean_FWHM, x_psf, y_psf, n_xy_psf, debug, COLOR='cyan'
	endif
	;
	S8PSFF_end:
	;
;	stop
	;
	cd, old_directory
end
