pro S8_HWP_PSF_photometry, Event, ihwp, hwp_path, LED1_wid, group_colors, $
								dif_mag, n_phot_stars, phot_struct,x_psf, $
								y_psf, n_xy_psf, pol_status, $
								PSF_model
	;
	;	puts all HWP coadds on same exposure basis before finding stars
	;
	;	does PSF-fitting photometry for one coadded HWP image
	;
	;	uses pre-selected star list to try to fit star brightnesses
	;	uses post-fitting matching to be sure to put the right star's info back
	;			into the data structure in the right place
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_Struct
	d_ptr = PTR_Struct.PTR_Data
	s_ptr = PTR_Struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	;
	debug = 0
	pol_status = -1
	;
	region_flag = (*w_ptr).S8_save_regions_flag
	analysis_dir = (*s_ptr).analysis_dir
;	step_dir_text = 'S8_Deep_PSF_Fitting'
	;
	h_wid = (*w_ptr).S8_history_textbox_wid
	nstar_wid = (*w_ptr).S8_nstars_textbox_wid
	;
	flatten_pre_flag = (*w_ptr).S8_flatten_images_flag
	calibrator_flag = (*w_ptr).S8_calibrator_flag	; if set, shut down extra bump for merging stars
	;
	method = 'Aper'							; method of final photometry: "Aper" or "PSF"
	;
	group_colors[ihwp+1] = 'pink'
	WCT_LED_stack, LED1_wid, group_colors
	;
	;	number of grid zones, sigma level - restart to the deep values after modification at the group or user level
	;
	ng_start = (*w_ptr).S8_textbox_group_start[1]
	ng = ng_start
	nsig_start = (*w_ptr).S8_textbox_group_start[0]
;	nsig = (nsig_start - 1.0) > 4.0					; one smaller than deep fit, but trapped at 4
	;
	(*w_ptr).S8_textbox_group_value[0] = nsig_start	; put sigma to screen; but don't reset group value
	S8_Show_Options_Textbox, Event, 0, nsig_start
	;
	;	for these coadded images, the PSF-creation sat limit is 6,000 ADU per image in the coadd,
	;		and for final fitting = 10,000 ADU per image
	;
	PSF_sat_limit = 7500L
	FIT_sat_limit = 13000L
	;
;	save_phot_regions = 1					; 1 to save region file for each HWP; 0 to skip
	;
	resid_flag = (*w_ptr).S8_residuals_mode
	save_intermediate_images = resid_flag			; 1 to save some debugging images; 0 for normal operations
	;
	simple_PSF_assist = 1					; 1 for no tight-groups, 0 to enable tight-group simultaneous fitting
	;
	debug = 0
	;
	start_of_time = SYSTIME(/SECONDS)
	;
	;	also, no maximum number of stars
	;
	max_stars = 1.0e10
	max_in_group = 150
	;
	;	read in the image
	;
	image_data = readfits(hwp_path, image_header, /SILENT)
	image = image_data
	;
	header = image_header
	;
	;	flatten image for ease of star detection
	;
	cd, current=old_directory
	if(save_intermediate_images eq 1) then begin
		file_out_image = dialog_pickfile(title='Original Image before star removal')
		writefits, file_out_image, image_data, image_header
		starting_directory = FILE_DIRNAME(file_out_image)
		cd, starting_directory
	endif
	;
	if(flatten_pre_flag ne 0) then begin
		image_data = model_background(image_data)
		if(save_intermediate_images eq 1) then begin
			file_out_flat = dialog_pickfile(title='Image after background flattening')
			writefits, file_out_flat,  find_image, image_header
		endif
	endif
	;
	;	get image size
	;
	image_size = SIZE(image_data)
	xsize = image_size[1]
	ysize = image_size[2]
	;
	xoff = abs(sxpar(image_header, 'MPP_XOFF'))
	yoff = abs(sxpar(image_header, 'MPP_YOFF'))
	;
	;	and number of images this image is scaled to
	;
	n_total_images = long(sxpar(image_header,"MPP_NSCL"))
	;
	;	if none, try another variable
	;
	if(n_total_images le 0) then n_total_images = long(sxpar(image_header,"MPP_DADD"))
	;
	if(n_total_images le 0) then begin
		pol_status = -1
		str = "Cannot determine N_TOTAL_IMAGES for "+FILE_BASENAME(hwp_path)
		print_text, Event, "ERROR - "+str
		SAll_add_to_history, Event, h_wid, 8, str
		goto, S8HWP_PSFF_end
	endif
	;
	;	check for coadding of input images
	;
	coadds = sxpar(image_header, 'NUMCOADD')
	if(coadds eq 0) then coadds = 1
	;
	PSF_sat_limit_total = double(PSF_sat_limit * n_total_images * coadds)
	FIT_sat_limit_total = double(FIT_sat_limit * n_total_images * coadds)
	;
	;	recover aperture photometry constants
	;
	PP4_ARAD = SXPAR(header, "PP4_ARAD")
	PP4_REF0 = SXPAR(header, "PP4_REF0")
	PP4_REF1 = SXPAR(header, "PP4_REF1")
	PP4_CLEN = SXPAR(header, "PP4_CLEN")
	PP6_APRD = SXPAR(header, "PP6_APRD")
	;
	ap_rad = PP4_ARAD * PP6_APRD			; times FWHM
	clean_rad = PP4_CLEN					; times ap_rad times FWHM
	ref_ap = [PP4_REF0,PP4_REF1]			; times ap_rad times FWHM
	;
	;	we'll skip the overlap regions for star fitting
	;
	if(xsize gt 1024) then xborder = CEIL((xsize - 1024)/1.5) else xborder = 4
	if(ysize gt 1026) then yborder = CEIL((ysize - 1026)/1.5) else yborder = 9
	;
	;	extract astrometry info, if present
	;
	extast,header,astr,noparams			;  if noparams = -1, no astrometry present
	;
	center_in_textbox, nstar_wid, 'Build PSF grid'
	group_colors[ihwp+1] = 'plum'
	WCT_LED_stack, LED1_wid, group_colors
	;
	;-------------------------------------------------------------------------------------------------
	;		constants to pass
	;
	sat_limit = PSF_sat_limit_total
	print_text, Event, "NSCL, Sat limit = "+STRING(n_total_images, sat_limit)
	;
	sigma 				= 8.0								; set high threshold for PSF star selection
	save_region_files 	= region_flag						; 0 - none, 1 - all
	;
	fix_bad_pixels_flag = 1									; fix coadded images for finding, PSF building
	;
	ap_vec 				= [0.6,0.65,0.7,0.75,$
						   0.8,0.85,0.9,0.95,$
						   1.0,1.05,1.1,1.15]			; times FWHM times ap_rad
	n_ap 				= N_ELEMENTS(ap_vec)
	;
	fit_scale 			= 1.75					; times fwhm
	rad_scale 			= 3.00					; times fwhm + 3
	;
	ronois				= 3.1 * sqrt(n_total_images)	; effective readout noise (in ADU)
	;
	step_dir_text 		= 'S8_Deep_PSF_Fitting'+PATH_SEP()+'HWP_Image_Fits'
	ng_textbox_wid 		= (*w_ptr).S8_ng_textbox_wid
	nsig_textbox_wid 	= (*w_ptr).S8_nsig_textbox_wid
	analysis_dir		= (*s_ptr).analysis_dir
	image_pathname		= hwp_path
	;
	sig_mag_cut			= 5.0					; keep only mags with uncertainties smaller
	;
	;----------------------------------------------------------------------------------------------
	g_create_PSF_grid, Event, image_data, image_header, image_pathname, sigma, status, $
				NG=ng, SAT_LIMIT=sat_limit, skysig, mean_FWHM, find_image, xborder, $
				yborder, sky, clean_rad, ap_rad, phpadu, apr, skyrad, badpix, $
				bright_limit, rad_PSF, fit_rad, PSF_model, PSF_gauss, PSF_mag, $
				ronois, this_ng, n_ap, ap_vec, ref_ap, rad_scale, n_total_psf_stars, $
				x_psf, y_psf, mag_psf, save_region_files, flatten_pre_flag, $
				fix_bad_pixels_flag, fit_scale, ng_textbox_wid, nsig_textbox_wid, $
				step_dir_text, analysis_dir
	;
	;	if bad status, stop
	;
	if(status eq -99) then stop
	;
	find_image = image								; recover original image
	;
	;	override ng (but not nsig) with return value
	;
	ng = this_ng
	;
	(*w_ptr).S8_textbox_group_value[1] = ng				; updated "group" values here for passing to other HWP fits
	(*w_ptr).S8_textbox_group_value[0] = nsig_start		;
	;
	nsig = float(nsig_start)
	pass2_sigma 		= nsig
	short_ap_vec		= [0.5,0.6,0.7]* mean_FWHM * ap_rad		; for aperture phot later - these are good, central values
	skyrad 				= [0.8,2.1] * mean_FWHM * ap_rad
	;
	;	(don't) load the "group" buffers
	;
	;(*w_ptr).S8_textbox_group_value[*] = (*w_ptr).S8_textbox_value[*]
	;
	S8_Show_Options_Textbox, Event, 1			; optional final argument overrides common block look up
	S8_Show_Options_Textbox, Event, 0
	;
	;-----------------------------------------------------------------------||
	;
	center_in_textbox, nstar_wid, 'PSF grid ready'
	group_colors[ihwp+1] = 'skyblue'
	WCT_LED_stack, LED1_wid, group_colors
	;
	psf_build_time = SYSTIME(/seconds) - start_of_time
	;
;	stop
	;
	;---------------------------------------------------------------------------------------------------
	;
	;	at this point, the PSFs have been fully built and their 2D functional forms determined
	;
	;	now, do the PSF fitting photometry:
	;
	;		1. recover the star list from the Deep PSF photometry
	;		2. using DAO grouping, group stars
	;		3. PSF fit stars to improve magnitude estimates
	;		4. for each star on the star list:
	;			4.1. select all stars in list within 10 FWHMs, call these a group
	;			4.2. PSF fit and remove all these stars to get better photometry
	;			4.3. PSF remove all *except* the target star
	;			4.4. PSF fitting photometry on the target star
	;			4.6. move the target star photometry to the final phot list
	;			4.7. go on to next star on list
	;		5. when done, write to HWP-PHOT database
	;
	;--------------------------------------------------------------------------------------------------
	;
	badpix = [-1.0e4,1.4*FIT_sat_limit_total]
	bright_limit = 1.4*FIT_sat_limit_total	; to halt bright/saturated stars from being fit
	;
	;	STEP 1: recover Deep phot star list
	;
	; 			look in the phot structure for positions and fluxes of all stars below the FIT sat limit
	;-----------------
	;
	x = phot_struct[*].x
	y = phot_struct[*].y
;	mag = phot_struct[*].mag
;	phot_mag = mag						; save a copy of original magnitude in phot database
;	sky = phot_struct[*].sky
;	errap = phot_struct[*].errmag
;	phot_err = errap
	n_phot = N_ELEMENTS(x)
	;
	fwhm = mean_FWHM
	;
	print_text, Event, 'PASS 1: found '+STRING(n_phot)+' stars in phot data base'
	;
	;			try to do aperture photometry on all the stars
	;
	debug = 0
	;
	;	first, check the image at the position of each star - reject stars with deleted data
	;			to prevent g_aper from blowing up - DPC - 20100518
	;
	test_image = find_image(ROUND(x),ROUND(y))
	;
	ind_good = WHERE(test_image gt badpix[0] and test_image lt bright_limit,ngood,COMPLEMENT=ind_bad,NCOMPLEMENT=nbad)
	;
	if(region_flag gt 0) then begin
		;
		region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		;
		file_reg = FILE_BASENAME(hwp_path,".fits") + '_A_Pass_1_good_stars_green_too-bright_red_outside.reg'
		region_path = region_dir + file_reg
		;
		openw, lu_reg, region_path, /GET_LUN
		;
		;	put good stars in green, border in red
		;
		if(ngood gt 0) then begin
			for igood = 0, ngood-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x[ind_good[igood]]+1,$
												y[ind_good[igood]]+1,1.5*mean_FWHM, 'green'
			endfor
		endif
		;
		if(nbad gt 0) then begin
			for ibad = 0, nbad-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x[ind_bad[ibad]]+1,$
												y[ind_bad[ibad]]+1,1.5*mean_FWHM, 'red'
			endfor
		endif
		;
		FREE_LUN, lu_reg
		;
	endif
	;
	if(nbad gt 0) then begin
		print_text, Event, "Found "+string(nbad)+" stars with deleted or bright data"
		;
		x[ind_bad] = 0.0				; should turn off star as being on image edge
		y[ind_bad] = 0.0
	endif
	;
	g_aper,find_image,x,y,mag,errap,sky,skyerr,phpadu,short_ap_vec,skyrad,badpix,bright_limit,$
				SILENT=(1-debug),/NO_BAD_PIX_FIX, /EXACT
	;
	nst = N_ELEMENTS(x)
	;
	if(nst ne n_phot) then begin
		print, "ERROR - nst and n_phot do not match. stopping"
		stop
	endif
	;
	;	find the first aperture with good data for each star
	;
	for ist = 0, N_ELEMENTS(short_ap_vec)-1 do begin
		ind_ok = WHERE(abs(errap[*,ist]) le sig_mag_cut and errap[*,ist] ne 0.0 and $
					FINITE(mag[*,ist]) and mag[*,ist] gt -10. and mag[*,ist] lt 25.0 and $
					x ne 0.0, n_ok)
		if(n_ok gt 0) then begin
			errap[0,ist] = errap[ind_ok[0],ist]
			mag[0,ist] = mag[ind_ok[0],ist]
		endif else begin
			errap[0,ist] = 99.0
			mag[0,ist] = -31.0
		endelse
	endfor
	;
	ind_good = WHERE(mag[0,*] gt -10 and mag[0,*] lt 25,ngood,COMPLEMENT=ind_bad,NCOMPLEMENT=nbad)
	if(region_flag gt 0) then begin
		;
		region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		;
		file_reg = FILE_BASENAME(hwp_path,".fits") + '_B_Pass_1_Aper_good_green_bad_red.reg'
		region_path = region_dir + file_reg
		;
		openw, lu_reg, region_path, /GET_LUN
		;
		;	put good stars in green, border in red
		;
		if(ngood gt 0) then begin
			for igood = 0, ngood-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x[ind_good[igood]]+1,$
												y[ind_good[igood]]+1,1.5*mean_FWHM, 'green'
			endfor
		endif
		;
		if(nbad gt 0) then begin
			for ibad = 0, nbad-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x[ind_bad[ibad]]+1,$
												y[ind_bad[ibad]]+1,1.5*mean_FWHM, 'red'
			endfor
		endif
		;
		FREE_LUN, lu_reg
		;
	endif
	;
	errap = REFORM(errap[0,*])
	mag = REFORM(mag[0,*])
	;
	;	for the stars with good aperture photometry, compare to the photometry database
	;	to determine offset mag and then apply to all stellar mags (instead of insisting
	;	that aperture phot work for all stars...)
	;
	ind_good = WHERE(errap lt 0.1*sig_mag_cut, ngood)
	dmag = fltarr(ngood)
	for igood = 0, ngood-1 do begin
		knd = ind_good[igood]
		dmag[igood] = phot_struct[knd].mag - mag[knd]
	endfor
	zz = median_filtered_mean(dmag)
	;
	dmag_2_minus_Mim = zz[0]
	;
	;	offset PSF fit mags from phot to this image before doing new PSF fits
	;
	mags = phot_struct[*].mag - dmag_2_minus_Mim
	errap = phot_struct[*].errmag
	;sky = phot_struct[*].sky		; this sky is 16x too high, so turn it off
	;
	nstars1 = N_ELEMENTS(errap)
	;
	;	STEP 2: group all found stars into groups
	;
	print_text,Event, "PASS 1: grouping "+STRING(format='(I)',nstars1)+" stars"
	;
;	stop
	;
	rcrit0 = rad_PSF + fit_rad
;	if(ng le 2 and calibrator_flag ne 1) then rcrit = rcrit + fit_rad * (3 - ng)
	;
	rcrit = rcrit0
	;
	pass1_group_loop = 0L
	star_id = indgen(nstars1)
	;
	regroup_pass1:
	;
	group, Event, x, y, rcrit, group_vector
	;
	;	find the largest group. If its size is > max_in_group stars, reduce rcrit and retry
	;
	ngroups = N_ELEMENTS(group_vector)
	this_max_stars = MAX(HISTOGRAM(group_vector,MIN=0))
	if(this_max_stars gt max_in_group) then begin
		print_text,Event, "WARNING: PASS 1 had a group with "+STRING(format='(I)',this_max_stars)+" stars, where only "+STRING(max_in_group, format='(I4)')+" allowed"
		;print, "   reducing rcrit and retrying"
		if(pass1_group_loop lt 20) then begin
			rcrit = 0.95 * rcrit
			pass1_group_loop++
			print_text, Event, "  new rcrit = "+STRING(format='(F6.1)',rcrit)+"  Restarting PASS1 grouping"
			goto, regroup_pass1
		endif else print_text, Event, "WARNING: PASS1 grouping radius decremented 20 times, but still found large groups"
	endif
	;
	;	STEP 3: For all groups, form PSFs, fit group stars
	;
	n_groups = N_ELEMENTS(UNIQ(group_vector,SORT(group_vector)))
	print_text, Event, "PASS 1: star list has "+STRING(ngroups)+" stars spread over "+STRING(n_groups)+" groups"
	;
	print_text, Event, 'PASS 1: starting PSF fitting and removing stars'
	;
	ind_x = WHERE(mags[0,*] lt -10.0,n_neg)
	if(n_neg gt 0) then begin
		print,"ERROR - mag brighter than -10 in HWP PSF Fitting - stopping'
		stop
	end
	;
	center_in_textbox, nstar_wid, 'PASS 1 PSF Ph.'
	group_colors[ihwp+1] = 'skyblue'
	WCT_LED_stack, LED1_wid, group_colors
	;
	iplot=0
	isave=save_intermediate_images
	print=0
	message1 = 'file to save stellar print info'
	message2 = 'HWP Image after PASS 1 star removal'
	idpc=0
	errmag = errap
	;
	;	test for valid PSF_model, halt if none
	;
	if(N_ELEMENTS(PSF_model) gt 0) then begin
		var_type = SIZE(PSF_model,/TYPE)
		if(var_type eq 0) then begin
			print_text, Event, 'ERROR - PSF_model is undefined. Stopping'
			stop
		endif
	endif else begin
		print_text, Event, 'ERROR - PSF_model does not exist. Stopping'
		stop
	endelse
	;
	n_before_fit = N_ELEMENTS(x)
	x_pos_save = x
	y_pos_save = y
	;
	g_fit_and_remove_stars, this_ng, find_image, resid_image, x, y, mags, errmag, sky, $
			star_id, group_vector, PSF_model, PSF_gauss, PSF_rad, PSF_mag, sum_resid, $
			phpadu, ronois, fit_rad, PRINT=print, MESSAGE1=message1, IPLOT=iplot, $
			ISAVE=isave, MESSAGE2=message2, RESID_HEADER=resid_header, $
			RESID_FILENAME=resid_filename, idpc, status
	;
	if(region_flag eq 1) then begin
		region_dir = (*s_ptr).analysis_dir + step_dir_text + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		;
		file_reg = FILE_BASENAME(hwp_path,'.fits') + '_HWP' + STRING(ihwp,format='(i2.2)') + '_residuals.fits'
		region_path = region_dir + file_reg
		;
		writefits, region_path, resid_image
	endif
	;
	;	check run status
	;
	if(status eq -1) then begin
		str = "PSF fitting failed for "+FILE_BASENAME(hwp_path)
		print_text, Event, "ERROR - "+str
		pol_status = -99
		SAll_add_to_history, Event, h_wid, 8, str
		goto, S8HWP_PSFF_end
	endif
	;
	;	look for case of no good stars fit
	;
	test = N_ELEMENTS(group_vector)
	if(test eq 1) then begin
		if(group_vector eq -1) then begin
			;
			;	no stars were fit - quit remaining processing
			;
			pol_status = -1
			str = "No stars fit (empty group vector) for "+FILE_BASENAME(hwp_path)
			print_text, Event, "ERROR - "+str
			SAll_add_to_history, Event, h_wid, 8, str
			goto, S8HWP_PSFF_end
		endif
	endif
	;
	ind_PSF_good = WHERE(errmag le sig_mag_cut and FINITE(mags) and mags gt -10. and mags lt 25.0, n_good_PSF, $
					COMPLEMENT=ind_bad_ap, NCOMPLEMENT=nbad_ap)
	;
	if(region_flag gt 0) then begin
		;
		region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		;
		file_reg = FILE_BASENAME(hwp_path,".fits") + '_C_Pass_1_PSF_Fits_good_green_bad-ap_yellow_bad_red.reg'
		region_path = region_dir + file_reg
		;
		openw, lu_reg, region_path, /GET_LUN
		;
		;	put good stars in green, border in red
		;
		if(n_good_PSF gt 0) then begin
			for igood = 0, n_good_PSF-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x[ind_PSF_good[igood]]+1,$
												y[ind_PSF_good[igood]]+1,1.5*mean_FWHM, 'green'
			endfor
		endif
		;
		if(nbad_ap gt 0) then begin
			for ibad = 0, nbad_ap-1 do begin
				printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x[ind_bad_ap[ibad]]+1,$
												y[ind_bad_ap[ibad]]+1,1.5*mean_FWHM, 'yellow'
			endfor
		endif
		;
		;
		;	and any that didn't survive fitting
		;
		if(n_good_PSF ne n_before_fit) then begin
			print_text, Event, "PASS 1: Lost "+STRING(n_before_fit - n_good_PSF)+" stars in the PSF fitting"
			;
			;	find the ones that differ.
			;
			for i_start = 0, n_before_fit-1 do begin
				dx = x - x_pos_save[i_start]
				dy = y - y_pos_save[i_start]
				dr = dx * dx + dy * dy
				ind = SORT(dr)
				if(sqrt(dr[ind[0]]) gt 0.5*mean_FWHM) then begin	; found one that didn't match, put to reg file
					printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x_pos_save[i_start]+1,$
												y_pos_save[i_start]+1,1.5*mean_FWHM, 'red'
				endif
			endfor
		endif
		;
		FREE_LUN, lu_reg
		;
	endif
	;
	if(nbad_ap gt 0) then begin
		print_text, Event, 'PASS 1: lost '+STRING(nbad_ap, format='(I4)')+' stars in PSF fitting'
		;
;		stop
		;
	endif
	;
	if(n_good_PSF gt 0) then begin
		xmerge = x[ind_PSF_good]
		ymerge = y[ind_PSF_good]
		magmerge = mags[ind_PSF_good]
		errmagmerge = errap[ind_PSF_good]
		skymerge = sky[ind_PSF_good]
		;
		center_in_textbox, nstar_wid, STRING(n_good_PSF,format='("P1:",i4)')
		group_colors[ihwp+1] = 'orange'
		WCT_LED_stack, LED1_wid, group_colors
		;
		;	select stars that were able to be fit
		;
		n_into_fit = N_ELEMENTS(x)
		nstars = n_good_PSF
		;
	endif else begin
		pol_status = -1
		str = "No stars with err_mag < "+STRING(sig_mag_cut,format='(F6.2)')+" mag for "+FILE_BASENAME(hwp_path)
		print_text, Event, "ERROR - "+str
		SAll_add_to_history, Event, h_wid, 8, str
		goto, S8HWP_PSFF_end
	endelse
	;
	print_text, Event,'PASS 1: completed PSF fitting photometry for '+STRING(nstars)+' stars'
	;
;	stop
	;----------------------------------------------------------------
	;
	;	STEP 2: PSF-assisted Aperture photometry
	;
	print_text,Event, 'PASS 2: starting PSF-assisted PSF fitting photometry'
	;
	group_colors[ihwp+1] = 'slateblue'
	WCT_LED_stack, LED1_wid, group_colors
	;
	n_merge = nstars
	;
	;	check for dupes, just in case
	;
	if(ng le 2 and calibrator_flag ne 1) then bump = 1.5 * (3 - ng) else bump = 1.0
	;
	fwhm_test = mean_FWHM * bump
	rad1 = (0.5*mean_FWHM * bump) > 1.5
	rad2 = (0.75*mean_FWHM * bump) > 2.5
	rad3 = mean_FWHM * bump
	rad4 = 2.5 * mean_FWHM * bump
	rad5 = rad_scale * mean_FWHM * bump
	;
	;	sort by x
	;
	ind_sort = SORT(xmerge)
	xmerge = xmerge[ind_sort]
	ymerge = ymerge[ind_sort]
	magmerge = magmerge[ind_sort]
	errmagmerge = errmagmerge[ind_sort]
	skymerge = skymerge[ind_sort]
	;
;	print_text, Event, 'PASS 2: debug - fit_rad, fwhm = '+STRING(fit_rad, mean_FWHM)
	;
	for istar = 0, n_merge-2 do begin
		;
		restart_jstar_loop:
		;
		for jstar = istar+1, n_merge-1 do begin
			dx = xmerge[jstar] - xmerge[istar]
			if(dx gt (rad5+3)) then goto, dupe_i_end_3
			dy = ymerge[istar] - ymerge[jstar]
			if(abs(dy) gt (rad5+3)) then goto, dupe_j_end_3
			;
			if(magmerge[jstar] lt -20.0) then goto, dupe_j_end_3
			if(magmerge[istar] lt -20.0) then goto, dupe_i_end_3
			;
			dm = abs(magmerge[istar] - magmerge[jstar])
			;
			rad = sqrt(dx * dx + dy * dy)
			;
			merge = 0
			if(rad le rad5 and dm gt 7.0) then merge = 1
			if(rad le rad4 and dm gt 5.0) then merge = 1
			if(rad le rad3 and dm gt 3.5) then merge = 1
			if(rad le rad2 and dm gt 2.0) then merge = 1
			if(rad le rad1) then merge = 1
			;
			if(merge eq 1) then begin
				;
				;	found a dupe, merge the stars
				;
				F_i = 10^(-0.4*magmerge[istar])
				F_j = 10^(-0.4*magmerge[jstar])
				F_total = F_i + F_j
				mag_total = -2.5*alog10(F_total)
				;
				;	weight x, y by fluxes
				;
				mean_x = (xmerge[istar]*F_i + xmerge[jstar]*F_j)/F_total
				mean_y = (ymerge[istar]*F_i + ymerge[jstar]*F_j)/F_total
				;
				if(magmerge[istar] lt magmerge[jstar]) then begin
					magmerge[istar] = mag_total
					xmerge[istar] = mean_x
					ymerge[istar] = mean_y
					magmerge[jstar] = -49.0d0
				endif else begin
					magmerge[jstar] = mag_total
					xmerge[jstar] = mean_x
					ymerge[jstar] = mean_y
					magmerge[istar] = -49.0d0
				endelse
				goto, restart_jstar_loop
				;
;				stop
			endif
			;
			dupe_j_end_3:
		endfor
		dupe_i_end_3:
	endfor
	;
	;		remove any dupes found
	;
	ind_nodupe = WHERE(abs(magmerge + 49.0d0) gt 1.0,nodupes,COMPLEMENT=ind_dupe,NCOMPLEMENT=ndupe)
	if(nodupes ne n_merge) then begin
		print_text, Event, 'PASS 2: Removing '+STRING(n_merge - nodupes)+' duplicate stars from list'
		if(region_flag gt 0) then begin
			;
			region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
			test_dir = FILE_INFO(region_dir)
			if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
			;
			file_reg = FILE_BASENAME(hwp_path,".fits") + '_D_Dupes_Removed_red_is_dupe.reg'
			region_path = region_dir + file_reg
			;
			openw, lu_reg, region_path, /GET_LUN
			;
			if(ndupe gt 0) then begin
				for ibad = 0, ndupe-1 do begin
					printf, lu_reg, FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',xmerge[ind_dupe[ibad]]+1,$
													ymerge[ind_dupe[ibad]]+1,1.5*mean_FWHM, 'red'
				endfor
			endif
			;
			FREE_LUN, lu_reg
			;
		endif
		xmerge = xmerge[ind_nodupe]
		ymerge = ymerge[ind_nodupe]
		magmerge = magmerge[ind_nodupe]
		skymerge = skymerge[ind_nodupe]
		errmagmerge = errmagmerge[ind_nodupe]
		n_merge = N_ELEMENTS(xmerge)
	endif
	;---------------------------------------------------------------
	;
	;	save as a region file the star locations that are not dupes
	;
;	test_dir = FILE_INFO(region_dir)
;	if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
;	;
;	file_reg = FILE_BASENAME(hwp_path) + '_P2_1_HWP' + STRING(ihwp,format='(i2.2)') + '_region.reg'
;	region_path = region_dir + file_reg
;	;
;	n_xy_psf = intarr(1,1)					; to save for region file
;	x_psf = fltarr(1,1,nodupes)
;	y_psf = fltarr(1,1,nodupes)
;	n_xy_psf[0,0] = nodupes
;	x_psf[0,0,*] = xmerge
;	y_psf[0,0,*] = ymerge
;	S8_Save_PSF_stars_region_file, Event, region_path, 0.33*rad_PSF, x_psf, y_psf, n_xy_psf, debug
	;
;	stop
	;----------------------------------------------------------------
	;
	;	group tight groups
	;
	nstars3 = N_ELEMENTS(xmerge)
	star_id = indgen(nstars3)
	;
	;	regroup the final star list, looking for very tight doubles (rcrit ~< fit_rad)
	;
	if(simple_PSF_assist eq 0) then begin
		rcrit = fit_rad
		group, Event, xmerge, ymerge, rcrit, group_vector4
;		save_group4 = group_vector4
		ngroups = N_ELEMENTS(UNIQ(group_vector4,SORT(group_vector4)))
		print_text,Event,'PASS 2: Found '+STRING(ngroups)+' tight groups among '+STRING(nstars3)+' stars'
	endif else begin
		group_vector4 = indgen(N_ELEMENTS(xmerge))
		ngroups = nstars3
		print_text, Event, 'PASS 2: Doing Simple PSF-Assist w/o tight groups on '+STRING(nstars3)+' stars. '
	endelse
	;
	;		Step 3: For each tight group, collect nearby stars, fit and remove
	;
	dx10 = (clean_rad * ap_rad * mean_FWHM > (rad_scale * mean_FWHM)) / 2.0
	dx10 >= 1.25*skyrad[1]	; at least as large as outer limit of sky radius
	dy10 = dx10
	dx10_2 = (dx10) * (dx10)
	;
	x_final = dblarr(20000)
	y_final = dblarr(20000)
	mag_final = dblarr(20000)
	sky_final = dblarr(20000)
	errmag_final = dblarr(20000)
	grp_final = intarr(20000)
	;
	nstar = 0L
	;
;	resid_save_image = find_image
	test_grp = -1
	;
	;		5 percent steps
	;
	n5 = CEIL(ngroups / 20.0)
	color = 'goldenrod light'
	;
	n_gauss_good = 0L
	n_gauss_binary = 0L
	n_gauss_bad = 0L
	;
	;	determine sky noise and detection threshold for passing to gaussian tester
	;
	g_sky, find_image, skymode, skysig, mfm_mean, mfm_stdev, /SILENT
	hmin = nsig * mfm_stdev
	;
	if(region_flag gt 0) then begin
			;
			region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
			test_dir = FILE_INFO(region_dir)
			if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
			;
			file_reg = FILE_BASENAME(hwp_path,".fits") + '_E_Gaussfit_Classify_green_good_yellow_double_red_bad.reg'
			region_path = region_dir + file_reg
			bad_star_path = region_dir + FILE_BASENAME(hwp_path,".fits") + '_bad_pass_2_stars.dat'
			;
			openw, lu_reg, region_path, /GET_LUN
			openw, lu_bad_stars, bad_star_path, /GET_LUN
	endif
	;
	print_text, Event, 'PASS 2: NGROUPS = '+STRING(ngroups,format='(i5)')
	;
	for igrp = 0, ngroups-1 do begin
		;
		if((igrp+1) eq n5 * fix((igrp+1) / n5)) then begin
			fraction = 5 * ((igrp+1) / n5)
			center_in_textbox, nstar_wid, 'P2: '+STRING(fraction,format='(I3)')+'%'
			if(group_colors[ihwp+1] eq 'goldenrod light') then begin
				group_colors[ihwp+1] = 'goldenrod dark'
			endif else group_colors[ihwp+1] = 'goldenrod light'
			WCT_LED_stack, LED1_wid, group_colors
		endif
		;
		;	find mean X, Y centers of this group
		;
		ind_strs = WHERE(group_vector4 eq igrp,nstrs)
		;
		this_x = [TOTAL(xmerge(ind_strs))/float(nstrs)]
		this_y = [TOTAL(ymerge(ind_strs))/float(nstrs)]
		this_sky = [TOTAL(skymerge[ind_strs])/float(nstrs)]
		this_mag = [magmerge[ind_strs[0]]]
		;
		;	STEP 2.1. collect all stars within ~8 FWHM of center star
		;
		n_10 = 0L
		;
		xfit = dblarr(1000)
		yfit = dblarr(1000)
		magfit = dblarr(1000)
		skyfit = dblarr(1000)
		errmagfit = dblarr(1000)
		;
		if(igrp eq test_grp) then start_time = SYSTIME(/SECONDS)
		;
		dx = xmerge - this_x[0]
		dy = ymerge - this_y[0]
		dr = dx * dx + dy * dy
		ind_dr = SORT(dr)
		ind_close = WHERE(dr[ind_dr] lt dx10_2,nclose)
		;
		n_10 = nclose - 1
		ind_keep = ind_dr[ind_close]
		;
		;	if not close groups, then the fitting star is the first one on the
		;	list. If close groups, will have to revisit this...
		;
		x_10 = xmerge[ind_keep]
		y_10 = ymerge[ind_keep]
		sky_10 = skymerge[ind_keep]
		;
		xmin = ((FLOOR(MIN(x_10) - rad_PSF)) < (FLOOR(x_10[0] - dx10))) > 0
		xmax = ((CEIL(MAX(x_10) + rad_PSF)) > (CEIL(x_10[0] + dx10))) < (xsize-1)
		ymin = ((FLOOR(MIN(y_10) - rad_PSF)) < (FLOOR(y_10[0] - dy10))) > 0
		ymax = ((CEIL(MAX(y_10) + rad_PSF)) > (CEIL(y_10[0] + dy10))) < (ysize-1)
		;
		nx_phot = xmax - xmin + 1
		ny_phot = ymax - ymin + 1
		;
		;	select this portion of the original image, only
		;
		sub_image = find_image[xmin:xmax, ymin:ymax]
		;
		;	and load up the surrounding stars into the star subtraction arrays
		;
		if(n_10 gt 0) then begin
			x_sub = x_10[1:nclose-1] - xmin
			y_sub = y_10[1:nclose-1] - ymin
			mag_sub = magmerge[ind_keep[1:nclose-1]]
			sky_sub = skymerge[ind_keep[1:nclose-1]]
			errmag_sub = errmagmerge[ind_keep[1:nclose-1]]
		endif
		;
		;	and central star's coordinates in the sub-image
		;
		x_cen = x_10[0] - xmin
		y_cen = y_10[0] - ymin
		;
		;	STEP 2.3.: Remove the non-central stars from the image
		;
		PSF_residuals = g_make_PSF(ng, this_x[0], this_y[0], PSF_model, $
						PSF_gauss, PSF_rad, PSF_mag, sum_resid, status)
		;
		;	test for bad PSF
		;
		if(status eq -1) then begin
			print, "ERROR in Step 8: s2.3call to g_make_PSF returned with status=-1"
			pol_status = -1
			goto, S8HWP_PSFF_end
		endif
		;
		mkhdr, hdr, PSF_residuals         ;Create a minimal FITS header
		sxaddpar, hdr, 'PHPADU', phpadu, 'Photons per Analog Digital Unit'
		sxaddpar, hdr, 'RONOIS', ronois, 'Readout Noise'
		sxaddpar, hdr, 'PSFRAD', PSF_rad, 'Radius where PSF is defined (pixels)'
		sxaddpar, hdr, 'FITRAD', fit_rad, 'Fitting Radius'
		sxaddpar, hdr, 'PSFMAG', PSF_mag, 'PSF Magnitude'
	 	sxaddpar, hdr, 'GAUSS1', PSF_gauss[0], 'Gaussian Scale Factor'
		sxaddpar, hdr, 'GAUSS2', PSF_gauss[1], 'Gaussian X Position'
		sxaddpar, hdr, 'GAUSS3', PSF_gauss[2], 'Gaussian Y Position'
		sxaddpar, hdr, 'GAUSS4', PSF_gauss[3], 'Gaussian Sigma: X Direction'
		sxaddpar, hdr, 'GAUSS5', PSF_gauss[4], 'Gaussian Sigma: Y Direction'
		;
		;	report gaussians that are too narrow - DPC 20090307
		;
		if(PSF_gauss[3] lt 0.35) then begin
		 	print, "G_FIT_AND_REMOVE_STARS: Warning - sigx = ",PSF_gauss[3]," for group ",igrp
		endif
		if(PSF_gauss[4] lt 0.35) then begin
		 	print, "G_FIT_AND_REMOVE_STARS: Warning - sigy = ",PSF_gauss[4]," for group ",igrp
		endif
		;
		sxaddpar, hdr, 'SUMRESID', sum_resid, 'Flux sum of resid array inside PSFRAD'
		;
		x_w_psf = this_x[0]
		y_w_psf = this_y[0]
		sxaddpar, hdr, 'X_W_PSF', x_w_psf, 'Flux-Weighted X position'
		sxaddpar, hdr, 'Y_W_PSF', y_w_psf, 'Flux-Weighted Y position'
		idsub = -1
		verbose = 0
		;
		;	list of all non-central stars to remove
		;
		;	remove non-central stars from the image, using their PSFs
		;
		if(n_10 gt 0) then begin
;			id_sub = intarr(N_ELEMENTS(x_sub))
			id_sub = -1							; says subtract all the stars on the list
			;
			;	remove their PSFs from the image
			;
			g_substar,sub_image,x_sub, y_sub, mag_sub,id_sub,PSF_residuals, hdr,VERBOSE = verbose
			if(igrp eq test_grp and save_intermediate_images eq 1) then begin
				file_out = dialog_pickfile(Title='HWP Image of removed neighbors, only')
				writefits, file_out, this_image, header
			endif
		endif
		;
		;  STEP 2.4.: Fit and remove central star
		;
		group_vector = intarr(nstrs)
		star_id = indgen(nstrs)
		;
		that_x = x_cen
		that_y = y_cen
		;
		;	if Aper phot do following:
		;
		if(method eq 'Aper') then begin
		;
			;	fit this star to be sure that its A/B and FWHM are similar to the image averages
			;		- if not, then this is probably a double and needs to be rejected somehow
			;
			if(n_gauss_good eq 0) then a_debug = 1 else a_debug = 0
			;
			;	kill everything outside of skyrad[1] (set to sky)
			;
			xp = findgen(nx_phot)
			yp = findgen(ny_phot)
			;
			gsub_image = sub_image
			;
			r2_test = (0.5*skyrad[1])^2
			r3_test = (skyrad[1])^2
			for ixx = 0, nx_phot-1 do begin
				for iyy = 0, ny_phot-1 do begin
					dr2 = (that_x - xp[ixx])^2 + (that_y - yp[iyy])^2
					if(dr2 gt r2_test) then gsub_image[ixx,iyy] = sky_10[0]
					if(dr2 gt r3_test) then sub_image[ixx,iyy] = sky_10[0]
				endfor
			endfor
			;
			;	also, backfill pixels far below sky
			;
			ind_zz = WHERE(gsub_image ne sky_10[0],nzz)
			if(nzz gt 0) then begin
				zz = median_filtered_mean(gsub_image[ind_zz])
				ind_low = WHERE((zz[0] - gsub_image)/zz[1] gt 4.0,nlow)
				if(nlow gt 0) then gsub_image[ind_low] = zz[0]
			endif
			;
			ind_yy = WHERE(sub_image ne sky_10[0],nyy)
			if(nyy gt 0) then begin
				yy = median_filtered_mean(sub_image[ind_yy])
				ind_ylow = WHERE((yy[0] - sub_image)/yy[1] gt 4.0, nylow)
				if(nylow gt 0) then sub_image[ind_ylow] = yy[0]
			endif
			;
			;	and remove high pixels in the outer 1/4 of the zone for the gaussian test
			;
			r4_test = (0.75*0.5*skyrad[1])^2
			for ixx = 0, nx_phot-1 do begin
				for iyy = 0, ny_phot-1 do begin
					dr3 = (that_x - xp[ixx])^2 + (that_y - yp[iyy])^2
					if(dr3 lt r2_test and dr3 ge r4_test) then begin
						if((gsub_image[ixx,iyy]-zz[0])/zz[1] gt 4.0) then begin
							gsub_image[ixx,iyy] = zz[0]
						endif
					endif
				endfor
			endfor
			;
			a_debug = 0
			threshold = hmin
			status = S8_gaussian_test_one_star(gsub_image, nx_phot, ny_phot, that_x, that_y, mean_FWHM, $
						a_debug, SIGMA=threshold)
			;
			if(status eq 0) then begin
				if(region_flag gt 0) then printf, lu_reg, $
							FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x_10[0]+1, y_10[0]+1,1.75*mean_FWHM, 'green'

				n_gauss_good++
				;
				this_apr = ap_rad * mean_FWHM
				skyrad = ref_ap * this_apr
				g_aper,sub_image,that_x,that_y,that_mag,that_errmag,that_sky,skyerr,phpadu,this_apr,$
					skyrad,badpix,bright_limit,$
					SILENT=(1-debug),/NO_BAD_PIX_FIX, /EXACT
			endif
			if(status eq 1) then begin
						if(region_flag gt 0) then printf, lu_reg, $
							FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x_10[0]+1, y_10[0]+1,1.75*mean_FWHM, 'yellow'
				n_gauss_binary++
			endif
			if(status eq -1) then begin
				if(region_flag gt 0) then printf, lu_reg, $
							FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x_10[0]+1, y_10[0]+1,1.75*mean_FWHM, 'red'
				n_gauss_bad++
			endif
			;
;			if(x_10[0] gt 670 and x_10[0] lt 680 and y_10[0] gt 395 and y_10[0] lt 405) then begin
;				print_text, Event, "Found the bad star"
;				qq = 0.0
;			endif
			;
		endif
		;
		if(igrp eq test_grp and simple_PSF_assist eq 0) then begin
			end_fit_time = SYSTIME(/seconds)
			print_text, Event, 'time to fit neighbors and one star = '+STRING(end_fit_time - start_fit_time)+' seconds'
			print_text, Event, 'time to find neighbors and fit them = '+STRING(end_fit_time - start_time)+' seconds'
			project_time = n_merge * (end_fit_time - start_time)
			print_text, Event, 'project completion of all stars in '+STRING(project_time)+' seconds'
		endif
		;
		;	STEP 2.5: Save the photometry to the final list, if it is good
		;
		if(status eq 0) then begin
			jnd_good = WHERE(that_errmag lt sig_mag_cut and FINITE(that_mag) and that_mag gt -10. and that_mag lt 25.,ngood,$
					COMPLEMENT=jnd_bad,NCOMPLEMENT=n_jnd_bad)		; better than 1.0 mag one sigma
			;
			;	save the rejected ones onto a file
			;
			if(region_flag eq 1 and n_jnd_bad gt 0) then begin
				for jbad = 0, n_jnd_bad-1 do begin
					x_bad = that_x[jnd_bad[jbad]] + xmin
					y_bad = that_y[jnd_bad[jbad]] + ymin
					mag_bad = that_mag[jnd_bad[jbad]]
					s_mag_bad = that_errmag[jnd_bad[jbad]]
					;
					printf, lu_reg, $
							FORMAT='("circle ",f8.1,1x,f8.1,1x,f4.1,"# color = ",a)',x_bad+1, y_bad+1,2.25*mean_FWHM, 'magenta'
					printf, lu_bad_stars, x_bad, y_bad, mag_bad, s_mag_bad, $
							FORMAT='(f8.1,1x,f8.1,f7.2,1x,f6.3)'
				endfor
			endif
			;
			if(ngood gt 0) then begin
				for igood = 0, ngood-1 do begin
					x_final[nstar] = that_x[jnd_good[igood]] + xmin
					y_final[nstar] = that_y[jnd_good[igood]] + ymin
					mag_final[nstar] = that_mag[jnd_good[igood]]
					;
					if(mag_final[nstar] lt -10.0) then begin
						print, 'ERROR - mag brighter than -10. Stopping '
						stop
					endif
					;
					sky_final[nstar] = that_sky[jnd_good[igood]]
					errmag_final[nstar] = that_errmag[jnd_good[igood]]
					grp_final[nstar] = igrp
					nstar++
				endfor
				;
			endif else begin
	;			print_text, Event, 'PASS 2: lost star at (x,y,mag) = '+STRING(this_x[0], this_y[0], this_mag[0])+' in final Aperture Photometry'
			endelse
		endif
		;
		;	STEP 2.6.: Go on to next star
		;
	endfor
	;
	if(region_flag gt 0) then FREE_LUN, lu_reg, lu_bad_stars
	;
	if(nstar ne nstars3) then print_text, Event, 'PASS 2: rejected '+STRING(nstars3 - nstar)+' star candidates in PSF-assisted Aperture Photometry'
	;
	print_text, Event, 'PASS 2: gaussian test of cleaned zones found '+$
		STRING(n_gauss_good, n_gauss_binary, n_gauss_bad, format='(I5.1,",",I5.1,",",I5.1)')+$
		" good, binary, bad stars"
	print_text, Event, 'PSF assisted Aperture photometry finished for '+STRING(nstar)+' stars'
	;
	;	if residuals to be saved, create file and save them
	;
	SXADDPAR, header, 'PSF_NG', ng, "Number of X, Y grid zones in psf build"
	SXADDPAR, header, 'PSF_SIG', nsig, "Detection threshold in sigma of sky"
	SXADDPAR, header, 'PSF_SAT', FIT_sat_limit_total, "Saturation Limit for allowed stars"
	;
	print_text,Event, 'Saving photometry in phot_structure, where '+STRING(n_phot_stars)+' stars reside'
	;
	;	but if no stars, dump out (20120404 DPC)
	;
	if(nstar eq 0) then begin
		;
		pol_status = -1
		str = "No stars survived for "+FILE_BASENAME(hwp_path)
		print_text, Event, "ERROR - "+str
		SAll_add_to_history, Event, h_wid, 8, str
		goto, S8HWP_PSFF_end
	endif
	;
	;	match these stars to those in the phot_structure
	;
	n_xy_psf = intarr(1,1)					; to save for region file
	x_psf = fltarr(1,1,nstar)
	y_psf = fltarr(1,1,nstar)
	;
	if(nstar gt 0) then begin
		for is = 0, nstar-1 do begin
			;
			;	match to the closest star in the phot_structure database
			;
			dmin = 1.0e4
			this_iph = -1
			for iph = 0, n_phot_stars-1 do begin
				dx = phot_struct[iph].x - x_final[is]
				dy = phot_struct[iph].y - y_final[is]
				dist = dx * dx + dy * dy
				if(dist lt dmin*dmin) then begin
					;
					;	test for whether the mag difference is reasonable (correct for calibration of coadd)
					;		- to within 3.0 mag
					;
					dmag = phot_struct[iph].mag - mag_final[is] + (3.0 - dif_mag)
					s_test = 0.1 > sqrt(errmag_final[is]^2 + phot_struct[iph].errmag^2)
					if(abs(dmag)/s_test le 20.0) then begin
						;
						;	and whether distance is within one FWHM
						;
						dist = sqrt(dist)
						if(dist lt fwhm) then begin
							;
							;	close enough
							;
							dmin = dist
							this_iph = iph
						endif
					endif
				endif
			endfor
			;
			if(this_iph ne -1) then begin
				;
				;	found match, load phot_struct entry there
				;
				phot_struct[this_iph].hwp_mag[ihwp] = mag_final[is]
				phot_struct[this_iph].hwp_magerr[ihwp] = errmag_final[is]
				phot_struct[this_iph].pol_flag = 1
				;
				x_psf[0,0,n_xy_psf[0,0]] = phot_struct[this_iph].x
				y_psf[0,0,n_xy_psf[0,0]] = phot_struct[this_iph].y
				n_xy_psf[0,0]++
				;
			endif else begin
				str = "No matching star in the Deep image catalog for HWP, star "+$
									STRING(ihwp,is, format='(I,1x,I)')+", probably OK to ignore"
				print_text, Event, "Warning -"+str
				SAll_add_to_history, Event, h_wid, 8,str
				str = "...X,Y, mag, magerr = "+STRING(x_final[is],y_final[is],mag_final[is],errmag_final[is],$
								format='(F7.2,1x,f7.2,1x,f7.3,1x,f7.3)')
				print_text, Event, str
				SAll_add_to_history, Event, h_wid, 8, str
				;
			endelse
			;
		endfor
		;
		group_colors[ihwp+1] = 'palegreen dark'
		WCT_LED_stack, LED1_wid, group_colors
		;
		print_text, Event, "Saved "+STRING(nstar,format='(I)')+" stars in phot struct for this hwp"
		;
		end_of_time = SYSTIME(/SECONDS)
		print_text,Event, "Done processing  in "+STRING(format='(I)',end_of_time - start_of_time)+" seconds"
		;
		pol_status = 0			; all worked !
		;
	endif else begin
		pol_status = -1
		str = "No stars fit (empty group3 vector) for "+FILE_BASENAME(hwp_path)
		print_text, Event, "ERROR - "+str
		SAll_add_to_history, Event, h_wid, 8, str
		goto, S8HWP_PSFF_end
	endelse
	;
	if(region_flag eq 1) then begin
		region_dir = (*s_ptr).analysis_dir + 'S8_Deep_PSF_Fitting' + PATH_SEP() + 'Region_Files' + PATH_SEP()
		test_dir = FILE_INFO(region_dir)
		if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
		;
		file_reg = FILE_BASENAME(hwp_path,'.fits') + '_E_Final_Phot_HWP' + STRING(ihwp,format='(i2.2)') + '_region.reg'
		region_path = region_dir + file_reg
		;
		x_psf = x_psf[0,0,0:(n_xy_psf[0,0]-1)]
		y_psf = y_psf[0,0,0:(n_xy_psf[0,0]-1)]
		S8_Save_PSF_stars_region_file, Event, region_path, 2.5*mean_FWHM, x_psf, y_psf, n_xy_psf, debug, COLOR='cyan'
	endif
	;
	S8HWP_PSFF_end:
end
