pro S8_PSF_fitting, Event, meta_flag, group_id, group_name, LED1_wid, $
		status
	;
	;	handles PSF-fitting of coadded image groups and meta-groups
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;---- debug ---
	;phot_status = 0
	;pol_status = 0
	;--------------
	;
	image_wid = (*w_ptr).S8_image_textbox_wid
	nstars_wid = (*w_ptr).S8_nstars_textbox_wid
	;
	save_PSF_models = (*w_ptr).S8_save_PSF_models_flag
	PSF_Model_Dir = ''
	;
	;	Clear LED1 to red with the number of groups to process
	;
	;	this is either 17, or if phot only flag set, 1
	;
	phot_only_flag = (*w_ptr).S8_phot_only_mode
	if(phot_only_flag) then count = 1 else count = 17
	;
	WIDGET_CONTROL, LED1_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "red"
	S8_LED_sensitize, Event, 1, 1
	;
	;	1. create names for deep and HWP files
	;
	coadd_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
	psf_dir = (*s_ptr).analysis_dir + 'S8_Deep_PSF_Fitting' + PATH_SEP()
	clean_group_name = replace_underscores(group_name)
	;
	deep_path = coadd_dir + clean_group_name + '_Deep_Coadd.fits'
	;
	hwp_path = strarr(16)
	for ihwp = 0, 15 do begin
		hwp_path[ihwp] = coadd_dir + clean_group_name + '_HWP_' + STRING(ihwp,format='(i2.2)') + '.fits'
	endfor
	;
	;	2. perform deep photometry, returning full list of info
	;
	group_colors[0] = 'blue'
	WCT_LED_stack, LED1_wid, group_colors
	;
	print_text, Event, "Starting PSF-Assisted Aperture Photometry of Deep Coadd image "+FILE_BASENAME(deep_path)
	;
	center_in_textbox, (*w_ptr).S8_image_textbox_wid, FILE_BASENAME(deep_path)
	;
	;	do the psf fitting photometry
	;
	S8_Deep_PSF_photometry, Event, deep_path, LED1_wid, group_colors, n_phot_stars, phot_struct, $
							x_psf, y_psf, n_xy_psf, rad_PSF, phot_status, phot_header, PSF_model
	;
	;	if save PSF models set, ask user about directory
	;
	if(save_PSF_models eq 1) then begin
		PSF_Model_Dir = dialog_pickfile(Title='Select/Create Directory for Saving PSF Models',/dir)
		;
		S8_Deep_PSF_Model = PSF_Model
		save, S8_Deep_PSF_Model, FILENAME = PSF_Model_Dir + PATH_SEP() + 'S8_Deep_PSF_Model.sav'
	endif
	;
	;	what band and mode was this? H-Pol, K-Pol, or J-Band, H-Band, or K-Band?	20130620 DPC
	;
	header = headfits(deep_path,/silent)
	filter = strtrim(sxpar(header,'FILTNME1'),2)
	case filter of									;regardless of filter, just call it extracted_Hmag
	  'H_HWP': pmode = 'H-Pol'
	  'K_HWP': pmode = 'K-Pol'
	  'H':pmode = 'H-Pol'				; legacy, probably
	  else:	begin
			  	filter2 = strtrim(sxpar(header,'FILTNME2'),2)
			  	case filter2 of
			  		'J': pmode = 'J-Band'
			  		'H': pmode = 'H-Band'
			  		'K': pmode = 'K-Band'
			  		'Ks': pmode = 'K-Band'
			  		else: begin
			  		    filter3 = strtrim(sxpar(header,'FILTNME3'),2)
			  				if(filter3 eq 'H2-On' or filter3 eq 'H2-Off') then begin
			  				   pmode = 'K-Band'
			  				endif else begin
			  				   print, 'No filter match - stopping'
                   stop
                endelse
				 		  end
				 endcase
		    end
	endcase
	;
	print_text, Event, "Finished Photometry of Deep Coadd image "+FILE_BASENAME(deep_path)+' of type '+pmode
	if(phot_status eq 0) then begin
		group_colors[0] = 'green'
	endif else begin
		group_colors[0] = 'white'
	endelse
	WCT_LED_stack, LED1_wid, group_colors
	;
	;	2.1. go get 2MASS & GLIMPSE data & match 2MASS to the psf results
	;		(GLIMPSE bootstraps off 2MASS by 2MASS name)
	;
;	stop
	;
	print_text, Event, "Merging in 2MASS PSC data"
	S8_Merge_2MASS, Event, phot_struct, two_mass_struct, dif_mag, glimpse_struct, pmode, status
	print_text, Event, "Returned from 2MASS merging"
	;
;	stop
	;
	;	2.4. if PHOTCAT option selected, create PHOTCAT deposit file
	;
	if( (*w_ptr).S8_PHOTCAT_mode) then begin
		print_text, Event, "Deep Photometry Data to be written as PHOTCAT file"
		;
		S8_Write_PHOTCAT_file, Event, deep_path, phot_struct, two_mass_struct, glimpse_struct, phot_header, pmode
		;
		group_colors[0] = 'green'
		WCT_LED_stack, LED1_wid, group_colors
	endif
	;
	if(phot_only_flag or (pmode ne 'H-Pol' and pmode ne 'K-Pol') ) then begin
		print_text, Event, "Phot-only mode selected, so skipping HWP image PSF-Fitting"
		goto, skip_pol
	endif
	;
	;	3. perform psf photometry for each HWP image
	;
	print_text, Event, "Starting PSF-Fitting Photometry for Coadded HWP images"
	;
	for ihwp = 0, 15 do begin
;	for ihwp = 6, 6 do begin
		;
		;help, phot_struct, /struct
		;
		group_colors[ihwp+1] = 'blue'
		WCT_LED_stack, LED1_wid, group_colors
		WIDGET_CONTROL, (*w_ptr).S8_image_textbox_wid, SET_VALUE=FILE_BASENAME(hwp_path[ihwp])
		;
		print_text, Event, "Starting PSF-Fitting for "+FILE_BASENAME(hwp_path[ihwp])
		;
		S8_HWP_PSF_photometry, Event, ihwp, hwp_path[ihwp], LED1_wid, group_colors, $
								dif_mag, n_phot_stars, phot_struct, x_psf, y_psf, n_xy_psf, pol_status, PSF_model
		;
		;	save PSF models if set
		;
		if(save_PSF_models eq 1) then begin
			;
			S8_HWP_PSF_Model = PSF_Model
			save, S8_HWP_PSF_Model, FILENAME = PSF_Model_Dir + PATH_SEP() + 'S8_HWP' + STRING(ihwp,format='(I2.2)') +'_PSF_Model.sav'
		endif
		;
		if(pol_status eq 0) then begin
			group_colors[ihwp+1] = 'green'
		endif else begin
			group_colors[ihwp+1] = 'white'
		endelse
		WCT_LED_stack, LED1_wid, group_colors
		;
	endfor
	;
	status = pol_status + phot_status
	goto, the_end
	;
	skip_pol:
	status = phot_status
	;
	the_end:
	;
	;	save the phot_structure, 2mass, trimmed_glimpse structures to disk, if selected
	;
	save_path = psf_dir + clean_group_name + '_phot_struct.sav'
	save, n_phot_stars, phot_struct, two_mass_struct, glimpse_struct, FILENAME=save_path
	;
	print_text, Event, "Saved "+FILE_BASENAME(save_path)+" to disk"
end