pro g_create_PSF_grid, Event, image_data, image_header, image_pathname, sigma, status, $
				NG=incoming_ng, SAT_LIMIT=sat_limit, skysig, mean_FWHM, find_image, xborder, $
				yborder, sky, clean_rad, ap_rad, phpadu, apr, skyrad, badpix, $
				bright_limit, rad_PSF, fit_rad, PSF_model, PSF_gauss, PSF_mag, $
				ronois, this_ng, n_ap, ap_vec, ref_ap, rad_scale, n_total_psf_stars, $
				x_psf, y_psf, mag_psf, save_region_files, flatten_pre_flag, $
				fix_bad_pixels_flag, fit_scale, ng_textbox_wid,nsig_textbox_wid,$
				step_dir_text, analysis_dir,LLQ=llq

;
;	program that will take an image and a nominal grid density
;	(ng) and develop the PSF(s) for the image
;
;	DPC	20100203	pulled in code from S4_PSF_fit.pro
;		20120601    many repairs and resetting apertures and sky annuli for bright stars with new fudge factors
;
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_Struct
w_ptr = PTR_struct.PTR_wid_id
;
if(KEYWORD_SET(llq)) then llq = 1 else llq = 0
;
rad_PSF_fudge = 0		; 20120601 - DPC - adds two more pixels radius to the computed PSF size
skyrad_fudge = 1.0		; 20120601 - DPC - scale factor to increase the sky zone annulus
;						; 20120603 - DPC - returned to unity to put change in all steps via S4
;
regions_flag = save_region_files
this_fix_bad_pixels_flag = fix_bad_pixels_flag

ng = ROUND(incoming_ng)
debug = 0
print_times = 0											; set to 1 for printing of subroutine run times
clean_PSF_flag = 1						; use '0' for no iterative cleaning
sparse_scale = 20					; set to ~20 to invoke rich histogram PSF star selection, else
;									;   the sparse setting uses all from brightest to faintest until enough found
start_time = SYSTIME(/SECONDS)
;
do_phot = 'Aper'					; final star mags computed via gaussians ('Gaus'), apertures ('Aper'),
faint_sigma = 4.00
;
badpix = 		[-1.0e4,2.0*sat_limit]	; 2x sat ought to allow most
bright_limit = 	sat_limit				; to halt bright/saturated stars from being used in PSF building
;
bad_pix_value = -1.0e6
;
image_size = SIZE(image_data)
xsize = image_size[1]
ysize = image_size[2]
;
image = image_data
;
border = 6					; pixels  in X, multiply by 3 for Y (to avoid ski jump)
;
;	Ignore the junk by making masking out extreme top and bottom - DPC - 20100109
;
if(xsize eq 1024 and ysize eq 1026) then begin
	image[*,0:8] = bad_pix_value
	image[*,1012:1025] = bad_pix_value
	xborder = border
	yborder = 3 * border
endif else begin
	xborder = CEIL((xsize - 1024)/2.)
	yborder = CEIL((ysize - 1024)/2.)
	image[*,0:(yborder-1)] = bad_pix_value
	image[*,(ysize-yborder):(ysize-1)] = bad_pix_value
	image[0:(xborder-1),*] = bad_pix_value
	image[(xsize-xborder):(xsize-1),*] = bad_pix_value
endelse
;
residual_image = image * 0.0
header = image_header
;
fwhm_pass = 0						; do two passes through finding & PSF building, to get fwhm right
;
;	extract astrometry info, if present
;
extast,header,astr,noparams			;  if noparams = -1, no astrometry present
;
;	STEP 1: determine mean sky, sky rms
;
if(this_fix_bad_pixels_flag eq 1) then begin
	print_text, Event, "Fixing bad pixels in image"
	first_time = systime(/seconds)
	find_image = fix_bad_pixels(image)
	;
	;	once fixed, turn off
	;
	this_fix_bad_pixels_flag = 0
	;
	if(print_times) then print_text, Event, "Time to fix bad pixels = "+STRING((systime(/seconds)-first_time))+" seconds"
endif else find_image = image
;
if(flatten_pre_flag eq 1) then begin
	print_text, Event, "Flattening image background"
	first_time = systime(/seconds)
	find_image = model_background(find_image, /MCS, /NOCOLF, /NOROWF)	; models, flattens background
	if(print_times gt 0) then print_text, Event, "Time to flatten image background = "+STRING((systime(/seconds)-first_time))+" seconds"
endif
;
;	debug = 0
;
first_time = systime(/seconds)
g_sky, find_image, skymode, skysig, mfm_mean, mfm_stdev, SILENT=(1-debug), LLQ=llq
if(debug ne 0) then print_text, Event, "Sky mean, stdev = " + STRING(mfm_mean, mfm_stdev) + " mode, skysig = " + STRING(skymode, skysig)
if(print_times gt 0) then print_text, Event, "Time to find mean sky of "+STRING(mfm_mean)+" was = "+STRING((systime(/seconds)-first_time))+" seconds"
;
sky = mfm_mean
;
skysig = mfm_stdev
;
print_text, Event, 'Mean, Stdev sky = '+STRING(sky, skysig, format='(f8.1,2x,f8.2)')
;
;	STEP 2: determine first guess at average FWHM
;
;	debug=0
;
first_time = systime(/seconds)
;
;	add test for narrow stars
;
i_merit = -1L
best_merit = -100.0
for i_narrow = 0, 1 do begin
	g_get_FWHM, image, find_image, skymode, skysig, mfm_mean, mfm_stdev, $
			mean_FWHM, sig_FWHM, sat_limit, debug, AB=ab, NSTARS=nFWstars, $
			NARROW = i_narrow, STATUS=FWstatus					; ab returns mean ab and sigma ab
	this_merit = alog10(float(nFWstars)) - alog10(sig_FWHM)
	if(this_merit gt best_merit) then begin
		best_merit = this_merit
		i_merit = i_narrow
	endif
endfor
;
if(i_merit ne -1L) then begin
	i_narrow = i_merit
	g_get_FWHM, image, find_image, skymode, skysig, mfm_mean, mfm_stdev, $
			mean_FWHM, sig_FWHM, sat_limit, debug, AB=ab, NSTARS=nFWstars, $
			NARROW = i_narrow, STATUS=FWstatus					; ab returns mean ab and sigma ab
endif else begin
	stop
endelse
;
if(print_times gt 0) then print_text, Event, "Time to measure FWHM = "+STRING((systime(/seconds)-first_time))+" seconds"
;
;	if no stars were measured for FWHM, then abort this
;
if(nFWstars eq 0 or FWstatus ne 0) then begin
	print_text, Event, 'ERROR - no stars survived FWHM measurement. Aborting this image'
	status = -99
	goto, GCPG_end
endif
;
print_text, Event, "Axis ratio A/B = "+STRING(ab[0],ab[1],format='(F6.2,"+/-",F6.2)')
;
fwhm = mean_FWHM
;
fwhm_loop:
;
; Set Parameters for Aperture Photometry and PSF Photometry
;
apr = 			[ap_rad*fwhm]
skyrad = 		ref_ap * ap_rad * fwhm 		;
phpadu = 		8.21
fit_rad = 		fit_scale*fwhm
rad_PSF = 		CEIL(rad_scale*fwhm)
;
rad_PSF += rad_PSF_fudge 	; 20120601 - DPC - for wider bright star skirt fitting
;fit_rad *= 0.8				; 20120601 - DPC - for better bright star phot
skyrad *= skyrad_fudge		; 20120601 - DPC - for larger sky comparison annulus
;
print_text, Event, "FWHM Pass "+STRING(fwhm_pass+1,format='(I2)')+": Mean FWHM, s_FWHM = "+STRING(format='(F8.2,2x,f8.2)',mean_FWHM, sig_FWHM)
;
;	STEP 3: find all the stars above 4 sigma, using the FWHM size just found
;
;--- set up limits - depends somewhat on FWHM - if narrow, must open sharpness limit
;
roundlim = [-1.0,0.90]
sharplim = [0.2,0.8]
;
if(mean_FWHM lt 3.0) then begin
	roundlim = [-0.9,0.8]
	sharplim = [0.15,0.9]
endif
;
fwhm = mean_FWHM
;
;	find all stars down to the faint star limit
;
this_hmin = faint_sigma * mfm_stdev
;
g_find, find_image, x_all, y_all, flux_all, sharp_all, roundness_all, this_hmin, fwhm, roundlim, $
		sharplim, /NO_BAD_PIX_FIX, SILENT=(1-debug)
x_all_save = x_all
y_all_save = y_all
flux_all_save = flux_all
sky_all = flux_all * 0.0
;
;	then begin finding the brighter stars
;
iloop=0
loop_start:
;
;		reload faint star list
;
x_all = x_all_save
y_all = y_all_save
flux_all = flux_all_save
sky_all = flux_all * 0.0
;
max_psf_stars = ng * ng * sparse_scale * 3
;
;	clear the reject star list counter
;
n_rej = 0
;
hmin = sigma * mfm_stdev
;
;	debug=0
;
;	find all stars down to hmin
;
;debug = 1
;
g_find, find_image, x, y, flux, sharp, roundness, hmin, fwhm, roundlim, sharplim, $
		/NO_BAD_PIX_FIX, SILENT=(1-debug);
;
;debug = 0
;
print_text, Event, "Found "+STRING(N_elements(x),format='(I4)')+" stars to "+STRING(sigma,format='(F5.1)')+$
		" sigma and "+STRING(N_elements(x_all),format='(I4)')+" stars to "+STRING(faint_sigma,format='(F5.1)')+" sigma"
;
;	keep two lists - (1) all stars above hmin
;
nstars = N_ELEMENTS(x)
;
;	use that list to identify as potential PSF candidates only those stars within non-border region
;
;	debug = 1
;
ind = WHERE(x gt xborder and x lt (xsize-xborder) and y gt yborder and y lt (ysize-yborder), good_stars, $
			COMPLEMENT=ind_reject, NCOMPLEMENT=reject_stars)
if(debug gt 0) then print_text,Event, "Number of non-border stars found = "+STRING(format='(I)',good_stars)
if(good_stars gt 0) then begin
	x_can = x[ind]
	y_can = y[ind]
	flux_can = flux[ind]
	;
	if(debug gt 0) then begin
		print, "Location A: Good PSF candidates X, Y, Flux "
		nt_can = N_ELEMENTS(x_can)
		for ican = 0, nt_can-1 do begin
			print, "             ",x_can[ican],y_can[ican],flux_can[ican]
		endfor
	endif
	;
endif
;
;	stop
;
;	keep the rejects from the bright star list on a new, reject list
;
if(reject_stars gt 0) then begin
	if(reject_stars eq 1) then begin
		x_rej = [x[ind_reject]]
		y_rej = [y[ind_reject]]
		flux_rej = [flux[ind_reject]]
		mags_rej = [flux[ind_reject]]		; for now
	endif else begin
		x_rej = x[ind_reject]
		y_rej = y[ind_reject]
		flux_rej = flux[ind_reject]
		mags_rej = flux[ind_reject]
	endelse
	n_rej = reject_stars
endif
;
if(debug gt 0) then print_text, Event, "bright stars avoiding buffer region = "+STRING(good_stars,format='(I5)')+", within buffer = "+STRING(reject_stars,format='(I5)')
;
;	do provisional aperture photometry to get rid of bad stars quickly
;
;this_apr =     [1.5*fwhm]
;this_skyrad = [1.5, 2.5]*fwhm
;
;	new set, to allow bright stars better sky fitting
;
this_apr =     [1.75]*fwhm
this_skyrad =  [3.5, 5.0]*fwhm
;
g_aper,image,x_can,y_can,mags_can,errap,sky,skyerr,phpadu,this_apr,this_skyrad,badpix,bright_limit,$
	SILENT=(1-debug), /NO_BAD_PIX_FIX, /EXACT
;
mags_can = REFORM(mags_can[0,*])
errap = REFORM(errap[0,*])
n_stars_aper = N_ELEMENTS(mags)
;
;	get rid of bad stars, by mag cut, errmag cut,
;		and sky and skyrms cuts  (added 20100721 - DPC)
;
test_mags_1 = mags_can gt -10.0 and mags_can lt 25.
test_mags_2 = errap gt 0.0 and errap lt 1.0
test_sky_1 = abs(sky - mfm_mean) lt 15.0*mfm_stdev
test_sky_2 = abs(sky) lt 2.0*abs(mfm_mean)
test_sky_3 = skyerr lt 5.0*mfm_stdev
;
ind_phot_good = WHERE(test_mags_1 and test_mags_2  $
		      and test_sky_1 and test_sky_2 and test_sky_3,phot_good_stars,$
		      COMPLEMENT=ind_phot_bad,NCOMPLEMENT=phot_bad_stars)
;
if(phot_good_stars gt 0) then begin
	x_can_new = x_can[ind_phot_good]
	y_can_new = y_can[ind_phot_good]
	flux_can_new = flux_can[ind_phot_good]
	mags_can_new = mags_can[ind_phot_good]
endif else begin
	print_text, Event, "ERROR - no stars survived initial photometry"
	stop
endelse
;
;	and put the "bad" stars on the reject list, so they aren't lost
;
if(phot_bad_stars gt 0) then begin
	if(n_rej eq 0) then begin
		x_rej = x_can[ind_phot_bad]
		y_rej = y_can[ind_phot_bad]
		flux_rej = flux_can[ind_phot_bad]
		mags_rej = mags_can[ind_phot_bad]
		n_rej = phot_bad_stars
	endif else begin
		x_rej = [x_rej,x_can[ind_phot_bad]]
		y_rej = [y_rej,y_can[ind_phot_bad]]
		flux_rej = [flux_rej,flux_can[ind_phot_bad]]
		mags_rej = [mags_rej,mags_can[ind_phot_bad]]
		n_rej += phot_bad_stars
	endelse
endif
;
;	test - if number of good_stars > max_stars, sort, trim, continue
;
;	debug=0
if(phot_good_stars gt max_psf_stars) then begin
	print_text, Event, "Found > max_psf_stars good stars, sorting and trimming list"
;	ind = REVERSE(SORT(flux_can))						; sort brightest to faintest
	ind = SORT(mags_can_new)
	ind_can = ind[0:(max_psf_stars-1)]
	x_can_2 = x_can_new[ind_can]
	y_can_2 = y_can_new[ind_can]
	flux_can_2 = flux_can_new[ind_can]
	mags_can_2 = mags_can_new[ind_can]
	if(debug gt 0) then begin
		print, "Location B: Trimmed PSF candidates X, Y, Flux "
		nt_can = N_ELEMENTS(x_can_2)
		for ican = 0, nt_can-1 do begin
			print, "             ",x_can_2[ican],y_can_2[ican],flux_can_2[ican]
		endfor
	endif
	;
	;	put the fainter remainders on the reject list
	;
	ind_rej = ind[max_psf_stars:(phot_good_stars-1)]
	;
	if(n_rej gt 0) then begin
		x_rej = [x_rej, x_can_new[ind_rej]]
		y_rej = [y_rej, y_can_new[ind_rej]]
		flux_rej = [flux_rej, flux_can_new[ind_rej]]
		mags_rej = [mags_rej, mags_can_new[ind_rej]]
		n_rej += (phot_good_stars - max_psf_stars)
	endif else begin
		x_rej = x_can_new[ind_rej]
		y_rej = y_can_new[ind_rej]
		flux_rej = flux_can_new[ind_rej]
		mags_rej = mags_can_new[ind_rej]
		n_rej = (good_stars - max_psf_stars)
	endelse
	;
	;	load the trimmed candiate list back into the candidate arrays
	;
	x_can_new = x_can_2
	y_can_new = y_can_2
	flux_can_new = flux_can_2
	mags_can_new = mags_can_2
	good_stars = max_psf_stars			;;;
endif
;
;	also test for too few stars
;
if(phot_good_stars lt sparse_scale * ng * ng) then begin
	;
	;	first, try to reduce sigma, but not below 5
	;
	if(sigma gt 5.0) then begin
		if(iloop lt 10) then begin
			;
			;	if nearly right number of stars, step by 0.5 mag, else by 1 mag or more
			;
			ratio = phot_good_stars / (sparse_scale * ng * ng)
			if(ratio lt 0.1) then dsig = 4.0 else dsig = 2.0
			if(ratio gt 0.2) then dsig = 1.0
			if(ratio gt 0.5) then dsig = 0.5
			sigma = (sigma - dsig) > 5.0
			print_text, Event, "Found too few good stars, decreasing sigma to "+STRING(sigma, format='(F6.1)')
			WIDGET_CONTROL, nsig_textbox_wid, SET_VALUE=STRING(sigma,format='(F6.1)')
			iloop++
			goto, loop_start
		endif
	endif
	;
	;	already sigma down to 5, try to lower ng
	;
	if(ng gt 1) then begin
		;
		;	what ng would give enough stars?
		;
		ng_min = FLOOR(sqrt(phot_good_stars / sparse_scale)) > 1
		ng = ng_min
		print_text, Event, "Too few good stars ("+STRING(phot_good_stars,format='(I)')+") so lowering NG to "+STRING(ng,format='(I2)')
		WIDGET_CONTROL, ng_textbox_wid, SET_VALUE=STRING(ng,format='(I2)')
		iloop++
		goto, loop_start
	endif
	;
endif
;
;	look for stars in the good list with deleted or saturated pixels - don't let them become PSF stars
;	20090405 - but, add them to the reject list
;
igood = 0L
dist = fit_scale * mean_FWHM
dist2 = dist / 2.
nstars = N_ELEMENTS(x_can_new)
;
nstars1 = nstars-1
for i = 0, nstars1 do begin
	xmin = 0 > ROUND(x_can_new[i] - dist)
	xmax = (xsize-1) < ROUND(x_can_new[i] + dist)
	ymin = 0 > ROUND(y_can_new[i] - dist)
	ymax = (ysize-1) < ROUND(y_can_new[i] + dist)
	nxa = xmax - xmin + 1
	nya = ymax - ymin + 1
	xaxis = findgen(nxa) + xmin + 0.005
	yaxis = findgen(nya) + ymin + 0.005
	xdist = x_can_new[i] - xaxis
	ydist = y_can_new[i] - yaxis
	;
	ibad = 0L
	nxa1 = nxa-1
	nya1 = nya-1
	for iy = 0, nya1 do begin
		for ix = 0, nxa1 do begin
			this_pix = image[(ix + xmin),(iy + ymin)]
			xydist = sqrt(xdist[ix]^2 + ydist[iy]^2)
			;
			;	if deleted data within fit_rad*FWHM, score 1x bad, if within 0.5 fit_rad*FWHM score 3x bad
			;
			if(xydist le dist) then begin
				if(this_pix eq -1.0e6) then ibad++
				if(this_pix ge sat_limit) then ibad +=5		; don't keep any saturated stars for PSF building
			endif
			;
			if(xydist le dist2) then begin
				if(this_pix eq -1.0e6) then ibad += 2
			endif
		endfor
	endfor
	;
	if(ibad ge 5) then begin				; e.g. two bad or any saturated pixels inside FWHM
		if(n_rej gt 0) then begin
;			if(n_rej eq 1) then begin
;				if(x_rej ne -1.0) then begin
					x_rej = [x_rej, x_can_new[i]]
					y_rej = [y_rej, y_can_new[i]]
					flux_rej = [flux_rej, flux_can_new[i]]
					mags_rej = [mags_rej, mags_can_new[i]]
		endif else begin
			x_rej = [x_can_new[i]]
			y_rej = [y_can_new[i]]
			flux_rej = [flux_can_new[i]]
			mags_rej = [mags_can_new[i]]
		endelse
		n_rej++
		;
	endif else begin
		x_can[igood] = x_can_new[i]
		y_can[igood] = y_can_new[i]
		flux_can[igood] = flux_can_new[i]
		mags_can[igood] = mags_can_new[i]
		igood++
	endelse
endfor
;
;	if none survived, then no candidates
;
if(igood gt 0) then begin
	x_can = x_can[0:(igood-1)]
	y_can = y_can[0:(igood-1)]
	flux_can = flux_can[0:(igood-1)]
	mags_can = mags_can[0:(igood-1)]
endif else begin
	print_text, Event, 'ERROR - No PSF candidate stars survived testing against bad pixels'
	status=-99
	goto, GCPG_end
endelse
;
;	debug = 1
;
if(debug gt 0) then begin
	print, "Location C: PSF candidates X, Y, Flux "
	nt_can = N_ELEMENTS(x_can)
	for ican = 0, nt_can-1 do begin
		print, "             ",x_can[ican],y_can[ican],flux_can[ican]
	endfor
endif
;
nstars = igood + n_rej			; total number of good and rejected bright stars
;
if(igood lt 10) then print_text, Event, "Warning - beginning g_aper with only "+STRING(nstars,format='(I)')+" good stars"
;
iloop_psf = 0L
;
if(n_rej gt 0) then begin
	x = [x_can, x_rej]
	y = [y_can, y_rej]
endif else begin
	x = x_can
	y = y_can
endelse
;
restart_PSF:
;
star_id = indgen(nstars)
;
;	STEP 4: perform aperture photometry on candidate and reject stars
;
;	** 20100430 - DPC - must use fairly tight apertures to allow good mags
;		for neighboring (overlapping) stars
;
;       ** 20100520 - DPC - do this on the bad-pixel-fixed image to recover best
;		guesses for magnitudes for all stars
;
;this_apr =    [1.5*fwhm]			- use the same values as above
;this_skyrad = [1.5, 2.5]*fwhm
;
g_aper,find_image,x,y,mags,errap,sky,skyerr,phpadu,this_apr,this_skyrad,badpix,bright_limit,$
	SILENT=(1-debug), /NO_BAD_PIX_FIX, /EXACT
;
mags = REFORM(mags[0,*])
errap = REFORM(errap[0,*])
n_stars_aper = N_ELEMENTS(mags)
if(n_stars_aper lt 10) then print_text, Event, "Warning - left g_aper with only "+STRING(n_stars_aper,format='(I)')+" stars"
;
;	which of the PSF candidate stars also have aperture mag undertainties under 0.3 mag?
;
;	stop
;
ind_20 = WHERE(errap[0:(igood-1)] le 0.3 and errap[0:(igood-1)] gt 0.0 $
			and FINITE(mags[0:(igood-1)]),count_20)
;
if(debug gt 0) then print_text, Event, "PSF candidate stars with photometry uncer. < 0.3 mag = "+$
	STRING(format='(I)',count_20)
;
;	debug = 1
;
if(debug gt 0) then begin
	print, "Location D: After g_aper PSF candidates X, Y, Mag, sky "
	nt_can = count_20
	for ican = 0, nt_can-1 do begin
		print, "             ",x[ind_20[ican]],y[ind_20[ican]],mags[ind_20[ican]],sky[ind_20[ican]]
	endfor
endif
;
;	keep the good, move others to reject list if just high uncertainties
;
bind_20 = WHERE(errap[0:(igood-1)] gt 0.3 and errap[0:(igood-1)] lt 2.0 and $
				FINITE(mags[0:(igood-1)]),bcount_20)
;
if(bcount_20 gt 0) then begin
	if(n_rej gt 0) then begin
		x_rej = [x_rej, x[bind_20]]
		y_rej = [y_rej, y[bind_20]]
		mags_rej = [mags[igood:(nstars-1)],mags[bind_20]]
		errap_rej = [errap[igood:(nstars-1)],errap[bind_20]]
		sky_rej = [sky[igood:(nstars-1)],sky[bind_20]]
	endif else begin
		if(bcount_20 eq 1) then begin
			x_rej = [x[bind_20]]
			y_rej = [y[bind_20]]
			mags_rej = [mags[bind_20]]
			errap_rej = [errap[bind_20]]
			sky_rej = [sky[bind_20]]
		endif else begin
			x_rej = x[bind_20]
			y_rej = y[bind_20]
			mags_rej = mags[bind_20]
			errap_rej = errap[bind_20]
			sky_rej = sky[bind_20]
		endelse
	endelse
	n_rej += bcount_20
	;
endif else begin
	if(igood ne nstars) then begin
		x_rej = x[igood:(nstars-1)]
		y_rej = y[igood:(nstars-1)]
		mags_rej = mags[igood:(nstars-1)]
		errap_rej = errap[igood:(nstars-1)]
		sky_rej = sky[igood:(nstars-1)]
		n_rej = nstars - igood
	endif
endelse
;
;	clean reject list to only keep ones with real magnitudes
;
if(n_rej gt 0) then begin
	;
	bc_ind = WHERE(mags_rej lt 25.0 and mags_rej gt -15.0 and $
					errap_rej gt 0.0 and errap_rej lt 10.0,nbc_count)
	;
	if(nbc_count gt 0) then begin
		x_rej = x_rej[bc_ind]
		y_rej = y_rej[bc_ind]
		mags_rej = mags_rej[bc_ind]
		errap_rej = errap_rej[bc_ind]
		sky_rej = sky_rej[bc_ind]
	endif
endif else begin
	nbc_count = 0
endelse
n_rej = nbc_count
;
if(count_20 gt 0) then begin
	x_can_20 = x[ind_20]
	y_can_20 = y[ind_20]
	mags_can_20 = mags[ind_20]
	errap_can_20 = errap[ind_20]
	sky_can_20 = sky[ind_20]
	;
	x_can = x_can_20
	y_can = y_can_20
	mags_can = mags_can_20
	errap_can = errap_can_20
	sky_can = sky_can_20
endif else begin
	;
	;
	;	--------- at this point, if there are no PSF candidates, need to set flag and return
	;
	status = -1
	print_text, Event, 'ERROR - g_create_psf_grid - found no PSF candidates'
;	stop
	goto, GCPG_end
endelse
;
;	debug = 1
;
n_can = N_ELEMENTS(x_can)
;
if(debug gt 0) then begin
	print, "Location E: After splitting PSF candidates list by e_mag < 0.3, good PSF candidate X, Y, Mag, sky values are"
	nt_can = n_can
	for ican = 0, nt_can-1 do begin
		print, "             ",x_can[ican],y_can[ican],mags_can[ican],sky_can[ican]
	endfor
endif
;
;	stop
;
;	associate faint/reject stars within clean_rad of the surviving candidate PSF stars
;
save_all = intarr(N_ELEMENTS(x_all))
;
rad_test = clean_rad * ap_rad * mean_FWHM
;
if(n_can gt 0) then begin
	n_can1 = n_can-1
	for is = 0, n_can1 do begin
		dx = x_all - x_can[is]
		dy = y_all - y_can[is]
		rad = sqrt(dx * dx + dy * dy)
		ind = SORT(rad)
		save_all[ind[0]] = -1			; index of the current bright star
		jnd = WHERE(rad lt rad_test and save_all ne -1,count_faint)
		if(count_faint gt 0) then begin
			save_all[jnd] = 1			; turn on save flag
		endif
	endfor
	;
	ind_faint = WHERE(save_all eq 1,faint_count)
	if(faint_count ne 0) then begin
		x_all = x_all[ind_faint]				; these are the x, y, flux of faint stars near the bright stars
		y_all = y_all[ind_faint]
		flux_all = flux_all[ind_faint]
		sky_all = sky_all[ind_faint]
		n_all = faint_count
	endif else n_all = 0
endif
;
;	remove any of those faint stars already appearing in the reject list
;
if(n_rej gt 0 and n_all gt 0) then begin
	save_all2 = intarr(n_all)
	n_rej1 = n_rej-1
	for is = 0, n_rej1 do begin
		dx = x_all - x_rej[is]
		dy = y_all - y_rej[is]
		rad = sqrt(dx * dx + dy * dy)
		ind = SORT(rad)
		jnd = WHERE(rad lt 0.75 and save_all2 ne -1, count_merge)
		if(count_merge gt 0) then begin
			save_all2[jnd] = -1						; reject flag
		endif
	endfor
	;
	ind_faint = WHERE(save_all2 ne -1, faint_count)
	if(faint_count ne 0) then begin
		x_all = x_all[ind_faint]
		y_all = y_all[ind_faint]
		flux_all = flux_all[ind_faint]
		sky_all = sky_all[ind_faint]
		n_all = faint_count
	endif else n_all = 0
endif
;
;	aperture phot the selected faint stars (using the bad-pixel-fixed image, again)
;
;	debug = 0
;
;	stop
;
g_aper,find_image,x_all,y_all,mags_all,errap_all,sky_all,skyerr_all,phpadu,apr,skyrad,badpix,bright_limit,$
	SILENT=(1-debug), /NO_BAD_PIX_FIX
;
mags_all = REFORM(mags_all[0,*])
errap_all = REFORM(errap_all[0,*])
;
;	remove bad ones
;
ind_ok = WHERE( FINITE(mags_all) ,nok)
if(nok gt 0) then begin
	mags_all = mags_all[ind_ok]
	errap_all = errap_all[ind_ok]
	;
	;	keep only good magnitudes on the list, though
	;
	ind_g_faint = WHERE(mags_all lt 25.0 and mags_all gt -15.0 and errap_all gt 0.0 and errap_all lt 10.0,ng_faint)
	if(ng_faint gt 0) then begin
		x_all = x_all[ind_g_faint]
		y_all = y_all[ind_g_faint]
		mags_all = mags_all[ind_g_faint]
		errap_all = errap_all[ind_g_faint]
		sky_all = sky_all[ind_g_faint]
	endif
	n_all = ng_faint
endif else n_all = 0
;
;	debug = 1
if(debug gt 0) then begin
	print, "Location F: After assoc. w/PSF candidates, faint star X, Y, Flux, sky "
	nt_can = n_all
	for ican = 0, nt_can-1 do begin
		print, "             ",x_all[ican],y_all[ican],flux_all[ican],sky_all[ican]
	endfor
endif
;
print_text, Event, "Number of PSF cand. = "+STRING(n_can)+", PSF rejects = "+STRING(n_rej)+", and faint nearby = "+STRING(n_all)+" stars"
;
;	stop
;
iloop_psf = 0L
;------------------------------------------------------------------------------------
;
;	STEP 5: select PSF stars, using zone system across image
;
;	debug = 1
;--------
;rad_PSF = CEIL(rad_scale*fwhm) ; + 3 - 4		; DPC 20100221 reduced by 4 pixels
;
if(debug gt 0) then print_text, Event, "Starting PSF star selection with rad_PSF = "+STRING(rad_PSF,format='(F5.2)')
;
npsf_stars = 15
;
;	pack candidates, rejects, and faint nearby stars into x,y,mag,errap arrays and set n_candidate number
;
;	first, the candidates, then the rejected stars
;
if(n_rej ne 0) then begin
	x = [x_can, x_rej]
	y = [y_can, y_rej]
	mags = [mags_can, mags_rej]
	errap = [errap_can, errap_rej]
	sky = [sky_can, sky_rej]
endif else begin
	x = x_can
	y = y_can
	mags = mags_can
	errap = errap_can
	sky = sky_can
endelse
;
;		finally, the surviving faint stars
;
if(n_all ne 0) then begin
	x = [x, x_all]
	y = [y, y_all]
	mags = [mags, mags_all]
	errap = [errap, errap_all]
	sky = [sky, sky_all]
endif
;
;	debug = 1
;
if(debug gt 0) then begin
	print, "Location G: full list to PSF selector X, Y, Flux, sky  "
	nt_can = N_ELEMENTS(x)
	for ican = 0, nt_can-1 do begin
		print, "             ",x[ican],y[ican],mags[ican], sky[ican]
	endfor
endif
;
;	stop
;
n_candidates = N_ELEMENTS(x_can)
;
;-----------------------------------------------------------------------||
;	save region file of initial PSF candidate star locations
;
if(fwhm_pass eq 0 and regions_flag gt 0) then begin
	;
	x_psf = fltarr(ng, ng, n_candidates)
	y_psf = fltarr(ng, ng, n_candidates)
	n_xy_psf = intarr(ng,ng)
	;
	n_xy_psf[0,0] = n_candidates
	x_psf[0,0,0:(n_candidates-1)] = x[0:(n_candidates-1)]
	y_psf[0,0,0:(n_candidates-1)] = y[0:(n_candidates-1)]
	;
	region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
	test_dir = FILE_INFO(region_dir)
	if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
	;
	file_reg = FILE_BASENAME(image_pathname) + '_untested_PSF_cand_stars.reg'
	region_path = region_dir + file_reg
	;
	S8_Save_PSF_stars_region_file, Event, region_path, 6.0*fwhm, x_psf, y_psf, n_xy_psf, COLOR='green',debug
endif
;
;----------------------------------------------------------------------
;	debug = 1
;
;************************** need to remove dupes before calling this routine *****
;					DPC 20100416
this_ng = FIX(ng)
g_select_PSF_stars, Event, npsf_stars, find_image, x, y, mags, errap, this_ng, $
				xborder, yborder, fwhm, id_PSF, clean_PSF_flag, npsf_stars_found, $
				n_candidates, grid_fwhm, ab, $
				debug
;
;	test - if no psf stars, turn off use flag for this image and exit
;
if(debug) then print_text, Event, "on return from g_select_psf_stars, npsf_stars_found = "+STRING(npsf_stars_found)
;
;debug = 0
;
;	stop
;
if(npsf_stars_found eq 0) then begin
	status = -99					; tells calling code to turn off use flag
	print_text, Event, "ERROR - p_create_psfgrid - no PSF stars found on fwhm_pass = "+STRING(fwhm_pass)
;	stop
	goto, GCPG_end
endif
;
;	save PSF stars for return to HWP image fitting
;
n_xy_psf_max = N_ELEMENTS(REFORM(id_PSF[0,0,*]))
;
x_psf = 	fltarr(ng,ng,n_xy_PSF_max)
y_psf = 	fltarr(ng,ng,n_xy_PSF_max)
mag_psf = 	fltarr(ng,ng,n_xy_PSF_max)
n_xy_psf = 	intarr(ng,ng)
;
ng1 = ng-1
for izy = 0, ng1 do begin
	for izx = 0, ng1 do begin
		ind = WHERE(REFORM(id_PSF[izx,izy,*]) ne -99,count)
		n_xy_psf[izx,izy] = count
		if(count gt 0) then begin
			x_psf[izx,izy,0:(count-1)] = x[id_PSF[izx,izy,ind]]
			y_psf[izx,izy,0:(count-1)] = y[id_PSF[izx,izy,ind]]
			mag_psf[izx,izy,0:(count-1)] = mags[id_PSF[izx,izy,ind]]
		endif
	endfor
endfor
;
;-----------------------------------------------------------------------||
;	save region file of initial PSF star locations for this image
;
if(fwhm_pass eq 0 and regions_flag gt 0) then begin
	region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
	test_dir = FILE_INFO(region_dir)
	if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
	;
	file_reg = FILE_BASENAME(image_pathname) + '_first_selected_PSF_stars.reg'
	region_path = region_dir + file_reg
	;
	S8_Save_PSF_stars_region_file, Event, region_path, 4.0*fwhm, x_psf, y_psf, n_xy_psf, COLOR='yellow', debug
endif
;----------------------------------------------------------------------
;
;	loop to here for clean_PSF_flag > 0
;
image_for_psf = find_image
;
;	debug = 0
;
iclean = 0
;
clean_loop:
;
;	number of psf stars?
;
ind_psf_stars = WHERE(id_psf ne -99, n_psf_stars)
;
debug = 1
if(debug gt 0) then begin
	print_text, Event, 'Starting PSF building/cleaning for '+STRING(n_psf_stars,format='(I4)')+' stars with FWHM, ICLEAN loop numbers '+STRING(fwhm_pass+1,iclean+1,format='(i1,",",i1)')
;	ind_test = WHERE(id_psf ne -99,ntest)
;	if(ntest gt 0) then begin
;		ind_200 = WHERE(id_psf[ind_test] eq 201,n201)
;		if(n201 gt 0) then begin
;			stop
;		endif
;	endif
endif
debug = 0
;
;	STEP 6: create PSF models, one per zone
;
first_apr = apr[0]
;
; trap bad values
;
if(N_ELEMENTS(x) eq 1) then begin
	if(x[0] eq 0.0) then begin
		stop
	endif
endif
;
;	debug=0
first_time = systime(/seconds)
n_x_before = N_ELEMENTS(x)
g_getpsf, Event, this_ng, image_for_psf, x, y, mags, sky, ronois, phpadu, gauss, $
	psf, id_psf, first_apr, rad_psf, fit_rad, fwhm, PSF_grid, badpix, sat_limit, debug, $
	grid_fwhm, status, /NO_BAD_PIX_FIX, LOOP_COUNT=iclean, FWHM_LOOP_COUNT=fwhm_pass
n_x_after = N_ELEMENTS(x)
if(n_x_before ne n_x_after) then begin
	print_text, Event, "ERROR - number of stars changed in g_getpsf. Stopping"
	stop
endif
;
if(print_times) then print_text, Event, "Time to complete g_getpsf = "+STRING( (systime(/seconds)-first_time) )+" seconds"
;
if(status ne 0) then begin
	status = -99
	print_text, Event, "ERROR - g_create_psf_grid - no stars found"; to turn off use flag
;	stop
	goto, GCPG_end
endif
;
;	if clean_PSF_flag, then for every PSF star, remove all nearby neighbors from the image array,
;	using the PSF model and putting the result in the image_for_psf array
;
;	debug=0
;
if(iclean le fwhm_pass) then begin
	print_text, Event, "Cleaning PSF candidates of nearby faint stars"
	;
	;
	first_time = systime(/seconds)
	n_x_before = N_ELEMENTS(x)
	g_remove_PSF_neighbors, Event, this_ng, find_image, image_for_psf, x, y, mags, sky, ronois, $
		phpadu, gauss, psf, id_psf, first_apr, rad_psf, fit_rad, PSF_grid, badpix, bright_limit, $
		fwhm, grid_fwhm, debug, status, /NO_BAD_PIX_FIX
	n_x_after = N_ELEMENTS(x)
	if(n_x_before ne n_x_after) then begin
		print_text, Event, "ERROR - number of stars changed in g_remove_PSF_neighbors. Stopping"
		stop
	endif
	if(print_times) then print_text, Event, "Time to complete g_remove_PSF_neighbors = "+STRING( (systime(/seconds)-first_time))+" seconds"
;	debug=0
	;
	;	redo aperture photometry on the PSF stars, after cleaning the junk around them
	;
	;		first, compress lists to just the PSF stars
	;
	ind_psf_stars = WHERE(id_psf ne -99, n_psf_stars)
	;
	print_text, Event, "... new aperture photometry on the "+STRING(n_psf_stars,format='(I4)')+" PSF candidate stars"
	psf_n = 0L
	psf_ind = lonarr(1)
	psf_x = fltarr(1)
	psf_y = fltarr(1)
	psf_mags = fltarr(1)
	psf_sky = fltarr(1)
	;
	old_id_psf = id_psf
	id_psf [*,*,*] = -99
	;
	for iyg = 0, ng-1 do begin
		for ixg = 0, ng-1 do begin
			;
			ind_psf = WHERE(old_id_psf[ixg,iyg,*] ne -99, ng_psf)
			if(ng_psf gt 0) then begin
				x_psf = x[old_id_psf[ixg,iyg,ind_psf]]
				y_psf = y[old_id_psf[ixg,iyg,ind_psf]]
				;
				n_before_x = N_ELEMENTS(x_psf)
				g_aper,image_for_psf,x_psf,y_psf,mags_psf,errap_psf,sky_psf,skyerr,phpadu,apr,skyrad,$
							badpix,bright_limit,$
							SILENT=(1-debug), /NO_BAD_PIX_FIX
				n_after_x = N_ELEMENTS(x_psf)
				if(n_before_x ne n_after_x) then begin
					print_text, Event, "ERROR - number of psf stars changed in g_aper. Stopping"
					stop
				endif
				;
				mags_psf = REFORM(mags_psf[0,*])
				errap_psf = REFORM(errap_psf[0,*])
				ind_good = WHERE(FINITE(mags_psf) and errap_psf lt 0.5 and $
						mags_psf gt -10.0 and mags_psf lt 25.0,ngood)
				;
				if(ngood gt 0) then begin
					mags_psf = mags_psf[ind_good]
					x_psf = x_psf[ind_good]
					y_psf = y_psf[ind_good]
					sky_psf = sky_psf[ind_good]
					;
					if(psf_n eq 0) then begin
						psf_x = [x_psf]
						psf_y = [y_psf]
						psf_mags = [mags_psf]
						psf_sky = [sky_psf]
						psf_n = N_ELEMENTS(x_psf)
						psf_ind = indgen(psf_n)
						id_psf[ixg,iyg,0:(psf_n - 1)] = psf_ind	; point to the good stars
						id_psf[ixg,iyg,psf_n:99] = -99
					endif else begin
						psf_x = [psf_x, x_psf]
						psf_y = [psf_y, y_psf]
						psf_mags = [psf_mags, mags_psf]
						psf_sky = [psf_sky, sky_psf]
						n_here = N_ELEMENTS(x_psf)
						n_tot = N_ELEMENTS(psf_x)
						psf_ind = indgen(n_here) + psf_n
						id_psf[ixg,iyg,0:(n_here - 1)] = psf_ind
						id_psf[ixg,iyg,n_here:99] = -99
						psf_n = n_tot
					endelse
				endif
				;
	;			stop
				;
			endif else begin
				id_psf[ixg,iyg,*] = -99
			endelse
		endfor
	endfor
	;
	;	reload the x, y, mags, sky arrays (id_psf already loaded)
	;
	x = psf_x
	y = psf_y
	mags = psf_mags
	sky = psf_sky
	;
;	print_text, Event, "...Finished Aperture Photometry of PSF stars.. finishing with "+STRING(psf_n,format='(I4)')+" surviving PSF stars"
	;
	iclean++
	;
	;	check quality of new id_psf array - shouldn't be any stars with id numbers
	;		exceeding number of stars in array
	;
	ind_test = WHERE(id_psf ne -99,n_test)
	if(n_test gt 0) then begin
		ind_bad_test = WHERE(id_psf[ind_test] ge n_test,n_error)
		if(n_error gt 0) then begin
			print_text, Event, "ERROR - id_psf array corrupted. Stopping"
			stop
		endif
	endif else begin
		print_text, Event, "ERROR - no good PSF stars. Turning off USE flag for this image"
		status = -99
		goto, GCPG_end
	endelse
	goto, clean_loop
endif
;
;	save image cleaned of nearest neighbors to PSF stars
;
if(save_region_files gt 0) then begin
	mosaic_dir = analysis_dir + step_dir_text + PATH_SEP() + 'PSF_Mosaics' + PATH_SEP()
	test_dir = FILE_INFO(mosaic_dir)
	if(NOT(test_dir.exists)) then FILE_MKDIR, mosaic_dir
	file_out = FILE_BASENAME(image_pathname,'.fits') + '_PSF_cleaned.fits'
	mosaic_path = mosaic_dir + file_out
	writefits, mosaic_path, image_for_psf
endif
;
if(fwhm_pass gt 0) then print_text, Event, "PSFs cleaned, saving PSF star locations"
;
;-----------------------------------------------------------------------||
;	save region file of final PSF star locations for this image
;
n_xy_psf_max = N_ELEMENTS(REFORM(id_PSF[0,0,*]))
;
x_psf = fltarr(ng,ng,n_xy_PSF_max)
y_psf = fltarr(ng,ng,n_xy_PSF_max)
mag_psf = fltarr(ng,ng,n_xy_PSF_max)
n_xy_psf = intarr(ng,ng)
;	;
n_total_PSF_stars = 0L
ng1 = ng-1
for izy = 0, ng1 do begin
	for izx = 0, ng1 do begin
		ind = WHERE(REFORM(id_PSF[izx,izy,*]) ne -99,count)
		n_xy_psf[izx,izy] = count
		if(count gt 0) then begin
			x_psf[izx,izy,0:(count-1)] = x[id_PSF[izx,izy,ind]]
			y_psf[izx,izy,0:(count-1)] = y[id_PSF[izx,izy,ind]]
			mag_psf[izx,izy,0:(count-1)] = mags[id_PSF[izx,izy,ind]]
			n_total_PSF_stars = n_total_PSF_stars + count
		endif
	endfor
endfor
;
;	debug = 0
;-----------------------------------------------------------------------------------------------------------------
;
if(fwhm_pass eq 1 and regions_flag gt 0) then begin
	region_dir = analysis_dir + step_dir_text + PATH_SEP() + 'Region_Files' + PATH_SEP()
	test_dir = FILE_INFO(region_dir)
	if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
	;
	file_reg = FILE_BASENAME(image_pathname) + '_final_PSF_stars.reg'
	region_path = region_dir + file_reg
	;
	S8_Save_PSF_stars_region_file, Event, region_path, 2.5*fwhm, x_psf, y_psf, n_xy_psf, COLOR='magenta',debug
;
endif
;-------------------------------------------------------------------------------------------------
;
;	PSF building is finished.
;
;	next, fetch the fwhm from the PSF database and if fwhm_pass = 0, then loop back to do it all again
;
;		can turn off by setting override = 1
;
override = 0
;
if(fwhm_pass eq 0 and override eq 0) then begin
	ng1 = ng-1
	for iyg = 0, ng1 do begin
		for ixg = 0, ng1 do begin
			if(ixg eq 0 and iyg eq 0) then begin
				sig_x = PSF_grid[ixg,iyg].gauss[3]
				sig_y = PSF_grid[ixg,iyg].gauss[4]
			endif else begin
				sig_x = [sig_x, PSF_grid[ixg,iyg].gauss[3]]
				sig_y = [sig_y, PSF_grid[ixg,iyg].gauss[4]]
			endelse
		endfor
	endfor
	;
	if(ng ne 1) then begin
		zz_x = median_filtered_mean(sig_x)
		zz_y = median_filtered_mean(sig_y)
		x_fwhm = zz_x[0] * 2.354
		y_fwhm = zz_y[0] * 2.354
	endif else begin
		x_fwhm = sig_x * 2.354
		y_fwhm = sig_y * 2.354
	endelse
	;
	mean_FWHM = sqrt(x_fwhm * y_fwhm) > 1.55	; trap at no smaller than 0.9 arcsec
	sig_FWHM = 0.01									; assumed
	;
	fwhm_pass = 1
	;
	;	reset lists
	;
	x_rej = -1.0
	y_rej = -1.0
	flux_rej = -1.
	sky_rej = -1.
	;
	print_text, Event, "Restarting finding, PSF-candidate selection, cleaning loop"
	goto, fwhm_loop
endif
;
;	STEP 7: Fit PSFs to determine location-dependent parameters
;
fit_final_psfs:
;
;------------------------------------------------------------------------------
;
;	save all the PSFs in a single image, as an ng x ng grid
;
if(regions_flag gt 0) then begin
	g_mosaic_psf_models, Event, this_ng, rad_PSF, fit_rad, PSF_grid, PSF_mosaic_image
	;
	mosaic_dir = analysis_dir + step_dir_text + PATH_SEP() + 'PSF_Mosaics' + PATH_SEP()
	test_dir = FILE_INFO(mosaic_dir)
	if(NOT(test_dir.exists)) then FILE_MKDIR, mosaic_dir
	;
	file_mosaic = FILE_BASENAME(image_pathname,'.fits') + '_PSF_mosaic.fits'
	mosaic_path = mosaic_dir + file_mosaic
	writefits, mosaic_path, PSF_mosaic_image
endif
;--------------------------------------------------------------------------------
;
if(ng gt 1) then print_text, Event, "Fitting PSF variations across FOV"
;
g_fit_PSFs, Event, this_ng, PSF_grid, PSF_model
;
;	exit location
;
status = 1					; normal exit value
;
print_text, Event, "Final number of stars making up the PSF = "+STRING(n_total_PSF_stars)
;
psf_build_time = SYSTIME(/SECONDS)
psf_time = psf_build_time - start_time
print_text, Event, "Time to build PSF(s) = "+STRING(psf_time)+" seconds"
;
GCPG_end:
;
end
