pro g_curve_of_growth, image, x, y, mags, sky, apr, skyrad, curve, $
						bad_pix_limits, bright_pix_limit, $
						iplot_curves, debug, bad_star_vec, $
						MAG_FINAL = magfinal, ERRMAG_FINAL = errmagfinal
;
;	does a curve of growth analysis of the stars in the
;	image pointed to by the x and y vectors
;
;	Does aperture photometry from 1.0 to 2.0 * radius
;
;	scales all stars to same curve, plots same, fits curve
;	and returns curve referenced to 1.0 * radius value
;
;	DPC	20080715
;		20090402 - skip aperture phot part if importing existing mags, errmags
;		20090407 - added histo/gaussian analysis to flag, remove doubles and stars with coma
;		20090408 - returns bad_star vector to flag stars rejected from COG analysis
;
;iplot_curves = 0
idebug = debug
;
;idebug = 1
;
if(N_ELEMENTS(magfinal) ne 0) then begin
	import = 1
	n_steps = N_ELEMENTS(REFORM(magfinal[0,*]))
	nstar = N_ELEMENTS(REFORM(magfinal[*,0]))
endif else begin
	import = 0
	n_steps = N_ELEMENTS(apr)
	nstar = N_ELEMENTS(x)
endelse
;
bad_star_vec = intarr(nstar)
;
curve = fltarr(2,n_steps)		; first is curve, second is uncert
;
phpadu = 8.21
;
if(idebug) then print, "Starting COG aperture photometry with apertures ",apr, " and sky zone ",skyrad
;
if(~import) then begin
	g_debug = 0
	g_aper,image,x,y,mags,errap,sky,skyerr,phpadu,apr,skyrad,bad_pix_limits,bright_pix_limit, $
		SILENT=(1-g_debug),NO_BAD_PIXEL_FIX=1
endif else begin
	mags = TRANSPOSE(magfinal)
	errap = TRANSPOSE(errmagfinal)
endelse
;
magsd = mags * 0.0
;
ind_15 = WHERE(REFORM(errap[0,*]) lt 0.15,count)
;
icount = 0L
max_mag = [0.]
;
for i = 0, nstar-1 do begin
	if(errap[0,i] lt 0.15 and errap[0,i] gt 0.00001) then begin
		;
		;	dump the vectors
		;
		if(idebug gt 0) then begin
			print, "G_COG: Star ",i," curve and uncertainties"
			print, mags[*,i]
			print, errap[*,i]
		endif
		;
		first = mags[0,i]
		;print, "first = ",first
		magsd[*,icount] = mags[*,i] - first
		errap[*,icount] = 0.0005 > sqrt(errap[*,i]^2 + errap[0,i]^2)
		;
		;	if any differences go positive (star becomes fainter), set errors large
		;
		ind = WHERE(magsd[*,icount] gt 0.0, ibad)
		;
		;	also, look for any star that doesn't have a real negative mag brightening
		;		(like a bad pixel)
		;
		zz = LINFIT(apr, REFORM(mags[*,i]),SIGMA=s_zz)
		if(zz[1] gt -4.0*s_zz[1]) then begin
			ibad++				; test for 4 sigma slope negative
			ind = indgen(n_steps)
		endif
		;
		if(ibad gt 0) then begin
			errap[ind,icount] = 10.0
			magsd[ind,icount] = 0.0
			;
			if(idebug gt 0) then print, "ibad = ",ibad," for star ",i
		endif
		;
		icount++
		if(icount eq 1) then max_mag = [first] else max_mag = [max_mag, first]
	endif
endfor
;
if(idebug gt 0) then print, "after first loop, icount = ",icount
;
if(idebug gt 0) then begin
	print, "G_COG: Number of stars with good aperture photometry [under 0.15 mag unc] = ",icount
	nstars = N_ELEMENTS(x)
	for is = 0, count-1 do begin
		print, "Star ",is," X, Y, mag[0], errmag[0] = ",x[ind_15[is]],y[ind_15[is]],mags[0,ind_15[is]],errap[0,ind_15[is]]
	endfor
endif
;
if(icount ne count) then begin
	print, "ERROR - G_CURVE_OF_GROWTH: icount and count not equal"
endif
;
max_max_mag = min(max_mag)				; brightest star mag for smallest aperture
dmax_mag = max_mag - max_max_mag		; mag of other stars relative to brightest
;
;print, "Max mag = ",max_mag
;print, "Icount = ",icount," n_steps = ",n_steps
;
if(icount ne 0) then begin
	dmags = dblarr(n_steps,icount)
	sdmags = dblarr(n_steps,icount)
	mean_mags = dblarr(n_steps)
	smean_mags = dblarr(n_steps)
	swt_mags = dblarr(n_steps)
	;
	dmags = magsd
	sdmags = errap
	;
	for iap = 0, n_steps-1 do begin
		mean_mags[iap] = 0.0d0
		smean_mags[iap] = 0.0d0
		swt_mags[iap] = 0.0d0
		;
		for ic = 0, icount-1 do begin
			;
			if(sdmags[iap,ic] le 0.15 and sdmags[iap,ic] gt 0.0001) then begin
				wt = 1.0d0/ (sdmags[iap,ic]^2)
				swt_mags[iap] = swt_mags[iap] + wt
				mean_mags[iap] = mean_mags[iap] + dmags[iap,ic] * wt
			endif else begin
				if(sdmags[iap,ic] le 0.0001) then begin
					print, "ERROR - uncertainty under 0.0001 mag for iap, ic = ",iap, ic
					stop
				endif
			endelse
		endfor
		if(swt_mags[iap] ne 0.0d0) then begin
			mean_mags[iap] = mean_mags[iap] / swt_mags[iap]
			smean_mags[iap] = 1./sqrt(swt_mags[iap])
		endif else begin
			mean_mags[iap] = 0.0
			smean_mags[iap] = 1.0
		endelse
	endfor
	;
	ymin = min(dmags)
	ymax = max(dmags)
	;
	xmin = min(apr)
	xmax = max(apr)
	;
	if(iplot_curves eq 1) then begin
		window, 13
		device, decomposed=0
		loadct,0, /SILENT
		tvlct, [0, 255, 0, 0, 255], [0, 0, 255, 0, 255], [0, 0, 0, 255, 255], 10
		;
		red = 11
		black = 10
		green = 12
		blue = 13
		white = 14
		;
		color = [blue, green, red, white]
		thick = [4,2,1,0.5]
		line = [0, 0, 1, 2]
		mag_ind = 0 > (3 < FIX(dmax_mag[0]))
		;
		ymin_p = (-1. > ymin) < (-0.5)
		;
		plot, apr, dmags[*,0], xrange=[xmin, xmax], yrange=[ymin_p,0.025], $
			ystyle=1,xtitle='aperture [in pixels]', ytitle='difference mag',psym=-4,$
			thick = thick[mag_ind], color = color[mag_ind]
		;
		if(icount gt 1) then begin
			for ic = 1, icount-1 do begin
				mag_ind = 0 > (3 < FIX(dmax_mag[ic]))
				oplot, apr, dmags[*,ic], psym=-4, thick = thick[mag_ind], color=color[mag_ind]
			endfor
		endif
		;
		;	plot means, uncertainties
		;
		oplot, apr, mean_mags, psym=2, thick = 6, color=11
		;
		cursor, xx, yy, /wait
		wait, 0.5
		;wdelete, 14
	endif
	;
	;	for each star, make sure its CoG is monotonically decreasing
	;
	mono_test = intarr(icount) * 0
	for istar = 0, icount-1 do begin
		;
		count_bad = 0
		for iap = 0, n_steps-2 do begin
			err = sqrt(sdmags[iap+1,istar]^2 + sdmags[iap,istar]^2)
			if((dmags[iap+1,istar] - dmags[iap,istar]) gt (0.0005 > (3.0*err)) ) then count_bad++
			if(iap gt 0 and dmags[iap+1,istar] eq 0.0 and dmags[iap,istar] eq 0.0) then count_bad++
		endfor
		if(count_bad ge 2) then mono_test[istar] = -1
	endfor
	;
	;	for last 3 bins, create histogram of occupancy from 0.05 to 1.5 mag dif in 0.0375 mag steps
	;	and 0.075 mag widths
	;
	low_mag = -0.14
	high_mag = -1.0
	mag_bin = 0.08
	mag_bin2 = mag_bin / 2.0
	mag_step = 0.04
	;
	n_bins = CEIL((low_mag - high_mag) / mag_step) + 1
	;
	v = fltarr(n_bins)
	mag_v_center = low_mag - (findgen(n_bins) + mag_bin2) * mag_step
	;
	;	populate bins
	;
	;	and keep track of how many are within the limits
	;
	n_in_range = 0
	for this_ap = n_steps-3, n_steps-1 do begin
		for istar = 0, icount-1 do begin
			;
			;	keep only monotonically decreasing stars
			;
			if(mono_test[istar] eq 0) then begin
				;
				this_dmag = dmags[this_ap, istar]
				;
				dif_mag = abs(this_dmag - mag_v_center)
				;
				ind = WHERE(dif_mag lt mag_bin2, bcount)
				if(bcount gt 0) then begin
					v[ind] = v[ind] + 1.0
					n_in_range = n_in_range + bcount
				endif
			endif
		endfor
	endfor
	;
	;	plot histogram
	;
	if(iplot_curves) then begin
		window, 22
		plot, mag_v_center, v, psym=10, xtitle='Offset magnitude', ytitle='Number of stars'
	endif
	;
	;	find first peak, gaussian fit it
	;
	;	locate channel where neighboring channels are lower and
	;	peak has enough stars
	;
	peak_ch = -1
	count_test = ((count + TOTAL(mono_test)) < n_in_range) < TOTAL(v)
	;
	for ich = 1, n_bins-2 do begin
		ch1 = ich - 1
		ch2 = ich
		ch3 = ich + 1
		;
		test1 = v(ch1) le v(ch2)
		test2 = v(ch3) le v(ch2)
		if(count_test gt 20) then begin
			test3 = v(ch2) ge 0.25*count_test
		endif else begin
			test3 = v(ch2) ge 0.20*count_test
		endelse
		test4 = NOT((v(ch1) eq 0.0) and (v(ch2) eq 0.0))
		test5 = NOT((v(ch3) eq 0.0) and (v(ch2) eq 0.0))
		;
		if(test1 and test2 and test3 and test4 and test5) then begin
			peak_ch = ich
			goto, peak_end
		endif
	endfor
	peak_end:
	center = mag_v_center[peak_ch]
	width = 2.0 * mag_step			; a guess
	;
	;	to avoid the next peak, work out from the first peak until
	;	values no longer decrease, then trim the fit channels to match
	;
	;	high end first
	;
	high_ch = 0 > (peak_ch - 3)				; default
	for ihigh = peak_ch, 1 do begin
		dv_dch = v[ihigh] - v[ihigh-1]
		if(dv_dch lt 0. or v[ihigh] eq 0.) then begin
			high_ch = ihigh
			goto, high_end
		endif
	endfor
	high_end:
	;
	low_ch = (n_bins-1) > (peak_ch + 3)
	for ilow = peak_ch, n_bins-2 do begin
		dv_dch = v[ilow] - v[ilow+1]
		if(dv_dch lt 0. or v[ilow] eq 0.) then begin
			low_ch = ilow
		 	goto, low_end
		 endif
	endfor
	low_end:
	;
	;	must have at least 4 channels
	;
	low_ch = low_ch < (n_bins - 1)
	high_ch = high_ch > 0
	;
	nch_fit = low_ch - high_ch + 1
	if(nch_fit lt 4) then begin
		if(high_ch eq 0) then begin
			low_ch = 3 > low_ch
		endif else begin
			high_ch = (high_ch - 1) > 0
			low_ch = (low_ch + 1) < (n_bins-1)
		endelse
	endif
	;
	;	gaussian fit
	;
	nterms=3
	v_fit = GAUSSFIT(mag_v_center[high_ch:low_ch], v[high_ch:low_ch], coeff, NTERMS=nterms)
	;
	if(abs(coeff[1] - mag_v_center[peak_ch]) gt 3.0*coeff[2]) then begin
		;
		;	bad fit, use peak, fixed width
		;
		coeff[1] = mag_v_center[peak_ch]
		coeff[2] = 3.0*mag_bin
	endif
	;
	;print, "coeffs: ",coeff
	;
	if(iplot_curves) then oplot, mag_v_center[high_ch:low_ch], v_fit
	;
	;	locate cutoff at center + 2.5 * sigma
	;
	;		make sure gaussian width is at least 1/2 bin wide!
	;
	coeff[2] = coeff[2] > (mag_bin / 2.0)
	;
	;	if number of stars is below 6, widen even more
	;
	if(count le 6) then coeff[2] = coeff[2] > (mag_bin * 3.0)
	;
	cutoff_mag = [-1.0 > (coeff[1] - 3.0 * coeff[2]),-0.025 < (coeff[1] + 3.0*coeff[2])]
	xx0 = [cutoff_mag[0], cutoff_mag[0]]
	xx1 = [cutoff_mag[1], cutoff_mag[1]]
	yy = [0., coeff[0]]
	;
	if(iplot_curves) then begin
		oplot, xx0, yy
		oplot, xx1, yy
		cursor, xx, yy, /wait
		wait, 0.5
	endif
	;
	;	which curves fall outside the cutoffs?
	;
	test_dmags = REFORM(dmags[n_steps-2,*])
	n_total_count = N_ELEMENTS(test_dmags)
	ind_bad = WHERE(test_dmags gt cutoff_mag[1] or test_dmags lt cutoff_mag[0] or mono_test eq -1,bad_count)
	if(bad_count gt 0) then begin
		;
		sdmags[*,ind_bad] = 10.0
		bad_star_vec[ind_bad] = 1
		;
		;	test for all bad
		;
		if(TOTAL(bad_star_vec) eq n_total_count) then begin
			print, "All COG star curves flagged as bad. This should not happen..."
			stop
		endif
	endif
	;
	;	next, clean inputs at each aperture - skipping first
	;
	cog = mean_mags
	s_cog = smean_mags
	;
	mag_with_ap = 0.05 / float(n_steps)
	thresh_with_ap = 0.3 / float(n_steps)
	;
	for iloop = 0, 6 do begin
		;
		threshold = 2.5 + 0.10 * iloop
		m_window = 0.20 - 0.020 * iloop
		t_mags = dmags
		t_errap = sdmags		; reload values
		;
		for ic = 1, n_steps-1 do begin
			;
			m_window_ap = m_window + mag_with_ap * float(ic)
			thresh_ap = threshold + thresh_with_ap * float(ic)
			;
			;	compare each star and its uncertainty to the mean, rejecting any with
			;	deviations beyond 5 sigma - [this isn't easy with very different uncertainties]
			;
			;	also mag window increases with aperture
			;
			for istar = 0, icount-1 do begin
				if(mono_test[istar] eq 0) then begin
					;
					;	compare star's mean to current mean
					;
					dif = abs(cog[ic] - t_mags[ic,istar])
					err = 0.005 > sqrt(s_cog[ic]^2 + t_errap[ic,istar]^2)
					if(dif gt thresh_ap*err or dif gt m_window_ap or t_errap[ic,istar] eq 10.0) then begin
						;
						;	too large, kill value
						;
						t_mags[ic,istar] = 99.0
						t_errap[ic,istar] = 99.0
						;
					endif
				endif
			endfor
			;
			;	recompute average, error
			;
			wt = 0.0d0
			ave = 0.0d0
			for istar = 0, icount-1 do begin
				;
				if(mono_test[istar] eq 0) then begin
					if(t_errap[ic,istar] le 0.15 and t_errap[ic,istar] gt 0.0001) then begin
						this_wt = 1.0d0/((0.005 > t_errap[ic,istar])^2)
						wt = wt + this_wt
						ave = ave + t_mags[ic,istar]*this_wt
					endif
				endif
			endfor
			if(wt gt 0.0) then begin
				cog[ic] = ave / wt
				s_cog[ic] = 1./sqrt(wt)
			endif else begin
				;
				;	if not first, use previous
				;
				if(ic ne 0) then begin
					cog[ic] = cog[ic-1]
					s_cog[ic] = s_cog[ic-1]
				endif else begin
					cog[ic] = 0.0
					s_cog[ic] = 1.0
				endelse
			endelse
			;
			;	next aperture
			;
		endfor
		;
		;	all done, loop again
		;
	endfor
	;
	;	and replot
	;
	first = WHERE(mono_test eq 0,mono_count)
	if(mono_count gt 0) then begin
		ic_first = first[0]
	endif else begin
		ic_first = 0
	endelse
	;
	if(iplot_curves eq 1) then begin
		window, 14
		device, decomposed=0
		loadct,0, /SILENT
		tvlct, [0, 255, 0, 0, 255], [0, 0, 255, 0, 255], [0, 0, 0, 255, 255], 10
		;
		red = 11
		black = 10
		green = 12
		blue = 13
		white = 14
		;
		color = [blue, green, red, white]
		thick = [4,2,1,0.5]
		line = [0, 0, 1, 2]
		mag_ind = 0 > (3 < FIX(dmax_mag[ic_first]))
		ind = WHERE(t_mags[*,ic_first] ne 99.0,count)
		;
		ymin_p = (-1.0) > (MIN(t_mags) < (-0.5))
		;
		plot, apr[ind], t_mags[ind,ic_first], xrange=[xmin, xmax], yrange=[ymin,0.025], $
			ystyle=1,xtitle='aperture [in pixels]', ytitle='difference mag',psym=-4,$
			thick = thick[mag_ind], color = color[mag_ind]
		if(icount gt 1) then begin
			for ic = 1, icount-1 do begin
				;
				if(mono_test[ic] eq 0) then begin
					ind = WHERE(t_mags[*,ic] ne 99.0)
					mag_ind = 0 > (3 < FIX(dmax_mag[ic]))
					oplot, apr[ind], t_mags[ind,ic], psym=-4, thick = thick[mag_ind], color=color[mag_ind]
				endif
			endfor
		endif
		;
		;	plot means, uncertainties
		;
		oplot, apr, cog, psym=2, thick = 6, color=11
		;
		cursor, xx, yy, /wait
		wait, 0.5
		;wdelete, 14
	endif
	;
	curve[0,*] = cog[*]
	curve[1,*] = s_cog[*]
	;
endif else begin
	print,"No Curve of Growth Computed in this grid zone"
	curve[0,*] = 0.0
	curve[1,*] = 0.5
endelse
;
end