pro g_defocus_FWHM, in_image, find_image, skymode, skysig, mfm_mean, mfm_stdev, $
			mean_FWHM, sig_FWHM, sat_limit, debug, AB = ab, NSTARS = nstars, $
			STATUS = status
;
;	uses FIND and gaussian fitting of the brightest
;	ten stars in the image to produce a good estimate
;	of the mean FWHM and its uncertainty
;
;	DPC	20080624
;	    20080724	modified to skip stars with pixel values beyond SAT_LIMIT
;		20100802	this version is for processing defocussed images - no gaussian fitting permited
;
;----------------------------------------------
;
;
debug = 0
status = 0
;
del_flag = 0
deleted_data = -1.0e6
;
i_size = SIZE(in_image)
nx = i_size[1]
ny = i_size[2]
;
del_loop:
;
if(del_flag eq 0) then image = in_image else image = fix_bad_pixels(in_image)
;
if(nx eq 1024) then coadd = 0 else if(nx gt 1024) then coadd = 1 else coadd = -1
case coadd of
	0: begin
		xborder = 4
		yborder = 20
	   end
	1: begin
		xborder = 30
		yborder = 105
	   end
   -1: begin
   		xborder = 5
   		yborder = 7
   	   end
endcase
;
roundlim = [-1.5, 1.5]
sharplim = [0.1, 0.9]
;
scale = 20.0					; high significance stars only
iloop = 0
guess_FWHM = 7.0				; vs 3.0 for focussed stars
;guess_FWHM = 1.5
;
min_stars = 6
big_loop = 0
;
loop_start:
;
hmin = scale*mfm_stdev
;
if(debug lt 2) then silent = 1 else silent = 0
;
g_find, find_image, x, y, flux, sharp, roundness, hmin, guess_FWHM, $
	roundlim, sharplim, SILENT=silent, /NO_BAD_PIX_FIX
;
nstars = N_ELEMENTS(x)
;
;	first, reject all within some pixels of X boundary and Y boundary
;
;p_reject = 30
;
;	test for no stars
;
n_flux = N_ELEMENTS(flux)
if(n_flux eq 0) then begin
	s_count = 0
	status = -2
	goto, no_stars
endif
;
ind = WHERE(FINITE(flux) and x gt xborder and x lt (nx-xborder) and y gt yborder $
        and y lt (ny-yborder) and flux lt sat_limit,good_count)
;
if(good_count lt min_stars) then begin
	if(scale gt 3.5) then begin
		scale = scale * 0.75
		print,"Too few stars in G_DEFOCUS_FWHM. Reducing sigma threshold to ",scale
;		guess_FWHM = guess_FWHM * 1.5
;		print, "Also resetting guess FWHM to ",guess_FWHM
		iloop++
		goto, loop_start
	endif else begin
		print,"Still too few stars with lowest allowed threshold scaling. Results may be poor quality."
		if(good_count gt 0) then goto, loop_exit
		print, "ERROR - No stars in G_DEFOCUS_FWHM - using default values."
		s_count = 0
		status=-3
		goto, no_stars
	endelse
endif
;
loop_exit:
;
xg = x[ind]
yg = y[ind]
fluxg = flux[ind]
;
;	remove any fluxes beyond sat limit
;
inds = WHERE(fluxg le sat_limit,ns)
if(ns gt 0) then begin
	xg = xg[inds]
	yg = yg[inds]
	fluxg = fluxg[inds]
endif else begin
	s_count = 0
	status=-4
	goto, no_stars
endelse
;
;	sort by flux before throwing out stars
;
ind_sort = REVERSE(SORT(fluxg))
xg = xg[ind_sort]
yg = yg[ind_sort]
fluxg = fluxg[ind_sort]
;
;	kill off any with fluxes less than 3 sigma and any with fluxs < 1/50th of brightest
;
ind_good = WHERE(fluxg ge 3.0*mfm_stdev and fluxg gt 0.02* fluxg[0], ngood)
if(ngood gt 0) then begin
	xg = xg[ind_good]
	yg = yg[ind_good]
	fluxg = fluxg[ind_good]
endif else begin
	s_count = 0
	status=-5
	goto, no_stars
endelse
good_count = ngood
;
;	next, remove any stars with neighbors within 5 fwhm
;
gc1 = good_count-1
no_keep = intarr(good_count)
;
dr_test = 5.0 * guess_FWHM
;
for is1 = 0, gc1 do begin
	dx = xg - xg[is1]
	dy = yg - yg[is1]
	;
	dr = sqrt(dx^2 + dy^2)
	;
	ind_s = SORT(dr)
	;
	ind_del = WHERE(dr[ind_s] lt dr_test,n_kill)	; skip self
	if(n_kill gt 1) then begin
		no_keep[ind_s[ind_del[1:(n_kill-1)]]] = 1
	endif
endfor
;
n_no_keep = TOTAL(no_keep)
if(n_no_keep ge good_count) then begin
	print, 'ERROR - no isolated stars in image'
	status=-1
	s_count = 0
	goto, the_end
endif
;
if(n_no_keep gt 0) then begin
	ind_keep = WHERE(no_keep eq 0)
	xg = xg[ind_keep]
	yg = yg[ind_keep]
	fluxg = fluxg[ind_keep]
endif
;
n_stars = N_ELEMENTS(xg) < 300
;
fwhm = fltarr(n_stars)
axis_ratio_save = fltarr(n_stars)
;
if(debug ge 1) then print, "Number of stars to try to fit = ",n_stars
;
s_count = 0L
box_size = CEIL(2.5 * guess_FWHM)		; pixels - vs 15 for focussed images
;n_edge = 0L
;n_left = 0L
;n_right = 0L
;n_bottom = 0L
;n_top = 0L
n_bad_fit = 0L
n_sat = 0L
n_del = 0L
;
x_save = fltarr(n_stars)
y_save = fltarr(n_stars)
pk_save = fltarr(n_stars)
;
n_good_fit = 0L
;
for istar = 0, n_stars-1 do begin
	x_star = xg[istar]
	y_star = yg[istar]
	;
	;	extract sub array
	;
	xmin = xborder > (ROUND(x_star) - box_size)
	xmax = (nx-xborder) < (ROUND(x_star) + box_size)
	ymin = yborder > (ROUND(y_star) - box_size)
	ymax = (ny-yborder) < (ROUND(y_star) + box_size)
	nsx = xmax - xmin + 1
	nsy = ymax - ymin + 1
	;
	sub_array = image[xmin:xmax, ymin:ymax]
	;
	;	test for any saturated pixels within sub_array or more than 1 deleted pixels in the original array
	;
	ind_sat = WHERE(sub_array ge sat_limit,sat_count)
	if(sat_count gt 1) then begin
		n_sat++
		goto, loop_end
	endif
	;
	;	count bad pixels
	;
	ind_bad = WHERE(sub_array eq deleted_data, del_count)
	if(del_count gt 0) then begin
		;
		;	any within 4 pixels of star center?
		;
		xmin2 = xborder > (ROUND(x_star) - 4)
		xmax2 = (nx-xborder) < (ROUND(x_star) + 4)
		ymin2 = yborder > (ROUND(y_star) - 4)
		ymax2 = (ny-yborder) < (ROUND(y_star) + 4)
		;
		sub_array2 = image[xmin2:xmax2, ymin2:ymax2]
		ind_bad2 = WHERE(sub_array2 eq deleted_data, del_count2)
		;
		if(del_count2 gt 1) then begin
			n_del++
			goto, loop_end
		endif
	endif
	;
	;	clean any bad pixels not near star center
	;
	if(del_count ne 0) then sub_array = fix_bad_pixels(sub_array)
	;
	xs = indgen(nsx) + xmin
	ys = indgen(nsy) + ymin
	;
	;	compute sky mean outside of star
	;
	ind_sky = WHERE( sqrt( (xs - x_star)^2 + (ys - y_star)^2 ) gt 2.3 * guess_FWHM, nsky)
	if(nsky gt 0) then begin
		zz_sky = median_filtered_mean(sub_array[ind_sky])
	endif else begin
		zz_sky = [0.,0.]
	endelse
	;
	;	subtract sky from subimage
	;
	marginal_image = sub_array - zz_sky[0]
	;
	;	peak value
	;
	z_peak = MAX(marginal_image)
	;
	;	limit weights to those pixels within the brighter
	;	of 5% of the peak or 5 sigma of the background
	;
	wt_limit = (0.05 * z_peak) > (5.0 * zz_sky[1])
	;
	;	compute x, y moment of the intensity
	;
	positive_marginal_image = marginal_image
    ;
	ind_pmi = WHERE(positive_marginal_image ge wt_limit, n_pmi, COMPLEMENT=ind_zero, $
				NCOMPLEMENT=n_zero)
	;
	if(n_zero gt 0) then begin
		positive_marginal_image[ind_zero] = 0.0
	endif
	;
	x_z = positive_marginal_image
	y_z = x_z
	x2_z = x_z
	y2_z = x_z
	;
	for ix = 0, nsx-1 do begin
		x_z[ix,*] *= xs[ix]
		x2_z[ix,*] *= xs[ix] * xs[ix]
	endfor
	;
	for iy = 0, nsy-1 do begin
		y_z[*,iy] *= ys[iy]
		y2_z[*,iy] *= ys[iy] * ys[iy]
	endfor
	;
	flux_total = TOTAL(positive_marginal_image)
	x_center = TOTAL(x_z) / flux_total
	y_center = TOTAL(y_z) / flux_total
	;
	x_var = TOTAL(x2_z - positive_marginal_image * x_center * x_center) / flux_total
	x_sig = sqrt(x_var)
	;
	y_var = TOTAL(y2_z - positive_marginal_image * y_center * y_center) / flux_total
	y_sig = sqrt(y_var)
	;
	coeffs = [zz_sky[0], z_peak, x_sig, y_sig, x_center, y_center, 0.]
;	stop
	;
	;
	;	compute centroid and widths for this star using moments of x, y distributions
	;
;	gfit = gauss2dfit(sub_array,coeffs,xs,ys,/TILT)
	;
	if(debug ge 1) then begin
		print, "Star ",istar,"  Background Level = ",coeffs[0],"  Peak Fit value = ",coeffs[1]
		print, "X, Y widths (FWHM) = ",coeffs[2]*2.3548,coeffs[3]*2.3548,"  X, Y centers = ",coeffs[4],coeffs[5]
		;print, "Position Angle = ",coeffs[6]
	endif
	;
	;if(istar eq 7) then stop
	;
	;	reject any star with any coeff = -NaN
	;
	ind = WHERE(FINITE(coeffs[*]), count)
	if(count ne 7) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	if(coeffs[1] lt 0.5*hmin or coeffs[1] gt 1.5*sat_limit) then begin
		n_bad_fit++
		goto, loop_end
	end
	;
	if(abs(coeffs[2]) ge 40. or coeffs[2] lt 0.2) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	if(abs(coeffs[3]) ge 40. or coeffs[3] lt 0.2) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	if(coeffs[0] le (mfm_mean - 5.0*mfm_stdev)) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	if(coeffs[4] lt 0.0 or coeffs[4] gt float(nx)) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	if(coeffs[5] lt 0.0 or coeffs[5] gt float(ny)) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	harm_fwhm = 2.3548 * sqrt(coeffs[2]*coeffs[3])
	;
	axis_ratio = (coeffs[2] > coeffs[3]) / (coeffs[2] < coeffs[3])
	;
;	if(axis_ratio gt 2.5) then begin
;		n_bad_fit++
;		goto, loop_end
;	endif
	;
	if(FINITE(harm_fwhm)) then begin
			;
			if(harm_fwhm ge 1.38) then begin				; trap any smaller than 0.8 arcsec as unreasonable
;			if(harm_fwhm ge 0.5) then begin					; for narrow stars
				fwhm[s_count] = harm_fwhm
				axis_ratio_save[s_count] = axis_ratio
				x_save[s_count] = coeffs[4]
				y_save[s_count] = coeffs[5]
				pk_save[s_count] = coeffs[1]
				s_count++
			endif
	endif else begin
		n_bad_fit++
		goto, loop_end
	endelse
	;
	;	if the number of good fits is > 30 then jump out
	;
	n_good_fit++
	if(n_good_fit gt 30) then goto, jump_out
	;
	loop_end:
endfor
;
jump_out:
;
if(s_count lt 6) then begin
	if(debug ge 1) then begin
		print,"Problems in g_defocus_fwhm - too few stars with good FWHM values:"
		print,"  Number of trial FWHM stars = ",n_stars
		print,"  Number of stars rejected due to having one or more saturated pixels = ",n_sat
		print,"  Number of stars rejected due to having nearby deleted data pixels = ",n_del
		print,"  Number of stars rejected with bad gaussian fits = ",n_bad_fit
		print,"  Number of stars with good gaussian fits = ",s_count
	endif
	;
	if(s_count eq 0) then begin
		if(del_flag eq 0) then begin
			del_flag = 1
			if(debug ge 1) then print, 'Cleaning image of bad pixels and trying again'
			goto, del_loop
		endif
;		window, 24
;		plot, x, y, psym=2
;		stop
		status = -1
	endif else begin
		;
		;	loop, if permitted
		;
		if(scale gt 3.5) then begin
			scale = scale * 0.75
			iloop++
			print,"Too few stars in G_DEFOCUS_FWHM. Iteration ",iloop," Reducing sigma threshold to ",scale
			guess_FWHM = harm_fwhm
			print, "Also resetting guess FWHM to ",guess_FWHM
			goto, loop_start
		endif else begin
			print,"Still too few stars with lowest allowed threshold scaling. Results may be poor quality."
		endelse
		status = 0
	endelse
endif else status = 0
;
no_stars:
;
if(s_count eq 0) then begin
	;
	;	no good stars...insert default values
	;
	mean_FWHM = 7.0
	sig_FWHM = 3.0
	mean_axis_rtio = 1.0
	ab = [0., 0.]
	;
	status=0
	;
endif else begin
	;
	;	some good stars
	;
	fwhm = fwhm[0:s_count-1]
	axis_ratio_save = axis_ratio_save[0:s_count-1]
	;
	;	done, find mean, sig
	;
	zz = median_filtered_mean(fwhm)
	;
	mean_FWHM = zz[0]
	sig_FWHM = zz[1]
	;
	yy = median_filtered_mean(axis_ratio_save)
	ab = yy
	mean_axis_ratio = yy[0]
	;
	status=0
	;
endelse
;
if(debug ge 1) then begin
	print, "for ",s_count," stars with decent defocussed PSF values..."
	if(s_count gt 0) then begin
		for istar = 0, s_count-1 do begin
			print, "star ",istar, " at X,Y = ",x_save[istar],y_save[istar]," has fwhm = ",fwhm[istar]
		endfor
	endif
endif
if(debug ge 1) then print, "mean FWHM, unc. = ", mean_FWHM, sig_FWHM
;
;	compare the resulting FWHM to the guess, if more than 3 sigmas differnt,
;	restart with new value - do this only once, though
;
if(big_loop eq 0) then begin
	dif_FWHM = abs(guess_FWHM - mean_FWHM)
	if(dif_FWHM gt 3.0 * sig_FWHM) then begin
		guess_FWHM = mean_FWHM
		big_loop++
		goto, loop_start
	endif
endif
;
the_end:
;
nstars = s_count
;
end
