function load_vec, data
;
;	loads array data into return vector
;
isize = SIZE(data)
;help, data
;print, isize
;
nx = isize[1]
ny = isize[2]
;
vec = dblarr(nx * ny)
for ix = 0, nx-1 do begin
	for iy = 0, ny-1 do begin
		ii = iy + ix*ny
		vec[ii] = data[ix,iy]
	endfor
endfor
;
return, vec
end

function fit_this_surf, ng, data, array
;
;	loads the array into the third position of the data array
;	as a vector - assumes X, Y already loaded into data
;
data_vec = load_vec(array)
data[2,*] = data_vec[*]
;print, data_vec
;
;	do a fit
;
if(ng gt 3) then begin
	model = SFIT(data, 2, KX=kx, /IRREGULAR, /MAX_DEGREE)
endif else begin
	model = SFIT(data, 1, KX=kx, /IRREGULAR, /MAX_DEGREE)
endelse
;
return, kx
end

pro g_fit_PSFs, Event, ng, PSF_grid, PSF_model ;, debug
;
;	fits the grid of PSFs across the FOV to a 2nd order model
;	and returns the fit structure to the calling program
;
;	DPC		20080717
;			20080724 modified to include 1st order variation (ng=2) and no variation (ng=1) cases
;			20080922 modified to make ng=3 click down to 1st order, not 2nd order
;			20100221 added switch for cleaning parameters with mfm
;
;-----------------------------------
;
mfm_flag = 1				; use 0 for no parameter cleaning - only for ng > 3
mfm_threshold = 4.0			; sigma
;
;	add option to convolve with pixel size (DPC 20100222)
;
convol_flag = 0
convol_value = 0.10
convol_sign = +1.		; +1 does convol, -1 does deconvol
;
;	first, determine grid parameters
;
;help, PSF_grid
gsize = SIZE(PSF_grid)
if(gsize[0] ne 1) then begin
	ngx = gsize[1]
	ngy = gsize[2]
endif else begin
	ngx = 1
	ngy = 1
endelse
nn = ngx * ngy
;print, gsize
;
;	and unpack one structure to determine the size of the residuals arrays
;	and the radius of the psf function
;
if(nn ne 1) then begin
	nxypsf = PSF_grid[0,0].nxypsf
	rad_psf = PSF_grid[0,0].radpsf
	fit_rad = PSF_grid[0,0].fitrad
endif else begin
	nxypsf = PSF_grid.nxypsf
	rad_psf = PSF_grid.radpsf
	fit_rad = PSF_grid.fitrad
endelse
;
case ng of
	0: begin									; ng = 0 ERROR
		print, "ERROR - entered g_fit_psfs.pro with NG=0 -- shouldn't happen"
		stop
	   end
	1: begin									; ng=1 no PSF variation case
		gauss_fit = dblarr(5)
		resid_fit = dblarr(nxypsf, nxypsf)
	   end
	2: begin									; ng=2 1st order surface fit
		gauss_fit = dblarr(5,3)
		resid_fit = dblarr(nxypsf, nxypsf, 3)	; k, y, x terms
	   end
 	3: begin									; ng=3 1st order surface fit
		gauss_fit = dblarr(5,3)
		resid_fit = dblarr(nxypsf, nxypsf, 3)	; k, y, x terms
   	   end
	else: begin									; ng > 3 -> 2nd order
			gauss_fit = dblarr(5,6)					; gaussian term x SFIT kx term
			resid_fit = dblarr(nxypsf, nxypsf,6)
	      end
endcase
;
psfmag = PSF_grid[0,0].psfmag	;**** this will need examination for ng>1  DPC 20090729
;
PSF_model = {	nxypsf: nxypsf, $
				psfmag: psfmag, $
				rad_psf: rad_psf, $
				fit_rad: fit_rad, $
				gauss_fit: gauss_fit, $
				resid_fit: resid_fit $
			 }
;
if(ng ge 2) then PSF_model.fit_rad = PSF_grid[0,0].fitrad else PSF_model.fit_rad = PSF_grid.fitrad
;
;
;	load data with x, y values
;
data = dblarr(3, nn)
;
if(nn ne 1) then begin
	for ix = 0, ngx-1 do begin
		for iy = 0, ngy-1 do begin
			ii = iy+ngy*ix
			data[0,ii] = PSF_grid[ix,iy].x_wt
			data[1,ii] = PSF_grid[ix,iy].y_wt
		endfor
	endfor
endif									; for fixed PSF, don't load fits
;
;	first, load and fit gauss data
;
if(ng ge 2) then begin
	for ig = 0, 4 do begin
		;
		;	if ng > 3 do mfm cleaning if flag set
		;
		if(mfm_flag gt 0 and ng gt 3) then begin
			vec = REFORM( PSF_grid[*,*].gauss[ig] )
			zz = median_filtered_mean(vec)
			;
			;	test for deviations
			;
			ind_fix = WHERE( abs(PSF_grid[*,*].gauss[ig] - zz[0])/zz[1] gt mfm_threshold, n_fix)
			if(n_fix gt 0) then begin
				PSF_grid[ind_fix].gauss[ig] = zz[0]
			endif
		endif
		;
		kx = fit_this_surf(ng, data, PSF_grid[*,*].gauss[ig])
		PSF_model.gauss_fit[ig,*] = kx[*]
	endfor
	;
	if(convol_flag eq 1) then begin
		;
		;	convolve widths with sampling due to pixelation, but preserve integral by
		;	lowering strength
		;
		area = PSF_model.gauss_fit[3,*] * PSF_model.gauss_fit[4,*] * PSF_model.gauss_fit[0,*]
		;
		PSF_model.gauss_fit[3,*] = sqrt(PSF_model.gauss_fit[3,*]^2 + convol_sign * (convol_value^2))
		PSF_model.gauss_fit[4,*] = sqrt(PSF_model.gauss_fit[4,*]^2 + convol_sign * (convol_value^2))
		;
		PSF_model.gauss_fit[0,*] = area / (PSF_model.gauss_fit[3,*] * PSF_model.gauss_fit[4,*])
	endif
endif else begin
	PSF_model.gauss_fit[*] = PSF_grid.gauss[*]
	if(convol_flag eq 1) then begin
		;
		area = PSF_model.gauss_fit[3] * PSF_model.gauss_fit[4] * PSF_model.gauss_fit[0]
		;
		PSF_model.gauss_fit[3] = sqrt(PSF_model.gauss_fit[3]^2 + convol_sign * (convol_value^2))
		PSF_model.gauss_fit[4] = sqrt(PSF_model.gauss_fit[4]^2 + convol_sign * (convol_value^2))
		;
		PSF_model.gauss_fit[0] = area / (PSF_model.gauss_fit[3] * PSF_model.gauss_fit[4])
	endif
endelse
;
;	then all the residuals
;;
if(ng ge 2) then begin
	for ix = 0, nxypsf-1 do begin
		for iy = 0, nxypsf-1 do begin
			if(mfm_flag gt 0 and ng gt 3) then begin
				vec = REFORM( PSF_grid[*,*].resid[ix,iy] )
				zz = median_filtered_mean(vec)
				;
				;	test for 5 sigma deviations
				;
				ind_fix = WHERE( abs(PSF_grid[*,*].resid[ix,iy] - zz[0])/zz[1] gt mfm_threshold, n_fix)
				if(n_fix gt 0) then begin
					PSF_grid[ind_fix].resid[ix,iy] = zz[0]
				endif
			endif
			;
			kx = fit_this_surf(ng, data, PSF_grid[*,*].resid[ix,iy])
			PSF_model.resid_fit[ix,iy,*] = kx[*]
		endfor
	endfor
endif else begin
	PSF_model.resid_fit[*,*] = PSF_grid.resid[*,*]
endelse
;
;stop
;
;if(debug) then begin
;	nlines = N_ELEMENTS(kx)
;	for ilines = 0, nlines-1 do begin
;		print_text, Event, "kx"+STRING(ilines)+") = "+STRING(kx[ilines])
;	endfor
;endif
;
end


