pro g_gaussian_fit_one_star, uncleaned_image, fwhm_in, x, y, mag, mag_unc, sky, sky_unc, debug, fwhm_out
;
;	does a 2d gaussian fit to one star in an image
;
;	based on engine in the "fast 4Q photometry" code
;
;	20090429	new code - DPC
;
;debug = 1
;
deleted_data = -1.0e6
image_size = SIZE(uncleaned_image)
nx = image_size[1]
ny = image_size[2]
;
this_x = x[0]
this_y = y[0]
;
if(debug gt 0) then begin
	print, " "
	print, "Starting gaussian fitting for star at X, Y, fwhm = ",this_x,this_y, fwhm
endif
;
x0 = FLOOR(this_x-15) > 0
x1 = CEIL(this_x+15) < (nx-1)
y0 = FLOOR(this_y-15) > 0
y1 = CEIL(this_y+15) < (ny-1)
;
ngx = x1 - x0 + 1
ngy = y1 - y0 + 1
xg = findgen(ngx) + x0
yg = findgen(ngy) + y0
;
;	create 2D versions
;
MAKE_2D, xg, yg, xxg, yyg
;
xg = reform(xg)
yg = reform(yg)
;
sub_image = uncleaned_image[x0:x1,y0:y1]
ind = -1
;
;	select sub_image locations outside of central fwhm
;
rad = sqrt((xxg-this_x)^2 + (yyg-this_y)^2)
;
ind_outer = WHERE(rad gt 3.0*fwhm_in, count_outer)
if(count_outer eq 0) then begin
	print, "ERROR - g_gaussian_fit_one_star has no pixels outside fwhm = ",fwhm
	goto, bad_fit
endif
;
;	clean pixels outside fwhm of both high and low spikes
;
zz = median_filtered_mean(sub_image[ind_outer])
ind = where(abs(sub_image[ind_outer] - zz[0]) gt 5.0*zz[1] ,count)
;
if(count ne 0) then begin
	if(debug gt 0) then print, "Cleaning ",count," background pixels"
	sub_image[ind_outer[ind]] = zz[0]
endif
;
;	and clean pixels within that limit of deleted values
;
ind_inner = WHERE(rad le 3.0*fwhm_in, count_inner)
;
ind_bad = WHERE(sub_image[ind_inner] eq deleted_data, count_bad)
if(count_bad gt 0) then begin
	sub_image = fix_bad_pixels(sub_image)
endif
;
if(debug) then begin
	min_image = MIN(sub_image)
	tx = ROUND(this_x)
	ty = ROUND(this_y)
	;
	big_sub_image = fltarr(ngx*15, ngy*15)
	for ix = 0, ngx-1 do begin
		for iy = 0, ngy-1 do begin
			ix0 = ix * 15
			ix1 = ix0 + 14
			iy0 = iy * 15
			iy1 = iy0 + 14
			big_sub_image[ix0:ix1,iy0:iy1] = sqrt(sub_image[ix,iy]-min_image+0.001)		;sqrt(abs(psf_image[ix,iy]))
		endfor
	endfor
	;if(this_debug) then
	z = display_image ( big_sub_image, 22, MESSAGE='Star ('+STRING(tx,ty,format='(i4,1x,i4)')+')', BIPOLAR=0)
	;
	iyn = ''
	print, "continue ?"
	read, iyn
	if(iyn ne 'y' and iyn ne 'Y') then stop
endif
;
gfit = gauss2dfit(sub_image,coeffs,xg,yg,/TILT)
;
if(debug gt 0) then begin
	print, "Background Level = ",coeffs[0]
	print, "Peak Fit value = ",coeffs[1]
	print, "X, Y widths (FWHM) = ",coeffs[2]*2.3548,coeffs[3]*2.3548
	print, "X, Y centers = ",coeffs[4],coeffs[5]
	print, "Position Angle = ",coeffs[6]
	;
	ymax = MAX(sub_image)
	print, "Maximum pixel value = ",ymax
endif
;
;	test quality of fit
;
if(coeffs[4] lt 0 or coeffs[4] gt (nx-1)) then goto, bad_fit
if(coeffs[5] lt 0 or coeffs[5] gt (ny-1)) then goto, bad_fit
if(coeffs[2] lt 0.20 or coeffs[3] lt 0.20) then goto, bad_fit
if(coeffs[2] gt 5.0*fwhm_in or coeffs[3] gt 5.0*fwhm_in) then goto, bad_fit
if(coeffs[1] lt 0.0) then goto, bad_fit
;
;   go to the center, compute offsets of all pixel values, plot
;
;xcen = coeffs[4] - cx + 15
;ycen = coeffs[5] - cy + 15
;print, "xcen, ycen = ",xcen,ycen
;rad_offset = FltArr(31*31)
;zed_offset = FltArr(31*31)
;for ix = 0, 30 do begin
;    for iy = 0, 30 do begin
;        if(sub_image[ix,iy] ne deleted_data) then begin
;            rad_offset(ix*31+iy) = sqrt((float(ix)-xcen)^2 + (float(iy)-ycen)^2)
;            zed_offset(ix*31+iy) = sub_image[ix,iy]
;        endif
;    endfor
;endfor
;
;   make model version, too
;
rad_wid = sqrt(coeffs[2]*coeffs[3])
;
fwhm_out = rad_wid * 2.354
;
;xmodel = FIndGen(31)*0.5
;zmodel = FltArr(31)
;for iz = 0, 30 do begin
;    r = xmodel(iz)
;    zmodel(iz) = coeffs[0]+coeffs[1]*exp(-(r^2/(2.0*rad_wid*rad_wid)))
;endfor
;
;
;window,12,title="PSF model and data",xsize=600,ysize=500
;y0 = -50. > (coeffs[0]-0.15*coeffs[1])
;y1 = 2. > (coeffs[0]+1.10*coeffs[1])
;;
;plot, rad_offset, zed_offset, psym=4,xrange=[0,15],yrange=[y0,y1],position=[0.125,0.125,0.95,0.95]
;;
;oplot, xmodel, zmodel, color=1, thick=2
;
;   use the model to compute the total counts and instrumental magnitude
;
flux = total(gfit)-N_Elements(gfit)*coeffs[0]
mag = -2.5 * alog10(flux)
;
;	and the background fluctuations
;
background = sub_image - gfit
back_vec = REFORM(background[*,*])
zz = median_filtered_mean(back_vec)
;
if(coeffs[1] lt 2.0*zz[1]) then goto, bad_fit		; if under 2 sigma background RMS
;
;	and photometric uncertainty
;
gain = 8.21
noise = (1./gain) * sqrt(!DPI * (zz[1]*gain*rad_wid*2.3548)^2 + gain * flux)
S_to_N = flux / noise
s_mag = 1. / S_to_N
;
;   report fit parameters
;
if(debug gt 0) then begin
	a = coeffs[2] > coeffs[3]
	b = coeffs[2] < coeffs[3]
	axis = a/b
	pa = -1.0 * coeffs[6] * (57.29578)
	rad_wid = rad_wid * 2.3548
	wid_results = string(format='("FWHM =",F6.3," pixels")',rad_wid)
	print,wid_results
	cen_results = string(format='("X,Y Center = ",F6.1,",",F6.1," pixels")',coeffs[4],coeffs[5])
	print,cen_results
	axis_results = string(format='("Axis ratio, PA = ",F4.2,",",F6.1)',axis,pa)
	print,axis_results
	mag_results = string(format='("Instrumental Mag = ",f7.3," +/- ",f6.3)',mag,s_mag)
	print,mag_results
	back_rms = STRING(zz[0],zz[1],format='("Background Mean, RMS = ",f9.1,f8.1," ADUs")')
	print,back_rms
endif
;
;stop
;
;	return parameters for good fit
;
x = coeffs[4]
y = coeffs[5]
mag = 25.0d0 + mag
mag_unc = s_mag
sky = zz[0] + coeffs[0]
sky_unc = zz[1]
goto, the_end
;
bad_fit:
;
if(debug gt 0) then begin
	print, "Failed fit for star at X,Y = ",this_x,this_y
	print, "...fit coeffs = ",coeffs
endif
;
mag = 25.0
mag_unc = 25.0
sky = 0.0
sky_unc = 1.0e6
fwhm_out = -1.0
;
the_end:
;
end