pro g_get_FWHM, in_image, find_image, skymode, skysig, mfm_mean, mfm_stdev, $
			mean_FWHM, sig_FWHM, sat_limit, debug, AB = ab, NSTARS = nstars, $
			NARROW = narrow, STATUS = status, F17=f17
;
;	uses FIND and gaussian fitting of the brightest
;	ten stars in the image to produce a good estimate
;	of the mean FWHM and its uncertainty
;
;	DPC	20080624
;	    20080724	modified to skip stars with pixel values beyond SAT_LIMIT
;		20110315	cleaned up alot. added nstars keyword on exit
;		20121230	added narrow keyword
;		20131005	added F17 keyword
;
;----------------------------------------------
;
if(KEYWORD_SET(narrow)) then i_narrow = 1 else i_narrow = 0
if(KEYWORD_SET(f17)) then i_17 = 1 else i_17 = 0
;
;i_narrow = 1				; for very narrow PSFs (short exposures) set this to '1', else normal set to '0'
;							;  DPC 20121209
;
;debug = 2
;
;	if the following flag is set, impose a 50:1 brightness limit. If set to 0, no limit imposed
;
b50_flag = 1			; normally 1
if(i_17 eq 1) then b50_flag = 0
;
del_flag = 0
deleted_data = -1.0e6
;
i_size = SIZE(in_image)
nx = i_size[1]
ny = i_size[2]
;
status = 0
;
del_loop:
;
if(del_flag eq 0) then image = in_image else image = fix_bad_pixels(in_image)
;
if(nx eq 1024) then coadd = 0 else if(nx gt 1024) then coadd = 1 else coadd = -1
case coadd of
	0: begin
		xborder = 4
		yborder = 20
	   end
	1: begin
		xborder = 30
		yborder = 105
	   end
   -1: begin
   		xborder = 5
   		yborder = 7
   	   end
endcase
;
roundlim = [-1.5, 1.5]
sharplim = [0.1, 0.9]
;
scale = 8.0
if(i_17 eq 1) then scale = 4.0
iloop = 0
;
if(i_narrow eq 1) then begin
	if(i_17 eq 0) then begin
		guess_FWHM = 1.3 		; f5 first guess
	endif else begin
		guess_FWHM = 4.5		; f17 first guess
	endelse
endif else begin
	if(i_17 eq 0) then begin
		guess_FWHM = 3.0
	endif else begin
		guess_FWHM = 10.0
	endelse
endelse
;
;
loop_start:
;
hmin = scale*mfm_stdev
;
if(debug lt 2) then silent = 1 else silent = 0
;
g_find, find_image, x, y, flux, sharp, roundness, hmin, guess_FWHM, $
	roundlim, sharplim, SILENT=silent, /NO_BAD_PIX_FIX
;
nstars = N_ELEMENTS(x)
;
;	first, reject all within some pixels of X boundary and Y boundary
;
;p_reject = 30
;
;	test for no stars
;
n_flux = N_ELEMENTS(flux)
if(n_flux eq 0) then begin
	s_count = 0
	status=-2
	goto, no_stars
endif
;
ind = WHERE(FINITE(flux) and x gt xborder and x lt (nx-xborder) and y gt yborder and y lt (ny-yborder),good_count)
;
if(debug gt 0) then print, 'Number of good_count stars = ',good_count
;
if(good_count lt 10) then begin
	if(scale gt 3.5) then begin
		scale = scale * 0.75
		print,"Too few stars in G_GET_FWHM. Reducing sigma threshold to ",scale
;		guess_FWHM = guess_FWHM * 1.5
;		print, "Also resetting guess FWHM to ",guess_FWHM
		iloop++
		goto, loop_start
	endif else begin
		print,"Still too few stars with lowest allowed threshold scaling. Results may be poor quality."
		if(good_count gt 0) then goto, loop_exit
		print, "ERROR - No stars in G_GET_FWHM - using default values."
		status=-3
		goto, no_stars
	endelse
endif
;
loop_exit:
;
xg = x[ind]
yg = y[ind]
fluxg = flux[ind]
;
;	remove any fluxes beyond sat limit
;
inds = WHERE(fluxg le sat_limit,ns)
if(ns gt 0) then begin
	xg = xg[inds]
	yg = yg[inds]
	fluxg = fluxg[inds]
endif else begin
	s_count = 0
	status=-4
	goto, no_stars
endelse
;
;	sort by flux before throwing out stars
;
ind_sort = REVERSE(SORT(fluxg))
xg = xg[ind_sort]
yg = yg[ind_sort]
fluxg = fluxg[ind_sort]
;
;	kill off any with fluxes less than 3 sigma and any with fluxs < 1/50th of brightest
;
test1 = fluxg ge 3.0*mfm_stdev
test2 = fluxg gt 0.02*fluxg[0]
;
ind_good = WHERE(test1 and (test1 or (1-b50_flag)), ngood)
;
if(debug gt 0) then print, 'Number of stars with more than 3 sigma and at least 2% of brightest = ',ngood
;
if(ngood gt 0) then begin
	xg = xg[ind_good]
	yg = yg[ind_good]
	fluxg = fluxg[ind_good]
endif else begin
	s_count = 0
	status=-5
	goto, no_stars
endelse
good_count = ngood
;
;	next, remove any stars with neighbors within 5 fwhm
;
gc1 = good_count-1
no_keep = intarr(good_count)
;
dr_test = 5.0 * guess_FWHM
;
if(i_17 eq 1) then dr_test = 2.0 * guess_FWHM
;
for is1 = 0, gc1 do begin
	dx = xg - xg[is1]
	dy = yg - yg[is1]
	;
	dr = sqrt(dx^2 + dy^2)
	;
	ind_s = SORT(dr)
	;
	ind_del = WHERE(dr[ind_s] lt dr_test,n_kill)	; skip self
	if(n_kill gt 1) then begin
		no_keep[ind_s[ind_del[1:(n_kill-1)]]] = 1
	endif
endfor
;
n_no_keep = TOTAL(no_keep)
if(n_no_keep ge good_count) then begin
	print, 'ERROR - no isolated stars in image'
	s_count = 0
	status=-1
;	if(debug gt 50) then stop
	goto, the_end
endif
;
if(n_no_keep gt 0) then begin
	ind_keep = WHERE(no_keep eq 0)
	xg = xg[ind_keep]
	yg = yg[ind_keep]
	fluxg = fluxg[ind_keep]
endif
;
n_stars = N_ELEMENTS(xg) < 300
;
fwhm = fltarr(n_stars)
axis_ratio_save = fltarr(n_stars)
;
if(debug gt 0) then print, "Number of stars to try to fit = ",n_stars
;
s_count = 0L
box_size = 15 * (1.0 + 3.0*i_17)	; pixels
;n_edge = 0L
;n_left = 0L
;n_right = 0L
;n_bottom = 0L
;n_top = 0L
n_bad_fit = 0L
n_sat = 0L
n_del = 0L
;
x_save = fltarr(n_stars)
y_save = fltarr(n_stars)
pk_save = fltarr(n_stars)
;
n_good_fit = 0L
;
for istar = 0, n_stars-1 do begin
	x_star = xg[istar]
	y_star = yg[istar]
	;
	;	extract sub array
	;
	xmin = xborder > (ROUND(x_star) - box_size)
	xmax = (nx-xborder) < (ROUND(x_star) + box_size)
	ymin = yborder > (ROUND(y_star) - box_size)
	ymax = (ny-yborder) < (ROUND(y_star) + box_size)
	nsx = xmax - xmin + 1
	nsy = ymax - ymin + 1
	;
	sub_array = image[xmin:xmax, ymin:ymax]
	;
	;	test for any saturated pixels within sub_array or more than 1 deleted pixels in the original array
	;
	ind_sat = WHERE(sub_array ge sat_limit,sat_count)
	if(sat_count gt 1) then begin
		n_sat++
		goto, loop_end
	endif
	;
	;	count bad pixels
	;
	ind_bad = WHERE(sub_array eq deleted_data, del_count)
	if(del_count gt 0) then begin
		;
		;	any within 4 pixels of star center?
		;
		xmin2 = xborder > (ROUND(x_star) - 4)
		xmax2 = (nx-xborder) < (ROUND(x_star) + 4)
		ymin2 = yborder > (ROUND(y_star) - 4)
		ymax2 = (ny-yborder) < (ROUND(y_star) + 4)
		;
		sub_array2 = image[xmin2:xmax2, ymin2:ymax2]
		ind_bad2 = WHERE(sub_array2 eq deleted_data, del_count2)
		;
		if(del_count2 gt 1) then begin
			n_del++
			goto, loop_end
		endif
	endif
	;
	;	clean any bad pixels not near star center
	;
	if(del_count ne 0) then sub_array = fix_bad_pixels(sub_array)
	;
	xs = indgen(nsx) + xmin
	ys = indgen(nsy) + ymin
	;
	;	gaussian fit this star
	;
	gfit = gauss2dfit(sub_array,coeffs,xs,ys,/TILT)
	;
	if(debug ge 1) then begin
		print, "Star ",istar,"  Background Level = ",coeffs[0],"  Peak Fit value = ",coeffs[1]
		print, "X, Y widths (FWHM) = ",coeffs[2]*2.3548,coeffs[3]*2.3548,"  X, Y centers = ",coeffs[4],coeffs[5]
		;print, "Position Angle = ",coeffs[6]
	endif
	;
	;	reject any star with any coeff = -NaN
	;
	ind = WHERE(FINITE(coeffs[*]), count)
	if(count ne 7) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	if(coeffs[1] lt 0.5*hmin or coeffs[1] gt 1.5*sat_limit) then begin
		n_bad_fit++
		goto, loop_end
	end
	;
	if(abs(coeffs[2]) ge 15.*(1.0+2.0*i_17) or coeffs[2] lt 0.2*(1.0+2.0*i_17)) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	if(abs(coeffs[3]) ge 15.*(1.0+2.0*i_17) or coeffs[3] lt 0.2*(1.0+2.0*i_17)) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	if(coeffs[0] le (mfm_mean - 3.0*mfm_stdev)) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	if(coeffs[4] lt 0.0 or coeffs[4] gt float(nx)) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	if(coeffs[5] lt 0.0 or coeffs[5] gt float(ny)) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	harm_fwhm = 2.3548 * sqrt(coeffs[2]*coeffs[3])
	;
	axis_ratio = (coeffs[2] > coeffs[3]) / (coeffs[2] < coeffs[3])
	;
	if(axis_ratio gt 2.5 and i_17 eq 0) then begin
		n_bad_fit++
		goto, loop_end
	endif
	;
	if(FINITE(harm_fwhm)) then begin
			;
			if(harm_fwhm ge (1.38 - 0.88*i_narrow)*(1.0+2.0*i_17)) then begin	; trap any smaller than 0.8 (or 0.3) arcsec as unreasonable
				;
				fwhm[s_count] = harm_fwhm
				axis_ratio_save[s_count] = axis_ratio
				x_save[s_count] = coeffs[4]
				y_save[s_count] = coeffs[5]
				pk_save[s_count] = coeffs[1]
				if(debug ge 1) then begin
					print, "Star ",s_count," x,y,pk,fwhm,axis = ",x_save[s_count],y_save[s_count],$
							pk_save[s_count],fwhm[s_count],axis_ratio_save[s_count]
				endif
				s_count++
			endif
	endif else begin
		n_bad_fit++
		goto, loop_end
	endelse
	;
	;	if the number of good fits is > 30 then jump out
	;
	n_good_fit++
	if(n_good_fit gt 30) then goto, jump_out
	;
	loop_end:
endfor
;
jump_out:
;
if(s_count lt 6) then begin
	if(debug ge 1) then begin
		print,"Problems in g_get_fwhm - too few stars with good FWHM fits:"
		print,"  Number of trial FWHM stars = ",n_stars
		print,"  Number of stars rejected due to having one or more saturated pixels = ",n_sat
		print,"  Number of stars rejected due to having nearby deleted data pixels = ",n_del
		print,"  Number of stars rejected with bad gaussian fits = ",n_bad_fit
		print,"  Number of stars with good gaussian fits = ",s_count
	endif
	;
	if(s_count eq 0) then begin
		if(del_flag eq 0) then begin
			del_flag = 1
			if(debug ge 1) then print, 'Cleaning image of bad pixels and trying again'
			goto, del_loop
		endif
;		window, 24
;		plot, x, y, psym=2
;		stop
		status = -1
	endif else begin
		;
		;	loop, if permitted
		;
		if(scale gt 3.5) then begin
			scale = scale * 0.75
;		guess_FWHM = guess_FWHM * 1.5
;		print, "Also resetting guess FWHM to ",guess_FWHM
			iloop++
			print,"Too few stars in G_GET_FWHM. Iteration ",iloop," Reducing sigma threshold to ",scale
			goto, loop_start
		endif else begin
			print,"Still too few stars with lowest allowed threshold scaling. Results may be poor quality."
		endelse
		status = 0
	endelse
endif else status = 0
;
no_stars:
;
if(s_count eq 0) then begin
	;
	;	no good stars...insert default values
	;
	mean_FWHM = 3.0
	sig_FWHM = 3.0
	mean_axis_ratio = 1.0
	ab = [0.,0.]
	;
	status=0
	;
endif else begin
	;
	;	some good stars
	;
	fwhm = fwhm[0:s_count-1]
	axis_ratio_save = axis_ratio_save[0:s_count-1]
	;
	;	done, find mean, sig
	;
	zz = median_filtered_mean(fwhm)
	;
	mean_FWHM = zz[0]
	sig_FWHM = zz[1]
	;
	yy = median_filtered_mean(axis_ratio_save)
	ab = yy
	mean_axis_ratio = yy[0]
	;
	status=0
	;
endelse

;
if(debug ge 1) then begin
	print, "for ",s_count," stars with decent gaussian fits..."
	if(s_count gt 0) then begin
		for istar = 0, s_count-1 do begin
			print, "star ",istar, " at X,Y = ",x_save[istar],y_save[istar]," has PK = ",pk_save[istar],$
						", fwhm = ",fwhm[istar],$
						",  and A/B = ",axis_ratio_save[istar]
		endfor
	endif
endif
if(debug ge 1) then print, "mean FWHM, unc. = ", mean_FWHM, sig_FWHM," mean A/B = ",mean_axis_ratio
;
the_end:
;
nstars = s_count
;
end
