pro g_getpsf, Event, ng, input_image, x, y, mag, sky, ronois, phpadu, gauss, psf, id_psf, $
			ref_apr, rad_psf, fit_rad, fwhm, PSF_grid, bad_pix_limits, bright_pix_limit, debug, $
			grid_fwhm, status, NO_BAD_PIX_FIX=no_bad_pix_fix, LOOP_COUNT=loop_count, $
			FWHM_LOOP_COUNT=fwhm_loop_count
;
;	fits PSF to selected stars
;
;	calls this_GETPSF for each fit zone, then loads results buffer (PSF_grid)
;
;	DPC	20080625
;
;---------------------------------------------------------------
;
;
;debug = 1
;
;	test for, remove NaNs in image
;
;ind_nan = WHERE(~FINITE(input_image),n_nan)
;if(n_nan gt 0) then begin
;	print, "found ",n_nan," NaNs in image - fixing"
;	input_image[ind_nan] = -1.0e6
;	input_image = fix_bad_pixels(input_image)
;endif
;
x_save = x
y_save = y
mag_save = mag
sky_save = sky
;
debug_by_zone = intarr(ng,ng)
;debug_by_zone[0,1] = 1
;
iplot = 0
isurface_plot = 0
do_COG = 0					; if 0, don't do COG correction; if 1 do COG correction
;
if(N_ELEMENTS(loop_count) eq 0) then loop_n = 0 else loop_n = loop_count	; which PSF build iteration ?
;																			; will use to set resid cutoff
if(N_ELEMENTS(fwhm_loop_count) eq 0) then fwhm_loop = 1 else fwhm_loop = fwhm_loop_count
;
;print_text, Event, 'in g_getpsf, loop count = '+STRING(loop_n)
;
;	how many zones? query size of id_psf
;
isize = SIZE(id_psf)
nx_zones = isize[1]
ny_zones = isize[2]
n_star_max = isize[3]		;  no more than 100, with unused positions set to -99
;
;	size of PSF residuals arrays
;
n = 2 * FIX(rad_psf + 0.5) + 1
nxypsf = 2 * n + 7						; 3 was 7 - dpc 20090507; back to 7 DPC 20090723
;
;		set up the data structure that will capture the PSF fits for all zones
;
grid_struct = {		x_wt: 0.0d0, $		; weighted X value of PSF stars in this PSF
					y_wt: 0.0d0, $		; weighted Y value
					nxypsf:  nxypsf, $		; number of rows, columns in residuals image
					radpsf: rad_psf, $	; radius of PSF in full image pixels
					fitrad: fit_rad, $	; radius for photometric fitting
					psfmag: 0.0d0, $	; psf zero point magnitude
					gauss: dblarr(5),$	; gaussian parameters
					scale: 0.0d0,$		; scale factor applied to gauss[0] and residuals
					resid: dblarr(nxypsf, nxypsf) $	; the residuals array (with 1/2 pixel sampling)
				}
PSF_grid = REPLICATE(grid_struct, nx_zones, ny_zones)	; an array, now, note...
;
cog_cor_by_zone = dblarr(nx_zones, ny_zones)
;
;	----- if debug, set up size of image to hold final array of PSFs
;
if(debug gt 0) then begin
	box = rad_psf * 2 + 1
	PSF_final_image = fltarr(((box+1)*nx_zones)+1, ((box+1)*ny_zones)+1)
endif
;
;	clean bad pixels
;
if(~KEYWORD_SET(no_bad_pix_fix)) then begin
	print, 'g_getpsf - fixing bad pixels'
	image = fix_bad_pixels(input_image)
endif else image = input_image
;
isize = SIZE(image)
ncol = isize[1]
nrow = isize[2]
xicen = ncol / 2.0
yicen = nrow / 2.0
;
;	save key values
;
s_psfmag = fltarr(nx_zones, ny_zones)
s_gain = fltarr(nx_zones,ny_zones)
s_resid = fltarr(nx_zones, ny_zones)
;
s_g0 = fltarr(nx_zones, ny_zones)
;
s_xcen = fltarr(nx_zones, ny_zones)
s_ycen = fltarr(nx_zones, ny_zones)
;
for ixz = 0, nx_zones-1 do begin
	for iyz = 0, ny_zones-1 do begin
		;
		debug = debug_by_zone[ixz,iyz]
		;
		;	select stars and pass to GETPSF routine
		;
		ind = WHERE(id_psf[ixz,iyz,*] ne -99,count)
		if(count eq 0) then begin
			print_text, Event, "G_GETPSF: No stars to fit for zone [x,y] = "+STRING(ixz,iyz)
			;
			;	if this is the only zone, set status=-1 and quit, else continue
			;
			if(nx_zones+ny_zones eq 2) then begin
				print_text, Event, "G_GETPSF: This is a single zone image, so no stars is fatal..."
				status=-1
				goto, the_end
			endif
			print_text, Event, "G_GETPSF: This is not a single zone image, so not fatal. Clearing PSF info for this zone"
				PSF_grid[ixz,iyz].x_wt = 0.0
				PSF_grid[ixz,iyz].y_wt = 0.0
				PSF_grid[ixz,iyz].gauss[*] = 0.0
				PSF_grid[ixz, iyz].scale = 0.0
				PSF_grid[ixz,iyz].resid = 0.0
				PSF_grid[ixz,iyz].fitrad = fit_rad
			goto, zone_end
		endif
		;
		xp = REFORM(x[id_psf[ixz,iyz,ind]])
		yp = REFORM(y[id_psf[ixz,iyz,ind]])
		magp = REFORM(mag[id_psf[ixz,iyz,ind]])
		skyp = REFORM(sky[id_psf[ixz,iyz,ind]])
		;
		;	order by brightness, so brightest star goes first
		;
		ind_mag = SORT(magp)
		;
		xp = xp[ind_mag]
		yp = yp[ind_mag]
		magp = magp[ind_mag]
		skyp = skyp[ind_mag]
		n_p = N_ELEMENTS(xp)
		;
		good_stars = N_ELEMENTS(xp)
		;
		;	dump values
		;
		if(debug gt 0) then begin
			;
            ;----------------------- tests 20100308 DPC
;			print, 'trimming PSF star list to the one star'
;			ind_st = WHERE(abs(xp - 77) gt 2.0 or abs(yp - 236) gt 2.0,nst)
;			if(nst eq 1) then begin
;				xp = xp[ind_st]
;				yp = yp[ind_st]
;				magp = magp[ind_st]
;				skyp = skyp[ind_st]
;				good_stars = N_ELEMENTS(xp)
;			endif
			;------------------------
			for ic = 0, good_stars-1 do begin
				print, "G_GETPSF: x,y,mag,sky = "+STRING(xp[ic],yp[ic],magp[ic],skyp[ic])
			endfor
		endif
		;
		id_psfp = indgen(good_stars)
		;
		;	prepare to catch any error thrown (esp if this_GETPSF kills off all PSF stars it is fed)
		;
		ierr = 0
		this_fwhm = grid_fwhm[ixz,iyz]
;		print, "grid_fwhm for cell [x,y] = ",ixz,iyz," = ",this_fwhm
;		print, "going into this_GETPSF with xp = ",xp
;		debug = 1
		;
		;
		CATCH, ierr
		;
		if(ierr ne 0) then begin
			print, "ERROR - ierr = ",ierr
			CATCH, /CANCEL
			print, "G_GETPSF: error thrown by this_GETPSF, caught by G_GET_PSF"
			;
;			if(ierr eq -161 and sum_resid lt 0.0) then begin
;				print, "Caught array problems with negative residuals. Ignoring and continuing"
;				goto, continue_this
;			endif
			;
			ne_psf = N_ELEMENTS(xp)
			if(ne_psf ne 0) then begin
				print,"     star   x     y     mag"
				for ie = 0, ne_psf-1 do begin
					print,ie,xp[ie],yp[ie],magp[ie]
				endfor
			endif
			print_text, Event, "G_GETPSF: No stars could be fit for zone [x,y] = "+STRING(ixz,iyz)
			;
			;	if this is the only zone, set status=-1 and quit, else continue
			;
			if(nx_zones+ny_zones eq 2) then begin
				print_text, Event, "G_GETPSF: This is a single zone image, so no stars is fatal..."
				status=-1
				goto, the_end
			endif
			print_text, Event, "G_GETPSF: This is not a single zone image, so not fatal. Clearing PSF info for this zone"
				PSF_grid[ixz,iyz].x_wt = 0.0
				PSF_grid[ixz,iyz].y_wt = 0.0
				PSF_grid[ixz,iyz].gauss[*] = 0.0
				PSF_grid[ixz, iyz].scale = 0.0
				PSF_grid[ixz,iyz].resid = 0.0
				PSF_grid[ixz,iyz].fitrad = fit_rad
			goto, zone_end
		endif
		;
		;
		;no_resid = (1 - loop_n) > 0	; first pass has gaussian only, residuals suppressed
		;
		no_resid = 0	; try, using full residuals
		;
;		debug = 1
		;
;		print," before calling this_GETPSF, error state = ",!ERROR_STATE
		this_GETPSF, image, xp, yp, magp, skyp, ronois, phpadu, gauss, psf, this_fwhm, id_psfp, $
				rad_psf, fit_rad, x_w_psf, y_w_psf, psfmag, sum_resid, resid, 'psf_temp.fits', $
				NO_RESID = no_resid, DEBUG=debug
;		print," after calling this_GETPSF, error state = ",!ERROR_STATE
		;
;		debug = 0
		;
		;-------------------------------------------------------
		;
		CATCH, /CANCEL
		;
		continue_this:
		;
		;	save results to structure
		;
		PSF_grid[ixz,iyz].x_wt = x_w_psf
		PSF_grid[ixz,iyz].y_wt = y_w_psf
		PSF_grid[ixz,iyz].gauss[*] = gauss[*]
		PSF_grid[ixz,iyz].psfmag = psfmag
		PSF_grid[ixz,iyz].fitrad = fit_rad
		;
		;	compute scale factor from the PSF creator
		;
		scale1 = 10^(-0.4*(15.0-psfmag))
		;--------------------------------------------
		xcen = fix((nxypsf/2.)-0.4995)
		ycen = xcen
		;
		sum_f = 0.0d0
;		sum_p = 0.0d0
		for jx = 0, nxypsf-1 do begin
			for jy = 0, nxypsf-1 do begin
				rad = sqrt(float(jx-xcen)^2 + float(jy-ycen)^2)/2.0
				if(rad le fit_rad) then begin
					sum_f = sum_f + resid[jx,jy]/4.0d0			; this is the sum w/i fit_rad
;					sum_p = sum_p + resid[jy,jy]/4.0d0
				endif else begin
;					if(rad le rad_psf) then begin
;						sum_p = sum_p + resid[jx,jy]/4.0d0
;					endif
				endelse
			endfor
		endfor
		;
		if(sum_f lt 0.0) then begin
			print,'G_GETPSF: Warning - net negative residuals = ',sum_f,' if first PSF build pass, ignore...'
		endif
		;
		sum_gauss = gauss[0] * 2.0d0 * !DPI * gauss[3] * gauss[4]
		;
		total_flux = (sum_f > 0.0) + sum_gauss
		scale2 = 10000.0d0 / total_flux		; this is the basic scaling for PSFmag
		;
		;	check that scales are within 1/2%
		;
		dif_scale = abs(scale1 - scale2)
;		if(dif_scale / scale1 gt 0.03) then print, 'G_GETPSF: > 3% different: scale1 (from PSF creator), scale2 (sum of gauss + resid) = ',scale1,scale2
		;
		;	rescale option - rescale to aperture phot on PSF model
		;
;		PSF_grid[ixz,iyz].scale = scale2
;		PSF_grid[ixz,iyz].gauss[0] = gauss[0] * scale2
;		PSF_grid[ixz,iyz].resid = resid * scale2
		;
		;	believe the PSF model - rescale only to common PSF scale, based on PSF build
		;
		PSF_grid[ixz,iyz].scale = scale1
		PSF_grid[ixz,iyz].gauss[0] = gauss[0] * scale1
		PSF_grid[ixz,iyz].resid = resid * scale1
		;
		PSF_grid[ixz,iyz].psfmag = 15.000d0
		;
		;	leave the PSF model as-is		DPC  20090729
		;
;		PSF_grid[ixz,iyz].scale = 1.0
;		PSF_grid[ixz,iyz].gauss[0] = gauss[0]
;		PSF_grid[ixz,iyz].resid = resid
;		;
;		PSF_grid[ixz,iyz].psfmag = psfmag
		;----------------------------------------------------------------
		s_psfmag[ixz,iyz] = PSF_grid[ixz,iyz].psfmag
		;
		;print, "G_GETPSF: for zone ix,iy = ",ixz,iyz," psfmag = ",psfmag
		;
		s_gain[ixz, iyz] = gauss[0] * 2.0d0 * !DPI * gauss[3] * gauss[4]
		s_xcen[ixz, iyz] = x_w_psf - xicen
		s_ycen[ixz, iyz] = y_w_psf - yicen
		s_g0[ixz, iyz] = gauss[0]
		;
		;	compute the correction due to the residuals
		;
		s_sum = total_flux
		ratio = (abs(sum_f) / s_sum)
		;
		;print_text, Event, "Ratio of residual power to gaussian power = "+STRING(ratio)
		;
		;	if large, call out error and set status to bad
		;
		;		set resid power cutoff based on loop iteration (high going to low)
		;
		ratio_cutoff = 0.40 - 0.025 * float(loop_n)
		if(abs(ratio) gt ratio_cutoff and fwhm_loop eq 1) then begin
			print_text, Event, "G_GETPSF: Loop iteration "+STRING(loop_n)+" Error - residuals "+STRING(ratio)+" > gaussian power cutoff - bad PSF"
;			status = -1
			;				turned off failed status 20100303 DPC
;			stop
			;
;			goto, the_end
		endif
		;
		s_resid[ixz, iyz] = sum_f
		;
		if(debug gt 0) then begin
			print, "G_GETPSF: for zone [ix,iy] = "+STRING(ixz, iyz)+$
				" power fraction in gaussian = "+STRING(s_gain[ixz, iyz]/s_sum)+$
				" power fraction in residuals = "+STRING(s_resid[ixz, iyz]/s_sum)
		;
			rdpsf, image_PSF, header_PSF, 'psf_temp.fits'
			;
			;	add to PSF_final_image, if debug by zone is all switched on
			;
			db_tot = TOTAL(debug_by_zone) - ng*ng
			if(db_tot ge 0) then begin
				;
				xoff = (ixz * (box+1)) + 1
				yoff = (iyz * (box+1)) + 1
				;
				PSF_final_image[xoff:xoff+box-1,yoff:yoff+box-1] = image_PSF / max(image_PSF)
				;
				PSF_final_header = header_PSF		; currently the last zone's header
			endif
		endif
		;
		;-------------- save results to prepare for variable PSF fitting ------------
		;
		zone_end:
	endfor
endfor
;
;	try to plot, fit surface from PSF fits
;		start with psfmag
;
data = fltarr(3,nx_zones * ny_zones)
iz = 0L
for ix = 0, nx_zones-1 do begin
	for iy = 0, ny_zones-1 do begin
		data[0,iz] = s_xcen[ix,iy]
		data[1,iz] = s_ycen[ix,iy]
		data[2,iz] = s_g0[ix,iy]
		iz++
	endfor
endfor
;
;	plot
;
if(isurface_plot eq 1) then begin
	window, 24
	SURFACE, s_g0, s_xcen, s_ycen, Charsize=2.0
	window, 23
	CONTOUR, s_g0, s_xcen, s_ycen, nlevels=12
	cursor, xx, yy, /wait
	wait, 0.5
	;
	;	try to fit the surface
	;
	model = SFIT(data, 2, KX=kx, /IRREGULAR, /MAX_DEGREE)
	;print, "kx = ",kx
	;
	window, 25
	model = REFORM(model,4,4)
	model = TRANSPOSE(model)	; since it gets formed as model[y,x]
	SURFACE, model, s_xcen, s_ycen, Charsize=2.0
	window, 26
	CONTOUR, model, s_xcen, s_ycen, nlevels=12
	cursor, xx, yy, /wait
	wait, 0.5
	;
	;	use coefficients to make finer gridded model
	;
	xgrid = findgen(100) * 10.0 - 500.0
	ygrid = findgen(100) * 10.0 - 500.0
	new_model = fltarr(100,100)
	;
	;	for no MAX_DEGREE...
	;
	;
	;	for MAX_DEGREE
	;
	for ix = 0, 99 do begin
		for iy = 0, 99 do begin
			new_model[ix,iy] = 	kx[0] + kx[1]*ygrid[iy] + kx[2]*(ygrid[iy]^2) + $
				 				kx[3] * xgrid[ix] + kx[4] * xgrid[ix] * ygrid[iy] + $
				 				kx[5] * (xgrid[ix]^2)
		endfor
	endfor
	;
	window, 28
	CONTOUR, new_model, xgrid, ygrid, NLEVELS=12
	window, 29
	SURFACE, new_model, xgrid, ygrid, charsize=2.0
endif
;
;---------------- other stuff
;
if(debug gt 1) then begin
	file_out = dialog_pickfile(Title="PSF Array Image")
	writefits, file_out, PSF_final_image, PSF_final_header
endif
;
;	print table of psfmags, gain, delta mag
;
s_dmag = fltarr(nx_zones * ny_zones)
s_fmag = fltarr(nx_zones * ny_zones)
s_psf_mag = fltarr(nx_zones * ny_zones)
;
i=0L
for ix = 0, nx_zones-1 do begin
	for iy = 0, ny_zones-1 do begin
		if(FINITE(s_psfmag[ix,iy]) and FINITE(s_gain[ix,iy]) and FINITE(s_resid[ix,iy])) then begin
			gmag = -2.5*alog10(s_gain[ix,iy])
			dmag = s_psfmag[ix,iy] - gmag
			rmag = -2.5*alog10(1.0d0-(s_resid[ix,iy]/(s_resid[ix,iy]+s_gain[ix,iy])))
			;
			scale = 10000.0d0 / (s_resid[ix,iy]+s_gain[ix,iy])
			g0scale = scale * s_g0[ix,iy]
			;
			final_dmag = dmag + rmag
			;
			if(FINITE(final_dmag)) then begin
				s_dmag[i] = dmag
				s_fmag[i] = final_dmag
				s_psf_mag[i] = s_psfmag[ix,iy]
				i++
			endif
			;
			if(debug gt 0 or NOT(FINITE(final_dmag))) then print, "G_GETPSF: Zone [x,y] = ",ix,iy," PSFmag = ",s_psfmag[ix,iy]," Gauss Mag  ",gmag,$
					" dif mag = ",dmag," Resid mag = ",rmag," final dif mag = ",final_dmag,$
					" COG cor mag = ",cog_cor_by_zone[ix,iy]," Scale = ",scale, " g0*scale = ",g0scale
		endif
	endfor
endfor
s_dmag = s_dmag[0:i-1]
s_fmag = s_fmag[0:i-1]
s_psf_mag = s_psf_mag[0:i-1]
;
if(debug gt 0 and i gt 1) then print, "G_GETPSF: mean, stdev of dif mag = ",MEAN(s_dmag),STDDEV(s_dmag)
if(i gt 1) then zz = median_filtered_mean(s_fmag)
if(debug gt 0 and i gt 1) then print, "G_GETPSF: mean, stdev, unc. of final dif mag = ",MEAN(s_fmag),STDDEV(s_fmag),zz[1]/sqrt(nx_zones * ny_zones - 1)
;
;	fit for dependence of dif mag on psf mag
;
;a = LINFIT(s_psf_mag, s_fmag, SIGMA=s_a)
;if(debug gt 0) then begin
;	print, "G_GETPSF: fit results (x = psfmag; y = final_mag)"
;	print, "G_GETPSF: Offset = ",a[0]," +/- ",s_a[0]
;	print, "G_GETPSF: Slope = ",a[1]," +/- ",s_a[1]
;endif
;
;xx = [min(s_psf_mag),max(s_psf_mag)]
;yy = [a[0]+a[1]*xx[0],a[0]+a[1]*xx[1]]
;;
;;	plot some stuff
;;
;if(iplot ne 0) then begin
;	window, 27
;	plot, s_psf_mag, s_fmag, xtitle="PSF Mag", ytitle="Final Mag", psym=2, xrange=[min(s_psf_mag),max(s_psf_mag)],$
;			yrange = [min(s_fmag),max(s_fmag)]
;	oplot, xx, yy, linestyle=2
;endif
;
if(debug gt 0) then print, "Leaving G_GETPSF with PSFmag = ",PSF_grid[0,0].psfmag
;
status = 0
the_end:
;
end
