pro g_remove_PSF_neighbors, Event, ng, image, image_for_psf, x, y, mag, sky, ronois, $
		phpadu, gauss, psf, id_psf, ref_apr, rad_psf, fit_rad, PSF_grid, badpix, bright_limit, $
		fwhm, grid_fwhm, debug, status, NO_BAD_PIX_FIX=no_bad_pix_fix
	;
	;	two-pass system -
	;
	;	first pass builds star list, 2nd pass removes all except central star
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_Struct
	d_ptr = PTR_Struct.PTR_Data
	s_ptr = PTR_Struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	;
;	this_debug = debug
	this_debug = 0
	;
	;	debug flag by zone
	;
	debug_by_zone = intarr(ng,ng)
;	debug_by_zone[2,2] = 1
	;
	ap_scale = (fit_rad / fwhm)
	phpadu = 8.21
	smag_cut = 1.0
	sig_cut = 3.5
	delay = 0.05				; sec
	sig_step = -0.25
	big_scale = 8.0				; times the FWHM
	big_loop_max = 3			; find, fit loops for each PSF star region
	;
	n_stars_removed = 0L
	;
	image_size = SIZE(image)
	ncol = image_size[1]
	nrow = image_size[2]
	;
	if(~KEYWORD_SET(no_bad_pix_fix)) then begin
		print, 'g_remove_psf_neighbors - fixing bad pixels'
		this_image = fix_bad_pixels(image)
	endif else this_image = image
	;
	image_for_psf = this_image
	;
	star_list = fltarr(5,10000)		; x, y, mag, central (0) or surrounding (1), errmag
	;
	;	for each grid zone
	;
	ngx = ng
	ngy = ng
	;
	;	list of ALL PSF stars (for avoidance later)
	;
	n_all_star_vec = 0L
	for igx = 0, ngx-1 do begin
		for igy = 0, ngy-1 do begin
			;
			ind = WHERE(REFORM(id_psf[igx,igy,*]) ne -99,count)
			if(count gt 0) then begin
				if(n_all_star_vec eq 0) then begin
					all_star_vec = REFORM(id_psf[igx,igy,ind])
				endif else begin
					all_star_vec = [all_star_vec, REFORM(id_psf[igx,igy,ind])]
				endelse
				n_all_star_vec += count
			endif
		endfor
	endfor
	;
	if(this_debug) then print, 'found a total of ',n_all_star_vec,' PSF stars'
	;
	for igx = 0, ngx-1 do begin
		;
		for igy = 0, ngy-1 do begin
			;
			;	debug by zone
			;
			this_debug = debug_by_zone[igx,igy]
			;
			n_star_list = 0L
			;
			if(this_debug gt 1) then print_text, Event, "In g_remove_psf_neighbors, grid x, y, fwhm = "+$
									STRING(igx,igy,grid_fwhm[igx,igy],format='(I4,1x,i4,1x,f8.3)')
			;
			;	rebuild the PSF for this grid zone
			;
			gauss = PSF_grid[igx,igy].gauss[*]
			resid = PSF_grid[igx,igy].resid
			nxypsf = PSF_grid[igx,igy].nxypsf
			psfrad = PSF_grid[igx,igy].radpsf
			;
			psfmag = PSF_grid[igx,igy].psfmag		; *** kludge 20090729 DPC
			;
			;zz = median_filtered_mean(REFORM(resid[*,*]))
			;if(this_debug) then print, "mean, rms of residuals = ",zz[0],zz[1]
			;
			g_remake_psf, gauss, resid, psfrad, psf_image, psf_mag, fit_rad
			;
			psfmag = PSF_grid[igx,igy].psfmag		; *** kludge 20090729 DPC
			;
			if(this_debug gt 1) then begin
				psf_size = SIZE(psf_image)
				;
				x_size = psf_size[1]
				y_size = psf_size[2]
				print, "PSF size nx,ny = ",x_size, y_size
				;
				;	do aperture photometry on it to get psfmag
				;
				xpsf = [gauss[1]] + (x_size - 1) / 2.0
				ypsf = [gauss[2]] + (y_size - 1) / 2.0
				apr = [ap_scale*fwhm]
	        	skyrad = [1.1*fwhm,2.0*fwhm] * ap_scale
	        	g_aper, psf_image,xpsf,ypsf,magpsf,errap,skypsf,skyerr,phpadu,apr,skyrad,badpix, $
					bright_limit, /EXACT, SILENT=1, /NO_BAD_PIX_FIX
					;
				if(this_debug) then print, "Aperture photometry on PSF model give psfmag = ",magpsf[0],$
				     " while the tabulated PSFmag = ",PSF_grid[igx,igy].psfmag
				;
				;	keep good ones
				;
				ind_ok = WHERE( FINITE(magspsf[0,*]) ,nok)
				if(nok gt 0) then begin
					magpsf = REFORM(magpsf[0,ind_ok])
				endif
				;
				if(this_debug) then begin
					min_image = MIN(psf_image)
					;
					big_sub_image = fltarr(x_size*12, y_size*12)
					for ix = 0, x_size-1 do begin
						for iy = 0, y_size-1 do begin
							ix0 = ix * 12
							ix1 = ix0 + 11
							iy0 = iy * 12
							iy1 = iy0 + 11
							big_sub_image[ix0:ix1,iy0:iy1] = sqrt(psf_image[ix,iy]-min_image+0.001)		;sqrt(abs(psf_image[ix,iy]))
						endfor
					endfor
					;if(this_debug) then
					z = display_image ( big_sub_image, 22, MESSAGE='PSF Model, Grid ['+STRING(igx,igy,format='(i1,",",i1)')+']', $
									BIPOLAR=0, XOFF=400)
					;
				endif
			endif
			;
			mkhdr, hdr, resid
			sxaddpar, hdr, 'GAUSS1', gauss[0], 'Gaussian Scale Factor'
			sxaddpar, hdr, 'GAUSS2', gauss[1], 'Gaussian X Position'
			sxaddpar, hdr, 'GAUSS3', gauss[2], 'Gaussian Y Position'
			sxaddpar, hdr, 'GAUSS4', gauss[3], 'Gaussian Sigma: X Direction'
			sxaddpar, hdr, 'GAUSS5', gauss[4], 'Gaussian Sigma: Y Direction'
			;
			;SXADDPAR, hdr, 'PSFMAG', 13.639  	;PSF_grid[igx,igy].psfmag
			;SXADDPAR, hdr, 'PSFMAG', magpsf[0]
			;SXADDPAR, hdr, 'PSFMAG', 15.00
			;SXADDPAR, hdr, 'PSFMAG', PSF_grid[igx,igy].psfmag
			;
			;	if psf_mag not finite, stop
			;
			if(~FINITE(psf_mag)) then begin
				print_text, Event, "G_REMOVE_PSF_NEIGHBORS: psf_mag is not finite, stopping."
				print, "in g_remove_psf_neighbors, psf_mag = ",psf_mag
				stop
			endif
			;
			;	force to 15.00
			;
;			psf_mag = 15.0
			;
			SXADDPAR, hdr, 'PSFMAG', psf_mag
			;
			SXADDPAR, hdr, 'PSFRAD', psfrad
			SXADDPAR, hdr, 'FITRAD', fit_rad
			SXADDPAR, hdr, 'NAXIS1', PSF_grid[igx,igy].nxypsf
			;
			SXADDPAR, hdr, 'PHPADU', 8.21
			SXADDPAR, hdr, 'RONOIS', 3.1		; RMS ADU per read - not true for coadds
			;
			;	for each PSF star
			;
			ind = WHERE(REFORM(id_psf[igx,igy,*]) ne -99,count)
			if(count gt 0) then begin
				star_vec = REFORM(id_psf[igx,igy,ind])
				;
				for istar = 0, count-1 do begin
					;
					if(this_debug gt 0) then begin
						print, " "
						print, "starting cleaning around PSF star number ",istar
						print, "------------------------------------------------------"
					endif
					;
					;	get x, y, mag location of this psf star
					;
					id_star = star_vec[istar]
					x_star = x[id_star]
					y_star = y[id_star]
					mag_star = mag[id_star]
					;
					;	if this star is a particular one, put in a full stop to allow breakpoint processing
					;
;					if( abs(x_star - 518) le 2.0 and abs(y_star - 644) le 2.0) then stop
					;
					if(this_debug gt 0) then print, " X, Y of this PSF star in original image = ",x_star, y_star
					;
					x_min = 0 > FLOOR(x_star - big_scale * fwhm)
					x_max = (ncol-1) < CEIL(x_star + big_scale * fwhm)
					y_min = 0 > FLOOR(y_star - big_scale * fwhm)
					y_max = (nrow-1) < CEIL(y_star + big_scale * fwhm)
					;
					x_size = x_max - x_min + 1
					y_size = y_max - y_min + 1
					;
					x_cen = x_star - x_min
					y_cen = y_star - y_min
					;
					sub_image = image_for_psf[x_min:x_max,y_min:y_max]
					;
					;	loop on finding, fitting, removal of stars until
					;	either N passes or no more stars
					;
					big_loop = 0
					big_loop_start:
						;
						;	find all stars in this sub_image
						;
						sub_size = SIZE(sub_image)
						;
						;	check for nan or deeply negative pixels, return them to mean if present)
						;
						zz = median_filtered_mean(REFORM(sub_image[*,*],sub_size[1]*sub_size[2]))
						;
						ind_nan = WHERE( ~FINITE(sub_image) or ((sub_image - zz[0])/zz[1] lt -10.), n_nan)
						if(n_nan gt 0) then sub_image[ind_nan] = zz[0]
						;
						if(this_debug gt 0) then print, "For iteration ",big_loop+1," of psf star ",istar," sky = ",zz[0]," +/- ",zz[1]
						;
						hmin = (sig_cut + sig_step * big_loop)*zz[1]
						roundlim = [-1.2,1.2]
						sharplim = [0.2,1.0]
						;
						if(fwhm lt 3.0) then begin
							roundlim = [-1.5,1.5]
							sharplim = [0.05,1.5]
						endif
						g_find, sub_image, xs, ys, fluxs, sharp, roundness, hmin, $
							fwhm*(1.0+0.1*big_loop), roundlim, sharplim,$
	                      	PRINT = 0, SILENT=1, /NO_BAD_PIX_FIX
	                    ;
	                    n_stars = N_ELEMENTS(xs)
	                    if(n_stars eq 1 and xs[0] eq -99.0) then goto, nostars
	                    ;
	                    ;	sort by brightness, keep only 1 star per 0.5*FWHM
	                    ;
;	                    ind_sort = REVERSE(SORT(fluxs))
;	                    xs = xs[ind_sort]
;	                    ys = ys[ind_sort]
;	                    fluxs = fluxs[ind_sort]
	                    ;
	                    ;	aperture photometry
	                    ;
	                    apr = [ap_scale*fwhm]*(1.0+0.3*big_loop)		; smaller apr on later loops
	                    skyrad = ap_scale*fwhm*[1.05,2.5] < ((sub_size[1] < sub_size[2])/2.)
	                    skyrad *= (1.0+0.3*big_loop)					; same with sky
	                    g_aper, sub_image,xs,ys,mags,errap,skys,skyerr,phpadu,apr,skyrad,badpix, $
	       					bright_limit, /EXACT, /SILENT, /NO_BAD_PIX_FIX
	       					;
	       				ind_ok = WHERE( FINITE(mags[0,*]),nok)
	       				if(nok gt 0) then begin
		       				mags = REFORM(mags[0,ind_ok])
		       				errap = REFORM(errap[0,ind_ok])
		       			endif else begin
		       				print, "problems in g_remove_psf_neighbors - no good aperture photometry. stopping"
		       				stop
		       			endelse
	       				;
	       				ind_stars = WHERE(abs(abs(mags) - 99.999) gt 3.0 and (errap lt smag_cut),n_neighbors)
	       				if(n_neighbors eq 0) then goto, nostars
	       				xs = xs[ind_stars]
	       				ys = ys[ind_stars]
	       				mags = mags[ind_stars]
	       				skys = skys[ind_stars]
	       				errmags = errap[ind_stars]
	       				;
;	       				r = sqrt( (x_star - xs)^2 + (y_star - ys)^2 )
;	       				;
;	       				;	also, ignore stars fainter than 3 mag below and within 1.5 FWHM of center star
;	       				;
;	       				ind_weak = WHERE( (mags - mag_star) gt 3.0 and (r le 1.5*fwhm), n_ignore, $
;	       					COMPLEMENT=ind_keep, NCOMPLEMENT=n_keep)
;	       				;
;	       				if(n_keep eq 0) then goto, nostars
;	       				xs = xs[ind_keep]
;	       				ys = ys[ind_keep]
;	       				mags = mags[ind_keep]
;	       				skys = skys[ind_keep]
;	       				errmags = errap[ind_keep]
	       				;
						if(this_debug gt 0) then begin
							print, "Number of stars found = ",n_neighbors
;							stop
						endif
						;
						if(this_debug gt 0 and big_loop eq 0) then begin
							;
							;	show image of first star data
							;	expand to make pixels visible
							;
							big_sub_image = fltarr(x_size*6, y_size*6)
							for ix = 0, x_size-1 do begin
								for iy = 0, y_size-1 do begin
									ix0 = ix * 6
									ix1 = ix0 + 5
									iy0 = iy * 6
									iy1 = iy0 + 5
									;
									big_sub_image[ix0:ix1,iy0:iy1] = 2.5*alog10(sub_image[ix,iy])
								endfor
							endfor
							if(this_debug) then z = display_image ( big_sub_image, 21, $
								MESSAGE='Image '+STRING(big_loop+1,format='(i3)')+' star '+STRING(istar+1,format='(i3)'), $
										XOFF=-300, BIPOLAR=1)
							;
							wshow, 21
							wait, delay
							;
						endif
						;
						;	group stars for fitting and removal
						;
						ng_stars = N_ELEMENTS(xs)
						rcrit = psfrad + fit_rad + 1.0
						if(ng_stars ge 2) then begin
							group, Event, xs, ys, rcrit, ngroup
						endif else begin
							ngroup = [0]
						endelse
						;
						;	how many groups?
						;
						ind_g = SORT(ngroup)
						ind_g_u = UNIQ(ngroup[ind_g])
						group_count = N_ELEMENTS(ind_g_u)
						;
						if(this_debug gt 0) then print, "found ",group_count," groups"
						;
						m_good_stars = 0
						for igroup = 0, group_count-1 do begin
                            if(this_debug gt 0) then print, "-- starting fitting for group ",igroup," --"
							ind_stars_in_group = WHERE(ngroup eq igroup, nst)
							;
							xg = xs[ind_stars_in_group]
							yg = ys[ind_stars_in_group]
							magg = mags[ind_stars_in_group]
							skyg = skys[ind_stars_in_group]
							errmagg = errmags[ind_stars_in_group]
							id = indgen(nst)
							;
							ibk=0
							;
							;	star vectors going into nstar
							;
							n_vec = N_ELEMENTS(magg)
							if(this_debug gt 0) then begin
								print, "Star mag, uncertainty vectors going into g_nstar"
								for img = 0, n_vec-1 do begin
									print, img, xg[img], yg[img], magg[img], errmagg[img], format='(i3,2x,f8.2,1x,f8.2,2x,f8.3,"+/-",f7.3)'
								endfor
							endif
							;
							g_nstar,sub_image,id,xg,yg,magg,skyg,igroup,phpadu,ronois,$
								resid,hdr,DEBUG=0, $
					           	errmagg,iter,chisq,peak,PRINT=print0,$
					           	SILENT=1, VARSKY = 0, ibk
					        ;
					        n_vec1 = N_ELEMENTS(magg)
					        if(this_debug gt 0) then begin
						        print, "Star mag vector coming out of g_nstar"
								for img = 0, n_vec1-1 do begin
									print, img, xg[img], yg[img], magg[img], errmagg[img], format='(i3,2x,f8.2,1x,f8.2,2x,f8.3,"+/-",f7.3)'
								endfor
							endif
					        ;
					        ind_good = WHERE(abs(abs(magg) - 99.99) gt 3.0 and errmagg lt smag_cut,nn_stars)
							;
							;	for each neighbor, use the psf in this grid zone to fit
							;	and remove the star
							;
							if(nn_stars ne 0) then begin
								xg = xg[ind_good]
								yg = yg[ind_good]
								magg = magg[ind_good]
								errmagg = errmagg[ind_good]
								;
								n_old = n_star_list
								star_list[0,n_star_list:n_star_list + nn_stars - 1] = xg[*] + x_min
								star_list[1,n_star_list:n_star_list + nn_stars - 1] = yg[*] + y_min
								star_list[2,n_star_list:n_star_list + nn_stars - 1] = magg[*]
								star_list[3,n_star_list:n_star_list + nn_stars - 1] = 1.0
								star_list[4,n_star_list:n_star_list + nn_stars - 1] = errmagg[*]
								;
								n_star_list = n_star_list + nn_stars
								;
                                if(this_debug gt 0) then print, "Adding ",nn_stars," to global star list. Total now = ",n_star_list
								;
								ind = indgen(nn_stars)
								;ind = [0,1]
								;
								;	flag central star
								;
								ind_stars = WHERE(abs(x_cen - xg) le 0.25*fwhm and abs(y_cen - yg) le 0.25*fwhm,n_center)
	                    		if(n_center gt 0) then begin
	                    			ind_central = ind_stars + n_old
	                    			star_list[3,ind_central] = 0.0
	                    			;
	                    			if(this_debug gt 0) then print, "flagging central star(s) from substar list for group ",igroup
	                    			;if(nn_stars gt n_center) then remove, ind_stars, xg, yg, magg,ind
	                    		endif else if(this_debug gt 0) then print, "did not remove central star from substar list for group ",igroup
	                    		;
	                    		;if(nn_stars - n_center gt 0) then begin
	                    		ind = -1			; remove all stars
								g_substar,sub_image,xg,yg,magg,ind,resid, hdr, VERBOSE = 0
								;endif
								m_good_stars++
							endif
						endfor
						;
						if(m_good_stars eq 0) then goto, nostars
						;
						if(this_debug gt 0) then begin
							;
							;	display subimage after local star removal
							;
							x_size = x_max - x_min + 1
							y_size = y_max - y_min + 1
							;
							big_sub_image = fltarr(x_size*6, y_size*6)
							for ix = 0, x_size-1 do begin
								for iy = 0, y_size-1 do begin
									ix0 = ix * 6
									ix1 = ix0 + 5
									iy0 = iy * 6
									iy1 = iy0 + 5
									big_sub_image[ix0:ix1,iy0:iy1] = sqrt(abs(alog10(sub_image[ix,iy])))
								endfor
							endfor
							;
							z = display_image ( big_sub_image, 23, MESSAGE='Clnd Img,loop '+STRING(big_loop+1,format='(i2)')+', str '+STRING(istar+1,format='(i3)'), $
									XOFF=75, BIPOLAR=0)
						;
							wshow, 23
							wait, delay
						endif
						;
						;	were any stars removed?
						;
						;	yes, loop again
						;
						big_loop++
						if(big_loop lt big_loop_max) then goto, big_loop_start
						;
						;	no, end loop
						;
					nostars:
					;
					if(this_debug gt 0) then print, "Finished with PSF star ",istar
					;
;					print_text, Event, "finished with PSF star "+STRING(istar)
					;
;					stop
					;
				endfor
				;
				;	with full star list, now remove all except those marked as "central"
				;
				;	do this on full-sized image
				;
				if(n_star_list gt 0) then begin
					ind_neighbors = WHERE(star_list[3,0:n_star_list-1] eq 1.0, n_neighbors)
					if(n_neighbors gt 0) then begin
						;
						if(this_debug gt 0) then print_text, Event, "Found and removed "+STRING(n_neighbors,format='(I4)')+" neighbors of PSF candidates"
						;
						n_stars_removed += n_neighbors
						;
						xs = star_list[0,ind_neighbors]
						ys = star_list[1,ind_neighbors]
						mags = star_list[2,ind_neighbors]
						errs = star_list[4,ind_neighbors]
						;
						;	ALL central PSF stars
						;
						x_psf = x[all_star_vec]
						y_psf = y[all_star_vec]
						npsf = N_ELEMENTS(x_psf)
						;
						;	regroup, refit, remove
						;
						;		first, test to see if grouping necessary
						;
						n_to_fit = N_ELEMENTS(xs)
						if(n_to_fit ge 2) then begin
							if(this_debug gt 0) then print, "grouping"
							group, Event, xs, ys, rcrit, ngroup
						endif else begin
							ngroup = [0]
						endelse
						;
						;	how many groups?
						;
						ind_g = SORT(ngroup)
						ind_g_u = UNIQ(ngroup[ind_g])
						group_count = N_ELEMENTS(ind_g_u)
						;
						if(this_debug gt 0) then print, "found ",group_count," groups"
						;
						for igroup = 0, group_count-1 do begin
							ind_stars_in_group = WHERE(ngroup eq igroup, nst)
							;
							if(this_debug gt 0) then print, "in group ",igroup," number of stars in group = ",nst
							;
							xg = xs[ind_stars_in_group]
							yg = ys[ind_stars_in_group]
							magg = mags[ind_stars_in_group]
							skyg = skys[ind_stars_in_group]
							errg = errs[ind_stars_in_group]
							;
							;	sort by brightness
							;
							ind_sort = SORT(magg)
							xg = xg[ind_sort]
							yg = yg[ind_sort]
							magg = magg[ind_sort]
							skyg = skyg[ind_sort]
							errg = errg[ind_sort]
							;
							;if(this_debug gt 0) then print, "... brightest star (likely PSF star) has x, y = ",xg[0],yg[0]
							;
							;	for each star, look within 0.25*FWHM of it, kill any star with less flux
		                    ;
		                    kill_star = intarr(nst)
		                    ;
		                    for jstar = 0, nst-1 do begin
		                    	if(kill_star[jstar] eq -1) then goto, loop_end
		                    	this_xs = xg[jstar]
		                    	this_ys = yg[jstar]
		                    	;
		                    	dist = sqrt((this_xs - xg)^2 + (this_ys - yg)^2)
		                    	ind = WHERE(dist lt 0.25*fwhm, count)
		                    	if(count gt 1) then begin				; the first one is itsself
		                    		jnd = WHERE(ind ne jstar)
		                    		kill_star[ind[jnd]] = -1
		                    	endif
		                    	loop_end:
		                    endfor
		                    ;
		                    ;	any kill_star values eq -1, kill them
		                    ;
		                    kill_ind = WHERE(kill_star eq -1, kill_count)
		                    if(kill_count gt 0) then begin
		                    	if(this_debug gt 0) then print, "Killing ",kill_count," close stars"
		                    	remove, kill_ind, xg, yg, magg, skyg, errg
		                    endif
		                    ;
		                    nst = N_ELEMENTS(xg)
		                	id = indgen(nst)
							;
							ibk=0
							;
							;if(this_debug gt 0) then print, "calling g_nstar "
	;		don't call nstar				g_nstar,image_for_psf,id,xg,yg,magg,skyg,igroup,phpadu,ronois,$
	;							resid,hdr,DEBUG=0, $
	;				           	errmagg,iter,chisq,peak,PRINT=print0,$
	;				           	SILENT=1, VARSKY = 0, ibk
							;
							;	for each neighbor, use the psf in this grid zone to fit
							;	and remove the star
							;
							ind_good = WHERE(abs(abs(magg) - 99.99) gt 3.0 and errg lt smag_cut,nn_stars)
							;
							if(this_debug gt 0) then print, 'Final pass, group ',igroup," with ",nn_stars," stars"
							;
							if(nn_stars gt 0) then begin
								xg = xg[ind_good]
								yg = yg[ind_good]
								magg = magg[ind_good]
								;
								if(this_debug gt 0) then begin
									print, 'About to subtract final list of stars from clean image'
									print,'  Star     x         y         mag'
									for iss = 0, nn_stars-1 do begin
										print, iss+1, xg[iss], yg[iss], magg[iss]
									endfor
									print, '---------------------------------------'
								endif
								;
								ind = indgen(nn_stars)
								;
								;	look for central stars
								;
								n_center_tot = 0
								for ipsf = 0, npsf-1 do begin
									ind_stars = WHERE(abs(x_psf[ipsf] - xg) le 0.25*fwhm and abs(y_psf[ipsf] - yg) le 0.25*fwhm,n_center)
	                    			if(n_center gt 0) then begin
	                    			    n_center_tot = n_center_tot + n_center
	                    				if(this_debug gt 0) then print, "removing PSF star",ipsf," from substar list for group ",igroup
	                    				if(nn_stars gt n_center) then begin
	                    					;
	                    					;	if only one star left and we want to remove it, jump to skip_removal
	                    					;
	                    					if(N_ELEMENTS(xg) le 1) then begin
	                    						   if(this_debug gt 0) then print, "one star is a PSF star, skipping removal"
	                    						   goto, skip_removal
	                    				    endif
	                    					remove, ind_stars, xg, yg, magg
	                    					;
	                    					nn_stars -= n_center					; reduce number of stars still on list by number removed
	                    				endif else begin
	                    					if(this_debug gt 0) then print, "only star is a PSF star, skipping removal"
	                    					goto, skip_removal
	                    				endelse
	                    			endif
	                    		endfor
	        					;
	        					if(nn_stars - n_center gt 0) then begin
	        						ind_sub = -1
	        						if(this_debug gt 0) then print, "Calling g_substar to remove ",(nn_stars-n_center)," stars"
									g_substar,image_for_psf,xg,yg,magg,ind_sub,resid,hdr, VERBOSE = 0
								endif
								skip_removal:
							endif else if(this_debug) then print, "...No stars to remove, so on to next group"
						endfor
						;
						;big_image = sqrt(abs(alog10(image_for_psf)))
						;z = display_image (big_image, 25, MESSAGE='Final Cleaned Image', BIPOLAR=0)
						;
	;					print, "Finished Making Cleaned image - save as FITS file and stop? (y/n)"
	;					iyn = ''
	;					read, iyn
	;					if(iyn eq 'y' or iyn eq 'Y') then begin
	;						;
	;					endif
					endif
				endif
			endif
		endfor
	endfor
	;
;	file_out = dialog_pickfile(Title="Save FITS file")
;	writefits, file_out, image_for_psf, hdr
;	stop
	;
	if(this_debug) then begin
		print, "leaving g_remove_psf_neighbors.pro"
		print, "  "
	endif
	;
	print_text, Event, "Found and removed "+STRING(n_stars_removed,format='(I4)')+" neighbors of PSF candidates"
	;
	;
;	stop
end
