pro meta_correction_factors, Event, meta_group_number, meta_group_name, output_summary_path
;
;	for designated meta-group:
;		1. access and ingest group-level scaling factors and filenames
;		2. access and ingest the relative scale factors (group-to-group)
;		3. output full list of images with input * relative scale factors
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
w_ptr = PTR_struct.PTR_wid_id
s_ptr = PTR_struct.PTR_summary
g_ptr = PTR_struct.PTR_group
;
plot_wid = (*w_ptr).S6D_relative_scales_draw_wid
;
device,decompose=0
color_names = ["black", "white", "red", "orange", "yellow", "green", "blue", "violet", "gray"]
color_values= [10,11,12,13,14,15,16,17,18,19]
device, decomposed=0
loadct, 13, /silent
tvlct,        [0,        255,     255,     255,    255,        50,      0,       148,    190], $
              [0,        255,       0,     165,    255,       205,      0,         0,    190], $
              [0,        255,       0,       0,      0,        50,    255,       211,    190], 10
;
;----------------------------------------------------------------
;
;	reserve room for up to 32 groups in this meta-group
;
file_list = strarr(32,200)
file_list_num = lonarr(32)	; number of images in each group
group_scale_factors = dblarr(32,200)
group_x_offsets = fltarr(32,200)
group_y_offsets = fltarr(32,200)
group_HWP_angles = fltarr(32,200)
;
;	get the group names from the meta-group buffer
;
ngroups = (*g_ptr).meta_group_numbers[meta_group_number]				; number of groups in this meta
group_ids = (*g_ptr).meta_group_groups[meta_group_number,0:ngroups-1]	; group numbers in this meta
group_names = (*g_ptr).group_names[group_ids]
;
;print, "group names in this meta = ",group_names
;
;	names of the summary scale files
;
group_paths = strarr(ngroups)
for ig = 0, ngroups-1 do begin
	group_paths[ig] = replace_underscores(FILE_BASENAME(group_names[ig]))
endfor
;
group_dir = (*s_ptr).analysis_dir + "S6_Scaling" + PATH_SEP()
group_paths = group_dir + group_paths + '_group_scale_factors.dat'
;
;	for each, open, read
;
print_text, Event, "Number of groups in this meta-group = "+STRING(ngroups,format='(I)')
;
n_total_images = 0L
for ig = 0, ngroups-1 do begin
	print_text, Event, "Reading Scale factors for group "+FILE_BASENAME(group_paths[ig])
	openr, lu_in, group_paths[ig], /GET_LUN
	;
	;print_text, Event, "Opened group scaling file "+FILE_BASENAME(group_paths[ig])
	;
	line = ''
	readf, lu_in, line						; 1st line
	;
	nline = 0L
	WHILE (~EOF(lu_in)) do begin
		readf, lu_in, line
		parts = STRSPLIT(line, ' ', /REGEX, /EXTRACT)
		;
		;	check that the file name has a "phot" in it, skip line if not
		;
		test = STRPOS(parts[0],"phot")
		if(test eq -1) then goto, skip_a_line
		;
		file_list[ig,nline] = STRTRIM(parts[0],2)
		sf = 0.0d0
		reads, parts[2], sf
		group_scale_factors[ig,nline] = sf
		file_list_num[ig] = nline
		;
		reads, parts[1], hwp
		group_HWP_angles[ig,nline] = hwp
		reads, parts[3], xoff
		group_x_offsets[ig,nline] = xoff
		reads, parts[4], yoff
		group_y_offsets[ig,nline] = yoff
		;
		nline++
		n_total_images++
		;
		skip_a_line:
	ENDWHILE
	n_total_images--
	;
	FREE_LUN, lu_in
endfor
;
;	all group files read - group-to-group relative scaling from match
;	file of first images
;
input_dir = (*s_ptr).analysis_dir + 'S6_Scaling' + PATH_SEP()
new_group_name = replace_underscores(meta_group_name)
input_path = input_dir + new_group_name + '_meta_match.dat'
;
print_text, Event, "Opening file of group-relative scale factors = "+FILE_BASENAME(input_path)
openr, lu_scale, input_path, /GET_LUN
;
;	read
;
rel_grp_id = lonarr(ngroups)
rel_grp_name = strarr(ngroups)
rel_grp_first_file = strarr(ngroups)
rel_grp_dmag = dblarr(ngroups)
rel_grp_flux = dblarr(ngroups)
rel_grp_x_offset = fltarr(ngroups)
rel_grp_y_offset = fltarr(ngroups)
;
line = ''
for irg = 0, ngroups-1 do begin
	line = ''
	readf, lu_scale, line, format='(a)'
	;
	;	parse
	;
	parts = STRSPLIT(line,' ',/extract)
	;
	reads, parts[1], id, format='(I)'
	rel_grp_id[irg] = id
	rel_grp_name[irg] = STRTRIM(parts[2],2)
	rel_grp_first_file[irg] = STRTRIM(parts[3],2)
	reads, parts[7], dmag, format='(F)'
	rel_grp_dmag[irg] = dmag
	;
	reads, parts[11], xoff
	reads, parts[12], yoff
	rel_grp_x_offset[irg] = xoff
	rel_grp_y_offset[irg] = yoff
	;
	;	flux of this file
	;
	rel_grp_flux[irg] = 10.0d0^(-0.40d0 * dmag)
	;
;	print, "irg = ",irg," group ID = ", rel_grp_id[irg]," group name = ",rel_grp_name[irg],$
;			" first file name = ",FILE_BASENAME(rel_grp_first_file[irg]),$
;			" difference mag = ",rel_grp_dmag[irg]," flux equiv. = ",rel_grp_flux[irg]
	;
endfor
;
FREE_LUN, lu_scale
;
for irg = 0, ngroups-1 do begin
	print_text, Event, "Scale factor for group "+STRING(irg+1,format='(I)')+" is "+STRING(rel_grp_flux[irg],$
					format='(F)')
endfor
;
;	match the orders of the groups
;
ind = lonarr(ngroups)
for ig = 0, ngroups-1 do begin
	jnd = WHERE(rel_grp_id[*] eq group_ids[ig],count)
	if(count eq 1) then begin
		ind[ig] = jnd[0]
	endif
endfor
;
;print, "group matched order = ",ind
;
;	put rel arrays into that order
;
rel_grp_dmag = rel_grp_dmag[ind]
rel_grp_x_offset = rel_grp_x_offset[ind]
rel_grp_y_offset = rel_grp_y_offset[ind]
;
relative_scale = dblarr(ngroups)
relative_scale[0] = 1.0d0
if(ngroups gt 2) then begin
	for ig = 1, ngroups-1 do begin					; in group_ids order
		relative_scale[ig] = rel_grp_flux[0] / rel_grp_flux[ig]
		;
		;print_text, Event, "Relative scale factor for group "+STRING(ig+1,format='(I)')+" relative to first = "+$
		;					STRING(relative_scale[ig],format='(F12.8)')
	endfor
endif else begin
	relative_scale[1] = rel_grp_flux[0] / rel_grp_flux[1]
endelse
;
;	open output file and start writing files and scale products
;
openw, lu_out, output_summary_path, /GET_LUN
;
printf, lu_out, format='(";;Meta-Group Scale factors (and X,Y offsets) for meta-group ",a)',meta_group_name
;
scale = dblarr(n_total_images)
c_scale = intarr(ngroups)
i_images = 0L
;
for ig = 0, ngroups-1 do begin
	;
	x_g_offset = rel_grp_x_offset[ig]
	y_g_offset = rel_grp_y_offset[ig]
	;
	nimages = file_list_num[ig]
	;
	printf, lu_out, format='(";;Images in ",a," had gain ",f12.8," and X,Y offsets ",f7.2,1x,f7.2," applied")',$
						group_names[ig],relative_scale[ig],x_g_offset, y_g_offset
	;
	for ii = 0, nimages-1 do begin
		;
		s_all = relative_scale[ig] * group_scale_factors[ig,ii]
		scale[i_images] = s_all
		;
		total_x_offset = x_g_offset + group_x_offsets[ig,ii]
		total_y_offset = y_g_offset + group_y_offsets[ig,ii]
		;
		i_images++
		printf, lu_out, file_list[ig,ii], group_HWP_angles[ig,ii], s_all, $
					total_x_offset, total_y_offset, format='(a,2x,f5.1,1x,f12.8,1x,f7.2,1x,f7.2)'
		if(ii eq nimages-1) then c_scale[ig] = i_images-1				; last image of this color
	endfor
endfor
;
FREE_LUN, lu_out
;
;print, "total images = ",n_total_images
;ind = WHERE(scale eq 0,count)
;if(count gt 0) then begin
;	print, "ERROR - scale factors zero at ind = ",ind
;endif
;
;	plot
;
ymax = MAX(scale)
ymin = MIN(scale)
dy = ymax - ymin
;
x = indgen(n_total_images)
xrange = [0, n_total_images+1]
;
WIDGET_CONTROL, plot_wid, GET_VALUE=win_no
wset, win_no
erase
;
max_deviation = (ymax-1.) > (1.-ymin)
n_deci = ROUND(max_deviation * 20.0)
;
yrange = [1-0.075*n_deci,1+0.075*n_deci]
dy = yrange[1]-yrange[0]
;
dx = xrange[1] - xrange[0]
xrange[0] = xrange[0]-0.02*dx
xrange[1] = xrange[1]+0.02*dx
dx = xrange[1] - xrange[0]
;
plot, x, scale, yrange=yrange, xrange=xrange, xstyle=1 , ystyle=1, $
		xtitle='Image Number (arb)', ytitle='Scale Factor',$
		color=10, background=18, /NODATA
;
iend = -1
for ig = 0, ngroups-1 do begin
	color = (ig MOD 6) + 12
	istart = iend+1
	iend = c_scale[ig]
	oplot, x[istart:iend], scale[istart:iend], color = color
	xo = xrange[0] + dx*(float(ig)/ngroups+0.05)
	yo = yrange[0] + 0.1 * dy
	XYOutS, xo, yo, FILE_BASENAME(group_names[ig]), color=color, charsize=1.25
endfor
;
print_text, Event, "Completed Meta-group scale factor determination for meta-group "+meta_group_name
;
end