pro mp_GPIPS_astrometry_soft, stars_x_tot, stars_y_tot, stars_mag_tot, mass_h_tot, mass_ra_tot, mass_dec_tot, $
                              f17, input_header, cd_soft, flag
  ;
  ;  Takes in 2MASS positions, exact positions and tries to generate a rough astrometry solution
  ;
  ;  MP  20081203  subprogram to try using all stars if normal astrometry is taking a long time
  ;

  n=1000;initial number of stars to use in 2MASS and found lists
  n_inc = 0; number of time n has been increased
  a=0; first star in DAOFIND list to use
  time_start = systime(1)

  new_n:
  if (f17 eq 1) then begin
    star_sep01 = 96.6; min star seperations (arcsec), will reset seperations if more stars are needed
    star_sep02 = 96.6
    star_sep12 = 96.6
  endif else begin
    star_sep01 = 180.; min star seperations (arcsec), will reset seperations if more stars are needed
    star_sep02 = 180.
    star_sep12 = 180.
  endelse

  star_count = size(stars_mag_tot) & star_count = star_count[1]
  mass_size = size(mass_H_tot) & mass_size = mass_size[1]

  found_stars = (star_count < n); use only n brightest stars
  ;found_stars = (star_count-bad_mag_count < 50); use only 50 brightest stars
  stars_x = stars_x_tot(0:found_stars-1)
  stars_y = stars_y_tot(0:found_stars-1)
  stars_mag = stars_mag_tot(0:found_stars-1)

  mass_stars = (mass_size < n); only use a maximum of n stars
  ;mass_stars = (mass_size-bad_mass_count < 50); only use a maximum of n stars
  mass_H_tot = reverse(mass_H_tot);put brightest stars at the beginning of list
  mass_ra_tot = reverse(mass_ra_tot)
  mass_dec_tot = reverse(mass_dec_tot)
  mass_H = mass_H_tot[0:mass_stars-1]
  mass_ra = mass_ra_tot[0:mass_stars-1]
  mass_dec = mass_dec_tot[0:mass_stars-1]

  ;find the distance from every 2MASS star from every other 2MASS star
  distarr = fltarr(mass_stars,mass_stars)
  for coll=0,mass_stars-1 do begin
    for row=0,mass_stars-1 do begin
      gcirc,1,mass_ra[coll],mass_dec[coll],mass_ra[row],mass_dec[row],stardist
      distarr[coll,row] = stardist
    endfor
  endfor

  ;define ratios the ratios as
  ;ratio1 = dist01/dist02
  ;ratio2 = dist01/dist12
  ;ratio3 = dist02/dist12

  ;take three bright stars (a,b,c) in detector image and find the ratio of the distances between the stars
  star_retry = 0
  closer:
  if (star_retry gt 0) then begin;can't find stars at this seperation, allow closer matches
    if keyword_set(F17) then begin
      star_sep01 = star_sep01 - 15.5
      star_sep02 = star_sep02 - 15.5
      star_sep12 = star_sep12 - 15.5
    endif else begin
      star_sep01 = star_sep01 - 32
      star_sep02 = star_sep02 - 32
      star_sep12 = star_sep12 - 32
    endelse
  endif

  ;new_stars_post_failure:;loops back here if these three stars do not work
  ;defined 'a' at beginning of program
  b=a+1;b=1
  c=b+1;c=2

  reject_stars:;if the three stars are bad for any reason go back here and try again
  if (c gt found_stars-1) then begin;for this a and b, all c exhausted, reset c and increment b
    b = b+1
    c = 0
    if (b gt found_stars-1) then begin;for this a, all b exhausted, reset b and increment a
      b = 0
      a = a+1
      if (a gt found_stars-1) then begin;exhausted all a, b, c allow closer matches
        star_retry = star_retry+1
        if (star_retry eq 5) then begin
          if (n_inc eq 2) then begin
            print,'Coordinate matching unsuccessful, skipping image'
            goto, skip_image
          endif else begin
            print,'Increasing number of 2MASS stars by 15 for ' + file_basename(input_image_name[image])
            n=n+1000
            n_inc = n_inc+1
            goto,new_n
          endelse
        endif
        print,'Coordinate matching unsuccessful, allowing closer stars for ' + file_basename(input_image_name[image])
        a = 0
        goto,closer
        return
      endif
    endif
  endif

  if (a eq b) or (b eq c) or (a eq c) then begin
    c=c+1
    goto,reject_stars;two of the stars are the same
  endif

  dist01 = sqrt((stars_x[a]-stars_x[b])^2+(stars_y[a]-stars_y[b])^2)
  dist02 = sqrt((stars_x[a]-stars_x[c])^2+(stars_y[a]-stars_y[c])^2)
  dist12 = sqrt((stars_x[b]-stars_x[c])^2+(stars_y[b]-stars_y[c])^2)
  if (dist01 lt star_sep01) or (dist12 lt star_sep12) or (dist02 lt star_sep02) then begin;insufficient seperation
    c=c+1
    goto,reject_stars
  endif

  ;We know Mimir is roughly sky true, so the star with the highest y coordinate should have the
  ;highest declination, and vice versa.
  high_y = max([stars_y[a],stars_y[b],stars_y[c]])
  dec_high_ind = where([stars_y[a],stars_y[b],stars_y[c]] eq high_y)
  low_y = min([stars_y[a],stars_y[b],stars_y[c]])
  dec_low_ind = where([stars_y[a],stars_y[b],stars_y[c]] eq low_y)

  ratio1 = dist01/dist02
  ratio2 = dist01/dist12
  ;ratio3 = dist02/dist12

  matches ='0,0,0'
  grades=[1]

  ;For each star ss, assume it is the brightest (star 0) and that star rr is the next brightest (star 1)
  ;Find the ratio of distance from rr to ss with the distance from ss to each other star (pp).
  ;If the ratios are with 1% of the pixel ratios save those indicies, if not move on.

  For ss=0,mass_stars-1 do begin
    for rr=0,mass_stars-1 do begin
      if (rr ne ss) then begin; don't use one star twice
        distssrr = distarr(ss,rr)
        for pp=0,mass_stars-1 do begin
          if ((rr ne pp) and (ss ne pp)) then begin; don't use one star twice
            distsspp = distarr(ss,pp)
            mass_ratio1 = distssrr/distsspp
            ;if (abs(ratio1-mass_ratio1)/ratio1 lt 0.01) then begin;possible match, check other ratios
            if (abs(ratio1-mass_ratio1) lt 0.05) then begin;
              distrrpp = distarr(rr,pp)
              mass_ratio2 = distssrr/distrrpp
              ;mass_ratio3 = distsspp/distrrpp
              ;if (abs(ratio2-mass_ratio2)/ratio2 lt 0.01) and (abs(ratio3-mass_ratio3)/ratio3 lt 0.01) then begin
              ;if (abs(ratio2-mass_ratio2)/ratio2 lt 0.01) then begin
              if (abs(ratio2-mass_ratio2) lt 0.05) then begin
                ;Check if declination order is correct
                check_dec = [mass_dec(ss),mass_dec(rr),mass_dec(pp)];,mass_dec(nn)]
                check_dec_high = where(max(check_dec) eq check_dec)
                check_dec_low = where(min(check_dec) eq check_dec)
                if ((check_dec_high eq dec_high_ind) and (check_dec_low eq dec_low_ind)) then begin
                  matches = [matches,string(ss,rr,pp)]
                  ;grade_new = 100^3.*(abs(ratio1-mass_ratio1)/ratio1)*(abs(ratio2-mass_ratio2)/ratio2)*(abs(ratio3-mass_ratio3)/ratio3)
                  grade_new = 100^3.*(abs(ratio1-mass_ratio1)/ratio1)*(abs(ratio2-mass_ratio2)/ratio2)
                  grades = [grades, grade_new]
                endif
              endif
            endif
          endif
        endfor
      endif
    endfor
  Endfor

  ;print,'Testing possible solutions'
  new_solution:
  grades_ind = sort(grades);grades_ind[0] has the best grade
  grades = grades[grades_ind]; put best grade at the beginning of the list
  matches = matches[grades_ind]; put best match at beginning of the list
  star_indicies = FIX(STRSPLIT(matches[0],' ',/EXTRACT))

  if (grades[0] eq 1.0) then begin
    c=c+1
    goto,reject_stars
  endif

  time_now = systime(1)
  if time_now - time_start gt 300 then begin;probably not working, go back to original
    flag = 0
    goto, skip_image
  endif

  solution_ra = [mass_ra[star_indicies[0]],mass_ra[star_indicies[1]],mass_ra[star_indicies[2]]]*15
  solution_dec = [mass_dec[star_indicies[0]],mass_dec[star_indicies[1]],mass_dec[star_indicies[2]]]
  solution_x = [stars_x[a],stars_x[b],stars_x[c]]
  solution_y = [stars_y[a],stars_y[b],stars_y[c]]
  starast, solution_ra, solution_dec, solution_x, solution_y, cd, HDR = input_header
  sxdelpar,input_header,'HISTORY';delete any previous history entries from STARAST.pro
  ;figure out coords for center pixel, and use it as reference pixel
  imsize = size(working_image)
  new_refx = float(imsize[1])/2.;allows for fractional pixels
  new_refy = float(imsize[2])/2.
  xyad, input_header, new_refx, new_refy, new_refra, new_refdec;new_refra already in degrees
  solution_ra = [new_refra,15.d*mass_ra[star_indicies[0]],15.d*mass_ra[star_indicies[1]]]
  solution_dec = [new_refdec,mass_dec[star_indicies[0]],mass_dec[star_indicies[1]]]
  solution_x = [new_refx,stars_x[a],stars_x[b]]
  solution_y = [new_refy,stars_y[a],stars_y[b]]
  starast, solution_ra, solution_dec, solution_x, solution_y, cd, HDR = input_header
  ast_hist_entry = sxpar(input_header,'HISTORY');save these history entries in case they are good
  sxdelpar,input_header,'HISTORY';delete history entries...again
  ;
  ;check coordinate description (CD) matrix elements
  if keyword_set(f17) then begin; F17 camera
    if ((abs(3600.0*cd[0]) lt 0.17) or (abs(3600.0*cd[0]) gt 0.19) or (abs(3600.0*cd[3]) lt 0.17) or (abs(3600.0*cd[3]) gt 0.19)  or (abs(cd[1]) gt 1.0e-6) or (abs(cd[2]) gt 1.0e-6)) then begin
        ;something is wrong with this solution, reject and try another
        grades[0] = 1.0; flunk
        goto,new_solution
      endif
  endif else begin; F5 camera
    if ((abs(3600.0*cd[0]) lt 0.55) or (abs(3600.0*cd[0]) gt 0.6) or (abs(3600.0*cd[3]) lt 0.55) or (abs(3600.0*cd[3]) gt 0.6) or (abs(cd[1]) gt 1.0e-6) or (abs(cd[2]) gt 1.0e-6)) then begin
      ;something is wrong with this solution, reject and try another
      grades[0] = 1.0; flunk
      goto,new_solution
    endif
  endelse
  ;
  flag = 1
  skip_image:
  cd_soft = cd
end