function ppol_check_solutions, Event, tolerance, poss_matches, header, platescale_est, $
								save_this, N_MATCHED_STARS=n_matched_stars, $
								STAR_MATCHES=ngood, STATUS=status, idebug
;
;good_match = 4.0 * tolerance
;
;idebug = 1
;
;i_narrow = 1			; for very narrow PSFs (short exposures), set this to '1', else set to '0'
;						; DPC  20121209
;
final_header = header
test_header = header
;
old_hist = sxpar(header, "HISTORY")
ncol = sxpar(header, "NAXIS1")
nrow = sxpar(header, "NAXIS2")
;
ncol_cen = ROUND(ncol/2.)
nrow_cen = ROUND(nrow/2.)

;[grade,x1,y1,x2,y2,x3,y3,ra1,dec1,ra2,dec2,ra3,dec3]

ntest = (N_ELEMENTS(poss_matches[*,0]) < 100000L)
;
if(ntest gt 100) then good_match = tolerance else good_match = 1.5 * tolerance	; was 2.0 20100804
if(ntest lt 50) then good_match = 2.0 * tolerance								; was 3.0 20100804
if(ntest lt 25) then good_match = 2.5 * tolerance								; was 4.0 20100804
;
if(idebug eq 1) then print_text, Event, "Entered PpolCheckSolutions with number of trial matches = "+STRING(ntest)
;
cd_matrix = dblarr(ntest,4); [cd1_1, cd1_2, cd2_1, cd2_2]
radec_cen = dblarr(ntest,2)

;
for matches = 0, ntest-1 do begin
  ;
  test_header = header
  ;
  row = poss_matches[matches,*]
  ;poss = row[1:12]

  star1 = [row[1], row[2]]
  star2 = [row[3], row[4]]
  star3 = [row[5], row[6]]
  mass0 = [row[7], row[8]]; decimal degrees
  mass1 = [row[9], row[10]]
  mass2 = [row[11], row[12]]

  solution_ra = [mass0[0],mass1[0],mass2[0]]
  solution_dec = [mass0[1],mass1[1],mass2[1]]
  solution_x = [star1[0],star2[0],star3[0]] + 1		; to go into non-IDL array referencing
  solution_y = [star1[1],star2[1],star3[1]] + 1

  starast, solution_ra, solution_dec, solution_x, solution_y, cd, HDR = test_header, projection='TAN'
  ;sxdelpar,test_header,'HISTORY';delete any previous history entries from STARAST.pro
  ;
  ;
  ;	also compute, save center image RA, Dec
  ;
  extast, test_header, astr_struct, noparams
  xy2ad,ncol_cen,nrow_cen, astr_struct, ra_cen, dec_cen
  ;
  radec_cen[matches,0] = ra_cen
  radec_cen[matches,1] = dec_cen
	;
  cd_matrix[matches, 0] = cd[0,0]
  cd_matrix[matches, 1] = cd[1,0]
  cd_matrix[matches, 2] = cd[0,1]
  cd_matrix[matches, 3] = cd[1,1]
  ;
  ;
	;	check platescale (roughly)
	;
	if(idebug eq 1) then begin
		this_platescale = 1./cd_matrix[matches,0]
	;	;
		print_text, Event,"Trial "+STRING(matches)+" came in with platescale "+STRING(row[0])+$
						" and computed platescale "+STRING(-1.0*this_platescale)
	;	;
	endif
endfor
;
ind = where(abs((cd_matrix[*,0] + cd_matrix[*,3])/cd_matrix[*,0]) lt good_match and $
		abs(abs(cd_matrix[*,0]*3600.0)-platescale_est)/platescale_est lt 2.0*good_match $
			and cd_matrix[*,0] lt 0.0,ngood)
;
if(idebug eq 1) then print_text, Event, "Entered with "+STRING(ntest)+" triangle sets to test. Passed "+$
								STRING(ngood)+" on to solution"
;
;platescales must be the same and opposite signs and close to estimated platescale

if(ngood le 0) then begin
	status = -1
	sxaddpar, final_header, 'GPCOORD', 0
	return,final_header
endif

save_this = dblarr(ngood*3,4)
;
new_cd = dblarr(ngood,4)
new_cd[*,0] = cd_matrix[ind,0]
new_cd[*,1] = cd_matrix[ind,1]
new_cd[*,2] = cd_matrix[ind,2]
new_cd[*,3] = cd_matrix[ind,3]

final_cd = dblarr(4)

zz = median_filtered_mean(new_cd[*,0])
final_cd[0] = zz[0]
zz = median_filtered_mean(new_cd[*,1])
final_cd[1] = zz[0]
zz = median_filtered_mean(new_cd[*,2])
final_cd[2] = zz[0]
zz = median_filtered_mean(new_cd[*,3])
final_cd[3] = zz[0]

sxaddpar, final_header, 'CD1_1', final_cd[0]
sxaddpar, final_header, 'CD1_2', final_cd[1]
sxaddpar, final_header, 'CD2_1', final_cd[2]
sxaddpar, final_header, 'CD2_2', final_cd[3]
;
cd = dblarr(2,2)
cd[0,0] = final_cd[0]
cd[0,1] = final_cd[1]
cd[1,0] = final_cd[2]
cd[1,1] = final_cd[3]
;
zz = median_filtered_mean(REFORM(radec_cen[ind,0]))
ra_cen = zz[0]
zz = median_filtered_mean(REFORM(radec_cen[ind,1]))
dec_cen = zz[0]
;
crval1 = [ra_cen, dec_cen]
crpix1 = [float(ncol_cen), float(nrow_cen)]
;
offset = 0.0					; different than in examples...
make_astr, astr_struct,CD = cd, crval = crval1, crpix = crpix1+offset, ctype = ['RA---','DEC--'] + 'TAN'

putast, final_header, astr_struct, equi=2000.0,cd_type=2

n_old = N_ELEMENTS(old_hist)
for i=0, n_old-1 do begin
	sxaddpar, final_header, "HISTORY", old_hist[i]
endfor
;
;	do tests on final results:
;
;	1. CD1_1 < 0 and CD2_2 > 0
;	2. rotation angle < 3 deg
;
CD1_1 = sxpar(final_header, 'CD1_1')
CD2_2 = sxpar(final_header, 'CD2_2')
CD1_2 = sxpar(final_header, 'CD1_2')
;
if(CD1_1 lt 0.0) then begin
	if(CD2_2 gt 0.0) then begin
		theta = atan(CD1_2 / CD1_1)
		if(abs(theta) lt (0.052)) then begin
			gpstatus = 1
		endif else begin
			gpstatus = 0
		endelse
	endif else gpstatus = 0
endif else gpstatus = 0
;
sxaddpar, final_header, 'GPCOORD', gpstatus
if(gpstatus eq 1) then status=0 else status = -1
;
return,final_header
;
end