function ppol_fetch_mass, Event, header, f17
;
;	DPC	20091211	added more mirror site calls to vizier call
;
;	20100518 - larger search radii
;	20100722 - added test for coadded image. if so, then use crval1,2 instead of telera, teledec
;
;--------------------------------------------------
;
if KEYWORD_SET(F17) then search = 4.0 else search = 14.0
;f5_search = 14.0
;f17_search = 4.0
;
;	test for presence of CD1_1 and/or MPP_NADD
;
CD1_1 = sxpar(header, 'CD1_1', COUNT=test1)
MPP_NADD = sxpar(header, 'MPP_NADD', COUNT=test2)
;
if(test1 gt 0 or test2 gt 0) then coadd = 1 else coadd = 0
;
;tel_ra = sxpar(header, 'TELRA')
;tel_dec = sxpar(header, 'TELDEC')

xsize = sxpar(header,'NAXIS1')
ysize = sxpar(header,'NAXIS2')
;crpix1 = sxpar(header,'CRPIX1')
crval1 = sxpar(header,'CRVAL1')
;crdelt1 = sxpar(header,'CRDELT1')
;crota1 = sxpar(header,'CROTA1')
;cfint1 = sxpar(header,'CFINT1')
;crpix2 = sxpar(header,'CRPIX2')
crval2 = sxpar(header,'CRVAL2')
;
;	don't forget to check epoch!
;
epoch = sxpar(header, "EQUINOX")
;
;	if mode is not coadd, then get rough coordinates from the
;	telescope ra and dec
;
if(coadd eq 0) then begin
	this_ra = 15.d*ten(double(strsplit(sxpar(header,'TELRA'),':',/EXTRACT))); telescope RA in degrees
	this_dec = ten(double(strsplit(sxpar(header,'TELDEC'),':',/EXTRACT))); telescope Dec in degrees
	;
	neg_yn = strpos(sxpar(header,'TELDEC'),'-') ne -1
	if(neg_yn eq 1) then this_dec = -1.0* abs(this_dec)			; strsplit drops the - sign for -00d
	;
	if(epoch ne 2000.0) then begin
		;
		;	must precess to J2000
		;
		precess, this_ra, this_dec, epoch, 2000
	endif
;
endif else begin
	this_ra = crval1
	this_dec = crval2
endelse
;
;this_mirror = ['CA', 'CfA', 'UK', 'Jp', 'CDS']		; CA having problems responding
;this_mirror = ['CfA', 'UK', 'Jp', 'CDS', 'CA']		; Jp earthquake 3/9/2011
this_mirror = ['CfA', 'UK', 'CDS', 'Jp', 'SAAO']  ; more CA response problems

;
superloop = 0L
;
superloop_start:
;
nloop = 0L
WHILE nloop lt 4320L do begin							; 1 day at 20 sec per try
	;
	vizier_flag = 0
	for imirror = 0, N_ELEMENTS(this_mirror)-1 do begin
	     if(vizier_flag eq 0) then begin
	         ; 20200517 DPC replaced 2MASS-PSC, which queries 2 catalogs, with call to PSC by designation
  	  		info = ppol_QueryVizier('II/246', [this_ra,this_dec], [search*(xsize/1024.0),search*(ysize/1026.0)], $
  	  		MIRROR=this_mirror[imirror], constraint='Qflg==AAA')
  	  		test = size(info)
  	  		if test[0] ne 0 then begin
  	  		   ;
  	  		   ; second test - are there valid HMAGS?
  	  		   ;
  	  		   ind_h = WHERE(info[*].hmag gt 6. and info[*].hmag lt 18.,ngood)
  	  		   ;
  	  		   if(ngood gt 2) then begin
  	  		     vizier_flag = 1
  	  		     goto, done
  	  		   endif else begin      ; if regular 2MASS-PSC had no stars, try the 6X 2MASS-PSC - 20200517 DPC
  	  		     new_info = ppol_QueryVizier('II/281', [this_ra,this_dec], [search*(xsize/1024.0),search*(ysize/1026.0)], $
  	  		       MIRROR=this_mirror[imirror], constraint='Qflg==AAA')
  	  		   endelse
  	  		   new_test = size(new_info)
  	  		   if new_test[0] ne 0 then begin
  	  		     ind_h = WHERE(new_info[*].hmag gt 6. and new_info[*].hmag lt 18.,new_ngood)
  	  		     if(new_ngood gt 2) then begin
  	  		       vizier_flag = 1
  	  		       info = new_info
  	  		       ngood = new_ngood
  	  		       goto, done
  	  		      endif
  	  		   endif
  	  		endif
	     endif
	endfor
	;
	if(vizier_flag eq 0) then begin
	    info = 99   ;No Vizier servers available
			;
			;	if no Vizier servers, wait 20s and retry
			;
			print, 'No Vizier Servers at ',SYSTIME(),' waiting 20s and retrying'
			wait, 20
			nloop++
	endif else goto, done
ENDWHILE
;
;help, info
;stop
;
done:
;
; DPC 20150611 - supposedly, at this point we have a good set of 2MASS data.
;           however, of late we've found the Vizier calls sometimes return junk,
;           so this 'info' structure must be tested. If found to be junk, the
;           mirror site search must be done again.
;
ind_bad = WHERE(info[*].hmag lt 4.0 or info[*].hmag gt 18.,nbad,NCOMPLEMENT=n_not_bad,COMPLEMENT=ind_not_bad)
if(ngood-nbad lt 3) then begin
    ;
    ; increment superloop index
    ; 
    superloop++
    if(superloop gt 10) then begin
        print,'ERROR- no Vizier Mirror site is willing to return good 2MASS data. Stopping'
        stop
    endif
    ;
    goto, superloop_start
endif
;
;  if nbad gt 0, remove those entries
;  
if(nbad gt 0) then begin
  if(n_not_bad gt 3) then begin
    info = info[ind_not_bad]
  endif else begin
    print, 'ERROR - no good 2MASS fetches - this should not happen. Stopping'
    stop
  endelse
endif
;
return,info

end
