function ppol_find_solutions, Event, tolerance, header, trimmed_mass_array, $
			trimmed_star_list, quad_criterion, STATUS=status, $
			idebug, image_pathname
;
COMMON saved_times, t0, skip_2mass
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_Struct
d_ptr = PTR_Struct.PTR_Data
s_ptr = PTR_Struct.PTR_summary
w_ptr = PTR_struct.PTR_wid_id
;
; DPC 20150611 - test for junk in trimmed mass array. stop if found
; 
ind_bad = WHERE(abs(trimmed_mass_array[3,*] -99.9) lt 0.1,nbad)
if(nbad gt 0) then begin
    print, 'Bad 2MASS array on entry to PPOL_FIND_SOLUTIONS'
    print, 'Likely a bad Vizier coordinate fetch. Stopping"
    stop
endif
;
;	make region file of mimir star list
;
;idebug = 1
i2mass = 0
;
nstars = N_ELEMENTS(trimmed_star_list[0,*])
;
mass_ra_tot = reform(trimmed_mass_array[0,*])
mass_dec_tot = reform(trimmed_mass_array[1,*])
num_mass = N_ELEMENTS(mass_ra_tot)
;
;find the distance of every 2MASS star to every other 2MASS star
;
;	first, convert all 2MASS coordinates into "standard coordinates"
;
;		what is the reference for this image?
;
xsize = sxpar(header, 'NAXIS1')
ysize = sxpar(header, 'NAXIS2')
test_crval1 = sxpar(header, 'CRVAL1')
test_crpix1 = sxpar(header, 'CRPIX1')
if(test_crval1 eq test_crpix1) then begin
	;
	;	"raw" image, get notional center coordinates from telescope coords
	;
	tel_ra = 15.d*ten(double(strsplit(sxpar(header,'TELRA'),':',/EXTRACT))); telescope RA in degrees
	tel_dec = ten(double(strsplit(sxpar(header,'TELDEC'),':',/EXTRACT))); telescope Dec in degrees
	;
	neg_yn = strpos(sxpar(header,'TELDEC'),'-') ne -1
	if(neg_yn eq 1) then tel_dec = -1.0d0*abs(tel_dec)					; strsplit drops - sign for -00
	;
	if(idebug gt 0) then begin
		print_text, Event, "tel_ra, tel_dec = "+STRING(tel_ra,tel_dec,format='(f12.5,1x,f12.5)')
	endif
	;
	xcen = ROUND(xsize / 2.0)
	ycen = ROUND(ysize / 2.0)
	plate_scale = 0.579
endif else begin
	;
	;	otherwise, fetch from the header
	;
	tel_ra = sxpar(header, 'CRVAL1')
	tel_dec = sxpar(header, 'CRVAL2')
	if(idebug gt 0) then begin
		print_text, Event, "tel_ra, tel_dec = "+STRING(tel_ra,tel_dec,format='(f12.5,1x,f12.5)')
	endif
	xcen = sxpar(header, 'CRPIX1')
	ycen = sxpar(header, 'CRPIX2')
	;
	;	is there a CD matrix? if so, calc plate scale from it
	;
	cd_00 = sxpar(header, "CD1_1",COUNT=count)
	if(count gt 0) then begin
		plate_scale = 3600.0 * abs(cd_00)
	endif else begin
		plate_scale = 0.58					; only used in region plotting
	endelse
endelse
;
epoch = sxpar(header, "EQUINOX")
if(epoch ne 2000.0) then begin
	;
	;	must precess to J2000
	;
	precess, tel_ra, tel_dec, epoch, 2000
endif
;
crval1  = [ tel_ra, tel_dec ]
;
;	reset center to zero
;
crpix1 = [0.0d0, 0.0d0]
;
; Convert RA, Dec to Eta, Xi
;
map_type = 2			; 'TAN'
wcssph2xy, 15.0*mass_ra_tot, mass_dec_tot, eta, xi, map_type, $
         latpole = 0.0, crxy = crpix1, ctype=['RA---TAN', 'DEC--TAN'], $
         crval = crval1
;
;	any near image edges should be rejected
;
n_2mass_new = 0L
x_border_size = 2
y_border_size = 20
;
;	but save pointer to original 2mass data
;
orig_2mass_ind = lonarr(num_mass)
;
for i2m = 0, num_mass-1 do begin
	epix = xcen - eta[i2m] * 3600.0 / plate_scale
	xpix = ycen + xi[i2m] * 3600.0 / plate_scale
	if(epix ge x_border_size and epix le (xsize - x_border_size) and $
		xpix ge y_border_size and xpix le (ysize - y_border_size)) then begin
			eta[n_2mass_new] = eta[i2m]
			xi[n_2mass_new] = xi[i2m]
			;
			orig_2mass_ind[n_2mass_new] = i2m
			;
			n_2mass_new++
	endif
endfor
;
num_mass = n_2mass_new
eta = eta[0:num_mass-1]
xi = xi[0:num_mass-1]
orig_2mass_ind = orig_2mass_ind[0:num_mass-1]		; points to original list of 2mass entries
;
;	dump to a region file for DS9 plotting
;
ifind=idebug
;
if(i2mass gt 0) then begin
	;
	;	write a region file for this image
	;
	region_dir = (*s_ptr).analysis_dir + 'S3_Astrometry' + PATH_SEP() + 'Region_Files' + PATH_SEP()
	test_dir = FILE_INFO(region_dir)
	if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
	;
	file_reg = FILE_BASENAME(image_pathname) + '_2MASS_stars.reg'
	region_path = region_dir + file_reg
	;
	openw, lu_find, region_path, /GET_LUN
	;
	printf, lu_find, 'global color=cyan font="helvetica 10 normal" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source'
	printf, lu_find, 'physical'
	for isf = 0, num_mass-1 do begin
		epix = xcen - eta[isf] * 3600.0 / plate_scale
		xpix = ycen + xi[isf] * 3600.0 / plate_scale
		printf, lu_find, (epix+1),(xpix+1),format='("circle(",f7.1,",",f7.1,",8)")'
	endfor
	FREE_LUN, lu_find
;	stop
endif
;
massdistarr = fltarr(num_mass,num_mass)
;
for coll=0,num_mass-1 do begin
  for row=coll+1,num_mass-1 do begin
	  ;
	  dra = eta[coll] - eta[row]
	  ddec = xi[coll] - xi[row]
	  ;
	  stardist = sqrt(dra * dra + ddec * ddec)
	  massdistarr[coll,row] = stardist
	  massdistarr[row,coll] = stardist
  endfor
endfor
;
num_mass_trials = num_mass*(num_mass-1)*(num_mass-2)
;
;	if none, exit with error
;
if(num_mass_trials le 0) then begin
	print_text, Event, "WARNING - not enough 2MASS stars found, resetting search"
	status = -2
	best_sets = [0]
	return, best_sets
endif
;
mass_trials = intarr(num_mass_trials,3)
mass_dists = fltarr(num_mass_trials,3)			; 1-2, 2-3, 3-1 distances then 1-2/2-3, 2-3/3-1, 3-1/1-2 ratios
mass_ratios = fltarr(num_mass_trials,2)			; M/L, S/L
count = 0L
;
for star1 = 0, num_mass-1 do begin
  for star2 = 0, num_mass-1 do begin
  	;
  	if(star1 eq star2) then goto, skip1
  	;
    for star3 = 0, num_mass-1 do begin
    	;
    	if(star1 eq star3) then goto, skip2
    	if(star2 eq star3) then goto, skip2
    	;
        mass_trials[count,*] = [star1, star2, star3]
        mass_dists[count,0] = massdistarr[star1,star2]
        mass_dists[count,1] = massdistarr[star2,star3]
        mass_dists[count,2] = massdistarr[star3,star1]
        ;
        ;	resort by length
        ;
        ind = SORT(REFORM(mass_dists[count,*]))
        mass_dists[count,*] = mass_dists[count,ind]
        mass_ratios[count,0] = mass_dists[count,1] / mass_dists[count,2]	; M/L
        mass_ratios[count,1] = mass_dists[count,0] / mass_dists[count,2]	; S/L
        count++
      ;
      skip2:
    endfor
    ;
    skip1:
  endfor
endfor
;
mass_trials = mass_trials[0:count-1,*]
mass_dists = mass_dists[0:count-1,*]
mass_ratios = mass_ratios[0:count-1,*]
;
if(idebug gt 0) then print_text, Event, "before sorting, cleaning, number of 2mass triangles = "+STRING(count)
;
;	but, this should be duped by a factor of 6, reduce duping
;
ind = SORT(REFORM(mass_ratios[*,0]))
;
ind_uniq = UNIQ(REFORM(mass_ratios[ind,0]))
;
num_2mass_triangles = N_ELEMENTS(ind_uniq)
mass_trials = mass_trials[ind[ind_uniq],*]
mass_dists = mass_dists[ind[ind_uniq],*]
mass_ratios = mass_ratios[ind[ind_uniq],*]
;
num_mass_trials = num_2mass_triangles
;
if(idebug gt 0) then print_text, Event, "after sorting, cleaning, number of 2mass triangles = "+STRING(num_2mass_triangles)
;
;--------------------- next, treat the Mimir targets similarly -------------
;
;find the distance of every target star to every other target star
;
num_target = N_ELEMENTS(trimmed_star_list[0,*])
targdistarr = fltarr(num_target,num_target)
for coll=0,num_target-1 do begin
  for row=coll,num_target-1 do begin
  	if(row eq coll) then begin
  		targdistarr[coll,row] = float(coll+1)						; to allow for no quad criterion
  	endif else begin
	  	dx = (trimmed_star_list[0,coll]-trimmed_star_list[0,row])
	  	dy = (trimmed_star_list[1,coll]-trimmed_star_list[1,row])
	    targdistarr[coll,row] = sqrt(dx^2 + dy^2)
	    targdistarr[row,coll] = targdistarr[coll,row]
	endelse
  endfor
endfor
;
if(quad_criterion eq 1) then begin
	ind1 = where(trimmed_star_list[2,*] eq 1, count1)
	ind2 = where(trimmed_star_list[2,*] eq 2, count2)
	ind3 = where(trimmed_star_list[2,*] eq 3, count3)
	ind4 = where(trimmed_star_list[2,*] eq 4, count4)

	num_combos = (count1*count2*count3 + count1*count2*count4 + count1*count3*count4 + count2*count3*count4)
	;
	if(num_combos eq 0) then begin
		status = -1
		best_sets = -1
		return, best_sets
	endif
endif else begin
	num_combos = num_target^3
	count1 = num_target
	count2 = num_target
	count3 = num_target
	ind1 = indgen(count1)
	ind2 = indgen(count2)
	ind3 = indgen(count3)
endelse

target_trials = lonarr(num_combos, 3)		; [0,0,0,0,0,0]
target_dists = fltarr(num_combos, 3)		; [dist12, dist23, dist31, ratio12_23, ratio23_31, ratio31_12]
target_ratios = fltarr(num_combos, 2)		; M/L, S/L

count = 0L

if(quad_criterion eq 0) then begin
 ;
 for star1 = 0, count1-1 do begin
    for star2 = 0, count2-1 do begin
    	if(star1 ne star2) then begin
      		for star3 = 0, count3-1 do begin
      			if(star3 ne star1) then begin
			      	is1 = ind1[star1]
			      	is2 = ind2[star2]
			      	is3 = ind3[star3]
			        target_trials[count,*] = [is1,is2,is3]
			        target_dists[count,0] = targdistarr[is1,is2]
			        target_dists[count,1] = targdistarr[is2,is3]
			        target_dists[count,2] = targdistarr[is3,is1]
			        ind = SORT(REFORM(target_dists[count,*]))
			        target_dists[count,*] = target_dists[count,ind]
			        target_ratios[count,0] = target_dists[count,1] / target_dists[count,2]
			        target_ratios[count,1] = target_dists[count,0] / target_dists[count,2]
			        count++
			     endif
		    endfor
		endif
    endfor
  endfor
endif else begin

	;generate target cases
	;quads 1, 2, 3
	if (count1 ne -1 and count2 ne -1 and count3 ne -1) then begin; in case there are no stars in a quadrant
	;
	  for star1 = 0, count1-1 do begin
	    for star2 = 0, count2-1 do begin
	      for star3 = 0, count3-1 do begin
	      	is1 = ind1[star1]
	      	is2 = ind2[star2]
	      	is3 = ind3[star3]
	        target_trials[count,*] = [is1,is2,is3]
	        target_dists[count,0] = targdistarr[is1,is2]
	        target_dists[count,1] = targdistarr[is2,is3]
	        target_dists[count,2] = targdistarr[is3,is1]
	        ind = SORT(REFORM(target_dists[count,*]))
	        target_dists[count,*] = target_dists[count,ind]
	        target_ratios[count,0] = target_dists[count,1] / target_dists[count,2]
	        target_ratios[count,1] = target_dists[count,0] / target_dists[count,2]
	        count++
	      endfor
	    endfor
	  endfor
	endif
	;quads 1, 2, 4
	if (count1 ne -1 and count2 ne -1 and count4 ne -1) then begin
	  for star1 = 0, count1-1 do begin
	    for star2 = 0, count2-1 do begin
	      for star4 = 0, count4-1 do begin
	        is1 = ind1[star1]
	      	is2 = ind2[star2]
	      	is4 = ind4[star4]
	        target_trials[count,*] = [is1,is2,is4]
	        target_dists[count,0] = targdistarr[is1,is2]
	        target_dists[count,1] = targdistarr[is2,is4]
	        target_dists[count,2] = targdistarr[is4,is1]
	        ind = SORT(REFORM(target_dists[count,*]))
	        target_dists[count,*] = target_dists[count,ind]
	        target_ratios[count,0] = target_dists[count,1] / target_dists[count,2]
	        target_ratios[count,1] = target_dists[count,0] / target_dists[count,2]
	        count++
	      endfor
	    endfor
	  endfor
	endif
	;quads 1, 3, 4
	if (count1 ne -1 and count4 ne -1 and count3 ne -1) then begin
	  for star1 = 0, count1-1 do begin
	    for star3 = 0, count3-1 do begin
	      for star4 = 0, count4-1 do begin
	        is1 = ind1[star1]
	      	is3 = ind3[star3]
	      	is4 = ind4[star4]
	        target_trials[count,*] = [is1,is3,is4]
	        target_dists[count,0] = targdistarr[is1,is3]
	        target_dists[count,1] = targdistarr[is3,is4]
	        target_dists[count,2] = targdistarr[is4,is1]
	        ind = SORT(REFORM(target_dists[count,*]))
	        target_dists[count,*] = target_dists[count,ind]
	        target_ratios[count,0] = target_dists[count,1] / target_dists[count,2]
	        target_ratios[count,1] = target_dists[count,0] / target_dists[count,2]
	        count++
	      endfor
	    endfor
	  endfor
	endif
	;quads 2, 3, 4
	if (count4 ne -1 and count2 ne -1 and count3 ne -1) then begin
	  for star2 = 0, count2-1 do begin
	    for star3 = 0, count3-1 do begin
	      for star4 = 0, count4-1 do begin
	        is2 = ind2[star2]
	      	is3 = ind3[star3]
	      	is4 = ind4[star4]
	        target_trials[count,*] = [is2,is3,is4]
	        target_dists[count,0] = targdistarr[is2,is3]
	        target_dists[count,1] = targdistarr[is3,is4]
	        target_dists[count,2] = targdistarr[is4,is2]
	        ind = SORT(REFORM(target_dists[count,*]))
	        target_dists[count,*] = target_dists[count,ind]
	        target_ratios[count,0] = target_dists[count,1] / target_dists[count,2]
	        target_ratios[count,1] = target_dists[count,0] / target_dists[count,2]
	        count++
	      endfor
	    endfor
	  endfor
	endif
endelse
;
;	sort the mimir triangles by the first ratio
;
ind = SORT(REFORM(target_ratios[*,0]))
target_trials = target_trials[ind,*]
target_dists = target_dists[ind,*]
target_ratios = target_ratios[ind,*]
;
n_mimir = count
;
if(idebug gt 0) then print_text, Event, "after sorting, number of Mimir triangles = "+STRING(n_mimir)
;
;	check for dupes
;
ind_uniq = UNIQ(REFORM(target_ratios[*,0]))
if(N_ELEMENTS(ind_uniq) ne n_mimir) then begin
	;
	;	were some dupes, remove them
	;
	target_trials = target_trials[ind_uniq,*]
	target_dists = target_dists[ind_uniq,*]
	target_ratios = target_ratios[ind_uniq,*]
	n_mimir = N_ELEMENTS(ind_uniq)
	;
	if(idebug gt 0) then print_text, Event, "found some dupes in Mimir triangles. After removal, "+$
									"number Mimir triangles = "+STRING(n_mimir)
endif
;
num_combos = n_mimir
;
dt = systime(/seconds) - t0
if(idebug gt 0) then print_text, Event, "elapsed time after set up of stars = "+STRING(dt)+" seconds"
;
;		dpc engine
;
;toler = 0.002							; 0.2% tolerance
toler = tolerance
;
saved_trial = lonarr(100000L < (num_combos * num_mass_trials), 3)	; it, im,
saved_platescale = fltarr(100000L < (num_combos * num_mass_trials))	;plate_scale
;
n_saved = 0L
;
it = 0L
im = 0L
n_bad_angles = 0L
n_good_angles = 0L
;
n_first = 0L
n_second = 0L
;
;	softening factor for second triangle test - if few Mimir stars (e.g., calibrator images), make quite soft
;
tscale = 1.5
if(n_mimir le 10) then tscale = 5.0	; was 10  20100804
if(n_mimir le 6) then tscale = 15.0	; was 30  20100804
;
;	for every mimir trial triangle...
;
ilast_2mass = 0L							; last pointer to 2mass list
for it = 0L, num_combos-1L do begin
	;
	;	first, check that interior angle is not too large or too small
	;		reasonable limits are 45-135 deg (updated to 10-170)
	;
	;		order the lengths
	;
	star_length = REFORM(target_dists[it,*])
;	for iss = 0, 2 do star_length[iss] = target_dists[it,iss]
	;
	ind_len = SORT(star_length)
	star_length = star_length[ind_len]	; shortest = 1, longest = 3
	;
	;	compute angle opposite longest side
	;
	num = star_length[0]^2 + star_length[1]^2 - star_length[2]^2
	denom = 2.0d0*star_length[0]*star_length[1]
	angle = acos(num/denom)
	angle = angle * 180.0 / !PI
	;print, "Angle = ",angle
	;
	if(angle gt 10. and angle le 170.) then begin
		n_good_angles++
		;
		;		check every 2mass triangle
		;
		;			starting with the last one showing a shorter M/L ratio than the
		;			last Mimir triangle
		;
		for im = ilast_2mass, num_mass_trials-1L do begin
			;
			;	first, compare mimir to 2mass ratios for (re)setting ilast_2mass
			;
			if(mass_ratios[im,0] lt 0.95*target_ratios[it,0]) then ilast_2mass = im		; save if so
			;
			;	compare M/L distance ratio for Mimir to same ratio for 2mass
			;
			if(abs((target_ratios[it,0]-mass_ratios[im,0])/(target_ratios[it,0] < mass_ratios[im,0])) lt toler) then begin
				;
				;	passed first test, try second (S/L) - much softer test (30 * toler - esp for few stars)
				;
				n_first++
				if(abs((target_ratios[it,1]-mass_ratios[im,1])/(target_ratios[it,1] < mass_ratios[im,1])) lt tscale*toler) then begin
					;
					;	passed second test
					;
					n_second++
					saved_trial[n_saved,0] = it
					saved_trial[n_saved,1] = im
					;
					;	compute plate scale for this match, using perimeter ratio
					;
					saved_platescale[n_saved] = (target_dists[it,2]+target_dists[it,1]+target_dists[it,0])/(mass_dists[im,2]+$
													mass_dists[im,1] + mass_dists[im,0])
					;
					;	only two plate scales are "correct" for Mimir: 0.58" and 0.18" per pixel
					;		these show up with platescales of 6224 pixels per degree and 2075 pixels per degree
					;
					;	accept only those within +/-4% of these values - had been 10% 20100804
					;
					pstest = abs(saved_platescale[n_saved])
					t1 = abs(pstest - 6224.0)/6224.0
					t2 = abs(pstest - 2075.0)/2075.0
					if(t1 lt 0.04 or t2 lt 0.05) then n_saved++ else begin
						if(idebug gt 2) then print_text, Event, "...bad final triangle test. pstest, t1, t2 = "+$
										STRING(pstest, t1, t2, format='(F12.4,1x,f12.4,1x,f12.4)')
					endelse
				endif  else begin
					;
					;	dump the test 2 failures
					;
					if(idebug gt 1) then begin
						print_text, Event, "Fail in test 2 - target_ratios(it,1), mass_ratios(im,1) = "+STRING(target_ratios[it,1],$
											mass_ratios[im,1],format='(F12.4,1x,f12.4)')+" for it,im = "+$
											STRING(it,im,format='(I4,1x,I4)')
					endif
				endelse
			endif else begin
				if(mass_ratios[im,0] gt 1.05*target_ratios[it,0]) then goto, loop_end			; difference too large, don't search more
			endelse
		endfor
	endif else n_bad_angles++
	loop_end:
endfor
;
if(idebug gt 0) then begin
	print_text, Event, "Number of good angles = "+STRING(n_good_angles)
	print_text, Event, "Number of bad angles = "+STRING(n_bad_angles)
	print_text, Event, "Number of triangles passing first test = "+STRING(n_first)
	print_text, Event, "Number of triangles passing second test = "+STRING(n_second)
	print_text, Event, "Number of triangles passing all tests = "+STRING(n_saved)
	;
	dt = systime(/seconds) - t0
	print_text, Event, "Elapsed time after computation of grades = "+STRING(dt)+" seconds"
endif
;
if(n_saved eq 0) then begin
	;
	;	no saved cases - abort this try
	;
	if(idebug gt 0) then print_text, Event, "PpolFindSolutions - n_good_matches = 0. Issuing retry."
	status = -1
	best_sets = -1
	return, best_sets
endif
;
saved_trial = saved_trial[0:n_saved-1,*]
saved_platescale = saved_platescale[0:n_saved-1]
;
if(idebug gt 0) then print_text, Event, "number of saved trials of triangles = "+STRING(n_saved)
;
;	implement platescale mfm test
;
zz = median_filtered_mean(saved_platescale[*])
if(idebug gt 0) then print_text, Event, "median plate scale = "+STRING(zz[0])+" +/-"+STRING(zz[1]/sqrt(n_saved))
;
ind = WHERE(abs((saved_platescale[*]-zz[0])/zz[1]) le 3.5,count)
if(count gt 0) then begin
	saved_trial = saved_trial[ind,*]
	saved_platescale = saved_platescale[ind]
	if(n_saved ne count) then begin
		if(idebug gt 0) then print_text, Event, "rejected "+STRING(n_saved-count)+" solutions by platescale cut"
	endif
	n_saved = count
endif else begin
	if(idebug gt 0) then print_text, Event, "no good solutions saved. Quitting"
end
;
;	next, for each unique triangle in the target stars, find the minimum grade
;
best_sets = dblarr(n_saved,13)
;
ifind=idebug
if(ifind eq 1) then begin
	file_find = 'C:\Documents and Settings\dan\Desktop\IDL\MSP_AST\data\two_mass_stars_found.reg'
	openw, lu_find, file_find, /GET_LUN
	printf, lu_find, 'global color=red font="helvetica 10 normal" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source'
	printf, lu_find, 'physical'
endif
;
for itrian = 0, n_saved-1 do begin
	;
	;	get all the trials for this triangle
	;
	this_triangle = saved_trial[itrian,0]	; which triangle (first index)
	;
	triangle_2mass = saved_trial[itrian,1]
	;
	;print, "Matched Mimir triangle ",this_triangle," to 2MASS triangle ",triangle_2mass
	;
	tc = this_triangle
	mc = triangle_2mass
	;
	;	save platescale
	;
	best_sets[itrian,0] = saved_platescale[itrian]
	;
	;	load x, y for mimir triangle
	;
	x = fltarr(3)
	y = fltarr(3)
	for ii = 0, 2 do begin
		x[ii] = trimmed_star_list[0,target_trials[tc,ii]]	; x
		y[ii] = trimmed_star_list[1,target_trials[tc,ii]]	; y
	endfor
	;
	;	order the stars 1,2,3
	;
	ind = ppol_order_stars(x,y)
	;
	for ii = 0, 2 do begin
		;
  		best_sets[itrian,2*ii+1]   = x[ind[ii]]	; x
  		best_sets[itrian,2*(ii+1)] = y[ind[ii]]	; y
  		if(ifind eq 1) then printf, lu_find, format='("circle(",f7.1,",",f7.1,",",i2.1,") # color=green")', best_sets[itrian,2*ii+1], $
								best_sets[itrian,2*(ii+1)], 4+3*ii
	endfor
	if(ifind eq 1) then begin
		printf, lu_find, format='("line(",F7.1,",",F7.1,",",F7.1,",",F7.1,") # line= 0 0 color=green")',$
			best_sets[itrian,1], best_sets[itrian,2], best_sets[itrian, 3], best_sets[itrian, 4]
		printf, lu_find, format='("line(",F7.1,",",F7.1,",",F7.1,",",F7.1,") # line= 0 0 color=green")',$
			best_sets[itrian,3], best_sets[itrian,4], best_sets[itrian, 5], best_sets[itrian, 6]
		printf, lu_find, format='("line(",F7.1,",",F7.1,",",F7.1,",",F7.1,") # line= 0 0 color=green")',$
			best_sets[itrian,1], best_sets[itrian,2], best_sets[itrian, 5], best_sets[itrian, 6]
	endif
	  ;
	  eta_plot = fltarr(3)
	  xi_plot = fltarr(3)
	  ;
	  point_orig_2mass = lonarr(3)
	  ;
	  for ii = 0, 2 do begin
	  	i2mass = mass_trials[mc,ii]
	  	;
	  	;	load pointer to original 2mass order
	  	;
	  	point_orig_2mass[ii] = orig_2mass_ind[i2mass]
	  	;
		x[ii] = eta[i2mass]						; x
		y[ii] = xi[i2mass]						; y
	  endfor
	  ;
	  ;	order the stars 1,2,3
	  ;
	  ind = ppol_order_stars(x,y)
	  ;
	  x = x[ind]
	  y = y[ind]
	  ;
	  point_orig_2mass = point_orig_2mass[ind]
	  ;
	  for ii = 0, 2 do begin
	  	eta_plot[ii] = xcen - x[ii] * 3600.0 / plate_scale
	  	xi_plot[ii] = ycen + y[ii] * 3600.0 / plate_scale
	  	if(ifind eq 1) then printf, lu_find, eta_plot[ii], xi_plot[ii],4+3*ii,format='("circle(",f7.1,",",f7.1,",",i2.1,") # color=red")'
	  endfor
	  ;
	  if(ifind eq 1) then begin
			printf, lu_find, format='("line(",F7.1,",",F7.1,",",F7.1,",",F7.1,") # line= 0 0 color=red")',$
				eta_plot[0], xi_plot[0], eta_plot[1], xi_plot[1]
			printf, lu_find, format='("line(",F7.1,",",F7.1,",",F7.1,",",F7.1,") # line= 0 0 color=red")',$
				eta_plot[1], xi_plot[1], eta_plot[2], xi_plot[2]
			printf, lu_find, format='("line(",F7.1,",",F7.1,",",F7.1,",",F7.1,") # line= 0 0 color=red")',$
				eta_plot[0], xi_plot[0], eta_plot[2], xi_plot[2]
	  endif
	;
  ;
  ;	convert back to RA, DEC from standard coords -- 2009/01/21 DPC - why not just reload their original coords?
  ;
;  WCSXY2SPH, x, y, ra, dec, crval = crval1, latpole = 0.0, crxy=crpix1, $
;  			ctype = ['RA---TAN','DEC--TAN']
  ;
  ; old way - recompute coordinates
  ;
;  best_sets[itrian,7] = ra[0]
;  best_sets[itrian,8] = dec[0]
;  best_sets[itrian,9] = ra[1]
;  best_sets[itrian,10] = dec[1]
;  best_sets[itrian,11] = ra[2]
;  best_sets[itrian,12] = dec[2]
;
	;
	;	new way - reload original 2mass coordinates
	;
  best_sets[itrian,7] = 15.0d0*mass_ra_tot[point_orig_2mass[0]]
  best_sets[itrian,8] = mass_dec_tot[point_orig_2mass[0]]
  best_sets[itrian,9] = 15.0d0*mass_ra_tot[point_orig_2mass[1]]
  best_sets[itrian,10] = mass_dec_tot[point_orig_2mass[1]]
  best_sets[itrian,11] = 15.0d0*mass_ra_tot[point_orig_2mass[2]]
  best_sets[itrian,12] = mass_dec_tot[point_orig_2mass[2]]
  ;
;  if(ifind eq 1) then begin
;  	print, "match set ",itrian,"  with platescale = ",best_sets[itrian,0]
;  	for ik = 0, 2 do begin
;  		print, '  Star ',ik+1,' Mimir (x,y) = ',best_sets[itrian,2*ik+1],best_sets[itrian,2*ik+2],$
;  					'  (Ra, Dec) = ',best_sets[itrian,2*ik+7],best_sets[itrian,2*ik+8]
;  	endfor
;  endif
endfor
;
if(ifind eq 1) then begin
	FREE_LUN, lu_find
	;
	dt = systime(/seconds) - t0
	print_text, Event, "Elapsed time after initial plate solutions = "+STRING(dt)+" seconds"
endif
;
status=0
return, best_sets

end
