function ppol_trim_mass_array, Event, mass_array, header, max_quad_count, skip_bands, sparse
;
;	extracts up to max_quad_count of 2mass stars per quadrant
;
;	if skip_bands > 0, then skips that many 1/2 max_quad_counts down the list
;		(bright star, dense star strategy)
;
;	if sparse = 1, then use all magnitudes
;
;------------------------------------------------------------------
;
;	deal with skipping...
;
min_2mass = ROUND( 0.5 * float(max_quad_count) * skip_bands )
max_2mass = min_2mass + max_quad_count
;
;------------------------------------------------------------------
filter = strtrim(sxpar(header,'FILTNME1'),2)
case filter of									;regardless of filter, just call it extracted_Hmag
  'H_HWP': extracted_Hmag = mass_array.Hmag
  'K_HWP': extracted_Hmag = mass_array.Kmag
  'H':extracted_Hmag = mass_array.Hmag
  'J':extracted_Hmag = mass_array.Jmag
  'Ks1': extracted_Hmag = mass_array.Kmag
  'Open': begin
            filter2 = strtrim(sxpar(header,'FILTNME2'),2)
            case filter2 of
              'PK50-2': begin
                          filter3 = strtrim(sxpar(header,'FILTNME3'),2)
                          case filter3 of
                            'IZYJ-Blocker' : begin
                                              filter4 = strtrim(sxpar(header,'FILTNME4'),2)
                                              case filter4 of
                                                'Open': extracted_Hmag = mass_array.Jmag   ; is IZYJ-Wide, closest to J
                                                 else: begin
                                                          print, 'No filter match - stopping'
                                                          stop
                                                        end
                                              endcase
                                            end
                             else: begin
                                        case filter3 of
                                          'H2-On': extracted_Hmag = mass_array.Kmag
                                          'H2-Off': extracted_Hmag = mass_array.Kmag
                                          else: begin
                                                  print, 'No filter match - stopping'
                                                   stop
                                                end
                                        endcase
                                   end
                           endcase
                        end
              'J': extracted_Hmag = mass_array.Jmag
              'H': extracted_Hmag = mass_array.Hmag
              'K': extracted_Hmag = mass_array.Kmag
              'Ks': extracted_Hmag = mass_array.Kmag
              else: begin
                      print, 'No filter match - stopping'
                      stop
                    end
           endcase
         end
   else: begin
              print, 'No filter match - stopping'
              stop
         end
endcase

mass_ra_tot = mass_array.RAJ2000/15.0d0				; convert it to decimal hours
mass_dec_tot = mass_array.DEJ2000
mass_h_tot = extracted_Hmag
mass_stars = size(mass_ra_tot)
num_mass = mass_stars[1]
;
avg_ra = mean(mass_ra_tot)
avg_dec = mean(mass_dec_tot)
;
;	20100326 - DPC - skip any/all 2MASS stars within 2 arcsec of each other
;
;cosdec = cos(avg_dec / 180.0 * !PI)
;;
;no_keep_list = intarr(num_mass)
;;
;for istar = 0, num_mass-1 do begin
;	dx = mass_ra_tot - mass_ra_tot[0]
;	dy = mass_dec_tot - mass_dec_tot[0]
;	;
;	dx *= (15.0 * cosdec * 3600.0)
;	dy *= 3600.0
;	;
;	dr = sqrt(dx^2 + dy^2)
;	ind_s = SORT(dr)
;	ind_close = WHERE(dr[ind_s] lt 2.0,nclose)
;	if(nclose gt 1) then begin
;		no_keep_list[ind_s[ind_close[1:(nclose-1)]]] = 1
;	endif
;endfor
;;
;no_keep_total = TOTAL(no_keep_list,/INTEGER)
;if(no_keep_total lt num_mass) then begin
;	if(no_keep_total gt 0) then begin
;		ind_good = WHERE(no_keep_list eq 0,nstars)
;		;
;		mass_ra_tot = mass_ra_tot[ind_good]
;		mass_dec_tot = mass_dec_tot[ind_good]
;		mass_h_tot = mass_h_tot[ind_good]
;		num_mass = nstars
;	endif
;endif else begin
;	;
;	;	all stars eliminated, must
;	;
;	n_mass = 0
;	goto, q5
;endelse

;divide 2MASS stars into 4 quadrants and limit to max_quad_count stars per quadrant


ind1 = where(mass_ra_tot ge avg_ra and mass_dec_tot ge avg_dec, quad1_count)
ind2 = where(mass_ra_tot lt avg_ra and mass_dec_tot ge avg_dec, quad2_count)
ind3 = where(mass_ra_tot lt avg_ra and mass_dec_tot lt avg_dec, quad3_count)
ind4 = where(mass_ra_tot ge avg_ra and mass_dec_tot lt avg_dec, quad4_count)

;for each quadrant limit to max_quad_count brightest stars
;
n_mass = 0L
new_mass_ra = mass_ra_tot
new_mass_dec = mass_dec_tot
new_mass_quad = intarr(N_ELEMENTS(mass_ra_tot))
if(quad1_count gt 0) then new_mass_quad[ind1] = 1
if(quad2_count gt 0) then new_mass_quad[ind2] = 2
if(quad3_count gt 0) then new_mass_quad[ind3] = 3
if(quad4_count gt 0) then new_mass_quad[ind4] = 4
new_mass_mag = mass_h_tot
;
if(sparse eq 0) then begin
	;
	;stop
	;
	if quad1_count gt max_quad_count then begin
	  bright_sort = sort(mass_h_tot[ind1])
	  ;
	  ;	if possible, select the stars in the desired brightness band
	  ;
	  bmin = min_2mass > 0
	  bmax = max_2mass < (quad1_count-1)
	  nb = bmax - bmin + 1
	  if(nb le 0) then goto, q2
	  ;
	  bright_sort = bright_sort[bmin:bmax]
	  ;
	  mass_ra_quad = mass_ra_tot[ind1[bright_sort]]
	  mass_dec_quad = mass_dec_tot[ind1[bright_sort]]
	  mass_mag_quad = mass_h_tot[ind1[bright_sort]]
	  ;
	  nn = N_ELEMENTS(mass_ra_quad)
	  new_mass_ra[n_mass: n_mass+nn-1] = mass_ra_quad[*]
	  new_mass_dec[n_mass: n_mass+nn-1] = mass_dec_quad[*]
	  new_mass_mag[n_mass: n_mass+nn-1] = mass_mag_quad[*]
	  new_mass_quad[n_mass: n_mass+nn-1] = 1
	  n_mass = n_mass + nn
	endif else begin
	  if(quad1_count ne 0) then begin
		  mass_ra_quad = mass_ra_tot[ind1]
		  mass_dec_quad = mass_dec_tot[ind1]
		  mass_mag_quad = mass_h_tot[ind1]
		  ;
		  nn = N_ELEMENTS(mass_ra_quad)
		  new_mass_ra[n_mass: n_mass+nn-1] = mass_ra_quad[*]
		  new_mass_dec[n_mass: n_mass+nn-1] = mass_dec_quad[*]
		  new_mass_mag[n_mass: n_mass+nn-1] = mass_mag_quad[*]
		  new_mass_quad[n_mass: n_mass+nn-1] = 1
		  n_mass = n_mass + nn
	  endif
	endelse
	;
	q2:
	;
	if quad2_count gt max_quad_count then begin
	  bright_sort = sort(mass_h_tot[ind2])
	  ;
	  bmin = min_2mass > 0
	  bmax = max_2mass < (quad2_count-1)
	  nb = bmax - bmin + 1
	  if(nb le 0) then goto, q3
	  ;
	  bright_sort = bright_sort[bmin:bmax]
	  ;
	  mass_ra_quad = mass_ra_tot[ind2[bright_sort]]
	  mass_dec_quad = mass_dec_tot[ind2[bright_sort]]
	  mass_mag_quad = mass_h_tot[ind2[bright_sort]]
	  ;
	  nn = N_ELEMENTS(mass_ra_quad)
	  new_mass_ra[n_mass: n_mass+nn-1] = mass_ra_quad[*]
	  new_mass_dec[n_mass: n_mass+nn-1] = mass_dec_quad[*]
	  new_mass_mag[n_mass: n_mass+nn-1] = mass_mag_quad[*]
	  new_mass_quad[n_mass: n_mass+nn-1] = 2
	  n_mass = n_mass + nn
	endif else begin
		if(quad2_count ne 0) then begin
		  mass_ra_quad = mass_ra_tot[ind2]
		  mass_dec_quad = mass_dec_tot[ind2]
		  mass_mag_quad = mass_h_tot[ind2]
		  ;
		  nn = N_ELEMENTS(mass_ra_quad)
		  new_mass_ra[n_mass: n_mass+nn-1] = mass_ra_quad[*]
		  new_mass_dec[n_mass: n_mass+nn-1] = mass_dec_quad[*]
		  new_mass_mag[n_mass: n_mass+nn-1] = mass_mag_quad[*]
		  new_mass_quad[n_mass: n_mass+nn-1] = 2
		  n_mass = n_mass + nn
	  endif
	endelse
	;
	q3:
	;
	if quad3_count gt max_quad_count then begin
	  bright_sort = sort(mass_h_tot[ind3])
	  ;
	  bmin = min_2mass > 0
	  bmax = max_2mass < (quad3_count-1)
	  nb = bmax - bmin + 1
	  if(nb le 0) then goto, q4
	  ;
	  bright_sort = bright_sort[bmin:bmax]
	  ;
	  mass_ra_quad = mass_ra_tot[ind3[bright_sort]]
	  mass_dec_quad = mass_dec_tot[ind3[bright_sort]]
	  mass_mag_quad = mass_h_tot[ind3[bright_sort]]
	  ;
	  nn = N_ELEMENTS(mass_ra_quad)
	  new_mass_ra[n_mass: n_mass+nn-1] = mass_ra_quad[*]
	  new_mass_dec[n_mass: n_mass+nn-1] = mass_dec_quad[*]
	  new_mass_mag[n_mass: n_mass+nn-1] = mass_mag_quad[*]
	  new_mass_quad[n_mass: n_mass+nn-1] = 3
	  n_mass = n_mass + nn
	endif else begin
		if(quad3_count ne 0) then begin
		  mass_ra_quad = mass_ra_tot[ind3]
		  mass_dec_quad = mass_dec_tot[ind3]
		  mass_mag_quad = mass_h_tot[ind3]
		  ;
		  nn = N_ELEMENTS(mass_ra_quad)
		  new_mass_ra[n_mass: n_mass+nn-1] = mass_ra_quad[*]
		  new_mass_dec[n_mass: n_mass+nn-1] = mass_dec_quad[*]
		  new_mass_mag[n_mass: n_mass+nn-1] = mass_mag_quad[*]
		  new_mass_quad[n_mass: n_mass+nn-1] = 3
		  n_mass = n_mass + nn
		 endif
	endelse
	;
	q4:
	;
	if quad4_count gt max_quad_count then begin
	  bright_sort = sort(mass_h_tot[ind4])
	  ;
	  bmin = min_2mass > 0
	  bmax = max_2mass < (quad4_count-1)
	  nb = bmax - bmin + 1
	  if(nb le 0) then goto, q5
	  ;
	  bright_sort = bright_sort[bmin:bmax]
	  ;
	  mass_ra_quad = mass_ra_tot[ind4[bright_sort]]
	  mass_dec_quad = mass_dec_tot[ind4[bright_sort]]
	  mass_mag_quad = mass_h_tot[ind4[bright_sort]]
	  ;
	  nn = N_ELEMENTS(mass_ra_quad)
	  new_mass_ra[n_mass: n_mass+nn-1] = mass_ra_quad[*]
	  new_mass_dec[n_mass: n_mass+nn-1] = mass_dec_quad[*]
	  new_mass_mag[n_mass: n_mass+nn-1] = mass_mag_quad[*]
	  new_mass_quad[n_mass: n_mass+nn-1] = 4
	  n_mass = n_mass + nn
	endif else begin
		if(quad4_count gt 0) then begin
	  		mass_ra_quad = mass_ra_tot[ind4]
	  		mass_dec_quad = mass_dec_tot[ind4]
	  		mass_mag_quad = mass_h_tot[ind4]
	  		;
			  nn = N_ELEMENTS(mass_ra_quad)
			  new_mass_ra[n_mass: n_mass+nn-1] = mass_ra_quad[*]
			  new_mass_dec[n_mass: n_mass+nn-1] = mass_dec_quad[*]
			  new_mass_mag[n_mass: n_mass+nn-1] = mass_mag_quad[*]
			  new_mass_quad[n_mass: n_mass+nn-1] = 4
			  n_mass = n_mass + nn
		endif
	endelse
	;
endif else begin
	n_mass = N_ELEMENTS(new_mass_ra)
endelse
;
q5:
;
if(n_mass eq 0) then begin
	print_text, Event, "Warning - no 2mass stars in this mag band. Resetting"
	trimmed_mass_array = [0., 0., 0., 0.]
;	stop
	return, trimmed_mass_array
endif
;
;	compress lists
;
new_mass_ra = new_mass_ra[0:n_mass-1]
new_mass_dec = new_mass_dec[0:n_mass-1]
new_mass_quad = new_mass_quad[0:n_mass-1]
new_mass_mag = new_mass_mag[0:n_mass-1]
;
;print, "Starting number of 2mass stars = ",num_mass,"  Finishing number of 2mass stars = ",n_mass

;new_mass_ra = [reform(mass_ra_quad1),reform(mass_ra_quad2),reform(mass_ra_quad3),reform(mass_ra_quad4)]
;new_mass_dec = [reform(mass_dec_quad1),reform(mass_dec_quad2),reform(mass_dec_quad3),reform(mass_dec_quad4)]
;new_mass_quad = [intarr(quad1_count < max_quad_count)+1,intarr(quad2_count < max_quad_count)+2,intarr(quad3_count < max_quad_count)+3,intarr(quad4_count < max_quad_count)+4]

trimmed_mass_array = transpose([[new_mass_ra],[new_mass_dec],[new_mass_quad],[new_mass_mag]])

return, trimmed_mass_array

end