function s10_read_polcats_into_structure, Event, files, band
;
;	takes a list of PHOTCAT files and reads all entries into one structure
;
;	20131202	added band (= 'H' or 'K') to increase testing for cluster replacement values
;	20140217 	turned on corrections for all clusters to boresight means for both bands
;
;---------------------------------------------------------
;
one_line = {pol_no: 0, phot_no: 0, g_desig: '', x_img: 0.0d0, y_img: 0.0d0, x_det: 0, y_det: 0, $
            ra: 0.0d0, dec: 0.0d0, H_mag: 0.0d0, sH_mag: 0.0d0, pH_mag: 0.0d0, sky: 0.0d0, P: 0.0d0, sP: 0.0d0, $
            PA: 0.0d0, GPA: 0.0d0, sPA: 0.0d0, Q: 0.0d0, sQ: 0.0d0, U: 0.0d0, sU: 0.0d0, nHWP: 0, UF: 1, t_id: 0L, t_desig: '',$
            RA_2: 0.0d0, Dec_2: 0.0d0, J2: 0.0d0, sJ2: 0.0d0, H2: 0.0d0, sH2: 0.0d0, K2: 0.0d0, $
            sK2: 0.0d0}
;
;	get number of lines in each file
;
nfiles = N_ELEMENTS(files)
nlines = 0L
for ifile = 0, nfiles-1 do begin
	openr, lu_in, files[ifile], /GET_LUN
	line = ''
	WHILE ~EOF(lu_in) do begin
		readf, lu_in, line
		ind_semi = STRPOS(line, ';')
		if(ind_semi ne -1) then goto, skip_count
		;
		nlines++
		skip_count:
	ENDWHILE
	FREE_LUN, lu_in
endfor
;
print_text, Event, "Building structure for "+STRING(nlines,format='(I)')+" entries"
;
;	dup the structure to that level
;
pol_struct = REPLICATE(one_line,nlines)
;
;	for each line of each file, read in all the information
;
this_line = 0L
for ifile = 0, nfiles-1 do begin
	openr, lu_file, files[ifile], /GET_LUN
	;
	;	DPC 20110311 - determine if the POLCAT was for M12 or M71 globular clusters
	;
	q_add = 0.0
	sq_add = 0.0
	u_add = 0.0
	su_add = 0.0
	;
	;	new values 20131202 - DPC
	;
;	if(STRPOS(STRUPCASE(FILE_BASENAME(files[ifile])),'M30') ne -1 and band eq 'H') then begin
;		print, 'Found an H-Band M30 POLCAT - ',FILE_BASENAME(files[ifile])
;		print, '  Correcting for boresight polarization'
;		q_add = -0.1441
;		u_add = -0.1859
;		sq_add = 0.048
;		su_add = 0.045
;	endif
;	if(STRPOS(STRUPCASE(FILE_BASENAME(files[ifile])),'M71') ne -1 and band eq 'H') then begin
;		print, 'Found an H-Band M71 POLCAT - ',FILE_BASENAME(files[ifile])
;		print, '  Correcting for boresight polarization'
;		q_add = -0.2598
;		u_add =  0.0399
;		sq_add = 0.018
;		su_add = 0.018
;	endif
;	;
	;	correct all Q, U and uncertainties to the values needed to bring this cluster to
	;	agreement with boresight instrumental polarization mean values - 20140217 DPC
	;
	get_cluster_name, files[ifile], this_cluster
	get_cluster_qu_corrections, this_cluster, band, q_add, sq_add, u_add, su_add, ier
	if(ier ne 0) then begin
		print, 'ERROR in S10 read_polcats_into_structure.pro - cluster QU corrections not found. Stopping'
		stop
	endif
	;
	use_uncerts = 1.0				; if zero, turns off uncertainties
	sq_add *= use_uncerts
	su_add *= use_uncerts
	;
	line = ''
	;
	;	then, for each remaining line, read its data into the
	;	structure
	;
	WHILE ~EOF(lu_file) do begin
		readf, lu_file, line
		ind_semi = STRPOS(line, ';')
		if(ind_semi ne -1) then goto, skip_parse
		;
		s10_parse_one_line_of_data, pol_struct, line, this_line
		;
		;	correct Q, U and uncertainties
		;
		pol_struct[this_line].Q += q_add
		pol_struct[this_line].U += u_add
		pol_struct[this_line].sQ = sqrt(pol_struct[this_line].sQ^2 + sq_add^2)
		pol_struct[this_line].sU = sqrt(pol_struct[this_line].sU^2 + su_add^2)
		;
		this_line++
		;
		skip_parse:
	ENDWHILE
	;
	FREE_LUN, lu_file
endfor
;
print_text, Event, "Read in "+STRING(this_line,format='(I)')+" lines of polarimetric data"
;
return, pol_struct
end
