pro S11_compute_P_PA_images, Event, deep_path, group_colors, ig, status
;
;	compute P, PA images and uncertainty images from Q, U and uncert
;	images.
;
;	uses pinst_corr flag to tell which images to fetch from S11
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
w_ptr = PTR_struct.PTR_wid_id
s_ptr = PTR_struct.PTR_summary
g_ptr = PTR_struct.PTR_group
;
dtor=!pi/180.0
;
pinst_corr_flag = (*w_ptr).S11_Pinst_mode
;
case pinst_corr_flag of
	0: suffix = '.fits
	1: suffix = '_cor.fits'
endcase
;
meta_flag = (*w_ptr).S11_meta_group_mode
;
LED1_wid = (*w_ptr).S11_LED1_groups_wid
;
print_text, Event, 'Starting Computation of P, PA images'
;
group_colors[ig] = 'orange'
WCT_LED_stack, LED1_wid, group_colors
;
deep_header = HEADFITS( deep_path, /silent)
;
;	read the raw Stokes Q, U and uncertainties
;
dir_in = FILE_DIRNAME( deep_path )
file_in = FILE_BASENAME( deep_path )
;
test_fits = STRPOS( STRUPCASE(file_in), 'DEEP_COADD.FITS')
if(test_fits ne -1) then begin
	QU_root = STRMID( file_in, 0, test_fits)
endif else begin
	print_text, Event, "ERROR - Could not parse filename"
	status = -1
	goto, S11CP_end
endelse
;
;		and directory
;
test_S7 = STRPOS( dir_in, 'S7_' )
if(test_S7 ne -1) then begin
	new_dir = STRMID( dir_in, 0, test_S7 ) + 'S11_Full_Field_Polarimetry'
	test = FILE_INFO(new_dir)
	if(test.exists eq 1 and test.directory eq 1) then begin
		dir_hwp = new_dir
	endif else begin
		print_text, Event, "ERROR - S11 Directory does not exist"
		status = -1
		goto, S11CP_end
	endelse
endif else begin
	print_text, Event, "ERROR - Deep filename does not contain 'S7_' "
	status = -1
	goto, S11CP_end
endelse
;
Q_path = dir_hwp + PATH_SEP() + QU_root + 'Q' + suffix
Q_header = HEADFITS( Q_path )
ncol = SXPAR(Q_header, 'NAXIS1')
nrow = SXPAR(Q_header, 'NAXIS2')
;
mpp_xoff = SXPAR(Q_header, 'MPP_XOFF')	; amount to add to x pix to get detector x pix
mpp_yoff = SXPAR(Q_header, 'MPP_YOFF')
;
ptr_QU = PTR_NEW( {			Q:		fltarr(ncol, nrow),	$
							sQ:		fltarr(ncol, nrow), $
							U:		fltarr(ncol, nrow),	$
							sU:		fltarr(ncol, nrow) } )
;
ptr_out = PTR_NEW( {		P:		fltarr(ncol, nrow), $
						 	sP:		fltarr(ncol, nrow), $
						  	PA:		fltarr(ncol, nrow), $
						 	sPA:	fltarr(ncol, nrow) } )
;
(*ptr_QU).Q = readfits(Q_path, Q_header, /silent)
;
sQ_path = dir_hwp + PATH_SEP() + QU_root + 'sQ'+suffix
(*ptr_QU).sQ = readfits(sQ_path, sQ_header, /silent)
;
U_path = dir_hwp + PATH_SEP() + QU_root + 'U'+suffix
(*ptr_QU).U = readfits(U_path, U_header, /silent)
;
sU_path = dir_hwp + PATH_SEP() + QU_root + 'sU'+suffix
(*ptr_QU).sU = readfits(sU_path, sU_header, /silent)
;
;	scale to percentages
;
(*ptr_QU).Q *= 100.0
(*ptr_QU).sQ *= 100.0
(*ptr_QU).U *= 100.0
(*ptr_QU).sU *= 100.0
;
print_text, Event, 'Read Stokes Images from disk'
;
;	for every pixel
;
for ix = 0, ncol-1 do begin
	for iy = 0, nrow-1 do begin
		;
		if( (*ptr_QU).sU[ix,iy] lt 1.0e3 and (*ptr_QU).sQ[ix,iy] lt 1.0e3) then begin
			ucor = [ (*ptr_QU).U[ix,iy], (*ptr_QU).sU[ix,iy] ]
			qcor = [ (*ptr_QU).Q[ix,iy], (*ptr_QU).sQ[ix,iy] ]
			;
			ap_uq2polpa, ucor, qcor, pcor, pacor
			;
		endif else begin
			pcor  = [0.0, 100.0]
			pacor = [0.0, 180.0]
			ucor  = [0.0, 1.0e6]
			qcor  = [0.0, 1.0e6]
		endelse
		;
		;check to see if Ricean correction can be done, if so do it
		;
		p_sig = pcor[1]
		p = pcor[0]
		pa = pacor[0]
		pa_sig = pacor[1]
		;
		u = ucor[0]
		q = qcor[0]
		;
		if(p ne 0.0) then begin
			if ((1.-(p_sig/p)^2) lt 0.0) then begin
				;
				;	no, zero polarization
				;
				p 		= 0.0
				pa 		= 0.0
				pa_sig	= 180.0
			endif else begin
				;
				;	yes, correct
				;
				p 		= p * SQRT(1.-(p_sig / p)^2)  ;	Ricean correction
				pa 		= (0.5 / dtor) * ATAN(u, q)
				pa 		= (720.0 + pa) mod 180.
				pa_sig 	= (0.0 > ((0.5 / dtor)*(p_sig / p))) < 180.0
			endelse
		endif else begin
			p = 0.0
			pa = 0.0
			pa_sig = 180.0
			p_sig = 100.0
		endelse
		;
		;	save results
		;
		(*ptr_out).P[ix,iy] = p
		(*ptr_out).sP[ix,iy] = p_sig
		(*ptr_out).PA[ix,iy] = pa
		(*ptr_out).sPA[ix,iy] = pa_sig
	endfor
endfor
;
;	write output files to disk *** should add PInst info to header here...
;
P_path = dir_hwp + PATH_SEP() + QU_root + 'P.fits'
p_header = Q_header
SXADDPAR, p_header, 'S11IMG','P','The Polarization [%] Image'
writefits, P_path, (*ptr_out).P, p_header
;
sP_path = dir_hwp + PATH_SEP() + QU_root + 'sP.fits'
sp_header = Q_header
SXADDPAR, sp_header, 'S11IMG','sP','The Polarization Uncertainty [%] Image'
writefits, sP_path, (*ptr_out).sP, sp_header
;
PA_path = dir_hwp + PATH_SEP() + QU_root + 'PA.fits'
pa_header = Q_header
SXADDPAR, pa_header, 'S11IMG','PA','The Equatorial Position Angle [deg] Image'
writefits, PA_path, (*ptr_out).PA, pa_header
;
sPA_path = dir_hwp + PATH_SEP() + QU_root + 'sPA.fits'
spa_header = Q_header
SXADDPAR, spa_header, 'S11IMG','sPA','The Equatorial Position Uncertainty Angle [deg] Image'
writefits, sPA_path, (*ptr_out).sPA, spa_header
;
print_text, Event, 'Wrote P and PA Images to disk'
group_colors[ig] = 'lawngreen'
WCT_LED_stack, LED1_wid, group_colors
;
PTR_FREE, ptr_QU, ptr_out
wait, 0.3
;
status = 0
;
S11CP_end:
;
end