pro S11_compute_UQ_images, Event, deep_path, group_colors, ig, status
;
;	from the HWP images pointed to by deep_path in S11,
;	compute Q_raw, U_raw, sQ_raw, sU_raw images
;
;	DPC	20110829
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
w_ptr = PTR_struct.PTR_wid_id
s_ptr = PTR_struct.PTR_summary
g_ptr = PTR_struct.PTR_group
LED1_wid = (*w_ptr).S11_LED1_groups_wid
;
print_text, Event, 'Starting Computation of Raw Q, U images'
group_colors[ig] = "violet"
WCT_LED_stack, LED1_wid, group_colors
;
;	U flip flag (set, means consistent with stars)
;
U_flip_flag = 1
;
;	parse name to get HWP images
;
dir_in = FILE_DIRNAME( deep_path )
file_in = FILE_BASENAME( deep_path )
;
test_fits = STRPOS( STRUPCASE(file_in), 'DEEP_COADD.FITS')
if(test_fits ne -1) then begin
	HWP_root = STRMID( file_in, 0, test_fits) + 'HWP_'
endif else begin
	print_text, Event, "ERROR - Could not parse filename"
	status = -1
	goto, S11CI_end
endelse
;
;		and directory
;
test_S7 = STRPOS( dir_in, 'S7_' )
if(test_S7 ne -1) then begin
	new_dir = STRMID( dir_in, 0, test_S7 ) + 'S11_Full_Field_Polarimetry'
	test = FILE_INFO(new_dir)
	if(test.exists eq 1 and test.directory eq 1) then begin
		dir_hwp = new_dir
	endif else begin
		print_text, Event, "ERROR - S11 Directory does not exist"
		status = -1
		goto, S11CI_end
	endelse
endif else begin
	print_text, Event, "ERROR - Deep filename does not contain 'S7_' "
	status = -1
	goto, S11CI_end
endelse
;
;	read the first file header, then use it as the template for making a
;	structure in heap
;
path_0 = dir_hwp + PATH_SEP() + HWP_root + '00_bg.fits'
;
header = HEADFITS(path_0)
;
ncol = SXPAR(header, 'NAXIS1')
nrow = SXPAR(header, 'NAXIS2')
;
;	create two large structures in heap - on for input, one for output
;
ptr_hwp = PTR_NEW( {     hwp_image: fltarr(ncol,nrow,16), $		; was dbl...
					     hwp_angle: fltarr(16), $
					     hwp_sky:	fltarr(16)} )
;
ptr_QU = PTR_NEW( { 	 I02_image:	fltarr(ncol, nrow), $
						 I13_image: fltarr(ncol, nrow), $
						 I_image:	fltarr(ncol, nrow),	$
						 Q_image:	fltarr(ncol, nrow), $
						 U_image:	fltarr(ncol, nrow), $
						 sQ_image:	fltarr(ncol, nrow), $
						 sU_image:	fltarr(ncol, nrow) } )
;
;	and read
;
;	read the HWP images
;
ilast = 0
for ihwp = 0, 15 do begin
	hwp_in = dir_hwp + PATH_SEP() + HWP_root + STRING(ihwp,format='(i2.2)')+'_bg.fits'
	;
	print_text, Event, 'Reading HWP image '+FILE_BASENAME(hwp_in)
	;
	(*ptr_hwp).hwp_image[*,*,ihwp] = readfits(hwp_in, this_header, /silent)
	;
	;	read the header for HWP angle, sky (assumes all scaling already done)
	;
	(*ptr_hwp).hwp_angle[ihwp] = SXPAR(this_header, 'PPOL_HWP')
	(*ptr_hwp).hwp_sky[ihwp] = SXPAR(this_header, 'S11AVSKY')
	;
	if(ilast eq 0) then begin
		group_colors[ig] = "rosybrown light"
		ilast = 1
	endif else begin
		group_colors[ig] = "rosybrown dark"
		ilast = 0
	endelse
	WCT_LED_stack, LED1_wid, group_colors
endfor
;
;	inds for each of 0, 45, 90, 135
;
mod_angle = ROUND(10.0*(*ptr_hwp).hwp_angle) MOD 900
ind_0 = WHERE(mod_angle eq 0, n_0)
ind_45 = WHERE(mod_angle eq 225, n_45)
ind_90 = WHERE(mod_angle eq 450, n_90)
ind_135 = WHERE(mod_angle eq 675, n_135)
;
ptr_inten = PTR_NEW( fltarr(4,ncol,nrow) )
ptr_uncer = PTR_NEW( fltarr(4,ncol,nrow) )
;
;	compute intensity and uncertainty images for each of the 4 IPPAs
;
ilast = 0
for ippa = 0, 3 do begin
	print_text, Event, 'Starting processing of IPPA '+STRING(ippa+1,format='(I2)')+' of 4'
	case ilast of
		0:	begin
				group_colors[ig] = "goldenrod light"
				ilast = 1
			end
		1:	begin
				group_colors[ig] = "goldenrod dark"
				ilast = 0
			end
	endcase
	WCT_LED_stack, LED1_wid, group_colors
	;
	case ippa of
		0: ind = ind_0
		1: ind = ind_45
		2: ind = ind_90
		3: ind = ind_135
	endcase
	;
	;	for each pixel (vectorize later)
	;
	for ix = 0, ncol-1 do begin
		for iy = 0, nrow-1 do begin
			;
			;	get all intensities in the IPPA, compute uncertainty vector
			;
			inten = REFORM((*ptr_hwp).hwp_image[ix,iy,ind])		; 20120424 DPC added Reform
			unc = (*ptr_hwp).hwp_sky[ind]
			;
			;	propagate uncertainties, after testing for valid values
			;
			ind_good = WHERE(FINITE(inten) and FINITE(unc),ngood)
			;
			if(ngood eq 0) then begin
				(*ptr_inten)[ippa,ix,iy] = -1.0e6
				(*ptr_uncer)[ippa,ix,iy] = 1.0e6
				goto, pixel_loop_end
			endif
			if(ngood lt 4) then begin
				unc = unc[ind_good]
				inten = inten[ind_good]
			endif
			;
			unc = sqrt((unc + inten) / 8.21)		; 8.21 is conversion gain for Mimir
			;
			;	next, evaluate each inten to be sure it is w/i 7 sigma of others
			;
			zz = median_filtered_mean(inten)
			;
			if(~FINITE(zz[0]) or ~FINITE(zz[1])) then begin
				(*ptr_inten)[ippa,ix,iy] = -1.0e6
				(*ptr_uncer)[ippa,ix,iy] = 1.0e6
				goto, pixel_loop_end
			endif
			;
			ind_better = WHERE(abs(inten-zz[0])/zz[1] le 7.0,nbetter)
			if(nbetter eq 0) then begin
				(*ptr_inten)[ippa,ix,iy] = -1.0e6
				(*ptr_uncer)[ippa,ix,iy] = 1.0e6
				goto, pixel_loop_end
			endif
			;
			;	form weighted mean
			;
			wt = 0.0d0
			av = 0.0d0
			for ib = 0, nbetter-1 do begin
				this_ind = ind_better[ib]
				av += inten[this_ind] / (unc[this_ind]^2)
				wt += 1.0 / (unc[this_ind]^2)
			endfor
			;
			(*ptr_inten)[ippa,ix,iy] = av / wt
			(*ptr_uncer)[ippa,ix,iy] = 1.0 / sqrt(wt)
			;
			pixel_loop_end:
		endfor
	endfor
endfor
;
;	have IPPA intensity and uncertainty images, form Q, U, and uncertainty images
;
ind_good_02 = WHERE( (*ptr_inten)[0,*,*] gt -1.0e5 and (*ptr_inten)[2,*,*] gt -1.0e5,$
			ngood02,COMPLEMENT=ind_bad_02,NCOMPLEMENT=nbad02)
;
(*ptr_QU).I02_image = REFORM( (*ptr_inten)[0,*,*])
(*ptr_QU).I02_image += REFORM( (*ptr_inten)[2,*,*])
;
(*ptr_QU).Q_image = REFORM( (*ptr_inten)[0,*,*])
(*ptr_QU).Q_image -= REFORM( (*ptr_inten)[2,*,*])
;
if(ngood02 gt 0) then (*ptr_QU).Q_image[ind_good_02] /= abs((*ptr_QU).I02_image[ind_good_02])
if(nbad02 gt 0) then  (*ptr_QU).Q_image[ind_bad_02] = 0.0
;
if(ngood02 gt 0) then begin
	(*ptr_QU).sQ_image = sqrt( REFORM( (*ptr_uncer)[0,*,*]^2) + REFORM( (*ptr_uncer)[2,*,*]^2) )
	(*ptr_QU).sQ_image[ind_good_02] /= abs((*ptr_QU).I02_image[ind_good_02])
endif
if(nbad02 gt 0) then (*ptr_QU).sQ_image[ind_bad_02] = 1.0e6
;
ind_good_13 = WHERE( (*ptr_inten)[1,*,*] gt -1.0e5 and (*ptr_inten)[3,*,*] gt -1.0e5,$
			ngood13,COMPLEMENT=ind_bad_13,NCOMPLEMENT=nbad13)
;
(*ptr_QU).I13_image = REFORM( (*ptr_inten)[1,*,*])
(*ptr_QU).I13_image += REFORM( (*ptr_inten)[3,*,*])
;
(*ptr_QU).U_image = REFORM( (*ptr_inten)[1,*,*])
(*ptr_QU).U_image -= REFORM( (*ptr_inten)[3,*,*])
;
if(U_flip_flag eq 1) then (*ptr_QU).U_image *= -1.0
;
if(ngood13 gt 0) then (*ptr_QU).U_image[ind_good_13] /= abs((*ptr_QU).I13_image[ind_good_13])
if(nbad13 gt 0) then (*ptr_QU).U_image[ind_bad_13] = 0.0
;
if(ngood13 gt 0) then begin
	(*ptr_QU).sU_image = sqrt( REFORM( (*ptr_uncer)[1,*,*]^2) + REFORM( (*ptr_uncer)[3,*,*]^2) )
	(*ptr_QU).su_image[ind_good_13] /= abs((*ptr_QU).I13_image[ind_good_13])
endif
if(nbad13 gt 0) then (*ptr_QU).sU_image[ind_bad_13] = 1.0e6
;
;	average intensities
;
ind_good_I = WHERE( (*ptr_QU).sQ_image lt 1.0e5 and (*ptr_QU).sU_image lt 1.0e5, n_good_i, $
				COMPLEMENT=ind_bad_I, NCOMPLEMENT=n_bad_I)
;
(*ptr_QU).I_image = (*ptr_QU).I02_image
(*ptr_QU).I_image += (*ptr_QU).I13_image
(*ptr_QU).I_image /= 2.0
;
if(n_bad_I gt 0) then (*ptr_QU).I_image[ind_bad_I] = -1.0e6
;
group_colors[ig] = "yellow"
WCT_LED_stack, LED1_wid, group_colors
;
;	write all to disk
;
test_fits = STRPOS( STRUPCASE(file_in), 'DEEP_COADD.FITS')
if(test_fits ne -1) then begin
	QU_root = STRMID( file_in, 0, test_fits)
endif else begin
	print_text, Event, "ERROR - Could not parse filename"
	status = -1
	goto, S11CI_end
endelse
;
this_header = HEADFITS( deep_path )
;
Q_path = dir_hwp + PATH_SEP() + QU_root + 'Q.fits'
q_header = this_header
SXADDPAR, q_header, 'S11IMG','Q','The Q Image'
writefits, Q_path, (*ptr_QU).Q_image, q_header
;
sQ_path = dir_hwp + PATH_SEP() + QU_root + 'sQ.fits'
sq_header = this_header
SXADDPAR, sq_header, 'S11IMG','sQ','The Q uncertainty image'
writefits, sQ_path, (*ptr_QU).sQ_image, sq_header
;
U_path = dir_hwp + PATH_SEP() + QU_root + 'U.fits'
u_header = this_header
SXADDPAR, u_header, 'S11IMG','U','The U image'
writefits, U_path, (*ptr_QU).U_image, u_header
;
sU_path = dir_hwp + PATH_SEP() + QU_root + 'sU.fits'
su_header = this_header
SXADDPAR, su_header, 'S11IMG','sU','The U uncertainty image'
writefits, sU_path, (*ptr_QU).sU_image, su_header
;
I_path = dir_hwp + PATH_SEP() + QU_root + 'Intensity.fits'
i_header = this_header
SXADDPAR, i_header, 'S11IMG','I','The Intensity image'
writefits, I_path, (*ptr_QU).I_image, i_header
;
print_text, Event, 'Wrote Stokes Images to disk'
print_text, Event, 'Finished Computing Raw Q, U images'
;
;	clear all heap variables
;
PTR_FREE, ptr_QU, ptr_inten
;
status = 0
;
S11CI_end:
;
end