pro S11_remove_backgrounds, Event, deep_path, group_colors, ig, status
;
;	reads in the file of background points and info from
;	the background fitting of the deep image,
;	then, for each of the HWP coadds:
;
;	reads image, fits background, removes background,
;	saves mean of background in header, writes image
;	to S11 directory
;
;	DPC		20110829
;	DPC   20140319  updated to include moving grid points to match astrometry
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
w_ptr = PTR_struct.PTR_wid_id
s_ptr = PTR_struct.PTR_summary
g_ptr = PTR_struct.PTR_group
LED1_wid = (*w_ptr).S11_LED1_groups_wid
;
;	parse name to get background file
;
dir_in = FILE_DIRNAME( deep_path )
file_in = FILE_BASENAME( deep_path )
;
test_fits = STRPOS( STRUPCASE(file_in), '.FITS')
if(test_fits ne -1) then begin
	bg_file = STRMID( file_in, 0, test_fits) + '_bg_points.dat'
endif else begin
	print_text, Event, "ERROR - Could not parse filename"
	status = -1
	goto, S11RB_end
endelse
;
;		and directory
;
test_S7 = STRPOS( dir_in, 'S7_' )
if(test_S7 ne -1) then begin
	new_dir = STRMID( dir_in, 0, test_S7 ) + 'S11_Full_Field_Polarimetry'
	test = FILE_INFO(new_dir)
	if(test.exists eq 1 and test.directory eq 1) then begin
		dir_bg = new_dir
	endif else begin
		print_text, Event, "ERROR - S11 Directory does not exist"
		status = -1
		goto, S11RB_end
	endelse
endif else begin
	print_text, Event, "ERROR - Deep filename does not contain 'S7_' "
	status = -1
	goto, S11RB_end
endelse
;
path_bg = dir_bg + PATH_SEP() + bg_file
;
group_colors[ig] = "violet"
WCT_LED_stack, LED1_wid, group_colors
;
;	read
;
openr, lu_in, path_bg, /GET_LUN
;
header = strarr(100)
nh = 0L
xy_points = intarr(1000,2)
nxy = 0
;
WHILE ~EOF(lu_in) do begin
	line = ''
	readf, lu_in, line, format='(a)'
	;
	;	skip lines with double semicolons, keep lines with single colons to signify FITS HEADER - like line
	;
	test = STRPOS( line, ';;' )
	if(test eq -1) then begin
		test2 = STRPOS( line, ';' )
		if(test2 ne -1) then begin
			header[nh] = STRMID( line, 1)
			nh++
		endif else begin
			reads, line, i, x, y
			xy_points[nxy,0] = x
			xy_points[nxy,1] = y
			nxy++
		endelse
	endif
ENDWHILE
;
FREE_LUN, lu_in
print_text, Event, 'Read in '+STRING(nh,format='(i3)')+' lines and '+STRING(nxy, format='(i4)')+' data points'
;
;	extract key quantities from the header  
;
header = header[0:nh-1]
radius = SXPAR( header, 'GRIDRADI')
order = SXPAR( header, 'FITORDER' )
npts = SXPAR( header, 'NPTS')
;
; DPC 20140319 *** dpc add astrometry variables ***
; 
crpix1 = SXPAR( header, 'CRPIX1', COUNT=a1)
crpix2 = SXPAR( header, 'CRPIX2', COUNT=a2)
crval1 = SXPAR( header, 'CRVAL1', COUNT=a3)
crval2 = SXPAR( header, 'CRVAL2', COUNT=a4)
lonpole = SXPAR( header, 'LONPOLE', COUNT=a5)
latpole = SXPAR( header, 'LATPOLE', COUNT=a6)
;
if(a1+a2+a3+a4+a5+a6 ne 6) then begin
    print_text, Event, 'WARNING - grid points file does not contain full astrometry info'
endif
;
;	check that npts and nxy agree
;
if(npts ne nxy) then begin
	print_text, Event, 'ERROR - number of data points read does not match header number'
	status = -1
	goto, S11RB_end
endif
;
;	OK, points, radius, order all read in
;
;	begin loop of creating file name to HWP images, processing backgrounds, and saving
;	to the S11 directory
;
print_text, Event, 'Beginning HWP image processing loop'
;
;
ind = STRPOS( file_in, 'Deep_Coadd')
;
if(ind eq -1) then begin
	print_text, Event, 'ERROR - failed to find "Deep_Coadd" in file name'
	status = -1
	goto, S11RB_end
endif
;
; DPC 20140319 fetch the astrometry info from the Deep_Coadd image
; 
deep_header = HEADFITS(deep_path)
;
EXTAST, deep_header, deep_astr, noparams
if(noparams eq -1) then stop
;
print_text, Event, 'Successfully read '+file_in+' in S11_remove_backgrounds'
;
file_root = STRMID( file_in, 0, ind)
;
ilast = 0
for iHWP = 0, 15 do begin
	if(ilast eq 0) then begin
		group_colors[ig] = "rosybrown light"
		ilast = 1
	endif else begin
		group_colors[ig] = "rosybrown dark"
		ilast = 0
	endelse
	WCT_LED_stack, LED1_wid, group_colors
	;
	;
	this_image_in = dir_in + PATH_SEP() + file_root + 'HWP_'+ STRING( iHWP, format='(i2.2)') + '.fits'
	;
	print_text, Event, 'Starting background fitting of '+FILE_BASENAME(this_image_in)
	;
	ptr_image = PTR_NEW( readfits( this_image_in, this_header, /silent))
	;
	isize = SIZE( (*ptr_image) )
	ncol = isize[1]
	nrow = isize[2]
	;
	; get astrometry information for this image to allow grid point selections
	; 
	EXTAST, this_header, this_astr
	;
	;;         **** select grid points within this image (and some guard band) **** /// actually, not needed DPC
	;
	;	fit background
	;
	;		for each point, compute mfm value of the zone
	;
	mean_xy = fltarr(1000,3)
	rms_xy = fltarr(1000)
	nmean = 0L
	;
	for i = 0, nxy-1 do begin
		;
		mean_xy[nmean,*] = 0.0
		;
		cx = xy_points[i,0]
		cy = xy_points[i,1]
		;
		;	collect all the image pixels within radius of this point
		;
		x0 = 0 > (cx - radius)
		x1 = (cx + radius) < (ncol - 1)
		y0 = 0 > (cy - radius)
		y1 = (cy + radius) < (nrow - 1)
		;
		vec = fltarr( (x1 - x0 + 1) * (y1 - y0 + 1) )
		nvec = 0L
		;
		for ix = x0, x1 do begin
			for iy = y0, y1 do begin
				;
				dx = float(ix - cx)
				dy = float(iy - cy)
				dr = sqrt(dx * dx + dy * dy)
				if(dr le radius) then begin
					vec[nvec] = (*ptr_image)[ix,iy]
					nvec++
					;
					mean_xy[nmean,0] += float(ix)
					mean_xy[nmean,1] += float(iy)
				endif
			endfor
		endfor
		;
		mean_xy[nmean,0] /= float(nvec)
		mean_xy[nmean,1] /= float(nvec)
		;
		vec = vec[0:nvec-1]
		ind = WHERE(FINITE(vec),nfinite)
		if(nfinite gt 0) then begin
			zz = median_filtered_mean(vec[ind])
			if(FINITE(zz[0]) and FINITE(zz[1])) then begin
				mean_xy[nmean,2] = zz[0]
				rms_xy[nmean] = zz[1]
				nmean++
			endif
		endif
	endfor
	;
	;	mean sky and mean rms
	;
	zz_sky = median_filtered_mean(REFORM( mean_xy[0:nmean-1,2] ))
	zz_rms = median_filtered_mean(rms_xy[0:nmean-1])
	;
	mean_sky = zz_sky[0]
	sky_noise = zz_rms[0]
	;
	;	clean out baddies
	;
	ind_good = WHERE( abs(mean_xy[0:nmean-1,2] - mean_sky)/sky_noise lt 10.0,ngood)
	if(ngood ge 0) then begin
		mean_xy = mean_xy[ind_good,*]
	endif else begin
		print_text, Event, 'Too few good points for background fit'
		goto, HWP_loop_end
	endelse
	;
	;	have all the points. Next, select maximum fitting order
	;	in both dimensions.
	;
	degree = ((ngood - order) < order) < ngood
	;
	Result = SFIT( TRANSPOSE( mean_xy), degree, KX=coeffs, /IRREGULAR, /MAX_DEGREE )
	;
	;	create the model image (from original, times zero)
	;
	ptr_model = PTR_NEW( (*ptr_image) )
	;
	this_y = findgen(nrow)+0.5
	for ix = 0, ncol-1 do begin
		this_x = float(ix)+0.5
		;
		(*ptr_model)[ix,*] = 0.0
		;
		ik = 0
		for kx = 0, degree do begin
			for ky = 0, degree do begin
				if(kx+ky le degree) then begin
					(*ptr_model)[ix,*] += coeffs[ik] * (this_y^ky) * (this_x^kx)
					ik++
				endif
			endfor
		endfor
	endfor
	;
	;	also compute the difference image
	;
	ptr_diff = PTR_NEW( (*ptr_image) - (*ptr_model) )
	;
	;	put the mean background value into a new header variable
	;
	SXADDPAR, this_header,'S11AVSKY',mean_sky,'Mean sky level removed by Step 11'
	;
	;	create output file name
	;
	test_S7 = STRPOS( dir_in, 'S7_' )
	if(test_S7 ne -1) then begin
		new_dir = STRMID( dir_in, 0, test_S7 ) + 'S11_Full_Field_Polarimetry'
		test = FILE_INFO(new_dir)
		if(test.exists eq 1 and test.directory eq 1) then begin
			dir_out = new_dir
		endif else begin
			print_text, Event, "ERROR - S11 Directory does not exist"
			status = -1
			goto, S11RB_end
		endelse
	endif else begin
		print_text, Event, "ERROR - Deep filename does not contain 'S7_' "
		status = -1
		goto, S11RB_end
	endelse
	;
	this_image_out = dir_out + PATH_SEP() + file_root + 'HWP_'+ STRING( iHWP, format='(i2.2)') + '_bg.fits'
	;
	writefits, this_image_out, (*ptr_diff), this_header
	;
	print_text, Event, 'Wrote output image '+FILE_BASENAME(this_image_out)+' to S11 directory'
	;
	;	free heap
	;
	PTR_FREE, ptr_diff, ptr_model, ptr_image
	wait, 0.3
	;
	;	next HWP image
	;
	HWP_loop_end:
	;
endfor
;
;	all done
;
print_text, Event, 'Completed Background Removal of HWP images'
group_colors[ig] = "lawngreen"
WCT_LED_stack, LED1_wid, group_colors
;
status = 0
;
S11RB_end:
;
end

