pro s9_recover_field_info, Event, ind_grp, mean_jd, fwhm, n_use_zero, mean_sky, $
								sky_noise, field_pol, s_field_pol, rms_scale
;
;	collects key info pieces as needed by the s9 POLCAT writer
;
;	DPC		20110430	new code
;			20110629	added a/b ratio and scale rms
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
w_ptr = PTR_struct.PTR_wid_id
s_ptr = PTR_struct.PTR_summary
g_ptr = PTR_struct.PTR_group
;
;	get group name, analysis directory
;
group_name = (*g_ptr).group_names[ind_grp]
;
;--------------------------------------------
;
;	number of USE=0 images
;
n_in_group = (*g_ptr).group_numbers[ind_grp]
n_use_zero = ROUND(n_in_group - TOTAL( (*g_ptr).group_image_flags[ind_grp,0:n_in_group-1] ))
;
;print, 'Number of USE=0 images = ',n_use_zero
;
;----------------------------------------------
;
s5_dir = (*s_ptr).analysis_dir + 'S5_Star_Matching' + PATH_SEP()
s6_dir = (*s_ptr).analysis_dir + 'S6_Scaling' + PATH_SEP()
s7_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
;
s5_file = s5_dir + group_name + '_match.dat'
s6_file = s6_dir + group_name + '_group_scale_factors.dat'
s7_file = s7_dir + group_name + '_Deep_Coadd.fits'
;
image_start = (*g_ptr).group_images[ind_grp,0]
n_in_group = (*g_ptr).group_numbers[ind_grp]
image_last = (*g_ptr).group_images[ind_grp,n_in_group-1]
;
;	fetch JD from first, last images, form mean
;
openr, lu_match, s5_file, /GET_LUN
;
nmatch = 0L
WHILE ~EOF(lu_match) do begin
	line = ''
	readf, lu_match, line, format='(a)'
	;
	;	look for leading semicolon, skip if present
	;
	ind = STRPOS(line, ';')
	if(ind eq -1) then begin
		;
		;	nope, add to match array
		;
		if(nmatch eq 0L) then begin
			match_array = [line]
			nmatch = 1L
		endif else begin
			match_array = [match_array, line]
			nmatch++
		endelse
	endif
ENDWHILE
FREE_LUN, lu_match
;
;	parse match_array for first, last JD
;
parts_0 = STRSPLIT(match_array[0],' ',/REGEX,/EXTRACT)
parts_n = STRSPLIT(match_array[nmatch-1],' ',/REGEX,/EXTRACT)
;
jd_0 = 0.0d0
jd_n = 0.0d0
reads, parts_0[2], jd_0, format='(d)'
reads, parts_n[2], jd_n, format='(d)'
;
mean_jd = (jd_0 + jd_n) / 2.0d0
;
;print, 'JD mean = ',mean_jd
;
;-------------------------------------------------------------------
;
;	scaling quantities
;
openr, lu_scale, s6_file, /GET_LUN
;
nscale = 0L
WHILE ~EOF(lu_scale) do begin
	;
	;	collect the lines with leading semicolons to a header
	;
	line = ''
	readf, lu_scale, line, format='(a)'
	;
	;	look for leading semicolon, require to be present
	;
	ind = STRPOS(line, ';')
	ind2 = STRPOS(line, ':')		; due to an error, earlier
	if(ind ne -1 or ind2 ne -1) then begin
		;
		;	yep, add to scale array
		;
		if(nscale eq 0L) then begin
			scale_array = [STRMID(line,1)]
			nscale = 1L
		endif else begin
			scale_array = [scale_array, STRMID(line,1)]
			nscale++
		endelse
	endif else begin
		;
		;	add total scale factor to data array
		;
		parts = STRSPLIT(line,' ',/extract)
		this_scale = 0.0d0
		reads, parts[2], format='(f20.8)',this_scale
		if(N_elements(scale_vector) eq 0) then scale_vector = [this_scale] else scale_vector = [scale_vector, this_scale]
		;
	endelse
ENDWHILE
FREE_LUN, lu_scale
;
field_pol = SXPAR(scale_array, 'PP6_FPOL')
s_field_pol = SXPAR(scale_array, 'PP6_SFPL')
sky_noise = SXPAR(scale_array, 'PP6_SNOI')
;
;	compute scale rms
;
result = MOMENT(scale_vector, SDEV=rms_scale)
;
;-------------------------------------------------------------------
;
;	FWHM and sky level
;
coadd_ptr = PTR_NEW( readfits(s7_file,header,/silent) )
;
if(N_Elements(header) eq 0) then begin
	print, "ERROR - could not read file ",s7_file
	print, "Need to run individual files through to Step 7 first"
	stop
endif
;
fwhm = SXPAR(header, 'PSF_FWHM')		; in pixels
cd1_1 = SXPAR(header, 'CD1_1')
cd1_2 = SXPAR(header, 'CD1_2')
scale = sqrt(cd1_1^2 + cd1_2^2)
fwhm *= 3600.0d0 * scale				; now in arcsec
;
;	integration time
;
n_images = SXPAR(header, 'MPP_DADD')
tint = SXPAR(header, 'EXPTIME')
tread = SXPAR(header, 'MB_RDTIM')
t_eff = float(n_images) * (tint + tread)	; in sec
;
;	get sky background
;
y = model_background( (*coadd_ptr), /LOW_ORDER, SKY=sky)
;
mean_sky = sky / t_eff
;
;	done
;
end

