function ski_jump_fit_sub, image_input, ski_jump_code, short_fit_flag, MODEL=model
;
;   fits ski jump for an entire image, one column
;   at a time, returns residuals as an image
;
;   DPC 20050613     original code
;       20060129     modified to work with subframing
;		20081204	 modified to work in PPOL package
;						can return model image
;		20090129	 changed "swath" to "picket fence" to improve
;						bright star rejection
;		20091209	 added bad pixel detection and sidelining, to account for new col clock detection and tuned fit parameters
;		20100108	 improved fit for second order ski jump
;		20100109	 added ski_jump_code keyword (00, 10, 11, 02, 20, 22)
;		20111211	 added short_fit_flag keyword for fitting very short (in terms of rows) ski jumps - for extended objects
;
clean_image = image_input
;clean_image = fix_bad_pixels(image)	; turned off DPC 20091209
;clean_image = image
;
;	normal or abnormal?
;
normal_flag = 1									; is normal
if(ski_jump_code gt 11) then normal_flag = 0	; is abnormal
;
if(short_fit_flag eq 1) then normal_flag = 1	; an override
;
model = clean_image * 0.0
;
isize = size(clean_image)
im_col = isize[1]
im_row = isize[2]
;
bad_pixel_value = -1.0e6
cwidth = 13						; half width of column swath
pitch = 3						; keep only every 4 columns to try to step over stars
;
;if(normal_flag eq 0) then begin
;	cwidth = 11
;	patch = 2
;endif
;
cwidth = cwidth * pitch			; widen by pitch
;
center_y = (im_row - 2) / 2
;
if(im_col eq 1024 and im_row eq 1026) then begin
    ;
    if(normal_flag eq 1) then begin
    	;
    	if(short_fit_flag eq 0) then begin
    	;
	    	;	the normal set
	    	;
	    	;   full frame; establish 4 zones and boundaries
		    ;
		    nzones = 4
		    fit_region = intarr(2,nzones)
		    fit_order = intarr(nzones)
		    apply_region = intarr(2,nzones)     ; [start:stop, zone number]
	    	;
		    fit_region[0,0] = 0
		    fit_region[1,0] = 12		; was 15, then 10
		    apply_region[0,0] = 0
		    apply_region[1,0] = 8		; was 10
		    fit_order[0] = 5
		    ;
		    fit_region[0,1] = 8			; was 10, then 6
		    fit_region[1,1] = 60		; was 75, then 80
		    apply_region[0,1] = 9		; was 11
		    apply_region[1,1] = 45		; was 60
		    fit_order[1] = 7			; was 6, then 8
		    ;
		    fit_region[0,2] = 30		; was 50
		    fit_region[1,2] = 350
		    apply_region[0,2] = 46
		    apply_region[1,2] = 300
		    fit_order[2] = 6			; was 5
		    ;
		    fit_region[0,3] = 250
		    fit_region[1,3] = 511
		    apply_region[0,3] = 301
		    apply_region[1,3] = 511
		    fit_order[3] = 4
		endif else begin
			;
			;	the short fit set
			;
			;   2 zones and boundaries
		    ;
		    nzones = 3
		    fit_region = intarr(2,nzones)
		    fit_order = intarr(nzones)
		    apply_region = intarr(2,nzones)     ; [start:stop, zone number]
	    	;
		    fit_region[0,0] = 0
		    fit_region[1,0] = 12
		    apply_region[0,0] = 0
		    apply_region[1,0] = 8
		    fit_order[0] = 4
		    ;
		    fit_region[0,1] = 6
		    fit_region[1,1] = 25
		    apply_region[0,1] = 9
		    apply_region[1,1] = 20
		    fit_order[1] = 3
		    ;
		    fit_region[0,2] = 15
		    fit_region[1,2] = 511
		    apply_region[0,2] = 21
		    apply_region[1,2] = 511
		    fit_order[2] = 1
		    ;
		endelse
		;
	endif else begin
		;
		print, 'fitting abnormal jump'
		;
		;	the abnormal set
		;
		nzones = 6
	    fit_region = intarr(2,nzones)
	    fit_order = intarr(nzones)
	    apply_region = intarr(2,nzones)     ; [start:stop,
	    ;
	    reg = 0
		fit_region[0,reg] = 5
	    fit_region[1,reg] = 12
	    apply_region[0,reg] = 5
	    apply_region[1,reg] = 9
	    fit_order[reg] = 5
	    ;
	    reg++
	    fit_region[0,reg] = 8			; was 10, then 6
	    fit_region[1,reg] = 60		; was 75, then 80
	    apply_region[0,reg] = 10		; was 11
	    apply_region[1,reg] = 45		; was 60
	    fit_order[reg] = 7			; was 6, then 8
	    ;
	    reg++
	    fit_region[0,reg] = 35		; was 50
	    fit_region[1,reg] = 90
	    apply_region[0,reg] = 46
	    apply_region[1,reg] = 75
	    fit_order[reg] = 6			; was 5
	    ;
	    reg++
	    fit_region[0,reg] = 60		; was 50
	    fit_region[1,reg] = 160
	    apply_region[0,reg] = 76
	    apply_region[1,reg] = 95
	    fit_order[reg] = 6			; was 5
	    ;
	    reg++
	    fit_region[0,reg] = 70		; was 50
	    fit_region[1,reg] = 350
	    apply_region[0,reg] = 96
	    apply_region[1,reg] = 300
	    fit_order[reg] = 7			; was 5
	    ;
	    reg++
	    fit_region[0,reg] = 250
	    fit_region[1,reg] = 511
	    apply_region[0,reg] = 301
	    apply_region[1,reg] = 511
	    fit_order[reg] = 4
	endelse
endif else begin
	print_text, Event, "ERROR - didn't find full frame image"
    ;
    ;  sub frame; details depend on row number
    ;
;    if(im_row ge 256) then begin
;        nzones = 3
;        fit_region = intarr(2,nzones)
;        fit_order = intarr(nzones)
;        apply_region = intarr(2,nzones)
;        ;
;        fit_region[0,0] = 0
;        fit_region[1,0] = 12
;        apply_region[0,0] = 0
;        apply_region[1,0] = 8
;        fit_order[0] = 6
;        ;
;        fit_region[0,1] = 8
;        fit_region[1,1] = 35
;        apply_region[0,1] = 9
;        apply_region[1,1] = 28
;        fit_order[1] = 6
;        ;
;        fit_region[0,2] = 30
;        fit_region[1,2] = center_y-2
;        apply_region[0,2] = 29
;        apply_region[1,2] = center_y - 1
;        fit_order[2] = 2
;    endif else begin
;        nzones = 3
;        fit_region = intarr(2,nzones)
;        fit_order = intarr(nzones)
;        apply_region = intarr(2,nzones)
;        ;
;        fit_region[0,0] = 0
;        fit_region[1,0] = 12
;        apply_region[0,0] = 0
;        apply_region[1,0] = 8
;        fit_order[0] = 6
;        ;
;        fit_region[0,1] = 8
;        fit_region[1,1] = 20
;        apply_region[0,1] = 9
;        apply_region[1,1] = 18
;        fit_order[1] = 6
;        ;
;        fit_region[0,2] = 18
;        fit_region[1,2] = center_y-2
;        apply_region[0,2] = 19
;        apply_region[1,2] = center_y - 1
;        fit_order[2] = 2
;    endelse
endelse
;
fixed = model        ; clear result array
xmodel = findgen(center_y)
ymodel = fltarr(center_y)
yresid = ymodel
;
;   big loop on the columns in the image
;
;column_vec = dblarr(im_row)
column_vec = fltarr(im_row)
result = column_vec * 0.0
;
for icol = 0, im_col-1 do begin
	;
	;	clear buffers
	;
	ymodel = fltarr(center_y)
	yresid = ymodel
	;
;for icol = (im_col-1), 0, -1 do begin
	;
	;	column zone to extract
	;
	c1 = 0 > (icol-cwidth)
	c2 = (im_col-1) < (icol+cwidth)
    ;
    ;	determine the mean column cut by doing mfm at each row
    ;          ;
    iswath = clean_image[c1:c2:pitch,*]
    ;
    for irow = 0, im_row-1 do begin
    	rvec = REFORM(iswath[*,irow])
    	ind = WHERE(rvec ne bad_pixel_value, ngd)
    	if(ngd gt 0) then begin
	    	zz = median_filtered_mean(rvec[ind])
	    	;
	    	column_vec[irow] = zz[0]			; mean of these columns of data at this location
	    endif else column_vec[irow] = bad_pixel_value
    endfor
    ;
    ;column_vec = image[icol,0:im_row-1]	; - the old way - this column only
    cswath = clean_image[icol,0:im_row-1]
    data_column_vec = REFORM(cswath)
    ;
    cut_L = column_vec[0:center_y-1]					 ; skipping top two rows
    cut_U = dpc_reverse(column_vec[center_y:im_row-3])   ; swap the upper half order to look like lower half
    ;
    data_cut_L = data_column_vec[0:center_y-1]
    data_cut_U = dpc_reverse(data_column_vec[center_y:im_row-3])	; these are the unsmoothed data
    ;
;    if(icol eq 36) then stop
;    if(icol eq 60) then stop
;    if(icol eq 974) then stop
;    if(icol eq 1020) then stop
    ;
    ;   fit each in four zones - as per discovery of shape of ski jump
    ;
    ;---------------------------------------------------------------
    ;
    ;   for subframing, and especially after we tuned the detector in 2005/11,
    ;   probably good enough to fit 0-10 with 4th order, 11-center with 2nd order
    ;
    for iquad = 0, 1 do begin
        ;
        case iquad of
            0: begin
            	y = cut_U
            	data_y = data_cut_U
               end
            1: begin
            	y = cut_L
            	data_y = data_cut_L
               end
        endcase
       ;
       ;	fit to smoothed (mfm) column behavior
       ;
        for izone = 0, nzones-1 do begin
            xfit = fit_region[0,izone] + findgen(fit_region[1,izone] - fit_region[0,izone] + 1)
            y_sub = y[fit_region[0,izone]:fit_region[1,izone]]
            order = fit_order[izone]
         	low_fit = apply_region[0,izone]
         	high_fit = apply_region[1,izone]
         	;
         	;	test that the y values aren't all deleted
         	;
         	ind_good = WHERE(y_sub ne bad_pixel_value, ngood)
         	if(ngood gt 0) then begin
         		;
	            ;	loop 2 times to reject pixels more than 7 sigma out of mean of region
	            ;
	            for iloop = 0, 1 do begin
	            	yind = WHERE(y_sub ne bad_pixel_value, nok)
	            	if(nok gt 0) then begin
	            		zz = median_filtered_mean(y_sub[yind])
	            		bad_ind = WHERE(abs(y_sub - zz[0])/zz[1] gt 7.0,n_bad)
	            		if(n_bad gt 0) then begin
	            			y_sub[bad_ind] = bad_pixel_value
	            		endif
	            	endif
	            endfor
	            ;
	            ;   loop 3 times to reject pixels more than 5 sigma out of mean of fit
	            ;
;	            ninzone = N_ELEMENTS(xfit)
	            coeffs_sub = dblarr(order+1)		; clear to zero
	            for iloop = 0, 2 do begin
	                yind = WHERE(y_sub ne bad_pixel_value,tcount)
	                if(tcount ne 0) then begin
		                coeffs_sub = POLY_FIT(xfit[yind], y_sub[yind], order, /DOUBLE, YERROR=sigma, $
		                		YFIT=yfit, STATUS=status)
		                if(status eq 0 or status eq 2) then begin
		                	ybad = WHERE(abs(y_sub[yind] - yfit)/sigma gt 5.,count)
		                	if(count gt 0) then y_sub[yind[ybad]] = bad_pixel_value else goto, out
		                endif else begin
		                	coeffs_sub = dblarr(order+1) * 0.0
		                endelse
		            endif else begin
		            	coeffs_sub = dblarr(order+1) * 0.0
		            endelse
	            endfor
	            out:
	            ;
	            ;  apply the coefficients to the fit range for this particular column
	            ;
	            for ipix = low_fit, high_fit do begin
	            	ymodel[ipix] = 0.0
	                for ipower = 0, order do begin
	                    ymodel[ipix] += coeffs_sub[ipower] * (xmodel[ipix]^ipower)
	                endfor
	            endfor
	            ;
	            yresid[low_fit: high_fit] = data_y[low_fit: high_fit]	;	to retain bad pixels, if there
	            jind = WHERE(data_y[low_fit:high_fit] ne bad_pixel_value, jgood)
	            if(jgood gt 0) then begin
	            	;
	        		;	apply model to data values, but respect bad pixels
	        		;
	        		yresid[low_fit+jind] = data_y[low_fit+jind] - ymodel[low_fit+jind]
	        	endif
	        endif else begin
	        	yresid[low_fit:high_fit] = bad_pixel_value
	        	ymodel[low_fit:high_fit] = bad_pixel_value
        	endelse
            ;
            ;print, yresid[low_fit:high_fit]
            ;print, xmodel[low_fit:high_fit]
            ;print, yfit[yind]
            ;print, xfit[yind]
            ;
            ;window, 10
            ;plot, xfit[yind],y_sub[yind],yrange=[-0.1*max(y_sub[yind]),1.2*max(y_sub[yind])],color=(255*2^16)+(255*2^8)+255
            ;oplot, xfit[yind],yfit
            ;oplot, xfit[yind],y_sub[yind]-yfit-0.05*max(y_sub[yind])
            ;oplot, xmodel[low_fit:high_fit],yresid[low_fit:high_fit], color=100
            ;cursor, xx, yy, /wait
            ;wait, 1
            ;wdelete, 10
        endfor
        ;
        ;   pack into result array
        ;
        case iquad of
            0: begin
                yresid = dpc_reverse(yresid)
                ymodel = dpc_reverse(ymodel)
                result[center_y:im_row-3] = yresid[0:center_y-1]
                model[icol,center_y:im_row-3] = ymodel[0:center_y-1]
               end
            1: begin
            	result[0:center_y-1] = yresid[0:center_y-1]
            	model[icol,0:center_y-1] = ymodel[0:center_y-1]
               end
        endcase
    endfor
;    result[im_row-2:im_row-1] = bad_pixel_value
;    model[icol,im_row-2:im_row-1] = bad_pixel_value
    ;
    fixed[icol,0:im_row-1] = result
    ;
endfor
;
the_end:
;
;	mask, as per normal or abnormal
;
if(normal_flag) then begin
	fixed[*,im_row-10:im_row-1] = bad_pixel_value
	fixed[*,0:8] = bad_pixel_value
	;
	model[*,im_row-10:im_row-1] = bad_pixel_value
	model[*,0:8] = bad_pixel_value
endif else begin
	fixed[*,im_row-12:im_row-1] = bad_pixel_value
	fixed[*,0:9] = bad_pixel_value
	;
	model[*,im_row-12:im_row-1] = bad_pixel_value
	model[*,0:9] = bad_pixel_value
endelse
;
return, fixed
end
